@@ -1,7 +1,7 @@
 /* capture.c
  * Routines for packet capture windows
  *
- * $Id: capture.c,v 1.106 2000/05/26 22:08:13 guy Exp $
+ * $Id: capture.c,v 1.122 2000/08/22 06:04:40 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -50,7 +50,10 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
+
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -74,11 +77,6 @@
 #include <errno.h>
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
@@ -86,6 +84,10 @@
 #include <pcap.h>
 #endif
 
+#ifdef _WIN32
+#include <process.h>    /* For spawning child process */
+#endif
+
 #include "gtk/main.h"
 #include "gtk/gtkglobals.h"
 #include "packet.h"
@@ -96,6 +98,10 @@
 #include "prefs.h"
 #include "globals.h"
 
+#include "wiretap/libpcap.h"
+#include "wiretap/wtap-int.h"
+
+#include "packet-clip.h"
 #include "packet-eth.h"
 #include "packet-fddi.h"
 #include "packet-null.h"
@@ -105,10 +111,17 @@
 
 int sync_mode;	/* fork a child to do the capture, and sync between them */
 static int sync_pipe[2]; /* used to sync father */
+enum PIPES { READ, WRITE }; /* Constants 0 and 1 for READ and WRITE */
 int quit_after_cap; /* Makes a "capture only mode". Implies -k */
 gboolean capture_child;	/* if this is the child for "-S" */
+static int fork_child;	/* In parent, process ID of child */
 static guint cap_input_id;
 
+#ifdef _WIN32
+static guint cap_timer_id;
+static int cap_timer_cb(gpointer); /* Win32 kludge to check for pipe input */
+#endif
+
 static void cap_file_input_cb(gpointer, gint, GdkInputCondition);
 static void capture_delete_cb(GtkWidget *, GdkEvent *, gpointer);
 static void capture_stop_cb(GtkWidget *, gpointer);
@@ -122,15 +135,29 @@ typedef struct _loop_data {
   gint           max;
   gint           linktype;
   gint           sync_packets;
+  gboolean       from_pipe;    /* TRUE if we are capturing data from a pipe */
+  gboolean       modified;     /* TRUE if data in the pipe uses modified pcap headers */
+  gboolean       byte_swapped; /* TRUE if data in the pipe is byte swapped */
   packet_counts  counts;
   wtap_dumper   *pdh;
 } loop_data;
 
+#ifndef _WIN32
+static void adjust_header(loop_data *, struct pcap_hdr *, struct pcaprec_hdr *);
+static int pipe_open_live(char *, struct pcap_hdr *, loop_data *, char *);
+static int pipe_dispatch(int, loop_data *, struct pcap_hdr *);
+#endif
+
 /* Win32 needs the O_BINARY flag for open() */
 #ifndef O_BINARY
 #define O_BINARY	0
 #endif
 
+#ifdef _WIN32
+/* Win32 needs a handle to the child capture process */
+int child_process;
+#endif
+
 /* Open a specified file, or create a temporary file, and start a capture
    to the file in question. */
 void
@@ -147,44 +174,84 @@ do_capture(char *capfile_name)
 
   if (capfile_name != NULL) {
     /* Try to open/create the specified file for use as a capture buffer. */
-    cf.save_file_fd = open(capfile_name, O_RDWR|O_BINARY|O_TRUNC|O_CREAT, 0600);
+    cfile.save_file_fd = open(capfile_name, O_RDWR|O_BINARY|O_TRUNC|O_CREAT, 0600);
     is_tempfile = FALSE;
   } else {
     /* Choose a random name for the capture buffer */
-    cf.save_file_fd = create_tempfile(tmpname, sizeof tmpname, "ether");
+    cfile.save_file_fd = create_tempfile(tmpname, sizeof tmpname, "ether");
     capfile_name = g_strdup(tmpname);
     is_tempfile = TRUE;
   }
-  if (cf.save_file_fd == -1) {
-    simple_dialog(ESD_TYPE_WARN, NULL,
-	"The file to which the capture would be saved (\"%s\")"
+  if (cfile.save_file_fd == -1) {
+    if (is_tempfile) {
+      simple_dialog(ESD_TYPE_CRIT, NULL,
+	"The temporary file to which the capture would be saved (\"%s\")"
 	"could not be opened: %s.", capfile_name, strerror(errno));
+    } else {
+      simple_dialog(ESD_TYPE_CRIT, NULL,
+	file_open_error_message(errno, TRUE), capfile_name);
+    }
     return;
   }
-  close_cap_file(&cf, info_bar);
-  g_assert(cf.save_file == NULL);
-  cf.save_file = capfile_name;
+  close_cap_file(&cfile, info_bar);
+  g_assert(cfile.save_file == NULL);
+  cfile.save_file = capfile_name;
 
   if (sync_mode) {	/* do the capture in a child process */
-#ifndef _WIN32
-    int  fork_child;
     char ssnap[24];
     char scount[24];	/* need a constant for len of numbers */
     char save_file_fd[24];
     char errmsg[1024+1];
     int error;
+#ifdef _WIN32
+    char sync_pipe_fd[24];
+    char *filterstring;
+#endif
 
-    sprintf(ssnap,"%d",cf.snap); /* in lieu of itoa */
-    sprintf(scount,"%d",cf.count);
-    sprintf(save_file_fd,"%d",cf.save_file_fd);
+    sprintf(ssnap,"%d",cfile.snap); /* in lieu of itoa */
+    sprintf(scount,"%d",cfile.count);
+    sprintf(save_file_fd,"%d",cfile.save_file_fd);
+
+#ifdef _WIN32
+    /* Create a pipe for the child process */
+
+    if(_pipe(sync_pipe, 512, O_BINARY) < 0) {
+      /* Couldn't create the pipe between parent and child. */
+      error = errno;
+      unlink(cfile.save_file);
+      g_free(cfile.save_file);
+      cfile.save_file = NULL;
+      simple_dialog(ESD_TYPE_CRIT, NULL, "Couldn't create sync pipe: %s",
+                        strerror(error));
+      return;
+    }
+
+    /* Convert pipe write handle to a string and pass to child */
+    itoa(sync_pipe[WRITE], sync_pipe_fd, 10);
+    /* Convert filter string to a quote delimited string */
+    filterstring = g_new(char, strlen(cfile.cfilter) + 3);
+    sprintf(filterstring, "\"%s\"", cfile.cfilter);
+    filterstring[strlen(cfile.cfilter) + 2] = 0;
+    /* Spawn process */
+    fork_child = spawnlp(_P_NOWAIT, ethereal_path, CHILD_NAME, "-i", cfile.iface,
+                         "-w", cfile.save_file, "-W", save_file_fd,
+                         "-c", scount, "-s", ssnap,
+                         "-Z", sync_pipe_fd,
+                         strlen(cfile.cfilter) == 0 ? (const char *)NULL : "-f",
+                         strlen(cfile.cfilter) == 0 ? (const char *)NULL : filterstring,
+                         (const char *)NULL);
+    g_free(filterstring);
+    /* Keep a copy for later evaluation by _cwait() */
+    child_process = fork_child;
+#else
     signal(SIGCHLD, SIG_IGN);
     if (pipe(sync_pipe) < 0) {
       /* Couldn't create the pipe between parent and child. */
       error = errno;
-      unlink(cf.save_file);
-      g_free(cf.save_file);
-      cf.save_file = NULL;
-      simple_dialog(ESD_TYPE_WARN, NULL, "Couldn't create sync pipe: %s",
+      unlink(cfile.save_file);
+      g_free(cfile.save_file);
+      cfile.save_file = NULL;
+      simple_dialog(ESD_TYPE_CRIT, NULL, "Couldn't create sync pipe: %s",
 			strerror(error));
       return;
     }
@@ -203,14 +270,14 @@ do_capture(char *capfile_name)
        * -f "filter expression"
        */
       close(1);
-      dup(sync_pipe[1]);
-      close(sync_pipe[0]);
-      execlp(ethereal_path, CHILD_NAME, "-i", cf.iface,
-		"-w", cf.save_file, "-W", save_file_fd,
+      dup(sync_pipe[WRITE]);
+      close(sync_pipe[READ]);
+      execlp(ethereal_path, CHILD_NAME, "-i", cfile.iface,
+		"-w", cfile.save_file, "-W", save_file_fd,
 		"-c", scount, "-s", ssnap, 
-		"-m", medium_font, "-b", bold_font,
-		(cf.cfilter == NULL)? 0 : "-f",
-		(cf.cfilter == NULL)? 0 : cf.cfilter,
+		"-m", prefs.gui_font_name,
+		(cfile.cfilter == NULL)? 0 : "-f",
+		(cfile.cfilter == NULL)? 0 : cfile.cfilter,
 		(const char *)NULL);	
       snprintf(errmsg, sizeof errmsg, "Couldn't run %s in child process: %s",
 		ethereal_path, strerror(errno));
@@ -222,6 +289,7 @@ do_capture(char *capfile_name)
 	 our parent). */
       _exit(2);
     }
+#endif
 
     /* Parent process - read messages from the child process over the
        sync pipe. */
@@ -230,41 +298,41 @@ do_capture(char *capfile_name)
        open, and thus it completely closes, and thus returns to us
        an EOF indication, if the child closes it (either deliberately
        or by exiting abnormally). */
-    close(sync_pipe[1]);
+    close(sync_pipe[WRITE]);
 
     /* Close the save file FD, as we won't be using it - we'll be opening
        it and reading the save file through Wiretap. */
-    close(cf.save_file_fd);
+    close(cfile.save_file_fd);
 
     if (fork_child == -1) {
       /* We couldn't even create the child process. */
       error = errno;
-      close(sync_pipe[0]);
-      unlink(cf.save_file);
-      g_free(cf.save_file);
-      cf.save_file = NULL;
-      simple_dialog(ESD_TYPE_WARN, NULL, "Couldn't create child process: %s",
+      close(sync_pipe[READ]);
+      unlink(cfile.save_file);
+      g_free(cfile.save_file);
+      cfile.save_file = NULL;
+      simple_dialog(ESD_TYPE_CRIT, NULL, "Couldn't create child process: %s",
 			strerror(error));
       return;
     }
 
-    /* Read a byte count from "sync_pipe[0]", terminated with a
+    /* Read a byte count from "sync_pipe[READ]", terminated with a
        colon; if the count is 0, the child process created the
        capture file and we should start reading from it, otherwise
        the capture couldn't start and the count is a count of bytes
        of error message, and we should display the message. */
     byte_count = 0;
     for (;;) {
-      i = read(sync_pipe[0], &c, 1);
+      i = read(sync_pipe[READ], &c, 1);
       if (i == 0) {
 	/* EOF - the child process died.
 	   Close the read side of the sync pipe, remove the capture file,
 	   and report the failure.
 	   XXX - reap the child process and report the status in detail. */
-	close(sync_pipe[0]);
-	unlink(cf.save_file);
-	g_free(cf.save_file);
-	cf.save_file = NULL;
+	close(sync_pipe[READ]);
+	unlink(cfile.save_file);
+	g_free(cfile.save_file);
+	cfile.save_file = NULL;
 	simple_dialog(ESD_TYPE_WARN, NULL, "Capture child process died");
 	return;
       }
@@ -274,10 +342,10 @@ do_capture(char *capfile_name)
 	/* Child process handed us crap.
 	   Close the read side of the sync pipe, remove the capture file,
 	   and report the failure. */
-	close(sync_pipe[0]);
-	unlink(cf.save_file);
-	g_free(cf.save_file);
-	cf.save_file = NULL;
+	close(sync_pipe[READ]);
+	unlink(cfile.save_file);
+	g_free(cfile.save_file);
+	cfile.save_file = NULL;
 	simple_dialog(ESD_TYPE_WARN, NULL,
 			"Capture child process sent us a bad message");
 	return;
@@ -286,31 +354,40 @@ do_capture(char *capfile_name)
     }
     if (byte_count == 0) {
       /* Success.  Open the capture file, and set up to read it. */
-      err = start_tail_cap_file(cf.save_file, is_tempfile, &cf);
+      err = start_tail_cap_file(cfile.save_file, is_tempfile, &cfile);
       if (err == 0) {
 	/* We were able to open and set up to read the capture file;
 	   arrange that our callback be called whenever it's possible
 	   to read from the sync pipe, so that it's called when
 	   the child process wants to tell us something. */
-	cap_input_id = gtk_input_add_full(sync_pipe[0],
+#ifdef _WIN32
+	/* Tricky to use pipes in win9x, as no concept of wait.  NT can
+	   do this but that doesn't cover all win32 platforms.  GTK can do
+	   this but doesn't seem to work over processes.  Attempt to do
+	   something similar here, start a timer and check for data on every
+	   timeout. */
+	cap_timer_id = gtk_timeout_add(1000, cap_timer_cb, NULL);
+#else
+	cap_input_id = gtk_input_add_full(sync_pipe[READ],
 				       GDK_INPUT_READ|GDK_INPUT_EXCEPTION,
 				       cap_file_input_cb,
 				       NULL,
-				       (gpointer) &cf,
+				       (gpointer) &cfile,
 				       NULL);
+#endif
       } else {
 	/* We weren't able to open the capture file; complain, and
 	   close the sync pipe. */
-	simple_dialog(ESD_TYPE_WARN, NULL,
-			file_open_error_message(err, FALSE), cf.save_file);
+	simple_dialog(ESD_TYPE_CRIT, NULL,
+			file_open_error_message(err, FALSE), cfile.save_file);
 
 	/* Close the sync pipe. */
-	close(sync_pipe[0]);
+	close(sync_pipe[READ]);
 
 	/* Don't unlink the save file - leave it around, for debugging
 	   purposes. */
-	g_free(cf.save_file);
-	cf.save_file = NULL;
+	g_free(cfile.save_file);
+	cfile.save_file = NULL;
       }
     } else {
       /* Failure - the child process sent us a message indicating
@@ -320,7 +397,7 @@ do_capture(char *capfile_name)
 	simple_dialog(ESD_TYPE_WARN, NULL,
 		"Capture child process failed, but its error message was too big.");
       } else {
-	i = read(sync_pipe[0], msg, byte_count);
+	i = read(sync_pipe[READ], msg, byte_count);
 	if (i < 0) {
 	  simple_dialog(ESD_TYPE_WARN, NULL,
 		  "Capture child process failed: Error %s reading its error message.",
@@ -333,15 +410,14 @@ do_capture(char *capfile_name)
 	g_free(msg);
 
 	/* Close the sync pipe. */
-	close(sync_pipe[0]);
+	close(sync_pipe[READ]);
 
 	/* Get rid of the save file - the capture never started. */
-	unlink(cf.save_file);
-	g_free(cf.save_file);
-	cf.save_file = NULL;
+	unlink(cfile.save_file);
+	g_free(cfile.save_file);
+	cfile.save_file = NULL;
       }
     }
-#endif
   } else {
     /* Not sync mode. */
     capture_succeeded = capture();
@@ -351,20 +427,73 @@ do_capture(char *capfile_name)
     }
     if (capture_succeeded) {
       /* Capture succeeded; read in the capture file. */
-      if ((err = open_cap_file(cf.save_file, is_tempfile, &cf)) == 0) {
+      if ((err = open_cap_file(cfile.save_file, is_tempfile, &cfile)) == 0) {
         /* Set the read filter to NULL. */
-        cf.rfcode = NULL;
-        err = read_cap_file(&cf);
+        cfile.rfcode = NULL;
+        switch (read_cap_file(&cfile, &err)) {
+
+        case READ_SUCCESS:
+        case READ_ERROR:
+          /* Just because we got an error, that doesn't mean we were unable
+             to read any of the file; we handle what we could get from the
+             file. */
+          break;
+
+        case READ_ABORTED:
+          /* Exit by leaving the main loop, so that any quit functions
+             we registered get called. */
+          gtk_main_quit();
+          return;
+        }
       }
     }
     /* We're not doing a capture any more, so we don't have a save
        file. */
-    g_free(cf.save_file);
-    cf.save_file = NULL;
+    g_free(cfile.save_file);
+    cfile.save_file = NULL;
   }
 }
 
-#ifndef _WIN32
+#ifdef _WIN32
+/* The timer has expired, see if there's stuff to read from the pipe,
+   if so call the cap_file_input_cb */
+static gint
+cap_timer_cb(gpointer data)
+{
+  HANDLE handle;
+  DWORD avail = 0;
+  gboolean result, result1;
+  DWORD childstatus;
+
+  /* Oddly enough although Named pipes don't work on win9x,
+     PeekNamedPipe does !!! */
+  handle = (HANDLE) _get_osfhandle (sync_pipe[READ]);
+  result = PeekNamedPipe(handle, NULL, 0, NULL, &avail, NULL);
+
+  /* Get the child process exit status */
+  result1 = GetExitCodeProcess((HANDLE)child_process, &childstatus);
+
+  /* If the Peek returned an error, or there are bytes to be read
+     or the childwatcher thread has terminated then call the normal
+     callback */
+  if (!result || avail > 0 || childstatus != STILL_ACTIVE) {
+
+    /* avoid reentrancy problems and stack overflow */
+    gtk_timeout_remove(cap_timer_id);
+
+    /* And call the real handler */
+    cap_file_input_cb((gpointer) &cfile, 0, 0);
+
+    /* Return false so that the timer is not run again */
+    return FALSE;
+  }
+  else {
+    /* No data so let timer run again */
+    return TRUE;
+  }
+}
+#endif
+
 /* There's stuff to read from the sync pipe, meaning the child has sent
    us a message, or the sync pipe has closed, meaning the child has
    closed it (perhaps because it exited). */
@@ -384,13 +513,22 @@ cap_file_input_cb(gpointer data, gint source, GdkInputCondition condition)
   char sigmsg_buf[6+1+3+1];
   char *coredumped;
 
+#ifndef _WIN32
   /* avoid reentrancy problems and stack overflow */
   gtk_input_remove(cap_input_id);
+#endif
 
-  if ((nread = read(sync_pipe[0], buffer, 256)) <= 0) {
+  if ((nread = read(sync_pipe[READ], buffer, 256)) <= 0) {
     /* The child has closed the sync pipe, meaning it's not going to be
        capturing any more packets.  Pick up its exit status, and
        complain if it died of a signal. */
+#ifdef _WIN32
+    /* XXX - analyze the wait stuatus and display more information
+       in the dialog box? */
+    if (_cwait(&wstatus, child_process, _WAIT_CHILD) == -1) {
+      simple_dialog(ESD_TYPE_WARN, NULL, "Child capture process stopped unexpectedly");
+    }
+#else
     if (wait(&wstatus) != -1) {
       /* XXX - are there any platforms on which we can run that *don't*
          support POSIX.1's <sys/wait.h> and macros therein? */
@@ -488,10 +626,25 @@ cap_file_input_cb(gpointer data, gint source, GdkInputCondition condition)
 		"Child capture process %s: %s%s", msg, sigmsg, coredumped);
       }
     }
+#endif
       
     /* Read what remains of the capture file, and finish the capture.
        XXX - do something if this fails? */
-    err = finish_tail_cap_file(cf);
+    switch (finish_tail_cap_file(cf, &err)) {
+
+    case READ_SUCCESS:
+    case READ_ERROR:
+      /* Just because we got an error, that doesn't mean we were unable
+         to read any of the file; we handle what we could get from the
+         file. */
+      break;
+
+    case READ_ABORTED:
+      /* Exit by leaving the main loop, so that any quit functions
+         we registered get called. */
+      gtk_main_quit();
+      return;
+    }
 
     /* We're not doing a capture any more, so we don't have a save
        file. */
@@ -524,23 +677,266 @@ cap_file_input_cb(gpointer data, gint source, GdkInputCondition condition)
   /* Read from the capture file the number of records the child told us
      it added.
      XXX - do something if this fails? */
-  err = continue_tail_cap_file(cf, to_read);
+  switch (continue_tail_cap_file(cf, to_read, &err)) {
+
+  case READ_SUCCESS:
+  case READ_ERROR:
+    /* Just because we got an error, that doesn't mean we were unable
+       to read any of the file; we handle what we could get from the
+       file.
+
+       XXX - abort on a read error? */
+    break;
+
+  case READ_ABORTED:
+    /* Kill the child capture process; the user wants to exit, and we
+       shouldn't just leave it running. */
+#ifdef _WIN32
+    /* XXX - kill it. */
+#else
+    kill(fork_child, SIGTERM);	/* SIGTERM so it can clean up if necessary */
+#endif
+    break;
+  }
 
   /* restore pipe handler */
-  cap_input_id = gtk_input_add_full (sync_pipe[0],
+#ifdef _WIN32
+  cap_timer_id = gtk_timeout_add(1000, cap_timer_cb, NULL);
+#else
+  cap_input_id = gtk_input_add_full (sync_pipe[READ],
 				     GDK_INPUT_READ|GDK_INPUT_EXCEPTION,
 				     cap_file_input_cb,
 				     NULL,
 				     (gpointer) cf,
 				     NULL);
+#endif
 }
-#endif /* _WIN32 */
 
 /*
  * Timeout, in milliseconds, for reads from the stream of captured packets.
  */
 #define	CAP_READ_TIMEOUT	250
 
+#ifndef _WIN32
+/* Take carre of byte order in the libpcap headers read from pipes.
+ * (function taken from wiretap/libpcap.c) */
+static void
+adjust_header(loop_data *ld, struct pcap_hdr *hdr, struct pcaprec_hdr *rechdr)
+{
+  if (ld->byte_swapped) {
+    /* Byte-swap the record header fields. */
+    rechdr->ts_sec = BSWAP32(rechdr->ts_sec);
+    rechdr->ts_usec = BSWAP32(rechdr->ts_usec);
+    rechdr->incl_len = BSWAP32(rechdr->incl_len);
+    rechdr->orig_len = BSWAP32(rechdr->orig_len);
+  }
+
+  /* In file format version 2.3, the "incl_len" and "orig_len" fields were
+     swapped, in order to match the BPF header layout.
+
+     Unfortunately, some files were, according to a comment in the "libpcap"
+     source, written with version 2.3 in their headers but without the
+     interchanged fields, so if "incl_len" is greater than "orig_len" - which
+     would make no sense - we assume that we need to swap them.  */
+  if (hdr->version_major == 2 &&
+      (hdr->version_minor < 3 ||
+       (hdr->version_minor == 3 && rechdr->incl_len > rechdr->orig_len))) {
+    guint32 temp;
+
+    temp = rechdr->orig_len;
+    rechdr->orig_len = rechdr->incl_len;
+    rechdr->incl_len = temp;
+  }
+}
+
+/* Mimic pcap_open_live() for pipe captures 
+ * We check if "pipename" is "-" (stdin) or a FIFO, open it, and read the
+ * header.
+ * N.B. : we can't read the libpcap formats used in RedHat 6.1 or SuSE 6.3
+ * because we can't seek on pipes (see wiretap/libpcap.c for details) */
+static int
+pipe_open_live(char *pipename, struct pcap_hdr *hdr, loop_data *ld, char *ebuf)
+{
+  struct stat pipe_stat;
+  int         fd;
+  guint32     magic;
+  int         bytes_read, b;
+
+  if (strcmp(pipename, "-") == 0) fd = 0; /* read from stdin */
+  else if (stat(pipename, &pipe_stat) == 0 && S_ISFIFO(pipe_stat.st_mode)) {
+    if ((fd = open(pipename, O_RDONLY)) == -1) return -1;
+  } else return -1;
+
+  ld->from_pipe = TRUE;
+  /* read the pcap header */
+  if (read(fd, &magic, sizeof magic) != sizeof magic) {
+    close(fd);
+    return -1;
+  }
+
+  switch (magic) {
+  case PCAP_MAGIC:
+    /* Host that wrote it has our byte order, and was running
+       a program using either standard or ss990417 libpcap. */
+    ld->byte_swapped = FALSE;
+    ld->modified = FALSE;
+    break;
+  case PCAP_MODIFIED_MAGIC:
+    /* Host that wrote it has our byte order, but was running
+       a program using either ss990915 or ss991029 libpcap. */
+    ld->byte_swapped = FALSE;
+    ld->modified = TRUE;
+    break;
+  case PCAP_SWAPPED_MAGIC:
+    /* Host that wrote it has a byte order opposite to ours,
+       and was running a program using either standard or
+       ss990417 libpcap. */
+    ld->byte_swapped = TRUE;
+    ld->modified = FALSE;
+    break;
+  case PCAP_SWAPPED_MODIFIED_MAGIC:
+    /* Host that wrote it out has a byte order opposite to
+       ours, and was running a program using either ss990915
+       or ss991029 libpcap. */
+    ld->byte_swapped = TRUE;
+    ld->modified = TRUE;
+    break;
+  default:
+    /* Not a "libpcap" type we know about. */
+    close(fd);
+    return -1;
+  }
+
+  /* Read the rest of the header */
+  bytes_read = read(fd, hdr, sizeof(struct pcap_hdr));
+  if (bytes_read <= 0) {
+    close(fd);
+    return -1;
+  }
+  while (bytes_read < sizeof(struct pcap_hdr))
+  {
+    b = read(fd, ((char *)&hdr)+bytes_read, sizeof(struct pcap_hdr) - bytes_read);
+    if (b <= 0) {
+      close(fd);
+      return -1;
+    }
+    bytes_read += b;
+  }
+  if (ld->byte_swapped) {
+    /* Byte-swap the header fields about which we care. */
+    hdr->version_major = BSWAP16(hdr->version_major);
+    hdr->version_minor = BSWAP16(hdr->version_minor);
+    hdr->snaplen = BSWAP32(hdr->snaplen);
+    hdr->network = BSWAP32(hdr->network);
+  }
+  if (hdr->version_major < 2) {
+    close(fd);
+    return -1;
+  }
+
+  return fd;
+}
+
+/* We read one record from the pipe, take care of byte order in the record
+ * header, write the record in the capture file, and update capture statistics. */
+static int
+pipe_dispatch(int fd, loop_data *ld, struct pcap_hdr *hdr)
+{
+  struct wtap_pkthdr whdr;
+  struct pcaprec_modified_hdr rechdr;
+  int bytes_to_read, bytes_read, b;
+  u_char pd[WTAP_MAX_PACKET_SIZE];
+  int err;
+
+  /* read the record header */
+  bytes_to_read = ld->modified ? sizeof rechdr : sizeof rechdr.hdr;
+  bytes_read = read(fd, &rechdr, bytes_to_read);
+  if (bytes_read <= 0) {
+    close(fd);
+    ld->go = FALSE;
+    return 0;
+  }
+  while (bytes_read < bytes_to_read)
+  {
+    b = read(fd, ((char *)&rechdr)+bytes_read, bytes_to_read - bytes_read);
+    if (b <= 0) {
+      close(fd);
+      ld->go = FALSE;
+      return 0;
+    }
+    bytes_read += b;
+  }
+  /* take care of byte order */
+  adjust_header(ld, hdr, &rechdr.hdr);
+  if (rechdr.hdr.incl_len > WTAP_MAX_PACKET_SIZE) {
+    close(fd);
+    ld->go = FALSE;
+    return 0;
+  }
+  /* read the packet data */
+  bytes_read = read(fd, pd, rechdr.hdr.incl_len);
+  if (bytes_read <= 0) {
+    close(fd);
+    ld->go = FALSE;
+    return 0;
+  }
+  while (bytes_read < rechdr.hdr.incl_len)
+  {
+    b = read(fd, pd+bytes_read, rechdr.hdr.incl_len - bytes_read);
+    if (b <= 0) {
+      close(fd);
+      ld->go = FALSE;
+      return 0;
+    }
+    bytes_read += b;
+  }
+  /* dump the packet data to the capture file */
+  whdr.ts.tv_sec = rechdr.hdr.ts_sec;
+  whdr.ts.tv_usec = rechdr.hdr.ts_usec;
+  whdr.caplen = rechdr.hdr.incl_len;
+  whdr.len = rechdr.hdr.orig_len;
+  whdr.pkt_encap = ld->linktype;
+  wtap_dump(ld->pdh, &whdr, NULL, pd, &err);
+
+  /* Set the initial payload to the packet length, and the initial
+     captured payload to the capture length (other protocols may
+     reduce them if their headers say they're less). */
+  pi.len = whdr.len;
+  pi.captured_len = whdr.caplen;
+    
+  /* update capture statistics */
+  switch (ld->linktype) {
+    case WTAP_ENCAP_ETHERNET:
+      capture_eth(pd, 0, &ld->counts);
+      break;
+    case WTAP_ENCAP_FDDI:
+    case WTAP_ENCAP_FDDI_BITSWAPPED:
+      capture_fddi(pd, &ld->counts);
+      break;
+    case WTAP_ENCAP_TR:
+      capture_tr(pd, 0, &ld->counts);
+      break;
+    case WTAP_ENCAP_NULL:
+      capture_null(pd, &ld->counts);
+      break;
+    case WTAP_ENCAP_PPP:
+      capture_ppp(pd, 0, &ld->counts);
+      break;
+    case WTAP_ENCAP_RAW_IP:
+      capture_raw(pd, &ld->counts);
+      break;
+    case WTAP_ENCAP_LINUX_ATM_CLIP:
+      capture_clip(pd, &ld->counts);
+      break;
+    /* XXX - FreeBSD may append 4-byte ATM pseudo-header to DLT_ATM_RFC1483,
+       with LLC header following; we should implement it at some
+       point. */
+  }
+
+  return 1;
+}
+#endif
+
 /* Do the low-level work of a capture.
    Returns TRUE if it succeeds, FALSE otherwise. */
 int
@@ -549,21 +945,31 @@ capture(void)
   GtkWidget  *cap_w, *main_vb, *count_lb, *sctp_lb, *tcp_lb, *udp_lb, *icmp_lb,
              *ospf_lb, *gre_lb, *netbios_lb, *ipx_lb, *vines_lb, *other_lb, *stop_bt;
   pcap_t     *pch;
-  gchar       err_str[PCAP_ERRBUF_SIZE], label_str[32];
+  int         pcap_encap;
+  int         snaplen;
+  gchar       err_str[PCAP_ERRBUF_SIZE], label_str[64];
   loop_data   ld;
   bpf_u_int32 netnum, netmask;
   time_t      upd_time, cur_time;
   int         err, inpkts;
-  char        errmsg[1024+1];
-#ifdef linux
+  char        errmsg[4096+1];
+#ifndef _WIN32
+  static const char ppamsg[] = "can't find PPA for ";
+  char       *libpcap_warn;
+#endif
   fd_set      set1;
   struct timeval timeout;
-  int         pcap_fd;
+#ifdef linux
+  int         pcap_fd = 0;
 #endif
 #ifdef _WIN32 
   WORD wVersionRequested; 
   WSADATA wsaData; 
 #endif
+#ifndef _WIN32
+  int         pipe_fd = -1;
+  struct pcap_hdr hdr;
+#endif
 
   /* Initialize Windows Socket if we are in a WIN32 OS 
      This needs to be done before querying the interface for network/netmask */
@@ -580,8 +986,9 @@ capture(void)
 
   ld.go             = TRUE;
   ld.counts.total   = 0;
-  ld.max            = cf.count;
+  ld.max            = cfile.count;
   ld.linktype       = WTAP_ENCAP_UNKNOWN;
+  ld.from_pipe      = FALSE;
   ld.sync_packets   = 0;
   ld.counts.sctp    = 0;
   ld.counts.tcp     = 0;
@@ -596,9 +1003,10 @@ capture(void)
   ld.pdh            = NULL;
 
   /* Open the network interface to capture from it. */
-  pch = pcap_open_live(cf.iface, cf.snap, 1, CAP_READ_TIMEOUT, err_str);
+  pch = pcap_open_live(cfile.iface, cfile.snap, 1, CAP_READ_TIMEOUT, err_str);
 
   if (pch == NULL) {
+#ifdef _WIN32
     /* Well, we couldn't start the capture.
        If this is a child process that does the capturing in sync
        mode or fork mode, it shouldn't do any UI stuff until we pop up the
@@ -607,26 +1015,80 @@ capture(void)
     if (!capture_child) {
       while (gtk_events_pending()) gtk_main_iteration();
     }
+
+    /* On Win32 OSes, the capture devices are probably available to all
+       users; don't warn about permissions problems.
+
+       Do, however, warn that Token Ring and PPP devices aren't supported. */
     snprintf(errmsg, sizeof errmsg,
-      "The capture session could not be initiated (%s).\n"
-      "Please check to make sure you have sufficient permissions, and that\n"
-      "you have the proper interface specified.", err_str);
+	"The capture session could not be initiated (%s).\n"
+	"Please check that you have the proper interface specified.\n"
+	"\n"
+	"Note that the driver Ethereal uses for packet capture on Windows\n"
+	"doesn't support capturing on Token Ring interfaces, and doesn't\n"
+	"support capturing on PPP/WAN interfaces in Windows NT/2000.\n",
+	err_str);
     goto error;
+#else
+    /* try to open cfile.iface as a pipe */
+    pipe_fd = pipe_open_live(cfile.iface, &hdr, &ld, err_str);
+
+    if (pipe_fd == -1) {
+      /* Well, we couldn't start the capture.
+	 If this is a child process that does the capturing in sync
+	 mode or fork mode, it shouldn't do any UI stuff until we pop up the
+	 capture-progress window, and, since we couldn't start the
+	 capture, we haven't popped it up. */
+      if (!capture_child) {
+	while (gtk_events_pending()) gtk_main_iteration();
+      }
+
+      /* If we got a "can't find PPA for XXX" message, warn the user (who
+         is running Ethereal on HP-UX) that they don't have a version
+	 of libpcap patched to properly handle HP-UX (the patched version
+	 says "can't find /dev/dlpi PPA for XXX" rather than "can't find
+	 PPA for XXX"). */
+      if (strncmp(err_str, ppamsg, sizeof ppamsg - 1) == 0)
+	libpcap_warn =
+	  "\n\n"
+	  "You are running Ethereal with a version of the libpcap library\n"
+	  "that doesn't handle HP-UX network devices well; this means that\n"
+	  "Ethereal may not be able to capture packets.\n"
+	  "\n"
+	  "To fix this, you will need to download the source to Ethereal\n"
+	  "from ethereal.zing.org if you have not already done so, read\n"
+	  "the instructions in the \"README.hpux\" file in the source\n"
+	  "distribution, download the source to libpcap if you have not\n"
+	  "already done so, patch libpcap as per the instructions, rebuild\n"
+	  "and install libpcap, and then build Ethereal (if you have already\n"
+	  "built Ethereal from source, do a \"make distclean\" and re-run\n"
+	  "configure before building).";
+      else
+	libpcap_warn = "";
+      snprintf(errmsg, sizeof errmsg,
+	  "The capture session could not be initiated (%s).\n"
+	  "Please check to make sure you have sufficient permissions, and that\n"
+	  "you have the proper interface or pipe specified.%s", err_str,
+	  libpcap_warn);
+      goto error;
+    }
+#endif
   }
 
-  if (cf.cfilter) {
+  /* capture filters only work on real interfaces */
+  if (cfile.cfilter && !ld.from_pipe) {
     /* A capture filter was specified; set it up. */
-    if (pcap_lookupnet (cf.iface, &netnum, &netmask, err_str) < 0) {
+    if (pcap_lookupnet (cfile.iface, &netnum, &netmask, err_str) < 0) {
       snprintf(errmsg, sizeof errmsg,
         "Can't use filter:  Couldn't obtain netmask info (%s).", err_str);
       goto error;
     }
-    if (pcap_compile(pch, &cf.fcode, cf.cfilter, 1, netmask) < 0) {
+    if (pcap_compile(pch, &cfile.fcode, cfile.cfilter, 1, netmask) < 0) {
       snprintf(errmsg, sizeof errmsg, "Unable to parse filter string (%s).",
 	pcap_geterr(pch));
       goto error;
     }
-    if (pcap_setfilter(pch, &cf.fcode) < 0) {
+    if (pcap_setfilter(pch, &cfile.fcode) < 0) {
       snprintf(errmsg, sizeof errmsg, "Can't install filter (%s).",
 	pcap_geterr(pch));
       goto error;
@@ -634,14 +1096,25 @@ capture(void)
   }
 
   /* Set up to write to the capture file. */
-  ld.linktype = wtap_pcap_encap_to_wtap_encap(pcap_datalink(pch));
+#ifndef _WIN32
+  if (ld.from_pipe) {
+    pcap_encap = hdr.network;
+    snaplen = hdr.snaplen;
+  } else
+#endif
+  {
+    pcap_encap = pcap_datalink(pch);
+    snaplen = pcap_snapshot(pch);
+  }
+  ld.linktype = wtap_pcap_encap_to_wtap_encap(pcap_encap);
   if (ld.linktype == WTAP_ENCAP_UNKNOWN) {
-    strcpy(errmsg, "The network you're capturing from is of a type"
-             " that Ethereal doesn't support.");
+    snprintf(errmsg, sizeof errmsg,
+	"The network you're capturing from is of a type"
+	" that Ethereal doesn't support (data link type %d).", pcap_encap);
     goto error;
   }
-  ld.pdh = wtap_dump_fdopen(cf.save_file_fd, WTAP_FILE_PCAP,
-		ld.linktype, pcap_snapshot(pch), &err);
+  ld.pdh = wtap_dump_fdopen(cfile.save_file_fd, WTAP_FILE_PCAP,
+      ld.linktype, snaplen, &err);
 
   if (ld.pdh == NULL) {
     /* We couldn't set up to write to the capture file. */
@@ -659,19 +1132,27 @@ capture(void)
 
     default:
       if (err < 0) {
-        sprintf(errmsg, "The file to which the capture would be"
+        snprintf(errmsg, sizeof(errmsg),
+		     "The file to which the capture would be"
                      " saved (\"%s\") could not be opened: Error %d.",
- 			cf.save_file, err);
+ 			cfile.save_file, err);
       } else {
-        sprintf(errmsg, "The file to which the capture would be"
+        snprintf(errmsg, sizeof(errmsg),
+		     "The file to which the capture would be"
                      " saved (\"%s\") could not be opened: %s.",
- 			cf.save_file, strerror(err));
+ 			cfile.save_file, strerror(err));
       }
       break;
     }
     goto error;
   }
 
+  /* XXX - capture SIGTERM and close the capture, in case we're on a
+     Linux 2.0[.x] system and you have to explicitly close the capture
+     stream in order to turn promiscuous mode off?  We need to do that
+     in other places as well - and I don't think that works all the
+     time in any case, due to libpcap bugs. */
+
   if (capture_child) {
     /* Well, we should be able to start capturing.
 
@@ -686,7 +1167,8 @@ capture(void)
   }
 
   cap_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(cap_w), "Ethereal: Capture / Playback");
+  gtk_window_set_title(GTK_WINDOW(cap_w), "Ethereal: Capture");
+  gtk_window_set_modal(GTK_WINDOW(cap_w), TRUE);
 
   /* Container for capture display widgets */
   main_vb = gtk_vbox_new(FALSE, 1);
@@ -753,45 +1235,66 @@ capture(void)
   gtk_widget_show(stop_bt);
 
   gtk_widget_show(cap_w);
-  gtk_grab_add(cap_w);
 
   upd_time = time(NULL);
 #ifdef linux
-  pcap_fd = pcap_fileno(pch);
+  if (!ld.from_pipe) pcap_fd = pcap_fileno(pch);
 #endif
   while (ld.go) {
     while (gtk_events_pending()) gtk_main_iteration();
-#ifdef linux
-    /*
-     * Sigh.  The semantics of the read timeout argument to
-     * "pcap_open_live()" aren't particularly well specified by
-     * the "pcap" man page - at least with the BSD BPF code, the
-     * intent appears to be, at least in part, a way of cutting
-     * down the number of reads done on a capture, by blocking
-     * until the buffer fills or a timer expires - and the Linux
-     * libpcap doesn't actually support it, so we can't use it
-     * to break out of the "pcap_dispatch()" every 1/4 of a second
-     * or so.
-     *
-     * Thus, on Linux, we do a "select()" on the file descriptor for the
-     * capture, with a timeout of CAP_READ_TIMEOUT milliseconds, or
-     * CAP_READ_TIMEOUT*1000 microseconds.
-     */
-    FD_ZERO(&set1);
-    FD_SET(pcap_fd, &set1);
-    timeout.tv_sec = 0;
-    timeout.tv_usec = CAP_READ_TIMEOUT*1000;
-    if (select(pcap_fd+1, &set1, NULL, NULL, &timeout) != 0) {
-      /*
-       * "select()" says we can read from it without blocking; go for
-       * it.
-       */
-      inpkts = pcap_dispatch(pch, 1, capture_pcap_cb, (u_char *) &ld);
-    } else
-      inpkts = 0;
-#else
-    inpkts = pcap_dispatch(pch, 1, capture_pcap_cb, (u_char *) &ld);
+
+#ifndef _WIN32
+    if (ld.from_pipe) {
+      FD_ZERO(&set1);
+      FD_SET(pipe_fd, &set1);
+      timeout.tv_sec = 0;
+      timeout.tv_usec = CAP_READ_TIMEOUT*1000;
+      if (select(pipe_fd+1, &set1, NULL, NULL, &timeout) != 0) {
+	/*
+	 * "select()" says we can read from the pipe without blocking; go for
+	 * it. We are not sure we can read a whole record, but at least the
+	 * begninning of one. pipe_dispatch() will block reading the whole
+	 * record.
+	 */
+	inpkts = pipe_dispatch(pipe_fd, &ld, &hdr);
+      } else
+	inpkts = 0;
+    }
+    else
 #endif
+    {
+#ifdef linux
+      /*
+       * Sigh.  The semantics of the read timeout argument to
+       * "pcap_open_live()" aren't particularly well specified by
+       * the "pcap" man page - at least with the BSD BPF code, the
+       * intent appears to be, at least in part, a way of cutting
+       * down the number of reads done on a capture, by blocking
+       * until the buffer fills or a timer expires - and the Linux
+       * libpcap doesn't actually support it, so we can't use it
+       * to break out of the "pcap_dispatch()" every 1/4 of a second
+       * or so.
+       *
+       * Thus, on Linux, we do a "select()" on the file descriptor for the
+       * capture, with a timeout of CAP_READ_TIMEOUT milliseconds, or
+       * CAP_READ_TIMEOUT*1000 microseconds.
+       */
+      FD_ZERO(&set1);
+      FD_SET(pcap_fd, &set1);
+      timeout.tv_sec = 0;
+      timeout.tv_usec = CAP_READ_TIMEOUT*1000;
+      if (select(pcap_fd+1, &set1, NULL, NULL, &timeout) != 0) {
+	/*
+	 * "select()" says we can read from it without blocking; go for
+	 * it.
+	 */
+	inpkts = pcap_dispatch(pch, 1, capture_pcap_cb, (u_char *) &ld);
+      } else
+	inpkts = 0;
+#else
+      inpkts = pcap_dispatch(pch, 1, capture_pcap_cb, (u_char *) &ld);
+#endif
+    }
     if (inpkts > 0)
       ld.sync_packets += inpkts;
     /* Only update once a second so as not to overload slow displays */
@@ -881,11 +1384,16 @@ capture(void)
       simple_dialog(ESD_TYPE_WARN, NULL,
 		"The file to which the capture was being"
 		" saved (\"%s\") could not be closed: %s.",
-		cf.save_file, wtap_strerror(err));
+		cfile.save_file, wtap_strerror(err));
       break;
     }
   }
-  pcap_close(pch);
+#ifndef _WIN32
+  if (ld.from_pipe)
+    close(pipe_fd);
+  else
+#endif
+    pcap_close(pch);
 
   gtk_grab_remove(GTK_WIDGET(cap_w));
   gtk_widget_destroy(GTK_WIDGET(cap_w));
@@ -895,13 +1403,13 @@ capture(void)
 error:
   /* We can't use the save file, and we have no wtap_dump stream
      to close in order to close it, so close the FD directly. */
-  close(cf.save_file_fd);
+  close(cfile.save_file_fd);
 
   /* We couldn't even start the capture, so get rid of the capture
      file. */
-  unlink(cf.save_file); /* silently ignore error */
-  g_free(cf.save_file);
-  cf.save_file = NULL;
+  unlink(cfile.save_file); /* silently ignore error */
+  g_free(cfile.save_file);
+  cfile.save_file = NULL;
   if (capture_child) {
     /* This is the child process for a sync mode capture.
        Send the error message to our parent, so they can display a
@@ -909,9 +1417,9 @@ error:
     send_errmsg_to_parent(errmsg);
   } else {
     /* Display the dialog box ourselves; there's no parent. */
-    simple_dialog(ESD_TYPE_WARN, NULL, errmsg);
+    simple_dialog(ESD_TYPE_CRIT, NULL, errmsg);
   }
-  if (pch != NULL)
+  if (pch != NULL && !ld.from_pipe)
     pcap_close(pch);
 
   return FALSE;
@@ -1001,6 +1509,9 @@ capture_pcap_cb(u_char *user, const struct pcap_pkthdr *phdr,
     case WTAP_ENCAP_RAW_IP:
       capture_raw(pd, &ld->counts);
       break;
+    case WTAP_ENCAP_LINUX_ATM_CLIP:
+      capture_clip(pd, &ld->counts);
+      break;
     /* XXX - FreeBSD may append 4-byte ATM pseudo-header to DLT_ATM_RFC1483,
        with LLC header following; we should implement it at some
        point. */
