@@ -1,7 +1,7 @@
 /* dfilter.c
  * Routines for display filters
  *
- * $Id: dfilter.c,v 1.34 2000/04/14 05:39:36 gram Exp $
+ * $Id: dfilter.c,v 1.38 2000/09/13 20:30:48 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,39 +31,17 @@
 # include <sys/types.h>
 #endif
 
-#ifndef _STDIO_H
 #include <stdio.h>
-#endif
-
-#ifndef _STRING_H
 #include <string.h>
-#endif
+#include <glib.h>
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
-#ifndef __G_LIB_H__
-#include <glib.h>
-#endif
-
-#ifndef __PROTO_H__
 #include "proto.h"
-#endif
-
-#ifndef __DFILTER_H__
 #include "dfilter.h"
-#endif
-
-#ifndef __UTIL_H__
 #include "util.h"
-#endif
-
 #include "dfilter-int.h"
 #include "dfilter-grammar.h"
 
@@ -102,6 +80,8 @@ static void clear_byte_array(gpointer data, gpointer user_data);
  */
 #define g_array_index_ptr(a,s,i)      (((guint8*) (a)->data) + (i*s))
 
+extern int hf_text_only; /* in proto.c */
+
 void
 dfilter_init(void)
 {
@@ -112,7 +92,11 @@ dfilter_init(void)
 
 	/* Add the header field and protocol abbrevs to the symbol table */
 	num_symbols = proto_registrar_n();
+	
 	for (i=0; i < num_symbols; i++) {
+		if (i == hf_text_only) {
+			continue;
+		}
 		s = proto_registrar_get_abbrev(i);
 		g_assert(s);		/* Not Null */
 		g_assert(s[0] != 0);	/* Not empty string */
@@ -121,7 +105,6 @@ dfilter_init(void)
 			g_message("Already have abbreviation \"%s\"", s);
 			g_assert(0);
 		}
-		/*g_message("Adding %s", s);*/
 		symbol = DFILTER_LEX_ABBREV_OFFSET + i;
 		g_tree_insert(dfilter_tokens, s, GINT_TO_POINTER(symbol));
 	}
@@ -225,10 +208,19 @@ dfilter_new(void)
 	df->node_memchunk = g_mem_chunk_new("df->node_memchunk",
 		sizeof(dfilter_node), 20 * sizeof(dfilter_node), G_ALLOC_ONLY);
 	df->list_of_byte_arrays = NULL;
+	df->list_of_strings = NULL;
 
 	return df;
 }
 
+static void
+free_string(gpointer data, gpointer user_data)
+{
+	char *string = data;
+	if (string)
+		g_free(string);
+}
+
 /* Frees all memory used by dfilter, and frees dfilter itself */
 void
 dfilter_destroy(dfilter *df)
@@ -249,8 +241,15 @@ dfilter_destroy(dfilter *df)
 		g_slist_free(df->list_of_byte_arrays);
 	}
 
+	/* clear the allocated strings */
+	if (df->list_of_strings) {
+		g_slist_foreach(df->list_of_strings, free_string, NULL);
+		g_slist_free(df->list_of_strings);
+	}
+
 	df->dftree = NULL;
 	df->list_of_byte_arrays = NULL;
+	df->list_of_strings = NULL;
 
 	/* Git rid of memchunk */
 	if (df->node_memchunk)
@@ -403,6 +402,21 @@ check_logical(gint operand, GNode *a, GNode *b, proto_tree *ptree, const guint8
 	return FALSE;
 }
 
+
+static void
+free_array_of_byte_arrays(GArray *array)
+{
+	int		i, len;
+	GByteArray	*ba_ptr;
+
+	len = array->len;
+
+	for (i = 0; i < len ; i++) {
+		ba_ptr = g_array_index(array, GByteArray*, i);
+		g_byte_array_free(ba_ptr, TRUE);
+	}
+}
+
 /* this is inefficient. I get arrays for both a and b that represent all the values present. That is,
  * if a is bootp.option, e.g., i'll get an array showing all the bootp.option values in the protocol
  * tree. Then I'll get an array for b, which more than likely is a single int, and then I'll compare
@@ -432,12 +446,22 @@ check_relation(gint operand, GNode *a, GNode *b, proto_tree *ptree, const guint8
 
 	retval =  node_a->check_relation_func(operand, vals_a, vals_b);
 
+	/* Free GByteArrays alloated by fill_array_bytes_variable() */
+	if (node_a->fill_array_variable_func == fill_array_bytes_variable) {
+		free_array_of_byte_arrays(vals_a);
+	}
+
+	if (node_b->fill_array_variable_func == fill_array_bytes_variable) {
+		free_array_of_byte_arrays(vals_b);
+	}
+
 	g_array_free(vals_a, FALSE);
 	g_array_free(vals_b, FALSE);
 
 	return retval;
 }
 
+
 static gboolean
 check_existence_in_ptree(dfilter_node *dnode, proto_tree *ptree)
 {
@@ -450,32 +474,103 @@ check_existence_in_ptree(dfilter_node *dnode, proto_tree *ptree)
 static GArray*
 get_values_from_ptree(dfilter_node *dnode, proto_tree *ptree, const guint8 *pd)
 {
-	GArray		*array;
-	int		parent_protocol;
-	proto_tree_search_info sinfo;
+	GArray		*result_array;
+	GPtrArray	*finfo_array;
+	int		i, len;
+	field_info	*finfo;
 
+	/* Prepare the array for results */
 	g_assert(dnode->elem_size > 0);
-	array = g_array_new(FALSE, FALSE, dnode->elem_size);
+	result_array = g_array_new(FALSE, FALSE, dnode->elem_size);
 
-	sinfo.target = dnode->value.variable;
-	sinfo.result.array = array;
-	sinfo.packet_data = pd;
-	sinfo.traverse_func = dnode->fill_array_func;
-
-	/* Find the proto_tree subtree where we should start searching.*/
-	if (proto_registrar_is_protocol(sinfo.target)) {
-		proto_find_protocol_multi(ptree, sinfo.target,
-				(GNodeTraverseFunc)proto_get_field_values, &sinfo);
+	/* Cull the finfos from the proto_tree */
+	finfo_array = proto_get_finfo_ptr_array(ptree, dnode->value.variable);
+	if (!finfo_array) {
+		return result_array;
 	}
-	else {
-		parent_protocol = proto_registrar_get_parent(sinfo.target);
-		if (parent_protocol >= 0) {
-			proto_find_protocol_multi(ptree, parent_protocol,
-					(GNodeTraverseFunc)proto_get_field_values, &sinfo);
+
+	len = g_ptr_array_len(finfo_array);
+
+	for (i = 0; i < len; i++) {
+		finfo = g_ptr_array_index(finfo_array, i);
+		dnode->fill_array_variable_func(finfo, result_array, pd);
+	}
+
+	g_ptr_array_free(finfo_array, FALSE);
+
+	return result_array;
+}
+
+
+void
+fill_array_numeric_variable(field_info *finfo, GArray *array, const guint8 *pd)
+{
+	g_array_append_val(array, finfo->value.numeric);
+}
+
+void
+fill_array_floating_variable(field_info *finfo, GArray *array, const guint8 *pd)
+{
+	g_array_append_val(array, finfo->value.floating);
+}
+
+void
+fill_array_ether_variable(field_info *finfo, GArray *array, const guint8 *pd)
+{
+	/* hmmm, yes, I *can* copy a pointer instead of memcpy() */
+	g_array_append_val(array, finfo->value.ether);
+}
+
+void
+fill_array_ipv4_variable(field_info *finfo, GArray *array, const guint8 *pd)
+{
+	/* hmmm, yes, I *can* copy a pointer instead of memcpy() */
+	g_array_append_val(array, finfo->value.ipv4);
+}
+
+void
+fill_array_ipv6_variable(field_info *finfo, GArray *array, const guint8 *pd)
+{
+	/* hmmm, yes, I *can* copy a pointer instead of memcpy() */
+	g_array_append_val(array, finfo->value.ipv6);
+}
+
+void
+fill_array_string_variable(field_info *finfo, GArray *array, const guint8 *pd)
+{
+	g_array_append_val(array, finfo->value.string);
+}
+
+void
+fill_array_bytes_variable(field_info *finfo, GArray *array, const guint8 *pd)
+{
+	GByteArray		*barray;
+	guint			read_start, pkt_end;
+
+	if (bytes_offset < 0) {
+		/* Handle negative byte offsets */
+		bytes_offset = finfo->length + bytes_offset;
+		if (bytes_offset < 0) {
+			return;
 		}
 	}
 
-	return array;
+	/* Check to make sure offset exists for this field */
+	if (bytes_offset >= finfo->length) {
+		return;
+	}
+
+	pkt_end = finfo->start + finfo->length;
+	read_start = finfo->start + bytes_offset;
+
+	/* Check to make sure entire length requested is inside field */
+	if (pkt_end < read_start + bytes_length) {
+		return;
+	}
+
+	barray = g_byte_array_new();
+	g_byte_array_append(barray, pd + read_start, bytes_length);
+	g_array_append_val(array, barray);
 }
 
 static GArray*
@@ -486,107 +581,11 @@ get_values_from_dfilter(dfilter_node *dnode, GNode *gnode)
 	g_assert(dnode->elem_size > 0);
 	array = g_array_new(FALSE, FALSE, dnode->elem_size);
 
-	g_node_traverse(gnode, G_IN_ORDER, G_TRAVERSE_ALL, -1, dnode->fill_array_func, array);
+	g_node_traverse(gnode, G_IN_ORDER, G_TRAVERSE_ALL, -1,
+			dnode->fill_array_value_func, array);
 	return array;
 }
 
-gboolean fill_array_numeric_variable(GNode *gnode, gpointer data)
-{
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
-	field_info		*fi = (field_info*) (gnode->data);
-
-	if (fi->hfinfo->id == sinfo->target) {
-		g_array_append_val(sinfo->result.array, fi->value.numeric);
-	}
-
-	return FALSE; /* FALSE = do not end traversal of GNode tree */
-}
-
-gboolean fill_array_floating_variable(GNode *gnode, gpointer data)
-{
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
-	field_info		*fi = (field_info*) (gnode->data);
-
-	if (fi->hfinfo->id == sinfo->target) {
-		g_array_append_val(sinfo->result.array, fi->value.floating);
-	}
-
-	return FALSE; /* FALSE = do not end traversal of GNode tree */
-}
-
-gboolean fill_array_ether_variable(GNode *gnode, gpointer data)
-{
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
-	field_info		*fi = (field_info*) (gnode->data);
-
-	if (fi->hfinfo->id == sinfo->target) {
-		g_array_append_val(sinfo->result.array, fi->value.ether);
-	}
-
-	return FALSE; /* FALSE = do not end traversal of GNode tree */
-}
-
-gboolean fill_array_ipv4_variable(GNode *gnode, gpointer data)
-{
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
-	field_info		*fi = (field_info*) (gnode->data);
-
-	if (fi->hfinfo->id == sinfo->target) {
-		g_array_append_val(sinfo->result.array, fi->value.ipv4);
-	}
-
-	return FALSE; /* FALSE = do not end traversal of GNode tree */
-}
-gboolean fill_array_ipv6_variable(GNode *gnode, gpointer data)
-{
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
-	field_info		*fi = (field_info*) (gnode->data);
-
-	if (fi->hfinfo->id == sinfo->target) {
-		g_array_append_val(sinfo->result.array, fi->value.ipv6);
-	}
-
-	return FALSE; /* FALSE = do not end traversal of GNode tree */
-}
-
-gboolean fill_array_bytes_variable(GNode *gnode, gpointer data)
-{
-	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
-	field_info		*fi = (field_info*) (gnode->data);
-	GByteArray		*barray;
-	guint			read_start, pkt_end;
-
-	if (fi->hfinfo->id == sinfo->target) {
-		if (bytes_offset < 0) {
-			/* Handle negative byte offsets */
-			bytes_offset = fi->length + bytes_offset;
-			if (bytes_offset < 0) {
-				goto FAIL;
-			}
-		}
-
-		/* Check to make sure offset exists for this field */
-		if (bytes_offset >= fi->length) {
-			goto FAIL;
-		}
-
-		pkt_end = fi->start + fi->length;
-		read_start = fi->start + bytes_offset;
-
-		/* Check to make sure entire length requested is inside field */
-		if (pkt_end < read_start + bytes_length) {
-			goto FAIL;
-		}
-
-		barray = g_byte_array_new();
-		g_byte_array_append(barray, sinfo->packet_data + read_start, bytes_length);
-		g_array_append_val(sinfo->result.array, barray);
-	}
-
- FAIL:
-	return FALSE; /* FALSE = do not end traversal of GNode tree */
-}
-
 gboolean fill_array_numeric_value(GNode *gnode, gpointer data)
 {
 	GArray		*array = (GArray*)data;
@@ -646,6 +645,16 @@ gboolean fill_array_bytes_value(GNode *gnode, gpointer data)
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
 }
 
+gboolean fill_array_string_value(GNode *gnode, gpointer data)
+{
+	GArray		*array = (GArray*)data;
+	dfilter_node	*dnode = (dfilter_node*) (gnode->data);
+
+	g_array_append_val(array, dnode->value.string);
+
+	return FALSE; /* FALSE = do not end traversal of GNode tree */
+}
+
 gboolean check_relation_numeric(gint operand, GArray *a, GArray *b)
 {
 	int	i, j, len_a, len_b;
@@ -1015,3 +1024,41 @@ gboolean check_relation_bytes(gint operand, GArray *a, GArray *b)
 	g_assert_not_reached();
 	return FALSE;
 }
+
+gboolean check_relation_string(gint operand, GArray *a, GArray *b)
+{
+	int	i, j, len_a, len_b;
+	char	*ptr_a, *ptr_b;
+
+	len_a = a->len;
+	len_b = b->len;
+
+
+	switch(operand) {
+	case TOK_EQ:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = g_array_index(a, char*, i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = g_array_index(b, char*, j);
+				if (strcmp(ptr_a, ptr_b) == 0)
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_NE:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = g_array_index(a, char*, i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = g_array_index(b, char*, j);
+				if (strcmp(ptr_a, ptr_b) != 0)
+					return TRUE;
+			}
+		}
+		return FALSE;
+	}
+
+	g_assert_not_reached();
+	return FALSE;
+}
+
