@@ -1,7 +1,7 @@
 /* dfilter.h
  * Definitions for display filters
  *
- * $Id: dfilter.h,v 1.16 2000/04/14 05:39:38 gram Exp $
+ * $Id: dfilter.h,v 1.18 2000/08/11 13:34:40 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -26,9 +26,7 @@
 #ifndef __DFILTER_H__
 #define __DFILTER_H__
 
-#ifndef __PROTO_H__
 #include "proto.h"
-#endif
 
 /* dfilter_error_msg is NULL if there was no error during dfilter_compile,
  * otherwise it points to a displayable error message. */
@@ -45,6 +43,10 @@ typedef struct {
 	/* list of byte arrays we allocate during parse. We can traverse this list
 	 * faster than the tree when we go back and free the byte arrays */
 	GSList *list_of_byte_arrays;
+
+	/* List of strings allocated during parse. */
+	GSList *list_of_strings;
+
 } dfilter;
 
 /* Initialization of the symbol table. Called once during program startup */
