@@ -1,7 +1,7 @@
 /* file.h
  * Definitions for file structures and routines
  *
- * $Id: file.h,v 1.68 2000/05/19 23:06:07 gram Exp $
+ * $Id: file.h,v 1.76 2000/08/21 15:45:21 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -30,27 +30,15 @@
 #include <sys/types.h>
 #endif
 
-#ifndef __WTAP_H__
 #include "wiretap/wtap.h"
-#endif
 
 #ifdef HAVE_LIBPCAP
-#ifndef lib_pcap_h
 #include <pcap.h>
 #endif
-#endif
 
-#ifndef __DFILTER_H__
 #include "dfilter.h"
-#endif
-
-#ifndef __COLORS_H__
 #include "gtk/colors.h"		/* XXX */
-#endif
-
-#ifndef __PRINT_H__
 #include "print.h"
-#endif
 
 #include <errno.h>
 
@@ -69,7 +57,16 @@
 
 typedef struct bpf_program bpf_prog;
 
+/* Current state of file. */
+typedef enum {
+	FILE_CLOSED,		/* No file open */
+	FILE_READ_IN_PROGRESS,	/* Reading a file we've opened */
+	FILE_READ_ABORTED,	/* Read aborted by user */
+	FILE_READ_DONE		/* Read completed */
+} file_state;
+
 typedef struct _capture_file {
+  file_state   state;     /* Current state of capture file */
   int          filed;     /* File descriptor of capture file */
   gchar       *filename;  /* Name of capture file */
   gboolean     is_tempfile; /* Is capture file a temporary file? */
@@ -83,7 +80,6 @@ typedef struct _capture_file {
   guint32      esec;      /* Elapsed seconds */
   guint32      eusec;     /* Elapsed microseconds */
   guint32      snap;      /* Captured packet length */
-  gboolean     update_progbar; /* TRUE if we should update the progress bar */
   long         progbar_quantum; /* Number of bytes read per progress bar update */
   long         progbar_nextstep; /* Next point at which to update progress bar */
   gchar       *iface;     /* Interface */
@@ -113,17 +109,26 @@ typedef struct _capture_file {
   FILE        *print_fh;  /* File we're printing to */
 } capture_file;
 
+/* Return values from "read_cap_file()", "continue_tail_cap_file()",
+   and "finish_tail_cap_file()". */
+typedef enum {
+	READ_SUCCESS,	/* read succeeded */
+	READ_ERROR,	/* read got an error */
+	READ_ABORTED	/* read aborted by user */
+} read_status_t;
+
 int  open_cap_file(char *, gboolean, capture_file *);
 void close_cap_file(capture_file *, void *);
-int  read_cap_file(capture_file *);
+read_status_t read_cap_file(capture_file *, int *);
 int  start_tail_cap_file(char *, gboolean, capture_file *);
-int  continue_tail_cap_file(capture_file *, int);
-int  finish_tail_cap_file(capture_file *);
+read_status_t continue_tail_cap_file(capture_file *, int, int *);
+read_status_t finish_tail_cap_file(capture_file *, int *);
 /* size_t read_frame_header(capture_file *); */
-int  save_cap_file(char *, capture_file *, gboolean, guint);
+int  save_cap_file(char *, capture_file *, gboolean, gboolean, guint);
 
 int filter_packets(capture_file *cf, gchar *dfilter);
 void colorize_packets(capture_file *);
+void redissect_packets(capture_file *cf);
 int print_packets(capture_file *cf, print_args_t *print_args);
 void change_time_formats(capture_file *);
 gboolean find_packet(capture_file *cf, dfilter *sfcode);
@@ -144,7 +149,7 @@ int file_mv(char *from, char *to);
 /* Copies a file. Returns 0 on failure, 1 on success */
 int file_cp(char *from, char *to);
 
-char *file_open_error_message(int, int);
+char *file_open_error_message(int, gboolean);
 char *file_read_error_message(int);
 char *file_write_error_message(int);
 
