@@ -1,7 +1,7 @@
 /* globals.h
  * Global defines, etc.
  *
- * $Id: globals.h,v 1.17 2000/01/25 04:31:16 guy Exp $
+ * $Id: globals.h,v 1.21 2000/08/20 07:53:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -26,21 +26,10 @@
 #ifndef __GLOBALS_H__
 #define __GLOBALS_H__
 
-#ifndef _STDIO_H_
 #include <stdio.h>
-#endif
-
-#ifndef __PACKET_H__
 #include "packet.h"
-#endif
-
-#ifndef __FILE_H__
 #include "file.h"
-#endif
-
-#ifndef __TIMESTAMP_H__
 #include "timestamp.h"
-#endif
 
 #define MIN_PACKET_SIZE 68	/* minimum amount of packet data we can read */
 
@@ -74,14 +63,11 @@
 # define MIN(x, y) ((x) < (y) ? (x) : (y))
 #endif
 
-extern FILE        *data_out_file;
 extern packet_info  pi;
-extern capture_file cf;
+extern capture_file cfile;
 extern guint        main_ctx, file_ctx;
 extern gchar        comp_info_str[256];
 extern gchar       *ethereal_path;
-extern gchar       *medium_font;
-extern gchar       *bold_font;
 extern gchar       *last_open_dir;
 extern gboolean     auto_scroll_live;
 extern int          g_resolving_actif;
