@@ -1,7 +1,7 @@
 /* capture_dlg.c
  * Routines for packet capture windows
  *
- * $Id: capture_dlg.c,v 1.25 2000/05/08 07:13:39 guy Exp $
+ * $Id: capture_dlg.c,v 1.32 2000/08/23 20:55:44 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -42,11 +42,6 @@
 #include <time.h>
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
@@ -106,10 +101,10 @@ void
 capture_prep_cb(GtkWidget *w, gpointer d)
 {
   GtkWidget     *if_cb, *if_lb,
-                *count_lb, *count_cb, *main_vb, *if_hb, *count_hb,
-                *filter_hb, *filter_bt, *filter_te,
-                *file_hb, *file_bt, *file_te,
-                *caplen_hb,
+                *count_lb, *count_cb, *main_vb,
+                *filter_bt, *filter_te,
+                *file_bt, *file_te,
+                *caplen_hb, *table,
                 *bbox, *ok_bt, *cancel_bt, *snap_lb,
                 *snap_sb, *sync_cb, *auto_scroll_cb, *resolv_cb;
   GtkAccelGroup *accel_group;
@@ -131,8 +126,7 @@ capture_prep_cb(GtkWidget *w, gpointer d)
 			err_str);
   }
   
-  cap_open_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(cap_open_w), "Ethereal: Capture Preferences");
+  cap_open_w = dlg_window_new("Ethereal: Capture Preferences");
   gtk_signal_connect(GTK_OBJECT(cap_open_w), "destroy",
 	GTK_SIGNAL_FUNC(capture_prep_destroy_cb), NULL);
 
@@ -142,84 +136,78 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   accel_group = gtk_accel_group_new();
   gtk_window_add_accel_group(GTK_WINDOW(cap_open_w), accel_group);
   
-  /* Container for each row of widgets */
   main_vb = gtk_vbox_new(FALSE, 3);
   gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
   gtk_container_add(GTK_CONTAINER(cap_open_w), main_vb);
   gtk_widget_show(main_vb);
   
+  /* Table : container of the first 4 rows */
+  table = gtk_table_new (4, 2, FALSE);
+  gtk_table_set_row_spacings(GTK_TABLE (table), 5);
+  gtk_table_set_col_spacings(GTK_TABLE (table), 5);
+  gtk_container_add(GTK_CONTAINER(main_vb), table);
+  gtk_widget_show(table);
+
   /* Interface row */
-  if_hb = gtk_hbox_new(FALSE, 3);
-  gtk_container_add(GTK_CONTAINER(main_vb), if_hb);
-  gtk_widget_show(if_hb);
   
   if_lb = gtk_label_new("Interface:");
-  gtk_box_pack_start(GTK_BOX(if_hb), if_lb, FALSE, FALSE, 0);
+  gtk_table_attach_defaults(GTK_TABLE(table), if_lb, 0, 1, 0, 1);
   gtk_widget_show(if_lb);
   
   if_cb = gtk_combo_new();
   if (if_list != NULL)
     gtk_combo_set_popdown_strings(GTK_COMBO(if_cb), if_list);
-  if (cf.iface)
-    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry), cf.iface);
+  if (cfile.iface)
+    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry), cfile.iface);
   else if (if_list)
     gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry), if_list->data);
-  gtk_box_pack_start(GTK_BOX(if_hb), if_cb, FALSE, FALSE, 0);
+  gtk_table_attach_defaults(GTK_TABLE(table), if_cb, 1, 2, 0, 1);
   gtk_widget_show(if_cb);
   
   free_interface_list(if_list);
 
   /* Count row */
-  count_hb = gtk_hbox_new(FALSE, 3);
-  gtk_container_add(GTK_CONTAINER(main_vb), count_hb);
-  gtk_widget_show(count_hb);
   
   count_lb = gtk_label_new("Count:");
-  gtk_box_pack_start(GTK_BOX(count_hb), count_lb, FALSE, FALSE, 0);
+  gtk_table_attach_defaults(GTK_TABLE(table), count_lb, 0, 1, 1, 2);
   gtk_widget_show(count_lb);
   
   count_list = g_list_append(count_list, count_item1);
-  if (cf.count) {
-    snprintf(count_item2, 15, "%d", cf.count);
+  if (cfile.count) {
+    snprintf(count_item2, 15, "%d", cfile.count);
     count_list = g_list_append(count_list, count_item2);
   }
 
   count_cb = gtk_combo_new();
   gtk_combo_set_popdown_strings(GTK_COMBO(count_cb), count_list);
-  gtk_box_pack_start(GTK_BOX(count_hb), count_cb, FALSE, FALSE, 0);
+  gtk_table_attach_defaults(GTK_TABLE(table), count_cb, 1, 2, 1, 2);
   gtk_widget_show(count_cb);
 
   while (count_list)
     count_list = g_list_remove_link(count_list, count_list);
 
   /* Filter row */
-  filter_hb = gtk_hbox_new(FALSE, 3);
-  gtk_container_add(GTK_CONTAINER(main_vb), filter_hb);
-  gtk_widget_show(filter_hb);
   
   filter_bt = gtk_button_new_with_label("Filter:");
   gtk_signal_connect(GTK_OBJECT(filter_bt), "clicked",
     GTK_SIGNAL_FUNC(filter_browse_cb), NULL);
-  gtk_box_pack_start(GTK_BOX(filter_hb), filter_bt, FALSE, TRUE, 0);
+  gtk_table_attach_defaults(GTK_TABLE(table), filter_bt, 0, 1, 2, 3);
   gtk_widget_show(filter_bt);
   
   filter_te = gtk_entry_new();
-  if (cf.cfilter) gtk_entry_set_text(GTK_ENTRY(filter_te), cf.cfilter);
+  if (cfile.cfilter) gtk_entry_set_text(GTK_ENTRY(filter_te), cfile.cfilter);
   gtk_object_set_data(GTK_OBJECT(filter_bt), E_FILT_TE_PTR_KEY, filter_te);
-  gtk_box_pack_start(GTK_BOX(filter_hb), filter_te, TRUE, TRUE, 0);
+  gtk_table_attach_defaults(GTK_TABLE(table), filter_te, 1, 2, 2, 3);
   gtk_widget_show(filter_te);
   
   /* File row */
-  file_hb = gtk_hbox_new(FALSE, 1);
-  gtk_container_add(GTK_CONTAINER(main_vb), file_hb);
-  gtk_widget_show(file_hb);
   
   file_bt = gtk_button_new_with_label("File:");
-  gtk_box_pack_start(GTK_BOX(file_hb), file_bt, FALSE, FALSE, 3);
+  gtk_table_attach_defaults(GTK_TABLE(table), file_bt, 0, 1, 3, 4);
   gtk_widget_show(file_bt);
   
   file_te = gtk_entry_new();
-  gtk_box_pack_start(GTK_BOX(file_hb), file_te, TRUE, TRUE, 3);
+  gtk_table_attach_defaults(GTK_TABLE(table), file_te, 1, 2, 3, 4);
   gtk_widget_show(file_te);
 
   gtk_signal_connect(GTK_OBJECT(file_bt), "clicked",
@@ -235,7 +223,7 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   gtk_box_pack_start(GTK_BOX(caplen_hb), snap_lb, FALSE, FALSE, 6);
   gtk_widget_show(snap_lb);
 
-  adj = (GtkAdjustment *) gtk_adjustment_new((float) cf.snap,
+  adj = (GtkAdjustment *) gtk_adjustment_new((float) cfile.snap,
     MIN_PACKET_SIZE, WTAP_MAX_PACKET_SIZE, 1.0, 10.0, 0.0);
   snap_sb = gtk_spin_button_new (adj, 0, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (snap_sb), TRUE);
@@ -246,18 +234,12 @@ capture_prep_cb(GtkWidget *w, gpointer d)
   sync_cb = dlg_check_button_new_with_label_with_mnemonic(
 		"_Update list of packets in real time", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sync_cb), sync_mode);
-#ifdef _WIN32
-  gtk_widget_set_sensitive(sync_cb, FALSE);
-#endif
   gtk_container_add(GTK_CONTAINER(main_vb), sync_cb);
   gtk_widget_show(sync_cb);
 
   auto_scroll_cb = dlg_check_button_new_with_label_with_mnemonic(
 		"_Automatic scrolling in live capture", accel_group);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(auto_scroll_cb), auto_scroll_live);
-#ifdef _WIN32
-  gtk_widget_set_sensitive(auto_scroll_cb, FALSE);
-#endif
   gtk_container_add(GTK_CONTAINER(main_vb), auto_scroll_cb);
   gtk_widget_show(auto_scroll_cb);
 
@@ -438,17 +420,17 @@ capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
     g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(if_cb)->entry)));
   if_name = strtok(if_text, " \t");
   if (if_name == NULL) {
-    simple_dialog(ESD_TYPE_WARN, NULL,
+    simple_dialog(ESD_TYPE_CRIT, NULL,
       "You didn't specify an interface on which to capture packets.");
     g_free(if_name);
     return;
   }
-  if (cf.iface)
-    g_free(cf.iface);
-  cf.iface = g_strdup(if_name);
+  if (cfile.iface)
+    g_free(cfile.iface);
+  cfile.iface = g_strdup(if_name);
   g_free(if_text);
 
-  /* XXX - don't try to get clever and set "cf.filter" to NULL if the
+  /* XXX - don't try to get clever and set "cfile.filter" to NULL if the
      filter string is empty, as an indication that we don't have a filter
      and thus don't have to set a filter when capturing - the version of
      libpcap in Red Hat Linux 6.1, and versions based on later patches
@@ -457,10 +439,10 @@ capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
      no filter is set, which means no packets arrive as input on that
      socket, which means Ethereal never sees any packets. */
   filter_text = gtk_entry_get_text(GTK_ENTRY(filter_te));
-  if (cf.cfilter)
-    g_free(cf.cfilter);
+  if (cfile.cfilter)
+    g_free(cfile.cfilter);
   g_assert(filter_text != NULL);
-  cf.cfilter = g_strdup(filter_text); 
+  cfile.cfilter = g_strdup(filter_text); 
 
   save_file = gtk_entry_get_text(GTK_ENTRY(file_te));
   if (save_file && save_file[0]) {
@@ -471,13 +453,13 @@ capture_prep_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
     save_file = NULL;
   }
 
-  cf.count = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(count_cb)->entry)));
+  cfile.count = atoi(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(count_cb)->entry)));
 
-  cf.snap = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(snap_sb));
-  if (cf.snap < 1)
-    cf.snap = WTAP_MAX_PACKET_SIZE;
-  else if (cf.snap < MIN_PACKET_SIZE)
-    cf.snap = MIN_PACKET_SIZE;
+  cfile.snap = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(snap_sb));
+  if (cfile.snap < 1)
+    cfile.snap = WTAP_MAX_PACKET_SIZE;
+  else if (cfile.snap < MIN_PACKET_SIZE)
+    cfile.snap = MIN_PACKET_SIZE;
 
   sync_mode = GTK_TOGGLE_BUTTON (sync_cb)->active;
 
