@@ -1,7 +1,7 @@
 /* display_opts.c
  * Routines for packet display windows
  *
- * $Id: display_opts.c,v 1.7 2000/05/08 05:51:37 guy Exp $
+ * $Id: display_opts.c,v 1.16 2000/08/23 06:55:32 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -44,19 +44,14 @@
 
 #include <time.h>
 
-#ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
-# include "snprintf.h"
-#endif
-
 #ifdef HAVE_SYS_SOCKIO_H
 # include <sys/sockio.h>
 #endif
 
+#ifdef NEED_SNPRINTF_H
+# include "snprintf.h"
+#endif
+
 #include "globals.h"
 #include "timestamp.h"
 #include "packet.h"
@@ -65,7 +60,7 @@
 #include "ui_util.h"
 #include "dlg_utils.h"
 
-extern capture_file  cf;
+extern capture_file  cfile;
 extern GtkWidget *packet_list;
 
 /* Display callback data keys */
@@ -74,10 +69,11 @@ extern GtkWidget *packet_list;
 #define E_DISPLAY_TIME_DELTA_KEY "display_time_delta"
 #define E_DISPLAY_AUTO_SCROLL_KEY "display_auto_scroll"
 #define E_DISPLAY_NAME_RESOLUTION_KEY "display_name_resolution"
-#define E_DISPLAY_IP_DSCP_KEY "display_ip_dscp"
 
 static void display_opt_ok_cb(GtkWidget *, gpointer);
 static void display_opt_apply_cb(GtkWidget *, gpointer);
+static void get_display_options(GtkWidget *);
+static void update_display(void);
 static void display_opt_close_cb(GtkWidget *, gpointer);
 static void display_opt_destroy_cb(GtkWidget *, gpointer);
 
@@ -89,7 +85,8 @@ static void display_opt_destroy_cb(GtkWidget *, gpointer);
  */
 static GtkWidget *display_opt_w;
 
-static ts_type prev_timestamp_type;
+static ts_type initial_timestamp_type;
+static ts_type current_timestamp_type;
 
 void
 display_opt_cb(GtkWidget *w, gpointer d) {
@@ -102,12 +99,18 @@ display_opt_cb(GtkWidget *w, gpointer d) {
     return;
   }
 
-  /* Save the current timestamp type, so that "Cancel" can put it back
-     if we've changed it with "Apply". */
-  prev_timestamp_type = timestamp_type;
+  /* Save the timestamp type value as of when the dialog box was first popped
+     up, so that "Cancel" can put it back if we've changed it with "Apply". */
+  initial_timestamp_type = timestamp_type;
 
-  display_opt_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(display_opt_w), "Ethereal: Display Options");
+  /* Save the current timestamp type so that we know whether it has changed;
+     we don't want to redisplay the time fields unless we've changed the way 
+     they should be displayed (as redisplaying the time fields could be 
+     expensive - we have to scan through all the packets and rebuild the 
+     packet list).*/
+  current_timestamp_type = timestamp_type;
+
+  display_opt_w = dlg_window_new("Ethereal: Display Options");
   gtk_signal_connect(GTK_OBJECT(display_opt_w), "destroy",
 	GTK_SIGNAL_FUNC(display_opt_destroy_cb), NULL);
 
@@ -168,15 +171,7 @@ display_opt_cb(GtkWidget *w, gpointer d) {
 		      button);
   gtk_box_pack_start(GTK_BOX(main_vb), button, TRUE, TRUE, 0);
   gtk_widget_show(button);
-  
-  button = dlg_check_button_new_with_label_with_mnemonic(
-		"Decode IPv4 TOS field as _DiffServ Field", accel_group);
-  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), g_ip_dscp_actif);
-  gtk_object_set_data(GTK_OBJECT(display_opt_w), E_DISPLAY_IP_DSCP_KEY,
-		      button);
-  gtk_box_pack_start(GTK_BOX(main_vb), button, TRUE, TRUE, 0);
-  gtk_widget_show(button);
-  
+    
   /* Button row: OK, Apply, and Cancel buttons */
   bbox = gtk_hbutton_box_new();
   gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
@@ -216,42 +211,23 @@ display_opt_cb(GtkWidget *w, gpointer d) {
 
 static void
 display_opt_ok_cb(GtkWidget *ok_bt, gpointer parent_w) {
-  GtkWidget *button;
-
-  button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
-                                              E_DISPLAY_TIME_ABS_KEY);
-  if (GTK_TOGGLE_BUTTON (button)->active)
-    timestamp_type = ABSOLUTE;
-
-  button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
-                                              E_DISPLAY_TIME_REL_KEY);
-  if (GTK_TOGGLE_BUTTON (button)->active)
-    timestamp_type = RELATIVE;
-
-  button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
-                                              E_DISPLAY_TIME_DELTA_KEY);
-  if (GTK_TOGGLE_BUTTON (button)->active)
-    timestamp_type = DELTA;
-
-  button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
-					     E_DISPLAY_AUTO_SCROLL_KEY);
-  auto_scroll_live = (GTK_TOGGLE_BUTTON (button)->active);
-
-  button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
-					     E_DISPLAY_NAME_RESOLUTION_KEY);
-  g_resolving_actif = (GTK_TOGGLE_BUTTON (button)->active);
-
-  button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
-					     E_DISPLAY_IP_DSCP_KEY);
-  g_ip_dscp_actif = (GTK_TOGGLE_BUTTON (button)->active);
+  get_display_options(GTK_WIDGET(parent_w));
 
   gtk_widget_destroy(GTK_WIDGET(parent_w));
 
-  change_time_formats(&cf);
+  update_display();
 }
 
 static void
 display_opt_apply_cb(GtkWidget *ok_bt, gpointer parent_w) {
+  get_display_options(GTK_WIDGET(parent_w));
+
+  update_display();
+}
+
+static void
+get_display_options(GtkWidget *parent_w)
+{
   GtkWidget *button;
 
   button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
@@ -277,20 +253,37 @@ display_opt_apply_cb(GtkWidget *ok_bt, gpointer parent_w) {
 					     E_DISPLAY_NAME_RESOLUTION_KEY);
   g_resolving_actif = (GTK_TOGGLE_BUTTON (button)->active);
 
-  button = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(parent_w),
-					     E_DISPLAY_IP_DSCP_KEY);
-  g_ip_dscp_actif = (GTK_TOGGLE_BUTTON (button)->active);
-
-  change_time_formats(&cf);
 }
 
 static void
-display_opt_close_cb(GtkWidget *close_bt, gpointer parent_w) {
+update_display(void)
+{
+  if (timestamp_type != current_timestamp_type) {
+      /* Time stamp format changed; update the display.
 
-  if (timestamp_type != prev_timestamp_type) {
-    timestamp_type = prev_timestamp_type;
-    change_time_formats(&cf);
+         XXX - redissecting the packets could actually be faster;
+	 we have to find the row number for each frame, in order to
+	 update the time stamp columns, and doing that is linear in
+	 the row number, which means the whole process is N^2 in
+	 the number of rows, whilst redissecting the packets is only
+	 linear in the number of rows (assuming you're using our
+	 CList code, or the GTK+ 1.2.8 CList code, or other CList
+	 code which doesn't have to scan the entire list to find the
+	 last element), even though the latter involves doing more work
+	 per packet. */
+    current_timestamp_type = timestamp_type;
+    change_time_formats(&cfile);
   }
+}
+
+static void
+display_opt_close_cb(GtkWidget *close_bt, gpointer parent_w)
+{
+  /* Revert the timestamp type to the value it has when we started. */
+  timestamp_type = initial_timestamp_type;
+
+  /* Update the display if either of those changed. */
+  update_display();
 
   gtk_grab_remove(GTK_WIDGET(parent_w));
   gtk_widget_destroy(GTK_WIDGET(parent_w));
