@@ -1,7 +1,7 @@
 /* file_dlg.c
  * Dialog boxes for handling files
  *
- * $Id: file_dlg.c,v 1.24 2000/06/02 03:35:39 gram Exp $
+ * $Id: file_dlg.c,v 1.31 2000/08/21 15:45:32 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -37,34 +37,16 @@
 
 #include <string.h>
 
-#ifndef __GLOBALS_H__
 #include "globals.h"
-#endif
-#ifndef __GTKGLOBALS_H__
 #include "gtkglobals.h"
-#endif
-
-#ifndef __KEYS_H__
 #include "keys.h"
-#endif
-
 #include "filter_prefs.h"
 #include "ui_util.h"
-
-#ifndef __DIALOG_H__
 #include "simple_dialog.h"
-#endif
-
-#ifndef __MENU_H__
 #include "menu.h"
-#endif
-
 #include "file_dlg.h"
 #include "dlg_utils.h"
-
-#ifndef __UTIL_H__
 #include "util.h"
-#endif
 
 static void file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs);
 static void file_open_destroy_cb(GtkWidget *win, gpointer user_data);
@@ -94,25 +76,15 @@ file_open_cmd_cb(GtkWidget *w, gpointer data) {
     return;
   }
 
-  /* XXX - GTK+'s file selection dialog box doesn't let you set the
-     initial directory it should show; it always uses the current
-     directory.  We want to start out by showing the user the files
-     in the last directory in which they looked, so we have to "chdir()"
-     there.
-
-     This means means that if Ethereal dumps core, the core file will be
-     dumped in whatever directory it last "chdir()"red to, rather
-     than in the directory in which you started it.
-
-     It also means, for better or worse, that *all* file selection
-     dialog boxes will start in that directory. */
-  if (last_open_dir)
-	  chdir(last_open_dir);
-
   file_open_w = gtk_file_selection_new ("Ethereal: Open Capture File");
   gtk_signal_connect(GTK_OBJECT(file_open_w), "destroy",
 	GTK_SIGNAL_FUNC(file_open_destroy_cb), NULL);
 
+  /* If we've opened a file, start out by showing the files in the directory
+     in which that file resided. */
+  if (last_open_dir)
+    gtk_file_selection_complete(GTK_FILE_SELECTION(file_open_w), last_open_dir);
+
   resolv_cb = dlg_check_button_new_with_label_with_mnemonic(
 		  "Enable name resolution", NULL);
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(resolv_cb), g_resolving_actif);
@@ -175,29 +147,23 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
   filter_te = gtk_object_get_data(GTK_OBJECT(w), E_RFILTER_TE_KEY);
   rfilter = gtk_entry_get_text(GTK_ENTRY(filter_te));
   if (dfilter_compile(rfilter, &rfcode) != 0) {
-    simple_dialog(ESD_TYPE_WARN, NULL, dfilter_error_msg);
+    simple_dialog(ESD_TYPE_CRIT, NULL, dfilter_error_msg);
     return;
   }
 
   /* Perhaps the user specified a directory instead of a file.
-   * See if we can chdir to it. */
-  if (chdir(cf_name) == 0) {
-	int	cf_name_len;
-
+     Check whether they did. */
+  if (test_for_directory(cf_name) == EISDIR) {
+	/* It's a directory - set the file selection box to display that
+	   directory, don't try to open the directory as a capture file. */
 	g_free(last_open_dir);
-
-	/* Append a slash, even if not needed */
-	cf_name_len = strlen(cf_name);
-	last_open_dir = g_malloc(cf_name_len + 2);
-	strcpy(last_open_dir, cf_name);
-	strcat(last_open_dir, "/");
-	g_free(cf_name);
-    	gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), last_open_dir);
+	last_open_dir = cf_name;
+	gtk_file_selection_complete(GTK_FILE_SELECTION(fs), last_open_dir);
     	return;
   }
 
   /* Try to open the capture file. */
-  if ((err = open_cap_file(cf_name, FALSE, &cf)) != 0) {
+  if ((err = open_cap_file(cf_name, FALSE, &cfile)) != 0) {
     /* We couldn't open it; don't dismiss the open dialog box,
        just leave it around so that the user can, after they
        dismiss the alert box popped up for the open error,
@@ -211,7 +177,7 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
   /* Attach the new read filter to "cf" ("open_cap_file()" succeeded, so
      it closed the previous capture file, and thus destroyed any
      previous read filter attached to "cf"). */
-  cf.rfcode = rfcode;
+  cfile.rfcode = rfcode;
 
   /* Set the global resolving variable */
   resolv_cb = gtk_object_get_data(GTK_OBJECT(w), E_FILE_RESOLVE_KEY);
@@ -221,7 +187,24 @@ file_open_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
   gtk_widget_hide(GTK_WIDGET (fs));
   gtk_widget_destroy(GTK_WIDGET (fs));
 
-  err = read_cap_file(&cf);
+  switch (read_cap_file(&cfile, &err)) {
+
+  case READ_SUCCESS:
+  case READ_ERROR:
+    /* Just because we got an error, that doesn't mean we were unable
+       to read any of the file; we handle what we could get from the
+       file. */
+    break;
+
+  case READ_ABORTED:
+    /* The user bailed out of re-reading the capture file; the
+       capture file has been closed - just free the capture file name
+       string and return (without changing the last containing
+       directory). */
+    g_free(cf_name);
+    return;
+  }
+    
   /* Save the name of the containing directory specified in the path name,
      if any; we can write over cf_name, which is a good thing, given that
      "get_dirname()" does write over its argument. */
@@ -269,13 +252,13 @@ file_open_destroy_cb(GtkWidget *win, gpointer user_data)
 /* Close a file */
 void
 file_close_cmd_cb(GtkWidget *widget, gpointer data) {
-  close_cap_file(&cf, info_bar);
+  close_cap_file(&cfile, info_bar);
 }
 
 void
 file_save_cmd_cb(GtkWidget *w, gpointer data) {
   /* If the file's already been saved, do nothing.  */
-  if (cf.user_saved)
+  if (cfile.user_saved)
     return;
 
   /* Do a "Save As". */
@@ -284,8 +267,10 @@ file_save_cmd_cb(GtkWidget *w, gpointer data) {
 
 /* XXX - can we make these not be static? */
 static gboolean filtered;
+static gboolean marked;
 static int filetype;
 static GtkWidget *filter_cb;
+static GtkWidget *mark_cb;
 static GtkWidget *ft_om;
 
 static gboolean
@@ -294,7 +279,7 @@ can_save_with_wiretap(int ft)
   /* To save a file with Wiretap, Wiretap has to handle that format,
      and its code to handle that format must be able to write a file
      with this file's encapsulation type. */
-  return wtap_dump_can_open(ft) && wtap_dump_can_write_encap(ft, cf.lnk_t);
+  return wtap_dump_can_open(ft) && wtap_dump_can_write_encap(ft, cfile.lnk_t);
 }
 
 /* Generate a list of the file types we can save this file as.
@@ -307,7 +292,11 @@ can_save_with_wiretap(int ft)
    "filtered" is TRUE if we're to save only the packets that passed
    the display filter (in which case we have to save it using Wiretap)
    and FALSE if we're to save the entire file (in which case, if we're
-   saving it in the type it has already, we can just copy it). */
+   saving it in the type it has already, we can just copy it). 
+
+   "marked" is TRUE if we have to save only the marked packets,
+   the same remark as "filtered" applies.
+*/
 static void
 set_file_type_list(GtkWidget *option_menu)
 {
@@ -325,8 +314,8 @@ set_file_type_list(GtkWidget *option_menu)
   /* Check all file types. */
   index = 0;
   for (ft = 0; ft < WTAP_NUM_FILE_TYPES; ft++) {
-    if (filtered || ft != cf.cd_t) {
-      /* Filtered, or a different file type.  We have to use Wiretap. */
+    if (filtered || marked || ft != cfile.cd_t) {
+      /* Filtered, marked or a different file type.  We have to use Wiretap. */
       if (!can_save_with_wiretap(ft))
         continue;	/* We can't. */
     }
@@ -354,7 +343,7 @@ select_file_type_cb(GtkWidget *w, gpointer data)
   int new_filetype = (int)data;
 
   if (filetype != new_filetype) {
-    /* We can select only the filtered packets to be saved iff we can
+    /* We can select only the filtered or marked packets to be saved if we can
        use Wiretap to save the file. */
     gtk_widget_set_sensitive(filter_cb, can_save_with_wiretap(new_filetype));
     filetype = new_filetype;
@@ -375,6 +364,20 @@ toggle_filtered_cb(GtkWidget *widget, gpointer data)
   }
 }
 
+static void
+toggle_marked_cb(GtkWidget *widget, gpointer data)
+{
+  gboolean new_marked;
+
+  new_marked = GTK_TOGGLE_BUTTON (widget)->active;
+
+  if (marked != new_marked) {
+    /* They changed the state of the "marked" button. */
+    marked = new_marked;
+    set_file_type_list(ft_om);
+  }
+}
+
 /*
  * Keep a static pointer to the current "Save Capture File As" window, if
  * any, so that if somebody tries to do "File:Save" or "File:Save As"
@@ -396,12 +399,18 @@ file_save_as_cmd_cb(GtkWidget *w, gpointer data)
 
   /* Default to saving all packets, in the file's current format. */
   filtered = FALSE;
-  filetype = cf.cd_t;
+  marked   = FALSE;
+  filetype = cfile.cd_t;
 
   file_save_as_w = gtk_file_selection_new ("Ethereal: Save Capture File As");
   gtk_signal_connect(GTK_OBJECT(file_save_as_w), "destroy",
 	GTK_SIGNAL_FUNC(file_save_as_destroy_cb), NULL);
 
+  /* If we've opened a file, start out by showing the files in the directory
+     in which that file resided. */
+  if (last_open_dir)
+    gtk_file_selection_complete(GTK_FILE_SELECTION(file_save_as_w), last_open_dir);
+
   /* Connect the ok_button to file_save_as_ok_cb function and pass along a
      pointer to the file selection box widget */
   ok_bt = GTK_FILE_SELECTION (file_save_as_w)->ok_button;
@@ -423,6 +432,14 @@ file_save_as_cmd_cb(GtkWidget *w, gpointer data)
   gtk_widget_set_sensitive(filter_cb, can_save_with_wiretap(filetype));
   gtk_widget_show(filter_cb);
 
+  mark_cb = gtk_check_button_new_with_label("Save only marked packets");
+  gtk_container_add(GTK_CONTAINER(main_vb), mark_cb);
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(mark_cb), FALSE);
+  gtk_signal_connect(GTK_OBJECT(mark_cb), "toggled",
+		     GTK_SIGNAL_FUNC(toggle_marked_cb), NULL);
+  gtk_widget_set_sensitive(mark_cb, can_save_with_wiretap(filetype));
+  gtk_widget_show(mark_cb);
+
   /* File type row */
   ft_hb = gtk_hbox_new(FALSE, 3);
   gtk_container_add(GTK_CONTAINER(main_vb), ft_hb);
@@ -462,8 +479,8 @@ file_save_as_ok_cb(GtkWidget *w, GtkFileSelection *fs) {
   gtk_widget_destroy(GTK_WIDGET (fs));
 
   /* Write out the packets (all, or only the ones that are currently
-     displayed) to the file with the specified name. */
-  save_cap_file(cf_name, &cf, filtered, filetype);
+     displayed or marked) to the file with the specified name. */
+  save_cap_file(cf_name, &cfile, filtered, marked, filetype);
 
   /* If "save_cap_file()" saved the file name we handed it, it saved
      a copy, so we should free up our copy. */
@@ -484,12 +501,13 @@ file_reload_cmd_cb(GtkWidget *w, gpointer data) {
   GtkWidget *filter_te;
   gchar *filename;
   gboolean is_tempfile;
+  int err;
 
   filter_te = gtk_object_get_data(GTK_OBJECT(w), E_DFILTER_TE_KEY);
 
-  if (cf.dfilter)
-    g_free(cf.dfilter);
-  cf.dfilter = g_strdup(gtk_entry_get_text(GTK_ENTRY(filter_te)));
+  if (cfile.dfilter)
+    g_free(cfile.dfilter);
+  cfile.dfilter = g_strdup(gtk_entry_get_text(GTK_ENTRY(filter_te)));
 
   /* If the file could be opened, "open_cap_file()" calls "close_cap_file()"
      to get rid of state for the old capture file before filling in state
@@ -499,21 +517,37 @@ file_reload_cmd_cb(GtkWidget *w, gpointer data) {
      a temporary file, mark it as not being a temporary file, and then
      reopen it as the type of file it was.
 
-     Also, "close_cap_file()" will free "cf.filename", so we must make
+     Also, "close_cap_file()" will free "cfile.filename", so we must make
      a copy of it first. */
-  filename = strdup(cf.filename);
-  is_tempfile = cf.is_tempfile;
-  cf.is_tempfile = FALSE;
-  if (open_cap_file(filename, is_tempfile, &cf) == 0)
-    read_cap_file(&cf);
-  else {
-    /* The open failed, so "cf.is_tempfile" wasn't set to "is_tempfile".
-       Instead, the file was left open, so we should restore "cf.is_tempfile"
+  filename = strdup(cfile.filename);
+  is_tempfile = cfile.is_tempfile;
+  cfile.is_tempfile = FALSE;
+  if (open_cap_file(filename, is_tempfile, &cfile) == 0) {
+    switch (read_cap_file(&cfile, &err)) {
+
+    case READ_SUCCESS:
+    case READ_ERROR:
+      /* Just because we got an error, that doesn't mean we were unable
+         to read any of the file; we handle what we could get from the
+         file. */
+      break;
+
+    case READ_ABORTED:
+      /* The user bailed out of re-reading the capture file; the
+         capture file has been closed - just free the capture file name
+         string and return (without changing the last containing
+         directory). */
+      g_free(filename);
+      return;
+    }
+  } else {
+    /* The open failed, so "cfile.is_tempfile" wasn't set to "is_tempfile".
+       Instead, the file was left open, so we should restore "cfile.is_tempfile"
        ourselves.
 
        XXX - change the menu?  Presumably "open_cap_file()" will do that;
        make sure it does! */
-    cf.is_tempfile = is_tempfile;
+    cfile.is_tempfile = is_tempfile;
   }
   /* "open_cap_file()" made a copy of the file name we handed it, so
      we should free up our copy. */
