@@ -1,7 +1,7 @@
 /* goto_dlg.c
  * Routines for "go to frame" window
  *
- * $Id: goto_dlg.c,v 1.6 2000/05/02 08:04:31 guy Exp $
+ * $Id: goto_dlg.c,v 1.11 2000/08/23 06:55:54 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -36,9 +36,7 @@
 #include <unistd.h>
 #endif
 
-#ifndef __G_LIB_H__
 #include <glib.h>
-#endif
 
 #include "proto.h"
 #include "dfilter.h"
@@ -64,8 +62,7 @@ goto_frame_cb(GtkWidget *w, gpointer d)
   GtkWidget     *goto_frame_w, *main_vb, *fnumber_hb, *fnumber_lb, *fnumber_te,
                 *bbox, *ok_bt, *cancel_bt;
 
-  goto_frame_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(goto_frame_w), "Ethereal: Go To Frame");
+  goto_frame_w = dlg_window_new("Ethereal: Go To Frame");
   
   /* Container for each row of widgets */
   main_vb = gtk_vbox_new(FALSE, 3);
@@ -145,20 +142,20 @@ goto_frame_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
        XXX - what about negative numbers (which "strtoul()" allows)?
        Can we hack up signal handlers for the widget to make it
        reject attempts to type in characters other than digits? */
-    simple_dialog(ESD_TYPE_WARN, NULL,
+    simple_dialog(ESD_TYPE_CRIT, NULL,
 		"The frame number you entered isn't a valid number.");
     return;
   }
 
-  switch (goto_frame(&cf, fnumber)) {
+  switch (goto_frame(&cfile, fnumber)) {
 
   case NO_SUCH_FRAME:
-    simple_dialog(ESD_TYPE_WARN, NULL, "There is no frame with that frame number.");
+    simple_dialog(ESD_TYPE_CRIT, NULL, "There is no frame with that frame number.");
     return;
 
   case FRAME_NOT_DISPLAYED:
     /* XXX - add it to the display filter? */
-    simple_dialog(ESD_TYPE_WARN, NULL, "The frame with that frame number is not currently being displayed.");
+    simple_dialog(ESD_TYPE_CRIT, NULL, "The frame with that frame number is not currently being displayed.");
     return;
 
   case FOUND_FRAME:
