@@ -1,7 +1,7 @@
 /* menu.c
  * Menu routines
  *
- * $Id: menu.c,v 1.30 2000/05/02 07:44:37 guy Exp $
+ * $Id: menu.c,v 1.44 2000/09/09 08:17:51 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -53,7 +53,9 @@
 #include "prefs_dlg.h"
 #include "packet_win.h"
 #include "print.h"
-#include "follow.h"
+#include "follow_dlg.h"
+#include "help_dlg.h"
+#include "proto_dlg.h"
 #include "keys.h"
 #include "plugins.h"
 
@@ -102,15 +104,23 @@ static GtkItemFactoryEntry menu_items[] =
   {"/File/<separator>", NULL, NULL, 0, "<Separator>"},
   {"/File/_Quit", "<control>Q", GTK_MENU_FUNC(file_quit_cmd_cb), 0, NULL},
   {"/_Edit", NULL, NULL, 0, "<Branch>" },
+#if 0
+  /* Un-#if this when we actually implement Cut/Copy/Paste. */
   {"/Edit/Cut", "<control>X", NULL, 0, NULL},
   {"/Edit/Copy", "<control>C", NULL, 0, NULL},
   {"/Edit/Paste", "<control>V", NULL, 0, NULL},
   {"/Edit/<separator>", NULL, NULL, 0, "<Separator>"},
+#endif
   {"/Edit/_Find Frame...", "<control>F", GTK_MENU_FUNC(find_frame_cb), 0, NULL},
   {"/Edit/_Go To Frame...", "<control>G", GTK_MENU_FUNC(goto_frame_cb), 0, NULL},
   {"/Edit/<separator>", NULL, NULL, 0, "<Separator>"},
-  {"/Edit/_Preferences...", NULL, GTK_MENU_FUNC(prefs_cb), E_PR_PG_NONE, NULL},
+  {"/Edit/_Mark Frame", "<control>M", GTK_MENU_FUNC(mark_frame_cb), 0, NULL},
+  {"/Edit/Mark _All Frames", NULL, GTK_MENU_FUNC(mark_all_frames_cb), 0, NULL},
+  {"/Edit/_Unmark All Frames", NULL, GTK_MENU_FUNC(unmark_all_frames_cb), 0, NULL},
+  {"/Edit/<separator>", NULL, NULL, 0, "<Separator>"},
+  {"/Edit/_Preferences...", NULL, GTK_MENU_FUNC(prefs_cb), 0, NULL},
   {"/Edit/_Filters...", NULL, GTK_MENU_FUNC(filter_dialog_cb), 0, NULL},
+  {"/Edit/P_rotocols...", NULL, GTK_MENU_FUNC(proto_cb), 0, NULL},
 #ifdef HAVE_LIBPCAP
   {"/_Capture", NULL, NULL, 0, "<Branch>" },
   {"/Capture/_Start...", "<control>K", GTK_MENU_FUNC(capture_prep_cb), 0, NULL},
@@ -130,6 +140,8 @@ static GtkItemFactoryEntry menu_items[] =
 /*  {"/Tools/Graph", NULL, NULL, 0, NULL}, future use */
   {"/Tools/_Summary", NULL, GTK_MENU_FUNC(summary_open_cb), 0, NULL},
   {"/_Help", NULL, NULL, 0, "<LastBranch>" },
+  {"/Help/_Help", NULL, GTK_MENU_FUNC(help_cb), 0, NULL},
+  {"/Help/<separator>", NULL, NULL, 0, "<Separator>"},
   {"/Help/_About Ethereal...", NULL, GTK_MENU_FUNC(about_ethereal), 0, NULL}
 };
 
@@ -139,7 +151,6 @@ static int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
 /* packet list popup */
 static GtkItemFactoryEntry packet_list_menu_items[] =
 {
-	{"/Match Selected", NULL, GTK_MENU_FUNC(match_selected_cb), 0, NULL},
 	{"/Follow TCP Stream", NULL, GTK_MENU_FUNC(follow_stream_cb), 0, NULL},
 	{"/Filters...", NULL, GTK_MENU_FUNC(filter_dialog_cb), 0, NULL},
 	{"/<separator>", NULL, NULL, 0, "<Separator>"},
@@ -151,20 +162,28 @@ static GtkItemFactoryEntry packet_list_menu_items[] =
 
 static GtkItemFactoryEntry tree_view_menu_items[] =
 {
-	{"/Match Selected", NULL, GTK_MENU_FUNC(match_selected_cb), 0, NULL},
-	{"/Resolve Name", NULL, GTK_MENU_FUNC(resolve_name_cb), 0, NULL},
 	{"/Follow TCP Stream", NULL, GTK_MENU_FUNC(follow_stream_cb), 0, NULL},
 	{"/Filters...", NULL, GTK_MENU_FUNC(filter_dialog_cb), 0, NULL},
 	{"/<separator>", NULL, NULL, 0, "<Separator>"},
+	{"/Resolve Name", NULL, GTK_MENU_FUNC(resolve_name_cb), 0, NULL},
+	{"/Protocol Properties...", NULL, GTK_MENU_FUNC(properties_cb), 0, NULL},
+	{"/Match Selected", NULL, GTK_MENU_FUNC(match_selected_cb), 0, NULL},
+	{"/<separator>", NULL, NULL, 0, "<Separator>"},
 	{"/Collapse All", NULL, GTK_MENU_FUNC(collapse_all_cb), 0, NULL},
 	{"/Expand All", NULL, GTK_MENU_FUNC(expand_all_cb), 0, NULL}
 };
 
+static GtkItemFactoryEntry hexdump_menu_items[] =
+{
+	{"/Follow TCP Stream", NULL, GTK_MENU_FUNC(follow_stream_cb), 0, NULL},
+	{"/Filters...", NULL, GTK_MENU_FUNC(filter_dialog_cb), 0, NULL}
+};
 
 static int initialize = TRUE;
 static GtkItemFactory *factory = NULL;
 static GtkItemFactory *packet_list_menu_factory = NULL;
 static GtkItemFactory *tree_view_menu_factory = NULL;
+static GtkItemFactory *hexdump_menu_factory = NULL;
 
 static GSList *popup_menu_list = NULL;
 
@@ -204,16 +223,26 @@ menus_init(void) {
     gtk_item_factory_create_items_ac(tree_view_menu_factory, sizeof(tree_view_menu_items)/sizeof(tree_view_menu_items[0]), tree_view_menu_items, NULL, 2);
     gtk_object_set_data(GTK_OBJECT(popup_menu_object), PM_TREE_VIEW_KEY, tree_view_menu_factory->widget);
     popup_menu_list = g_slist_append((GSList *)popup_menu_list, tree_view_menu_factory);
+
+    hexdump_menu_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);
+    gtk_item_factory_create_items_ac(hexdump_menu_factory, sizeof(hexdump_menu_items)/sizeof(hexdump_menu_items[0]), hexdump_menu_items, NULL, 2);
+    gtk_object_set_data(GTK_OBJECT(popup_menu_object), PM_HEXDUMP_KEY, hexdump_menu_factory->widget);
+    popup_menu_list = g_slist_append((GSList *)popup_menu_list, hexdump_menu_factory);
     
     factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", grp);
     gtk_item_factory_create_items_ac(factory, nmenu_items, menu_items, NULL,2);
     set_menus_for_unsaved_capture_file(FALSE);
     set_menus_for_capture_file(FALSE);
+#if 0
+    /* Un-#if this when we actually implement Cut/Copy/Paste.
+       Then make sure you enable them when they can be done. */
     set_menu_sensitivity("/Edit/Cut", FALSE);
     set_menu_sensitivity("/Edit/Copy", FALSE);
     set_menu_sensitivity("/Edit/Paste", FALSE);
+#endif
     set_menus_for_captured_packets(FALSE);
     set_menus_for_selected_packet(FALSE);
+    set_menus_for_selected_tree_row(FALSE);
   }
 }
 
@@ -261,14 +290,14 @@ set_menu_object_data (gchar *path, gchar *key, gpointer data) {
   }
 }
 
-void
-popup_menu_handler(GtkWidget *widget, GdkEvent *event)
+gint
+popup_menu_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
 	GtkWidget *menu = NULL;
 	GdkEventButton *event_button = NULL;
 
-	if(widget == NULL || event == NULL) {
-		return;
+	if(widget == NULL || event == NULL || data == NULL) {
+		return FALSE;
 	}
 	
 	/*
@@ -278,14 +307,17 @@ popup_menu_handler(GtkWidget *widget, GdkEvent *event)
 	 * I guess, like a CList with one column(?) and the expander widget
 	 * as a pixmap.
 	 */
-	menu = widget;
+	menu = (GtkWidget *)data;
 	if(event->type == GDK_BUTTON_PRESS) {
 		event_button = (GdkEventButton *) event;
 		
 		if(event_button->button == 3) {
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event_button->button, event_button->time);
+			gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
+			return TRUE;
 		}
 	}
+	return FALSE;
 }
 
 /* Enable or disable menu items based on whether you have a capture file
@@ -334,11 +366,30 @@ void
 set_menus_for_selected_packet(gboolean have_selected_packet)
 {
   set_menu_sensitivity("/File/Print Packet", have_selected_packet);
+  set_menu_sensitivity("/Edit/Mark Frame", have_selected_packet);
+  set_menu_sensitivity("/Edit/Mark All Frames", have_selected_packet);
+  set_menu_sensitivity("/Edit/Unmark All Frames", have_selected_packet);
   set_menu_sensitivity("/Display/Collapse All", have_selected_packet);
   set_menu_sensitivity("/Display/Expand All", have_selected_packet);
   set_menu_sensitivity("/Display/Show Packet In New Window", have_selected_packet);
   set_menu_sensitivity("/Tools/Follow TCP Stream",
       have_selected_packet ? (pi.ipproto == 6) : FALSE);
   set_menu_sensitivity("/Resolve Name", 
-      have_selected_packet && !g_resolving_actif);
+      have_selected_packet && !g_resolving_actif);  
+}
+
+/* Enable or disable menu items based on whether a tree row is selected. */
+void
+set_menus_for_selected_tree_row(gboolean have_selected_tree)
+{
+  gboolean properties = FALSE;
+  if (finfo_selected) {
+	header_field_info *hfinfo = finfo_selected->hfinfo;
+	if (hfinfo->parent == -1) {
+	  properties = prefs_is_registered_protocol(hfinfo->abbrev);
+	} else {
+	  properties = prefs_is_registered_protocol(proto_registrar_get_abbrev(hfinfo->parent));
+	}
+  }
+  set_menu_sensitivity("/Protocol Properties...", have_selected_tree && properties);
 }
