@@ -1,7 +1,7 @@
 /* print_dlg.c
  * Dialog boxes for printing
  *
- * $Id: print_dlg.c,v 1.17 2000/05/08 07:13:40 guy Exp $
+ * $Id: print_dlg.c,v 1.21 2000/08/23 06:56:20 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -29,26 +29,11 @@
 
 #include <errno.h>
 
-#ifndef __GLOBALS_H__
 #include "globals.h"
-#endif
-
-#ifndef __KEYS_H__
 #include "keys.h"
-#endif
-
-#ifndef __PRINT_H__
 #include "print.h"
-#endif
-
-#ifndef __PREFS_DLG_H__
 #include "prefs_dlg.h"
-#endif
-
-#ifndef __DIALOG_H__
 #include "simple_dialog.h"
-#endif
-
 #include "ui_util.h"
 #include "dlg_utils.h"
 
@@ -119,8 +104,7 @@ file_print_cmd_cb(GtkWidget *widget, gpointer data)
     return;
   }
 
-  print_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(print_w), "Ethereal: Print");
+  print_w = dlg_window_new("Ethereal: Print");
   gtk_signal_connect(GTK_OBJECT(print_w), "destroy",
 	GTK_SIGNAL_FUNC(print_destroy_cb), NULL);
 
@@ -508,7 +492,7 @@ print_ok_cb(GtkWidget *ok_bt, gpointer parent_w)
   gtk_widget_destroy(GTK_WIDGET(parent_w));
 
   /* Now print the packets */
-  if (!print_packets(&cf, &print_args)) {
+  if (!print_packets(&cfile, &print_args)) {
     if (print_args.to_file)
       simple_dialog(ESD_TYPE_WARN, NULL,
         file_write_error_message(errno), print_args.dest);
@@ -590,8 +574,8 @@ file_print_packet_cmd_cb(GtkWidget *widget, gpointer data) {
   print_args.print_summary = FALSE;
   print_args.print_hex = FALSE;
   print_args.expand_all = TRUE;
-  proto_tree_print(TRUE, &print_args, (GNode*) cf.protocol_tree, cf.pd,
-		cf.current_frame, fh);
+  proto_tree_print(TRUE, &print_args, (GNode*) cfile.protocol_tree, cfile.pd,
+		cfile.current_frame, fh);
   print_finale(fh, prefs.pr_format);
   close_print_dest(print_args.to_file, fh);
 }
