@@ -1,7 +1,7 @@
-/* dialog.c
- * Dialog box routines.
+/* simple_dialog.c
+ * Simple message dialog box routines.
  *
- * $Id: simple_dialog.c,v 1.2 2000/05/03 07:19:38 guy Exp $
+ * $Id: simple_dialog.c,v 1.6 2000/08/23 06:56:31 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -35,11 +35,6 @@
 #include <stdio.h>
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
@@ -81,25 +76,24 @@ simple_dialog(gint type, gint *btn_mask, gchar *msg_format, ...) {
   gchar      **icon;
 
   /* Main window */
-  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_container_border_width(GTK_CONTAINER(win), 7);
-
   switch (type) {
   case ESD_TYPE_WARN :
-    gtk_window_set_title(GTK_WINDOW(win), "Ethereal: Warning");
     icon = icon_excl_xpm;
+    win = dlg_window_new("Ethereal: Warning");
     break;
   case ESD_TYPE_CRIT :
-    gtk_window_set_title(GTK_WINDOW(win), "Ethereal: Critical");
     icon = icon_excl_xpm;
+    win = dlg_window_new("Ethereal: Error");
     break;
   case ESD_TYPE_INFO :
   default :
     icon = icon_ethereal_xpm;
-    gtk_window_set_title(GTK_WINDOW(win), "Ethereal: Information");
+    win = dlg_window_new("Ethereal: Information");
     break;
   }
 
+  gtk_container_border_width(GTK_CONTAINER(win), 7);
+
   gtk_object_set_data(GTK_OBJECT(win), bm_key, btn_mask);
 
   /* Container for our rows */
