@@ -1,6 +1,6 @@
-/* inet_pton.h
+/* inet_v6defs.h
  *
- * $Id: inet_v6defs.h,v 1.2 2000/01/10 17:32:51 gram Exp $
+ * $Id: inet_v6defs.h,v 1.4 2000/08/11 13:34:19 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -21,17 +21,17 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __INET_V6DEFS_H__
+#define __INET_V6DEFS_H__
+
 /*
- * Version of "inet_pton()" and "inet_ntop()", for the benefit of OSes that
+ * Versions of "inet_pton()" and "inet_ntop()", for the benefit of OSes that
  * don't have it.
  */
 extern int inet_pton(int af, const char *src, void *dst);
 extern const char *inet_ntop(int af, const void *src, char *dst,
     size_t size);
 
-struct in_addr;
-extern int inet_aton(const char* cp_arg, struct in_addr *addr);
-
 /*
  * Those OSes may also not have AF_INET6, so declare it here if it's not
  * already declared, so that we can pass it to "inet_ntop()" and "inet_pton()".
@@ -48,3 +48,5 @@ extern int inet_aton(const char* cp_arg, struct in_addr *addr);
 #ifndef __P
 #define __P(args)	args
 #endif
+
+#endif
