@@ -1,7 +1,7 @@
 /* packet-arp.c
  * Routines for ARP packet disassembly
  *
- * $Id: packet-arp.c,v 1.30 2000/05/31 05:06:50 guy Exp $
+ * $Id: packet-arp.c,v 1.33 2000/08/13 14:07:58 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -33,6 +33,7 @@
 
 #include <glib.h>
 #include "packet.h"
+#include "resolv.h"
 #include "packet-arp.h"
 #include "etypes.h"
 
@@ -388,7 +389,7 @@ dissect_atmarp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   gchar       *tha_str, *tsa_str, *tpa_str;
 
   if (!BYTES_ARE_IN_FRAME(offset, MIN_ATMARP_HEADER_SIZE)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -409,7 +410,7 @@ dissect_atmarp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   tot_len = MIN_ATMARP_HEADER_SIZE + ar_shtl + ar_ssl + ar_spln +
 				ar_thtl + ar_tsl + ar_tpln;
   if (!BYTES_ARE_IN_FRAME(offset, tot_len)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -563,8 +564,10 @@ dissect_arp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   int         sha_offset, spa_offset, tha_offset, tpa_offset;
   gchar       *sha_str, *spa_str, *tha_str, *tpa_str;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_arp, pd, offset, fd, tree);
+
   if (!BYTES_ARE_IN_FRAME(offset, MIN_ARP_HEADER_SIZE)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -580,7 +583,7 @@ dissect_arp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 
   tot_len = MIN_ARP_HEADER_SIZE + ar_hln*2 + ar_pln*2;
   if (!BYTES_ARE_IN_FRAME(offset, tot_len)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -637,6 +640,25 @@ dissect_arp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     }
   }
 
+  if ((ar_op == ARPOP_REPLY || ar_op == ARPOP_REQUEST) &&
+      ar_hln == 6 && ar_pln == 4) {
+
+    /* inform resolv.c module of the new discovered addresses */
+
+    u_int ip;
+
+    /* add sender address in all cases */
+
+    memcpy(&ip, &pd[spa_offset], sizeof(ip));
+    add_ether_byip(ip, &pd[sha_offset]);
+    
+    if (ar_op == ARPOP_REQUEST) {
+      /* add destination address */
+      memcpy(&ip, &pd[tpa_offset], sizeof(ip));
+      add_ether_byip(ip, &pd[tha_offset]);
+    }
+  }
+
   if (tree) {
     if ((op_str = match_strval(ar_op, op_vals)))
       ti = proto_tree_add_protocol_format(tree, proto_arp, NullTVB, offset, tot_len,
@@ -796,6 +818,6 @@ proto_register_arp(void)
 void
 proto_reg_handoff_arp(void)
 {
-	dissector_add("ethertype", ETHERTYPE_ARP, dissect_arp);
-	dissector_add("ethertype", ETHERTYPE_REVARP, dissect_arp);
+	old_dissector_add("ethertype", ETHERTYPE_ARP, dissect_arp);
+	old_dissector_add("ethertype", ETHERTYPE_REVARP, dissect_arp);
 }
