@@ -1,6 +1,6 @@
 /* packet-ascend.h
  *
- * $Id: packet-ascend.h,v 1.4 2000/05/19 23:06:08 gram Exp $
+ * $Id: packet-ascend.h,v 1.5 2000/08/11 13:34:18 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -22,4 +22,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_ASCEND_H__
+#define __PACKET_ASCEND_H__
+
 void dissect_ascend(tvbuff_t *, packet_info *, proto_tree *);
+
+#endif
