@@ -2,7 +2,7 @@
  * Routines for BGP packet dissection.
  * Copyright 1999, Jun-ichiro itojun Hagino <itojun@itojun.org>
  *
- * $Id: packet-bgp.c,v 1.23 2000/05/11 08:14:59 gram Exp $
+ * $Id: packet-bgp.c,v 1.27 2000/08/20 18:10:12 deniel Exp $
  * 
  * Supports:
  * RFC1771 A Border Gateway Protocol 4 (BGP-4)
@@ -49,25 +49,21 @@
 # include <netinet/in.h>
 #endif
 
-#ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
-# include "snprintf.h"
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
 #endif
 
 #include <string.h>
 #include <glib.h>
+
+#ifdef NEED_SNPRINTF_H
+# include "snprintf.h"
+#endif
+
 #include "packet.h"
 #include "packet-bgp.h"
 #include "packet-ipv6.h"
 
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-
 #define TCP_PORT_BGP			179
 
 static const value_string bgptypevals[] = {
@@ -188,6 +184,7 @@ static const value_string afnumber[] = {
 };
 
 static int proto_bgp = -1;
+static int hf_bgp_type = -1;
 
 static gint ett_bgp = -1;
 static gint ett_bgp_unfeas = -1;
@@ -1141,6 +1138,8 @@ dissect_bgp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     int           hlen;          /* BGP header length                */
     char          *typ;          /* BGP message type                 */
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_bgp, pd, offset, fd, tree);
+
     if (check_col(fd, COL_PROTOCOL))
 	col_add_str(fd, COL_PROTOCOL, "BGP");
 
@@ -1179,8 +1178,8 @@ dissect_bgp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     }
 
     if (tree) {
-	ti = proto_tree_add_text(tree, NullTVB, offset, END_OF_FRAME,
-		    "Border Gateway Protocol");
+        ti = proto_tree_add_item(tree, proto_bgp, NullTVB, offset, 
+				 END_OF_FRAME, FALSE);
 	bgp_tree = proto_item_add_subtree(ti, ett_bgp);
 
 	p = &pd[offset];
@@ -1244,9 +1243,11 @@ dissect_bgp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		    (hlen == 1) ? "byte" : "bytes");
 	    }
 
-	    proto_tree_add_text(bgp1_tree, NullTVB,
-		offset + i + offsetof(struct bgp, bgp_type), 1,
-		"Type: %s (%u)", typ, bgp.bgp_type);
+	    proto_tree_add_uint_format(bgp1_tree, hf_bgp_type, NullTVB, 
+				       offset + i + 
+				       offsetof(struct bgp, bgp_type), 1,
+				       bgp.bgp_type,
+				       "Type: %s (%u)", typ, bgp.bgp_type);
 
 	    CHECK_SIZE(i, hlen, l);
 
@@ -1279,6 +1280,13 @@ dissect_bgp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 void
 proto_register_bgp(void)
 {
+
+    static hf_register_info hf[] = {
+      { &hf_bgp_type,
+	{ "BGP message type", "bgp.type", FT_UINT8, BASE_HEX, 
+	  VALS(bgptypevals), 0x0, "BGP message type" }},
+    };
+
     static gint *ett[] = {
       &ett_bgp,
       &ett_bgp_unfeas,
@@ -1297,11 +1305,12 @@ proto_register_bgp(void)
     };
 
     proto_bgp = proto_register_protocol("Border Gateway Protocol", "bgp");
+    proto_register_field_array(proto_bgp, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
 }
 
 void
 proto_reg_handoff_bgp(void)
 {
-    dissector_add("tcp.port", TCP_PORT_BGP, dissect_bgp);
+    old_dissector_add("tcp.port", TCP_PORT_BGP, dissect_bgp);
 }
