@@ -2,7 +2,7 @@
  * Routines for the disassembly of the "Cisco Discovery Protocol"
  * (c) Copyright Hannes R. Boehm <hannes@boehm.org>
  *
- * $Id: packet-cdp.c,v 1.23 2000/05/31 05:06:56 guy Exp $
+ * $Id: packet-cdp.c,v 1.25 2000/08/13 14:08:04 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -101,6 +101,8 @@ dissect_cdp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     guint32 naddresses;
     int addr_length;
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_cdp, pd, offset, fd, tree);
+
     if (check_col(fd, COL_PROTOCOL))
         col_add_str(fd, COL_PROTOCOL, "CDP");
     if (check_col(fd, COL_INFO))
@@ -270,7 +272,7 @@ dissect_cdp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 				offset+=length;
 		}
 	}
-    	dissect_data(pd, offset, fd, cdp_tree);
+    	old_dissect_data(pd, offset, fd, cdp_tree);
     }
 }
 
