@@ -2,7 +2,7 @@
  * Routines for raw data (default case)
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-data.c,v 1.18 2000/05/12 06:23:33 gram Exp $
+ * $Id: packet-data.c,v 1.19 2000/08/07 03:20:27 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -40,8 +40,9 @@
  */
 int proto_data = -1;
 
+/* Remove this once all dissectors are converted to use tvbuffs */
 void
-dissect_data(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
+old_dissect_data(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
 	if (IS_DATA_IN_FRAME(offset) && tree) {
 		proto_tree_add_protocol_format(tree, proto_data, NullTVB, offset,
@@ -50,9 +51,8 @@ dissect_data(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	}
 }
 
-/* This will become dissect_data() once all dissectors are converted to use tvbuffs */
 void
-dissect_data_tvb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+dissect_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	int bytes;
 
