@@ -1,7 +1,7 @@
 /* packet-dns.c
  * Routines for DNS packet disassembly
  *
- * $Id: packet-dns.c,v 1.45 2000/05/31 05:07:00 guy Exp $
+ * $Id: packet-dns.c,v 1.53 2000/08/18 09:05:02 itojun Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -111,8 +111,16 @@ static gint ett_t_key_flags = -1;
 #define T_SRV           33              /* service location (RFC 2052) */
 #define T_ATMA          34              /* ??? */
 #define T_NAPTR         35              /* naming authority pointer (RFC 2168) */
-#define T_A6		38              /* IPv6 address with indirection (draft-ietf-ipngwg-dns-lookups-07) */
+#define T_A6		38              /* IPv6 address with indirection (RFC 2874) */
 #define T_OPT		41		/* OPT pseudo-RR (RFC 2671) */
+#define T_WINS		65281		/* Microsoft's WINS RR */
+#define T_WINS_R	65282		/* Microsoft's WINS-R RR */
+
+/* Class values */
+#define C_IN		1		/* the Internet */
+#define C_CS		2		/* CSNET (obsolete) */
+#define C_CH		3		/* CHAOS */
+#define C_HS		4		/* Hesiod */
 
 /* Bit fields in the flags */
 #define F_RESPONSE      (1<<15)         /* packet is response */
@@ -138,7 +146,17 @@ static gint ett_t_key_flags = -1;
 #define RCODE_NOTIMPL   (4<<0)
 #define RCODE_REFUSED   (5<<0)
 
-/* See RFC 1035 for all RR types for which no RFC is listed. */
+/* See RFC 1035 for all RR types for which no RFC is listed, except for
+   the ones with "???", and for the Microsoft WINS and WINS-R RRs, for
+   which one should look at
+
+http://www.windows.com/windows2000/en/server/help/sag_DNS_imp_UsingWinsLookup.htm
+   
+   and
+   
+http://www.microsoft.com/windows2000/library/resources/reskit/samplechapters/cncf/cncf_imp_wwaw.asp
+
+   which discuss them to some extent. */
 static char *
 dns_type_name (u_int type)
 {
@@ -181,7 +199,7 @@ dns_type_name (u_int type)
     "NAPTR",				/* RFC 2168 */
     NULL,
     NULL,
-    "A6",				/* draft-ietf-ipngwg-dns-lookups-07 */
+    "A6",				/* RFC 2874 */
     NULL,
     NULL,
     "OPT"				/* RFC 2671 */
@@ -202,7 +220,11 @@ dns_type_name (u_int type)
       return "GID";
     case 103:
       return "UNSPEC";
-      
+    case T_WINS:
+      return "WINS";
+    case T_WINS_R:
+      return "WINS-R";
+
       /* queries  */
     case 251:
       return "IXFR";	/* RFC 1995 */
@@ -214,6 +236,7 @@ dns_type_name (u_int type)
       return "MAILA";
     case 255:
       return "ANY";
+
     }
 
   return "unknown";
@@ -262,7 +285,7 @@ dns_long_type_name (u_int type)
     "Naming authority pointer",		/* RFC 2168 */
     NULL,
     NULL,
-    "IPv6 address with indirection",	/* draft-ietf-ipngwg-dns-lookups-07 */
+    "IPv6 address with indirection",	/* RFC 2874 */
     NULL,
     NULL,
     "EDNS0 option"			/* RFC 2671 */
@@ -284,7 +307,11 @@ dns_long_type_name (u_int type)
       return "GID";
     case 103:
       return "UNSPEC";
-      
+    case T_WINS:
+      return "WINS";
+    case T_WINS_R:
+      return "WINS-R";
+
       /* queries  */
     case 251:
       return "Request for incremental zone transfer";	/* RFC 1995 */
@@ -309,13 +336,16 @@ dns_class_name(int class)
   char *class_name;
   
   switch (class) {
-  case 1:
+  case C_IN:
     class_name = "inet";
     break;
-  case 3:
+  case C_CS:
+    class_name = "csnet";
+    break;
+  case C_CH:
     class_name = "chaos";
     break;
-  case 4:
+  case C_HS:
     class_name = "hesiod";
     break;
   default:
@@ -610,15 +640,23 @@ add_opt_rr_to_tree(proto_item *trr, int rr_type, int offset, const char *name,
 /*
  * SIG and KEY RR algorithms.
  */
-#define	DNS_ALGO_MD5		1	/* MD5/RSA */
-#define	DNS_ALGO_EDATE		253	/* Expiration date */
-#define	DNS_ALGO_PRIVATE	254	/* Private use */	
+#define	DNS_ALGO_RSAMD5		1	/* RSA/MD5 */
+#define	DNS_ALGO_DH		2	/* Diffie-Hellman */
+#define	DNS_ALGO_DSA		3	/* DSA */
+#define	DNS_ALGO_ECC		4	/* Elliptic curve crypto */
+#define	DNS_ALGO_INDIRECT	252	/* Indirect key */
+#define	DNS_ALGO_PRIVATEDNS	253	/* Private, domain name  */
+#define	DNS_ALGO_PRIVATEOID	254	/* Private, OID */	
 
 static const value_string algo_vals[] = {
-	  { DNS_ALGO_MD5,     "MD5/RSA" },
-	  { DNS_ALGO_EDATE,   "Expiration date" },
-	  { DNS_ALGO_PRIVATE, "Private use" },
-	  { 0,                NULL }
+	  { DNS_ALGO_RSAMD5,     "RSA/MD5" },
+	  { DNS_ALGO_DH,         "Diffie-Hellman" },
+	  { DNS_ALGO_DSA,        "DSA" },
+	  { DNS_ALGO_ECC,        "Elliptic curve crypto" },
+	  { DNS_ALGO_INDIRECT,   "Indirect key" },
+	  { DNS_ALGO_PRIVATEDNS, "Private, domain name" },
+	  { DNS_ALGO_PRIVATEOID, "Private, OID" },
+	  { 0,                   NULL }
 };
 
 static int
@@ -708,6 +746,11 @@ dissect_dns_answer(const u_char *pd, int offset, int dns_data_offset,
       proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Addr: %s",
 		     ip_to_str((guint8 *)dptr));
     }
+    if (class == C_IN) {
+      guint32 addr;
+      memcpy(&addr, dptr, sizeof(addr));
+      add_host_name(addr, name);
+    }
     break;
 
   case T_NS:
@@ -1435,6 +1478,163 @@ dissect_dns_answer(const u_char *pd, int offset, int dns_data_offset,
     }
     break;
 
+  case T_WINS:
+    {
+      int rr_len = data_len;
+      guint32 local_flag;
+      guint32 lookup_timeout;
+      guint32 cache_timeout;
+      guint32 nservers;
+
+      if (dns_tree != NULL) {
+	proto_item_set_text(trr, "%s: type %s, class %s", name, type_name,
+				class_name);
+	if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	  /* We ran past the end of the captured data in the packet. */
+	  return 0;
+	}
+	local_flag = pntohl(&pd[cur_offset]);
+	if (dns_tree != NULL) {
+	  proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Local flag: %s",
+		       local_flag ? "true" : "false");
+	}
+	cur_offset += 4;
+	rr_len -= 4;
+
+	if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	  /* We ran past the end of the captured data in the packet. */
+	  return 0;
+	}
+	lookup_timeout = pntohl(&pd[cur_offset]);
+	if (dns_tree != NULL) {
+	  proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Lookup timeout: %u seconds",
+		       lookup_timeout);
+	}
+	cur_offset += 4;
+	rr_len -= 4;
+
+	if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	  /* We ran past the end of the captured data in the packet. */
+	  return 0;
+	}
+	cache_timeout = pntohl(&pd[cur_offset]);
+	if (dns_tree != NULL) {
+	  proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Cache timeout: %u seconds",
+		       cache_timeout);
+	}
+	cur_offset += 4;
+	rr_len -= 4;
+
+	if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	  /* We ran past the end of the captured data in the packet. */
+	  return 0;
+	}
+	nservers = pntohl(&pd[cur_offset]);
+	if (dns_tree != NULL) {
+	  proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Number of WINS servers: %u",
+		       nservers);
+	}
+	cur_offset += 4;
+	rr_len -= 4;
+
+	while (rr_len != 0 && nservers != 0) {
+	  if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	    /* We ran past the end of the captured data in the packet. */
+	    return 0;
+	  }
+	  if (dns_tree != NULL) {
+	    proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "WINS server address: %s",
+		     ip_to_str((guint8 *)&pd[cur_offset]));
+	  }
+	  cur_offset += 4;
+	  rr_len -= 4;
+	  nservers--;
+	}
+      }
+    }
+    break;
+
+  case T_WINS_R:
+    {
+      int rr_len = data_len;
+      guint32 local_flag;
+      guint32 lookup_timeout;
+      guint32 cache_timeout;
+      char dname[MAXDNAME];
+      int dname_len;
+
+      if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	/* We ran past the end of the captured data in the packet. */
+	if (dns_tree != NULL) {
+	  proto_item_set_text(trr,
+		"%s: type %s, class %s, <Local flag goes past end of captured data in packet>",
+		       name, type_name, class_name);
+	}
+	return 0;
+      }
+      local_flag = pntohl(&pd[cur_offset]);
+      if (dns_tree != NULL) {
+	proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Local flag: %s",
+		       local_flag ? "true" : "false");
+      }
+      cur_offset += 4;
+      rr_len -= 4;
+
+      if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	/* We ran past the end of the captured data in the packet. */
+	if (dns_tree != NULL) {
+	  proto_item_set_text(trr,
+		"%s: type %s, class %s, <Lookup timeout goes past end of captured data in packet>",
+		       name, type_name, class_name);
+	}
+	return 0;
+      }
+      lookup_timeout = pntohl(&pd[cur_offset]);
+      if (dns_tree != NULL) {
+	proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Lookup timeout: %u seconds",
+		       lookup_timeout);
+      }
+      cur_offset += 4;
+      rr_len -= 4;
+
+      if (!BYTES_ARE_IN_FRAME(cur_offset, 4)) {
+	/* We ran past the end of the captured data in the packet. */
+	if (dns_tree != NULL) {
+	  proto_item_set_text(trr,
+		"%s: type %s, class %s, <Cache timeout goes past end of captured data in packet>",
+		       name, type_name, class_name);
+	}
+	return 0;
+      }
+      cache_timeout = pntohl(&pd[cur_offset]);
+      if (dns_tree != NULL) {
+	proto_tree_add_text(rr_tree, NullTVB, cur_offset, 4, "Cache timeout: %u seconds",
+		       cache_timeout);
+      }
+      cur_offset += 4;
+      rr_len -= 4;
+
+      dname_len = get_dns_name(pd, cur_offset, dns_data_offset, dname, sizeof(dname));
+      if (dname_len < 0) {
+	/* We ran past the end of the captured data in the packet. */
+	if (dns_tree != NULL) {
+	  proto_item_set_text(trr,
+		"%s: type %s, class %s, <Name result domain goes past end of captured data in packet>",
+		       name, type_name, class_name);
+	}
+	return 0;
+      }
+      if (fd != NULL)
+	col_append_fstr(fd, COL_INFO, " %s", dname);
+      if (dns_tree != NULL) {
+	proto_item_set_text(trr, "%s: type %s, class %s, name result domain %s",
+		     name, type_name, class_name, dname);
+	proto_tree_add_text(rr_tree, NullTVB, cur_offset, dname_len, "Name result domain: %s",
+			dname);
+      }
+    }
+    break;
+
     /* TODO: parse more record types */
 
   default:
@@ -1528,6 +1728,8 @@ dissect_dns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		  { RCODE_REFUSED,   "Refused"         },
 		  { 0,               NULL              } };
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_dns, pd, offset, fd, tree);
+
   dns_data_offset = offset;
 
   if (check_col(fd, COL_PROTOCOL))
@@ -1535,7 +1737,7 @@ dissect_dns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 
   if (pi.captured_len < DNS_HDRLEN) {
     col_add_str(fd, COL_INFO, "Short DNS packet");
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -1736,5 +1938,5 @@ proto_register_dns(void)
 void
 proto_reg_handoff_dns(void)
 {
-  dissector_add("udp.port", UDP_PORT_DNS, dissect_dns);
+  old_dissector_add("udp.port", UDP_PORT_DNS, dissect_dns);
 }
