@@ -1,6 +1,6 @@
 /* packet-eigrp.c
  *
- * $Id: packet-eigrp.c,v 1.4 2000/05/31 05:07:02 guy Exp $
+ * $Id: packet-eigrp.c,v 1.6 2000/08/13 14:08:09 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -92,6 +92,8 @@ dissect_eigrp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   proto_item *ti;
   guint16    cksum;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_eigrp, pd, offset, fd, tree);
+
   /* Avoids alignment problems on many architectures. */
   memcpy(&ih, &pd[offset], sizeof(e_eigrp));
   /* To do: check for runts, errs, etc. */
@@ -131,7 +133,7 @@ proto_register_eigrp(void)
 void
 proto_reg_handoff_eigrp(void)
 {
-    dissector_add("ip.proto", IP_PROTO_EIGRP, dissect_eigrp);
-    dissector_add("ddp.type", DDP_EIGRP, dissect_eigrp);
-    dissector_add("ipx.socket", IPX_SOCKET_EIGRP, dissect_eigrp);
+    old_dissector_add("ip.proto", IP_PROTO_EIGRP, dissect_eigrp);
+    old_dissector_add("ddp.type", DDP_EIGRP, dissect_eigrp);
+    old_dissector_add("ipx.socket", IPX_SOCKET_EIGRP, dissect_eigrp);
 }
