@@ -1,7 +1,7 @@
 /* ethertype.c
  * Routines for calling the right protocol for the ethertype.
  *
- * $Id: packet-ethertype.c,v 1.5 2000/05/31 05:07:03 guy Exp $
+ * $Id: packet-ethertype.c,v 1.7 2000/08/09 22:10:23 deniel Exp $
  *
  * Gilbert Ramirez <gram@xiexie.org>
  *
@@ -44,23 +44,33 @@
 static dissector_table_t ethertype_dissector_table;
 
 const value_string etype_vals[] = {
-    {ETHERTYPE_IP,     "IP"             },
-    {ETHERTYPE_IPv6,   "IPv6"           },
-    {ETHERTYPE_X25L3,  "X.25 Layer 3"   },
-    {ETHERTYPE_ARP,    "ARP"            },
-    {ETHERTYPE_REVARP, "RARP"           },
-    {ETHERTYPE_ATALK,  "Appletalk"      },
-    {ETHERTYPE_AARP,   "AARP"           },
-    {ETHERTYPE_IPX,    "Netware IPX/SPX"},
-    {ETHERTYPE_VINES,  "Vines"          },
-    {ETHERTYPE_TRAIN,   "Netmon Train"  },
-    {ETHERTYPE_LOOP,   "Loopback"       }, /* Ethernet Loopback */
-    {ETHERTYPE_PPPOED, "PPPoE Discovery"}, 
-    {ETHERTYPE_PPPOES, "PPPoE Session"  }, 
-    {ETHERTYPE_VLAN,   "802.1Q Virtual LAN" },
-    {ETHERTYPE_MPLS,   "MPLS label switched packet" },
-    {ETHERTYPE_MPLS_MULTI,   "MPLS multicast label switched packet" },
-    {0,                 NULL            } };
+    {ETHERTYPE_IP,		"IP"				},
+    {ETHERTYPE_IPv6,		"IPv6"				},
+    {ETHERTYPE_X25L3,		"X.25 Layer 3"			},
+    {ETHERTYPE_ARP,		"ARP"				},
+    {ETHERTYPE_REVARP,		"RARP"				},
+    {ETHERTYPE_DEC_LB,		"DEC LanBridge"			},
+    {ETHERTYPE_ATALK,		"Appletalk"			},
+    {ETHERTYPE_AARP,		"AARP"				},
+    {ETHERTYPE_IPX,		"Netware IPX/SPX"		},
+    {ETHERTYPE_VINES,		"Vines"				},
+    {ETHERTYPE_TRAIN,		"Netmon Train"			},
+    {ETHERTYPE_LOOP,		"Loopback"			}, /* Ethernet Loopback */
+    {ETHERTYPE_PPPOED,		"PPPoE Discovery"		}, 
+    {ETHERTYPE_PPPOES,		"PPPoE Session"			}, 
+    {ETHERTYPE_VLAN,		"802.1Q Virtual LAN"		},
+    {ETHERTYPE_MPLS,		"MPLS label switched packet"	},
+    {ETHERTYPE_MPLS_MULTI,	"MPLS multicast label switched packet" },
+    {ETHERTYPE_3C_NBP_DGRAM,	"3Com NBP Datagram"		},
+    {ETHERTYPE_DEC,		"DEC proto"			},
+    {ETHERTYPE_DNA_DL,		"DEC DNA Dump/Load"		},
+    {ETHERTYPE_DNA_RC,		"DEC DNA Remote Console"	},
+    {ETHERTYPE_DNA_RT,		"DEC DNA Routing"		},
+    {ETHERTYPE_LAT,		"DEC LAT"			},
+    {ETHERTYPE_DEC_DIAG,	"DEC Diagnostics"		},
+    {ETHERTYPE_DEC_CUST,	"DEC Customer use"		},
+    {ETHERTYPE_DEC_SCA,		"DEC LAVC/SCA"			},
+    {0,				NULL				} };
 
 void
 capture_ethertype(guint16 etype, int offset,
@@ -89,11 +99,8 @@ void
 ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_etype, packet_info *pinfo,
 		proto_tree *tree, proto_tree *fh_tree, int item_id)
 {
-	dissector_t	sub_dissector;
 	char		*description;
 	tvbuff_t	*next_tvb;
-	const guint8	*next_pd;
-	int		next_offset;
 	
 	/* Add to proto_tree */
 	if (tree) {
@@ -101,31 +108,27 @@ ethertype(guint16 etype, tvbuff_t *tvb, int offset_after_etype, packet_info *pin
 	}
 
 	next_tvb = tvb_new_subset(tvb, offset_after_etype, -1, -1);
-	tvb_compat(next_tvb, &next_pd, &next_offset);
 
 	/* Look for sub-dissector */
-	sub_dissector = dissector_lookup( ethertype_dissector_table, etype );
-
-	if (sub_dissector) {
-		/* Call sub-dissector */
-		sub_dissector(next_pd, next_offset, pinfo->fd, tree);
-	}
-	else {
-		/* Label rest of packet as "Data" */
-		dissect_data_tvb(next_tvb, pinfo, tree);
+	if (!dissector_try_port(ethertype_dissector_table, etype,
+	    next_tvb, pinfo, tree)) {
+		/* No sub-dissector found.
+		   Label rest of packet as "Data" */
+		dissect_data(next_tvb, pinfo, tree);
 
 		/* Label protocol */
 		switch(etype) {
-			case ETHERTYPE_LOOP:
-				if (check_col(pinfo->fd, COL_PROTOCOL)) {
-					col_add_fstr(pinfo->fd, COL_PROTOCOL, "LOOP");
-				}
-				break;
-			    default:
-				if (check_col(pinfo->fd, COL_PROTOCOL)) {
-					col_add_fstr(pinfo->fd, COL_PROTOCOL, "0x%04x", etype);
-				}
-				break;
+
+		case ETHERTYPE_LOOP:
+			if (check_col(pinfo->fd, COL_PROTOCOL)) {
+				col_add_fstr(pinfo->fd, COL_PROTOCOL, "LOOP");
+			}
+			break;
+		default:
+			if (check_col(pinfo->fd, COL_PROTOCOL)) {
+				col_add_fstr(pinfo->fd, COL_PROTOCOL, "0x%04x", etype);
+			}
+			break;
 		}
 		if (check_col(pinfo->fd, COL_INFO)) {
 			description = match_strval(etype, etype_vals);
