@@ -3,7 +3,7 @@
  *
  * Laurent Deniel <deniel@worldnet.fr>
  *
- * $Id: packet-fddi.c,v 1.36 2000/05/31 05:07:03 guy Exp $
+ * $Id: packet-fddi.c,v 1.38 2000/08/13 14:08:10 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -265,6 +265,8 @@ dissect_fddi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
   u_char     src_swapped[6], dst_swapped[6];
   tvbuff_t   *next_tvb;
 
+  CHECK_DISPLAY_AS_DATA(proto_fddi, tvb, pinfo, tree);
+
   pinfo->current_proto = "FDDI";
   if (check_col(pinfo->fd, COL_PROTOCOL))
     col_add_str(pinfo->fd, COL_PROTOCOL, "FDDI");
@@ -350,7 +352,7 @@ dissect_fddi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
       return;
       
     default :
-      dissect_data_tvb(next_tvb, pinfo, tree);
+      dissect_data(next_tvb, pinfo, tree);
       return;
 
   } /* fc */
