@@ -1,6 +1,6 @@
 /* packet-fddi.h
  *
- * $Id: packet-fddi.h,v 1.2 2000/05/11 08:15:08 gram Exp $
+ * $Id: packet-fddi.h,v 1.3 2000/08/11 13:34:13 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -22,5 +22,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_FDDI_H__
+#define __PACKET_FDDI_H__
+
 void capture_fddi(const u_char *, packet_counts *);
 void dissect_fddi(tvbuff_t*, packet_info*, proto_tree*, gboolean bitswapped);
+
+#endif
