@@ -2,7 +2,7 @@
  * Routines for ftp packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-ftp.c,v 1.16 2000/05/31 05:07:04 guy Exp $
+ * $Id: packet-ftp.c,v 1.19 2000/09/11 16:16:01 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -43,6 +43,7 @@
 #include <string.h>
 #include <glib.h>
 #include "packet.h"
+#include "strutil.h"
 
 static int proto_ftp = -1;
 static int hf_ftp_response = -1;
@@ -66,6 +67,8 @@ dissect_ftp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	int i2;
 	int max_data = pi.captured_len - offset;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_ftp, pd, offset, fd, tree);
+
 	memset(rr, '\0', sizeof(rr));
 	memset(rd, '\0', sizeof(rd));
 
@@ -203,7 +206,7 @@ proto_register_ftp(void)
 void
 proto_reg_handoff_ftp(void)
 {
-	dissector_add("tcp.port", TCP_PORT_FTPDATA, &dissect_ftpdata);
-	dissector_add("tcp.port", TCP_PORT_FTP, &dissect_ftp);
+	old_dissector_add("tcp.port", TCP_PORT_FTPDATA, &dissect_ftpdata);
+	old_dissector_add("tcp.port", TCP_PORT_FTP, &dissect_ftp);
 }
 
