@@ -3,7 +3,7 @@
  *
  * Laurent Deniel <deniel@worldnet.fr>
  *
- * $Id: packet-giop.c,v 1.14 2000/05/31 05:07:04 guy Exp $
+ * $Id: packet-giop.c,v 1.18 2000/09/01 16:02:36 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -214,6 +214,9 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   LocateRequestHeader locate_req;
   int i;
 
+  if (!proto_is_protocol_enabled(proto_giop))
+    return FALSE;
+
 #define END_OF_GIOP_MESSAGE (offset - first_offset - GIOP_HEADER_SIZE)
 
   if (!BYTES_ARE_IN_FRAME(offset, GIOP_HEADER_SIZE)) {
@@ -233,13 +236,30 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     return FALSE;
   }
 
+  if (check_col(fd, COL_PROTOCOL)) {
+    col_add_str(fd, COL_PROTOCOL, "GIOP");
+  }
+
+
   if (header.GIOP_version.major != GIOP_MAJOR ||
       ((minor_version = header.GIOP_version.minor) >  GIOP_MINOR)) {
     /* Bad version number; should we note that and dissect the rest
        as data, or should we return FALSE on the theory that it
        might have been some other packet that happened to begin with
        "GIOP"? */
-    dissect_data(pd, offset, fd, tree);
+    if (check_col(fd, COL_INFO)) {
+      col_add_fstr(fd, COL_INFO, "Version %d.%d",
+		header.GIOP_version.major, header.GIOP_version.minor);
+    }
+    if (tree) {
+      ti = proto_tree_add_item(tree, proto_giop, NullTVB, offset, 
+			  GIOP_HEADER_SIZE, FALSE);
+      clnp_tree = proto_item_add_subtree(ti, ett_giop);
+      proto_tree_add_text(clnp_tree, NullTVB, offset, GIOP_HEADER_SIZE,
+		"Version %d.%d not supported", 
+		header.GIOP_version.major, header.GIOP_version.minor);
+    }
+    old_dissect_data(pd, offset + GIOP_HEADER_SIZE, fd, tree);
     return TRUE;
   }
 
@@ -265,10 +285,6 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   else
     message_size = pletohl(&header.message_size);
 
-  if (check_col(fd, COL_PROTOCOL)) {
-    col_add_str(fd, COL_PROTOCOL, "GIOP");
-  }
-
   if (tree) {
     ti = proto_tree_add_item(tree, proto_giop, NullTVB, offset, 
 			  GIOP_HEADER_SIZE + message_size, FALSE);
@@ -284,13 +300,13 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	proto_tree_add_text(clnp_tree, NullTVB, offset +  6, 1, 
 			 "Flags: 0x%02x (%s%s)", 
 			 header.flags,
-			 (big_endian) ? "little" : "big",
+			 (big_endian) ? "big" : "little",
 			 (header.flags & 0x02) ? " fragment" : "");
 	break;
       case 0  :
 	proto_tree_add_text(clnp_tree, NullTVB, offset +  6, 1, 
 			 "Byte ordering: %s endian",
-			 (big_endian) ? "little" : "big");
+			 (big_endian) ? "big" : "little");
 	break;
       default :
 	break;
@@ -320,7 +336,7 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   offset += GIOP_HEADER_SIZE;
 
   if (!BYTES_ARE_IN_FRAME(offset, message_size)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return TRUE;
   }
 
@@ -445,7 +461,7 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	pntohl(&pd[offset]) : pletohl(&pd[offset]);
 
       if (sequence_length > message_size) {
-	dissect_data(pd, offset, fd, tree);
+	old_dissect_data(pd, offset, fd, tree);
 	return TRUE;
       }
        
@@ -514,7 +530,7 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	  pntohl(&pd[offset]) : pletohl(&pd[offset]);
 
 	if (sequence_length > message_size) {
-	  dissect_data(pd, offset, fd, tree);
+	  old_dissect_data(pd, offset, fd, tree);
 	  return TRUE;
 	}
 
@@ -553,7 +569,7 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	  pntohl(&pd[offset]) : pletohl(&pd[offset]);
 
 	if (sequence_length > message_size) {
-	  dissect_data(pd, offset, fd, tree);
+	  old_dissect_data(pd, offset, fd, tree);
 	  return TRUE;
 	}
 
@@ -573,7 +589,7 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	  pntohl(&pd[offset]) : pletohl(&pd[offset]);
 
 	if (sequence_length > message_size) {
-	  dissect_data(pd, offset, fd, tree);
+	  old_dissect_data(pd, offset, fd, tree);
 	  return TRUE;
 	}
 
@@ -707,7 +723,7 @@ dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   offset = first_offset + GIOP_HEADER_SIZE + message_size;
 
   if (IS_DATA_IN_FRAME(offset)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
   }
 
   return TRUE;
@@ -736,5 +752,5 @@ proto_register_giop(void)
 void
 proto_reg_handoff_giop(void)
 {
-  heur_dissector_add("tcp", dissect_giop);
+  old_heur_dissector_add("tcp", dissect_giop);
 }
