@@ -1,7 +1,7 @@
 /* packet-icq.c
  * Routines for ICQ packet disassembly
  *
- * $Id: packet-icq.c,v 1.15 2000/05/11 08:15:10 gram Exp $
+ * $Id: packet-icq.c,v 1.21 2000/08/13 14:08:15 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Johan Feyaerts
@@ -57,11 +57,6 @@
 #include <glib.h>
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
@@ -130,6 +125,8 @@ typedef struct _cmdcode {
 
 #define SRV_ACK			0x000a
 
+#define SRV_SILENT_TOO_LONG	0x001e
+
 #define SRV_GO_AWAY		0x0028
 
 #define SRV_NEW_UIN		0x0046
@@ -205,6 +202,7 @@ cmdcode serverMetaSubCmdCode[] = {
 
 cmdcode serverCmdCode[] = {
     { "SRV_ACK", SRV_ACK },
+    { "SRV_SILENT_TOO_LONG", SRV_SILENT_TOO_LONG },
     { "SRV_GO_AWAY", SRV_GO_AWAY },
     { "SRV_NEW_UIN", SRV_NEW_UIN },
     { "SRV_LOGIN_REPLY", SRV_LOGIN_REPLY },
@@ -217,7 +215,7 @@ cmdcode serverCmdCode[] = {
     { "SRV_NEW_USER", 180 },
     { "SRV_UPDATE_EXT", 200 },
     { "SRV_RECV_MESSAGE", SRV_RECV_MESSAGE },
-    { "SRV_X2", 230 },
+    { "SRV_END_OFFLINE_MESSAGES", 230 },
     { "SRV_NOT_CONNECTED", 240 },
     { "SRV_TRY_AGAIN", 250 },
     { "SRV_SYS_DELIVERED_MESS", SRV_SYS_DELIVERED_MESS },
@@ -229,7 +227,7 @@ cmdcode serverCmdCode[] = {
     { "SRV_UPDATE_FAIL", SRV_UPDATE_FAIL },
     { "SRV_AUTH_UPDATE", 500 },
     { "SRV_MULTI_PACKET", SRV_MULTI },
-    { "SRV_X1", 540 },
+    { "SRV_END_CONTACTLIST_STATUS", 540 },
     { "SRV_RAND_USER", SRV_RAND_USER },
     { "SRV_META_USER", SRV_META_USER },
     { NULL, -1 }
@@ -555,6 +553,36 @@ decrypt_v5(u_char *bfr, guint32 size,guint32 key)
     }
 }
 
+static void
+dissect_icqv4(const u_char *pd,
+	      int offset,
+	      frame_data *fd, 
+	      proto_tree *tree)
+{
+    /* Not really implemented yet */
+    if (check_col(fd, COL_PROTOCOL)) {
+	col_add_str(fd, COL_PROTOCOL, "ICQv4 (UDP)");
+    }
+    if (check_col(fd, COL_INFO)) {
+	col_add_str(fd, COL_INFO, "ICQ Version 4 protocol");
+    }
+}
+
+static void
+dissect_icqv3(const u_char *pd,
+	      int offset,
+	      frame_data *fd, 
+	      proto_tree *tree)
+{
+    /* Not really implemented yet */
+    if (check_col(fd, COL_PROTOCOL)) {
+	col_add_str(fd, COL_PROTOCOL, "ICQv3 (UDP)");
+    }
+    if (check_col(fd, COL_INFO)) {
+	col_add_str(fd, COL_INFO, "ICQ Version 3 protocol");
+    }
+}
+
 static void
 dissect_icqv2(const u_char *pd,
 	      int offset,
@@ -1241,11 +1269,15 @@ icqv5_cmd_login(proto_tree* tree,
 					CMD_SEND_MSG,
 					"Body");
 	subtree = proto_item_add_subtree(ti, ett_icq_body);
-	if (theTime!=-1)
+	if (theTime!=-1) {
+	    char *aTime = ctime(&theTime);
+
+	    aTime[strlen(aTime)-1] = '\0';
 	    proto_tree_add_text(subtree, NullTVB,
 				offset + CMD_LOGIN_TIME,
 				4,
-				"Time: %ld = %s", (long)theTime, ctime(&theTime));
+				"Time: %ld = %s", (long)theTime, aTime);
+	}
 	if (port!=-1)
 	    proto_tree_add_text(subtree, NullTVB,
 				offset + CMD_LOGIN_PORT,
@@ -1258,12 +1290,12 @@ icqv5_cmd_login(proto_tree* tree,
 				"Passwd: %s", password);
 	if (ipAddrp!=NULL)
 	    proto_tree_add_text(subtree, NullTVB,
-				offset + CMD_LOGIN_PASSWD + CMD_LOGIN_IP,
+				offset + CMD_LOGIN_PASSWD + passwdLen + CMD_LOGIN_IP,
 				4,
 				"IP: %s", ip_to_str(ipAddrp));
 	if (status!=-1)
 	    proto_tree_add_text(subtree, NullTVB,
-				offset + CMD_LOGIN_PASSWD + CMD_LOGIN_IP,
+				offset + CMD_LOGIN_PASSWD + passwdLen + CMD_LOGIN_STATUS,
 				4,
 				"Status: %s", findStatus(status));
     }
@@ -2060,22 +2092,10 @@ dissect_icqv5Client(const u_char *pd,
 					"Header");
 	icq_header_tree = proto_item_add_subtree(ti, ett_icq_header);
 					
-	proto_tree_add_uint_format(icq_header_tree,
-				   hf_icq_sessionid,
-				   NullTVB,
-				   offset+ICQ5_CL_SESSIONID,
-				   4,
-				   sessionid,
-				   "Session ID: 0x%08x",
-				   sessionid);
-	proto_tree_add_uint_format(icq_header_tree,
-				   hf_icq_checkcode,
-				   NullTVB,
-				   offset+ICQ5_CL_CHECKCODE,
-				   4,
-				   key,
-				   "Key: 0x%08x",
-				   key);
+	proto_tree_add_text(icq_header_tree, NullTVB,
+			    offset + ICQ_VERSION,
+			    2,
+			    "Version: %u", version);
 	proto_tree_add_uint_format(icq_header_tree,
 				   hf_icq_uin,
 				   NullTVB,
@@ -2084,14 +2104,34 @@ dissect_icqv5Client(const u_char *pd,
 				   uin,
 				   "UIN: %u (0x%08X)",
 				   uin, uin);
+	proto_tree_add_uint_format(icq_header_tree,
+				   hf_icq_sessionid,
+				   NullTVB,
+				   offset+ICQ5_CL_SESSIONID,
+				   4,
+				   sessionid,
+				   "Session ID: 0x%08x",
+				   sessionid);
+	proto_tree_add_text(icq_header_tree, NullTVB,
+			    offset + ICQ5_CL_CMD,
+			    2,
+			    "Command: %s (%u)", findClientCmd(cmd), cmd);
 	proto_tree_add_text(icq_header_tree, NullTVB,
 			    offset + ICQ5_CL_SEQNUM1,
 			    2,
-			    "Seqnum1: 0x%04x", seqnum1);
+			    "Seq Number 1: 0x%04x", seqnum1);
 	proto_tree_add_text(icq_header_tree, NullTVB,
-			    offset + ICQ5_CL_SEQNUM1,
+			    offset + ICQ5_CL_SEQNUM2,
 			    2,
-			    "Seqnum2: 0x%04x", seqnum2);
+			    "Seq Number 2: 0x%04x", seqnum2);
+	proto_tree_add_uint_format(icq_header_tree,
+				   hf_icq_checkcode,
+				   NullTVB,
+				   offset+ICQ5_CL_CHECKCODE,
+				   4,
+				   key,
+				   "Key: 0x%08x",
+				   key);
 	switch(cmd) {
 	case CMD_ACK:
 	    icqv5_cmd_ack(icq_tree,
@@ -2198,14 +2238,13 @@ dissect_icqv5Server(const u_char *pd,
     /* Server traffic is easy, not encrypted */
     proto_tree *icq_tree = NULL;
     proto_tree *icq_header_tree = NULL;
-    proto_tree *icq_decode_tree = NULL;
     proto_item *ti = NULL;
     const u_char* decr_pd;
     int changeCol = (pktsize==(guint32)-1);
 
     guint16 version, cmd;
     guint32 uin, sessionid;
-    guint16 seq_num1, seq_num2;
+    guint16 seqnum1, seqnum2;
     guint32 checkcode;
     
     uin = pletohl(&pd[ICQ5_SRV_UIN]);
@@ -2213,10 +2252,10 @@ dissect_icqv5Server(const u_char *pd,
     cmd = pletohs(&pd[ICQ5_SRV_CMD]);
     version = pletohs(&pd[ICQ_VERSION]);
     checkcode = pletohl(&pd[ICQ5_SRV_CHECKCODE]);
-    seq_num1 = pletohs(&pd[ICQ5_SRV_SEQNUM1]);
-    seq_num2 = pletohs(&pd[ICQ5_SRV_SEQNUM2]);
+    seqnum1 = pletohs(&pd[ICQ5_SRV_SEQNUM1]);
+    seqnum2 = pletohs(&pd[ICQ5_SRV_SEQNUM2]);
     if (pktsize == -1)
-	pktsize = fd->pkt_len - offset;
+	pktsize = END_OF_FRAME;
     decr_pd = pd;
     
     if (changeCol && check_col(fd, COL_INFO))
@@ -2243,6 +2282,10 @@ dissect_icqv5Server(const u_char *pd,
 					"Header");
 	icq_header_tree = proto_item_add_subtree(ti, ett_icq_header);
 					
+	proto_tree_add_text(icq_header_tree, NullTVB,
+			    offset + ICQ_VERSION,
+			    2,
+			    "Version: %u", version);
 	proto_tree_add_uint_format(icq_header_tree,
 				   hf_icq_sessionid,
 				   NullTVB,
@@ -2252,15 +2295,17 @@ dissect_icqv5Server(const u_char *pd,
 				   "Session ID: 0x%08x",
 				   sessionid);
 	proto_tree_add_text(icq_header_tree, NullTVB,
-			    offset+ICQ5_SRV_SEQNUM1,
+			    offset + ICQ5_SRV_CMD,
 			    2,
-			    "Seq Number1: 0x%04x",
-			    seq_num1);
+			    "Command: %s (%u)", findServerCmd(cmd), cmd);
 	proto_tree_add_text(icq_header_tree, NullTVB,
-			    offset+ICQ5_SRV_SEQNUM2,
+			    offset + ICQ5_SRV_SEQNUM1,
 			    2,
-			    "Seq Number2: 0x%04x",
-			    seq_num2);
+			    "Seq Number 1: 0x%04x", seqnum1);
+	proto_tree_add_text(icq_header_tree, NullTVB,
+			    offset + ICQ5_SRV_SEQNUM2,
+			    2,
+			    "Seq Number 2: 0x%04x", seqnum2);
 	proto_tree_add_uint_format(icq_header_tree,
 				   hf_icq_uin,
 				   NullTVB,
@@ -2331,6 +2376,7 @@ dissect_icqv5Server(const u_char *pd,
 			    fd);
 	    break;
 	case SRV_ACK:
+	case SRV_SILENT_TOO_LONG:
 	case SRV_GO_AWAY:
 	case SRV_NEW_UIN:
 	case SRV_BAD_PASS:
@@ -2353,14 +2399,6 @@ dissect_icqv5Server(const u_char *pd,
 	    fprintf(stderr,"Missing: %s\n", findServerCmd(cmd));
 	    break;
 	}
-
-	ti = proto_tree_add_text(icq_tree, NullTVB,
-				 offset,
-				 pktsize,
-				 "Decoded packet");
-        icq_decode_tree = proto_item_add_subtree(ti,
-						 ett_icq_decode);
-	proto_tree_add_hexdump(icq_decode_tree, offset, decr_pd, pktsize);
     }
 }
 
@@ -2389,11 +2427,19 @@ static void dissect_icq(const u_char *pd,
 {
   int version = 0;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_icq, pd, offset, fd, tree);
+
   version = pletohs(&pd[offset + ICQ_VERSION]);
   switch (version) {
   case 0x0005:
       dissect_icqv5(pd, offset, fd, tree);
       break;
+  case 0x0004:
+      dissect_icqv4(pd, offset, fd, tree);
+      break;
+  case 0x0003:
+      dissect_icqv3(pd, offset, fd, tree);
+      break;
   case 0x0002:
       dissect_icqv2(pd, offset, fd, tree);
       break;
@@ -2439,5 +2485,5 @@ proto_register_icq(void)
 void
 proto_reg_handoff_icq(void)
 {
-    dissector_add("udp.port", UDP_PORT_ICQ, dissect_icq);
+    old_dissector_add("udp.port", UDP_PORT_ICQ, dissect_icq);
 }
