@@ -2,7 +2,7 @@
  * Routines for imap packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-imap.c,v 1.7 2000/05/31 05:07:07 guy Exp $
+ * $Id: packet-imap.c,v 1.9 2000/08/13 14:08:16 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -60,6 +60,8 @@ dissect_imap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	int i2;
 	int max_data = pi.captured_len - offset;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_imap, pd, offset, fd, tree);
+
 	memset(rr, '\0', sizeof(rr));
 	memset(rd, '\0', sizeof(rd));
 
@@ -140,5 +142,5 @@ proto_register_imap(void)
 void
 proto_reg_handoff_imap(void)
 {
-  dissector_add("tcp.port", TCP_PORT_IMAP, dissect_imap);
+  old_dissector_add("tcp.port", TCP_PORT_IMAP, dissect_imap);
 }
