@@ -1,7 +1,7 @@
 /* packet-ip.c
  * Routines for IP and miscellaneous IP protocol packet disassembly
  *
- * $Id: packet-ip.c,v 1.92 2000/06/05 03:21:01 gram Exp $
+ * $Id: packet-ip.c,v 1.102 2000/08/13 14:08:16 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -38,18 +38,14 @@
 #include <stdio.h>
 #include <string.h>
 #include <glib.h>
-#include "packet.h"
-#include "resolv.h"
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
+#include "packet.h"
+#include "resolv.h"
+#include "prefs.h"
 #include "etypes.h"
 #include "ppptypes.h"
 #include "llcsaps.h"
@@ -57,8 +53,7 @@
 #include "packet-ipsec.h"
 
 static void dissect_icmp(const u_char *, int, frame_data *, proto_tree *);
-static void dissect_igmp(const u_char *, int, frame_data *, proto_tree *);
-
+static void dissect_igmp(tvbuff_t *, packet_info *, proto_tree *);
 
 /* Decode the old IPv4 TOS field as the DiffServ DS Field */
 gboolean g_ip_dscp_actif = TRUE;
@@ -775,26 +770,51 @@ static const true_false_string flags_set_truth = {
   "Not set"
 };
 
-static char *ip_checksum_state(e_ip *iph)
+static guint16 ip_checksum(const guint8 *ptr, int len)
 {
-    unsigned long Sum;
-    unsigned char *Ptr, *PtrEnd;
-    unsigned short word;
+	unsigned long Sum;
+	const unsigned char *Ptr, *PtrEnd;
 
-    Sum    = 0;
-    PtrEnd = (lo_nibble(iph->ip_v_hl) * 4 + (char *)iph);
-    for (Ptr = (unsigned char *) iph; Ptr < PtrEnd; Ptr += 2) {
-	memcpy(&word, Ptr, sizeof word);
-        Sum += word;
-    }
+	Sum    = 0;
+	PtrEnd = ptr + len;
+	for (Ptr = ptr; Ptr < PtrEnd; Ptr += 2) {
+		Sum += pntohs(Ptr);
+	}
+	Sum = (Sum & 0xFFFF) + (Sum >> 16);
+	Sum = (Sum & 0xFFFF) + (Sum >> 16);
 
-    Sum = (Sum & 0xFFFF) + (Sum >> 16);
-    Sum = (Sum & 0xFFFF) + (Sum >> 16);
+	return (guint16)~Sum;
+}
 
-    if (Sum != 0xffff)
-        return "incorrect";
+static guint16 ip_checksum_shouldbe(guint16 sum, guint16 computed_sum)
+{
+	guint32 shouldbe;
 
-    return "correct";
+	/*
+	 * The value that should have gone into the checksum field
+	 * is the negative of the value gotten by summing up everything
+	 * *but* the checksum field.
+	 *
+	 * We can compute that by subtracting the value of the checksum
+	 * field from the sum of all the data in the packet, and then
+	 * computing the negative of that value.
+	 *
+	 * "sum" is the value of the checksum field, and "computed_sum"
+	 * is the negative of the sum of all the data in the packets,
+	 * so that's -(-computed_sum - sum), or (sum + computed_sum).
+	 *
+	 * All the arithmetic in question is one's complement, so the
+	 * addition must include an end-around carry; we do this by
+	 * doing the arithmetic in 32 bits (with no sign-extension),
+	 * and then adding the upper 16 bits of the sum, which contain
+	 * the carry, to the lower 16 bits of the sum, and then do it
+	 * again in case *that* sum produced a carry.
+	 */
+	shouldbe = sum;
+	shouldbe += computed_sum;
+	shouldbe = (shouldbe & 0xFFFF) + (shouldbe >> 16);
+	shouldbe = (shouldbe & 0xFFFF) + (shouldbe >> 16);
+	return shouldbe;
 }
 
 void
@@ -802,14 +822,16 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   e_ip       iph;
   proto_tree *ip_tree, *field_tree;
   proto_item *ti, *tf;
-  gchar      tos_str[32];
   guint      hlen, optlen, len;
   guint16    flags;
   guint8     nxt;
+  guint16    ipsum;
+
+  OLD_CHECK_DISPLAY_AS_DATA(proto_ip, pd, offset, fd, tree);
 
   /* To do: check for errs, etc. */
   if (!BYTES_ARE_IN_FRAME(offset, IPH_MIN_LEN)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -835,31 +857,6 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   hlen = lo_nibble(iph.ip_v_hl) * 4;	/* IP header length, in bytes */
   
   if (tree) {
-
-    switch (IPTOS_TOS(iph.ip_tos)) {
-      case IPTOS_NONE:
-        strcpy(tos_str, "None");
-        break;
-      case IPTOS_LOWCOST:
-        strcpy(tos_str, "Minimize cost");
-        break;
-      case IPTOS_RELIABILITY:
-        strcpy(tos_str, "Maximize reliability");
-        break;
-      case IPTOS_THROUGHPUT:
-        strcpy(tos_str, "Maximize throughput");
-        break;
-      case IPTOS_LOWDELAY:
-        strcpy(tos_str, "Minimize delay");
-        break;
-      case IPTOS_SECURITY:
-        strcpy(tos_str, "Maximize security");
-        break;
-      default:
-        strcpy(tos_str, "Unknown.  Malformed?");
-        break;
-    }
-
     ti = proto_tree_add_item(tree, proto_ip, NullTVB, offset, hlen, FALSE);
     ip_tree = proto_item_add_subtree(ti, ett_ip);
 
@@ -902,8 +899,18 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     proto_tree_add_uint(ip_tree, hf_ip_ttl, NullTVB, offset +  8, 1, iph.ip_ttl);
     proto_tree_add_uint_format(ip_tree, hf_ip_proto, NullTVB, offset +  9, 1, iph.ip_p,
 	"Protocol: %s (0x%02x)", ipprotostr(iph.ip_p), iph.ip_p);
-    proto_tree_add_uint_format(ip_tree, hf_ip_checksum, NullTVB, offset + 10, 2, iph.ip_sum,
-        "Header checksum: 0x%04x (%s)", iph.ip_sum, ip_checksum_state((e_ip*) &pd[offset]));
+
+    ipsum = ip_checksum(&pd[offset], hlen);
+    if (ipsum == 0) {
+	proto_tree_add_uint_format(ip_tree, hf_ip_checksum, NullTVB, offset + 10, 2, iph.ip_sum,
+            "Header checksum: 0x%04x (correct)", iph.ip_sum);
+    }
+    else {
+	proto_tree_add_uint_format(ip_tree, hf_ip_checksum, NullTVB, offset + 10, 2, iph.ip_sum,
+            "Header checksum: 0x%04x (incorrect, should be 0x%04x)", iph.ip_sum,
+	    ip_checksum_shouldbe(iph.ip_sum, ipsum));
+    }
+
     proto_tree_add_ipv4(ip_tree, hf_ip_src, NullTVB, offset + 12, 4, iph.ip_src);
     proto_tree_add_ipv4(ip_tree, hf_ip_dst, NullTVB, offset + 16, 4, iph.ip_dst);
     proto_tree_add_ipv4_hidden(ip_tree, hf_ip_addr, NullTVB, offset + 12, 4, iph.ip_src);
@@ -940,18 +947,18 @@ dissect_ip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     if (check_col(fd, COL_INFO))
       col_add_fstr(fd, COL_INFO, "Fragmented IP protocol (proto=%s 0x%02x, off=%u)",
 	ipprotostr(iph.ip_p), iph.ip_p, (iph.ip_off & IP_OFFSET) * 8);
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
   /* do lookup with the subdissector table */
-  if (!dissector_try_port(ip_dissector_table, nxt, pd, offset, fd, tree)) {
+  if (!old_dissector_try_port(ip_dissector_table, nxt, pd, offset, fd, tree)) {
     /* Unknown protocol */
     if (check_col(fd, COL_PROTOCOL))
       col_add_str(fd, COL_PROTOCOL, "IP");
     if (check_col(fd, COL_INFO))
       col_add_fstr(fd, COL_INFO, "%s (0x%02x)", ipprotostr(iph.ip_p), iph.ip_p);
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
   }
 }
 
@@ -1002,6 +1009,8 @@ dissect_icmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   guint8     addr_entry_size = 0;
   int        i;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_icmp, pd, offset, fd, tree);
+
   /* Avoids alignment problems on many architectures. */
   memcpy(&ih, &pd[offset], sizeof(e_icmp));
   /* To do: check for runts, errs, etc. */
@@ -1083,7 +1092,7 @@ dissect_icmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     col_add_str(fd, COL_INFO, type_str);
 
   if (tree) {
-    ti = proto_tree_add_item(tree, proto_icmp, NullTVB, offset, 4, FALSE);
+    ti = proto_tree_add_item(tree, proto_icmp, NullTVB, offset, END_OF_FRAME, FALSE);
     icmp_tree = proto_item_add_subtree(ti, ett_icmp);
     proto_tree_add_uint_format(icmp_tree, hf_icmp_type, NullTVB, offset,      1, 
 			       ih.icmp_type,
@@ -1155,12 +1164,12 @@ dissect_icmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
 	   XXX - for now, just display it as data; not all dissection
 	   routines can handle a short packet without exploding. */
-	dissect_data(pd, offset + 8, fd, icmp_tree);
+	old_dissect_data(pd, offset + 8, fd, icmp_tree);
 	break;
 
       case ICMP_ECHOREPLY:
       case ICMP_ECHO:
-	dissect_data(pd, offset + 8, fd, icmp_tree);
+	old_dissect_data(pd, offset + 8, fd, icmp_tree);
 	break;
 
       case ICMP_RTRADVERT:
@@ -1173,7 +1182,7 @@ dissect_icmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	      "Preference level: %u", pntohl(&pd[offset + 12 + (i*8)]));
 	  }
 	} else
-	  dissect_data(pd, offset + 8, fd, icmp_tree);
+	  old_dissect_data(pd, offset + 8, fd, icmp_tree);
 	break;
 
       case ICMP_TSTAMP:
@@ -1196,68 +1205,65 @@ dissect_icmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 }
 
 static void
-dissect_igmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
+dissect_igmp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
   e_igmp     ih;
   proto_tree *igmp_tree;
   proto_item *ti;
-  guint16    cksum;
-  gchar      type_str[64] = "";
+  gchar      *type_str;
+
+  CHECK_DISPLAY_AS_DATA(proto_igmp, tvb, pinfo, tree);
+
+  pinfo->current_proto = "IGMP";
+  if (check_col(pinfo->fd, COL_PROTOCOL))
+    col_add_str(pinfo->fd, COL_PROTOCOL, "IGMP");
 
   /* Avoids alignment problems on many architectures. */
-  memcpy(&ih, &pd[offset], sizeof(e_igmp));
-  /* To do: check for runts, errs, etc. */
-  cksum = ntohs(ih.igmp_cksum);
+  memcpy(&ih, tvb_get_ptr(tvb, 0, sizeof(e_igmp)), sizeof(e_igmp));
   
   switch (lo_nibble(ih.igmp_v_t)) {
     case IGMP_M_QRY:
-      strcpy(type_str, "Router query");
+      type_str = "Router query";
       break;
     case IGMP_V1_M_RPT:
-      strcpy(type_str, "Host response (v1)");
+      type_str = "Host response (v1)";
       break;
     case IGMP_V2_LV_GRP:
-      strcpy(type_str, "Leave group (v2)");
+      type_str = "Leave group (v2)";
       break;
     case IGMP_DVMRP:
-      strcpy(type_str, "DVMRP");
+      type_str = "DVMRP";
       break;
     case IGMP_PIM:
-      strcpy(type_str, "PIM");
+      type_str = "PIM";
       break;
     case IGMP_V2_M_RPT:
-      strcpy(type_str, "Host response (v2)");
+      type_str = "Host response (v2)";
       break;
     case IGMP_MTRC_RESP:
-      strcpy(type_str, "Traceroute response");
+      type_str = "Traceroute response";
       break;
     case IGMP_MTRC:
-      strcpy(type_str, "Traceroute message");
+      type_str = "Traceroute message";
       break;
     default:
-      strcpy(type_str, "Unknown IGMP");
+      type_str = "Unknown IGMP";
   }
 
-  if (check_col(fd, COL_PROTOCOL))
-    col_add_str(fd, COL_PROTOCOL, "IGMP");
-  if (check_col(fd, COL_INFO))
-    col_add_str(fd, COL_INFO, type_str);
+  if (check_col(pinfo->fd, COL_INFO))
+    col_add_str(pinfo->fd, COL_INFO, type_str);
   if (tree) {
-    ti = proto_tree_add_item(tree, proto_igmp, NullTVB, offset, 8, FALSE);
+    ti = proto_tree_add_item(tree, proto_igmp, tvb, 0, 8, FALSE);
     igmp_tree = proto_item_add_subtree(ti, ett_igmp);
-    proto_tree_add_uint(igmp_tree, hf_igmp_version, NullTVB, offset,     1, 
+    proto_tree_add_uint(igmp_tree, hf_igmp_version, tvb, 0,     1, 
 			hi_nibble(ih.igmp_v_t));
-    proto_tree_add_uint_format(igmp_tree, hf_igmp_type, NullTVB,  offset    , 1, 
+    proto_tree_add_uint_format(igmp_tree, hf_igmp_type, tvb,  0    , 1, 
 			       lo_nibble(ih.igmp_v_t),
 			       "Type: %u (%s)",
 			       lo_nibble(ih.igmp_v_t), type_str);
-    proto_tree_add_uint_format(igmp_tree, hf_igmp_unused, NullTVB, offset + 1, 1,
-			       ih.igmp_unused,
-			       "Unused: 0x%02x",
-			       ih.igmp_unused);
-    proto_tree_add_uint(igmp_tree, hf_igmp_checksum, NullTVB, offset + 2, 2, 
-			cksum);
-    proto_tree_add_ipv4(igmp_tree, hf_igmp_group, NullTVB, offset + 4, 4, 
-			ih.igmp_gaddr);
+    proto_tree_add_item(igmp_tree, hf_igmp_unused, tvb,  1, 1, FALSE);
+    proto_tree_add_item(igmp_tree, hf_igmp_checksum, tvb,  2, 2, FALSE);
+    proto_tree_add_ipv4(igmp_tree, hf_igmp_group, tvb,  4, 4, ih.igmp_gaddr);
   }
 }
 
@@ -1275,7 +1281,7 @@ proto_register_igmp(void)
 			"" }},
 
 		{ &hf_igmp_unused,
-		{ "Unused",		"igmp.unused", FT_UINT8, BASE_DEC, NULL, 0x0,
+		{ "Unused",		"igmp.unused", FT_UINT8, BASE_HEX, NULL, 0x0,
 			"" }},
 
 		{ &hf_igmp_checksum,
@@ -1415,6 +1421,7 @@ proto_register_ip(void)
 		&ett_ip_option_route,
 		&ett_ip_option_timestamp,
 	};
+	module_t *ip_module;
 
 	proto_ip = proto_register_protocol ("Internet Protocol", "ip");
 	proto_register_field_array(proto_ip, hf, array_length(hf));
@@ -1422,16 +1429,23 @@ proto_register_ip(void)
 
 	/* subdissector code */
 	ip_dissector_table = register_dissector_table("ip.proto");
+
+	/* Register a configuration option for decoding TOS as DSCP */
+	ip_module = prefs_register_module("ip", "IP", NULL);
+	prefs_register_bool_preference(ip_module, "decode_tos_as_diffserv",
+	    "Decode IPv4 TOS field as DiffServ field",
+"Whether the IPv4 type-of-service field should be decoded as a Differentiated Services field",
+	    &g_ip_dscp_actif);
 }
 
 void
 proto_reg_handoff_ip(void)
 {
-	dissector_add("ethertype", ETHERTYPE_IP, dissect_ip);
-	dissector_add("ppp.protocol", PPP_IP, dissect_ip);
-	dissector_add("llc.dsap", SAP_IP, dissect_ip);
-	dissector_add("ip.proto", IP_PROTO_IPV4, dissect_ip);
-	dissector_add("ip.proto", IP_PROTO_IPIP, dissect_ip);
+	old_dissector_add("ethertype", ETHERTYPE_IP, dissect_ip);
+	old_dissector_add("ppp.protocol", PPP_IP, dissect_ip);
+	old_dissector_add("llc.dsap", SAP_IP, dissect_ip);
+	old_dissector_add("ip.proto", IP_PROTO_IPV4, dissect_ip);
+	old_dissector_add("ip.proto", IP_PROTO_IPIP, dissect_ip);
 }
 
 void
@@ -1464,5 +1478,5 @@ proto_register_icmp(void)
 void
 proto_reg_handoff_icmp(void)
 {
-	dissector_add("ip.proto", IP_PROTO_ICMP, dissect_icmp);
+	old_dissector_add("ip.proto", IP_PROTO_ICMP, dissect_icmp);
 }
