@@ -3,7 +3,7 @@
  *
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-ipp.c,v 1.10 2000/05/31 05:07:09 guy Exp $
+ * $Id: packet-ipp.c,v 1.12 2000/08/13 14:08:17 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -167,6 +167,8 @@ void dissect_ipp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	guint16 status_code;
 	gchar *status_fmt;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_ipp, pd, offset, fd, tree);
+
 	if (check_col(fd, COL_PROTOCOL))
 		col_add_str(fd, COL_PROTOCOL, "IPP");
 	if (check_col(fd, COL_INFO)) {
@@ -228,7 +230,7 @@ void dissect_ipp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		offset = parse_attributes(pd, offset, fd, ipp_tree);
 
 		if (IS_DATA_IN_FRAME(offset))
-			dissect_data(pd, offset, fd, ipp_tree);
+			old_dissect_data(pd, offset, fd, ipp_tree);
 	}
 }
 
@@ -584,6 +586,6 @@ proto_reg_handoff_ipp(void)
 
 	   Or should the HTTP dissector decide that the payload is
 	   IPP based on the MIME headers? */
-	dissector_add("tcp.port", 631, dissect_http);
+	old_dissector_add("tcp.port", 631, dissect_http);
 }
 
