@@ -1,7 +1,7 @@
 /* packet-ipsec.c
  * Routines for IPsec/IPComp packet disassembly 
  *
- * $Id: packet-ipsec.c,v 1.17 2000/06/05 03:21:02 gram Exp $
+ * $Id: packet-ipsec.c,v 1.20 2000/08/13 14:08:17 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -42,6 +42,10 @@
 #include "packet-ipsec.h"
 #include "packet-ip.h"
 #include "resolv.h"
+#include "prefs.h"
+
+/* Place AH payload in sub tree */
+gboolean g_ah_payload_in_subtree = FALSE;
 
 static int proto_ah = -1;
 static int hf_ah_spi = -1;
@@ -144,11 +148,13 @@ dissect_ah_old(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 void
 dissect_ah(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
-    proto_tree *ah_tree;
+    proto_tree *ah_tree, *next_tree = NULL;
     proto_item *ti;
     struct newah ah;
     int advance;
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_ah, pd, offset, fd, tree);
+
     memcpy(&ah, (void *) &pd[offset], sizeof(ah)); 
     advance = sizeof(ah) + ((ah.ah_len - 1) << 2);
 
@@ -176,15 +182,27 @@ dissect_ah(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			    (guint32)ntohl(ah.ah_seq));
 	proto_tree_add_text(ah_tree, NullTVB, offset + sizeof(ah), (ah.ah_len - 1) << 2,
 			    "ICV");
+
+	/* Decide where to place next protocol decode */
+	if (g_ah_payload_in_subtree) {
+		next_tree = ah_tree;
+	}
+	else {
+		next_tree = tree;
+	}
     }
 
     /* start of the new header (could be a extension header) */
     offset += advance;
 
-  /* do lookup with the subdissector table */
-  if (!dissector_try_port(ip_dissector_table, ah.ah_nxt, pd, offset, fd, tree)) {
-    dissect_data(pd, offset, fd, tree);
-  }
+    if (g_ah_payload_in_subtree) {
+	col_set_writable(fd, FALSE);
+    }
+
+    /* do lookup with the subdissector table */
+    if (!old_dissector_try_port(ip_dissector_table, ah.ah_nxt, pd, offset, fd, next_tree)) {
+      old_dissect_data(pd, offset, fd, next_tree);
+    }
 }
 
 static void
@@ -194,6 +212,8 @@ dissect_esp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     proto_item *ti;
     struct newesp esp;
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_esp, pd, offset, fd, tree);
+
     memcpy(&esp, (void *) &pd[offset], sizeof(esp)); 
 
     /*
@@ -220,7 +240,7 @@ dissect_esp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	proto_tree_add_uint(esp_tree, hf_esp_sequence, NullTVB,
 			    offset + offsetof(struct newesp, esp_seq), 4,
 			    (guint32)ntohl(esp.esp_seq));
-	dissect_data(pd, offset + sizeof(struct newesp), fd, esp_tree);
+	old_dissect_data(pd, offset + sizeof(struct newesp), fd, esp_tree);
     }
 }
 
@@ -232,6 +252,8 @@ dissect_ipcomp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     struct ipcomp ipcomp;
     char *p;
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_ipcomp, pd, offset, fd, tree);
+
     memcpy(&ipcomp, (void *) &pd[offset], sizeof(ipcomp)); 
 
     /*
@@ -277,7 +299,7 @@ dissect_ipcomp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		"CPI: %s (0x%04x)",
 		p, ntohs(ipcomp.comp_cpi));
 	}
-	dissect_data(pd, offset + sizeof(struct ipcomp), fd, ipcomp_tree);
+	old_dissect_data(pd, offset + sizeof(struct ipcomp), fd, ipcomp_tree);
     }
 }
 
@@ -317,6 +339,8 @@ proto_register_ipsec(void)
     &ett_ipcomp,
   };
 
+  module_t *ah_module;
+
   proto_ah = proto_register_protocol("Authentication Header", "ah");
   proto_register_field_array(proto_ah, hf_ah, array_length(hf_ah));
 
@@ -327,12 +351,19 @@ proto_register_ipsec(void)
   proto_register_field_array(proto_ipcomp, hf_ipcomp, array_length(hf_ipcomp));
 
   proto_register_subtree_array(ett, array_length(ett));
+
+  /* Register a configuration option for placement of AH payload dissection */
+  ah_module = prefs_register_module("ah", "AH", NULL);
+  prefs_register_bool_preference(ah_module, "place_ah_payload_in_subtree",
+	    "Place AH payload in subtree",
+"Whether the AH payload decode should be placed in a subtree",
+	    &g_ah_payload_in_subtree);
 }
 
 void
 proto_reg_handoff_ipsec(void)
 {
-  dissector_add("ip.proto", IP_PROTO_AH, dissect_ah);
-  dissector_add("ip.proto", IP_PROTO_ESP, dissect_esp);
-  dissector_add("ip.proto", IP_PROTO_IPCOMP, dissect_ipcomp);
+  old_dissector_add("ip.proto", IP_PROTO_AH, dissect_ah);
+  old_dissector_add("ip.proto", IP_PROTO_ESP, dissect_esp);
+  old_dissector_add("ip.proto", IP_PROTO_IPCOMP, dissect_ipcomp);
 }
