@@ -1,7 +1,7 @@
 /* packet-ipv6.c
  * Routines for IPv6 packet disassembly 
  *
- * $Id: packet-ipv6.c,v 1.39 2000/06/05 03:21:03 gram Exp $
+ * $Id: packet-ipv6.c,v 1.43 2000/08/13 14:08:06 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -67,6 +67,7 @@ static int hf_ipv6_nxt = -1;
 static int hf_ipv6_hlim = -1;
 static int hf_ipv6_src = -1;
 static int hf_ipv6_dst = -1;
+static int hf_ipv6_addr = -1;
 #ifdef TEST_FINALHDR
 static int hf_ipv6_final = -1;
 #endif
@@ -141,6 +142,8 @@ dissect_frag6(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
     int *fragstart) {
     struct ip6_frag frag;
     int len;
+    proto_item *ti;
+    proto_tree *rthdr_tree;
 
     memcpy(&frag, (void *) &pd[offset], sizeof(frag));
     len = sizeof(frag);
@@ -151,6 +154,39 @@ dissect_frag6(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
 	    ipprotostr(frag.ip6f_nxt), frag.ip6f_nxt,
 	    *fragstart, frag.ip6f_ident);
     }
+    if (tree) {
+	   ti = proto_tree_add_text(tree, NullTVB, offset, len,
+			   "Fragmention Header");
+	   rthdr_tree = proto_item_add_subtree(ti, ett_ipv6);
+
+	   proto_tree_add_text(rthdr_tree, NullTVB,
+			 offset + offsetof(struct ip6_frag, ip6f_nxt), 1,
+			 "Next header: %s (0x%02x)",
+			 ipprotostr(frag.ip6f_nxt), frag.ip6f_nxt);
+
+	#if 0
+	   proto_tree_add_text(rthdr_tree, NullTVB,
+			 offset + offsetof(struct ip6_frag, ip6f_reserved), 1,
+			 "Reserved: %u",
+			 frag.ip6f_reserved);
+	#endif
+
+	   proto_tree_add_text(rthdr_tree, NullTVB,
+			 offset + offsetof(struct ip6_frag, ip6f_offlg), 2,
+			 "Offset: %u",
+			 ntohs(frag.ip6f_offlg) & 0xfff8);
+
+	   proto_tree_add_text(rthdr_tree, NullTVB,
+			 offset + offsetof(struct ip6_frag, ip6f_offlg), 2,
+			 "More fragments: %s",
+				frag.ip6f_offlg & IP6F_MORE_FRAG ?
+				"Yes" : "No");
+
+	   proto_tree_add_text(rthdr_tree, NullTVB,
+			 offset + offsetof(struct ip6_frag, ip6f_ident), 4,
+			 "Identification: 0x%08x",
+			 frag.ip6f_ident);
+    }
     return len;
 }
 
@@ -259,8 +295,12 @@ dissect_ipv6(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
   struct ip6_hdr ipv6;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_ipv6, pd, offset, fd, tree);
+
   memcpy(&ipv6, (void *) &pd[offset], sizeof(ipv6)); 
 
+  pi.ipproto = ipv6.ip6_nxt; /* XXX make work TCP follow (ipproto = 6) */
+
   SET_ADDRESS(&pi.net_src, AT_IPv6, 16, &pd[offset + IP6H_SRC]);
   SET_ADDRESS(&pi.src, AT_IPv6, 16, &pd[offset + IP6H_SRC]);
   SET_ADDRESS(&pi.net_dst, AT_IPv6, 16, &pd[offset + IP6H_DST]);
@@ -305,6 +345,13 @@ dissect_ipv6(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 		offset + offsetof(struct ip6_hdr, ip6_hlim), 1,
 		ipv6.ip6_hlim);
 
+    proto_tree_add_ipv6_hidden(ipv6_tree, hf_ipv6_addr, NullTVB, 
+			       offset + offsetof(struct ip6_hdr, ip6_src), 16,
+			       ipv6.ip6_src.s6_addr8);
+    proto_tree_add_ipv6_hidden(ipv6_tree, hf_ipv6_addr, NullTVB, 
+			       offset + offsetof(struct ip6_hdr, ip6_dst), 16,
+			       ipv6.ip6_dst.s6_addr8);
+
     proto_tree_add_ipv6_format(ipv6_tree, hf_ipv6_src, NullTVB,
 		offset + offsetof(struct ip6_hdr, ip6_src), 16,
 		(guint8 *)&ipv6.ip6_src,
@@ -370,16 +417,16 @@ again:
     if (check_col(fd, COL_PROTOCOL))
       col_add_str(fd, COL_PROTOCOL, "IPv6");
     /* COL_INFO was filled in by "dissect_frag6()" */
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
   } else {
     /* do lookup with the subdissector table */
-    if (!dissector_try_port(ip_dissector_table, nxt, pd, offset, fd, tree)) {
+    if (!old_dissector_try_port(ip_dissector_table, nxt, pd, offset, fd, tree)) {
       /* Unknown protocol */
       if (check_col(fd, COL_PROTOCOL))
 	col_add_str(fd, COL_PROTOCOL, "IPv6");
       if (check_col(fd, COL_INFO))
 	col_add_fstr(fd, COL_INFO, "%s (0x%02x)", ipprotostr(nxt), nxt);
-      dissect_data(pd, offset, fd, tree);
+      old_dissect_data(pd, offset, fd, tree);
     }
   }
 }
@@ -422,6 +469,10 @@ proto_register_ipv6(void)
       { "Destination",		"ipv6.dst",
 				FT_IPv6, BASE_NONE, NULL, 0x0,
 				"Destination IPv6 Address" }},
+    { &hf_ipv6_addr,
+      { "Address",		"ipv6.addr",
+				FT_IPv6, BASE_NONE, NULL, 0x0,
+				"Source or Destination IPv6 Address" }},
 #ifdef TEST_FINALHDR
     { &hf_ipv6_final,
       { "Final next header",	"ipv6.final",
@@ -440,8 +491,8 @@ proto_register_ipv6(void)
 void
 proto_reg_handoff_ipv6(void)
 {
-	dissector_add("ethertype", ETHERTYPE_IPv6, dissect_ipv6);
-	dissector_add("ppp.protocol", PPP_IPV6, dissect_ipv6);
-	dissector_add("ip.proto", IP_PROTO_IPV6, dissect_ipv6);
-	dissector_add("ip.proto", IP_PROTO_NONE, dissect_ipv6_none);
+	old_dissector_add("ethertype", ETHERTYPE_IPv6, dissect_ipv6);
+	old_dissector_add("ppp.protocol", PPP_IPV6, dissect_ipv6);
+	old_dissector_add("ip.proto", IP_PROTO_IPV6, dissect_ipv6);
+	old_dissector_add("ip.proto", IP_PROTO_NONE, dissect_ipv6_none);
 }
