@@ -2,7 +2,7 @@
  * Routines for NetWare's IPX
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-ipx.h,v 1.7 2000/05/30 03:35:52 guy Exp $
+ * $Id: packet-ipx.h,v 1.10 2000/08/11 13:34:11 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,6 +31,9 @@
  *
  */
 
+#ifndef __PACKET_IPX_H__
+#define __PACKET_IPX_H__
+
 #define IPX_NODE_LEN	6
 
 typedef guint32	IPXNet;
@@ -142,5 +145,6 @@ gchar* ipxnet_to_string(const guint8 *ad);
 gchar* ipxnet_to_str_punct(const guint32 ad, char punct);
 
 void capture_ipx(const u_char *, int, packet_counts *);
-void dissect_ipx(const u_char *, int, frame_data *, proto_tree *);
+void dissect_ipx(tvbuff_t *, packet_info *, proto_tree *);
 
+#endif
