@@ -1,7 +1,7 @@
 /* packet-irc.c
  * Routines for MSX irc packet dissection
  *
- * $Id: packet-irc.c,v 1.6 2000/05/31 05:07:11 guy Exp $
+ * $Id: packet-irc.c,v 1.8 2000/08/13 14:08:18 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -78,6 +78,8 @@ dissect_irc(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	int start, cur, len;
 	const u_char *i;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_irc, pd, offset, fd, tree);
+
 	if (check_col(fd, COL_PROTOCOL))
 	col_add_str(fd, COL_PROTOCOL, "IRC");
 
@@ -165,6 +167,6 @@ proto_register_irc(void)
 void
 proto_reg_handoff_irc(void)
 {
-	dissector_add("tcp.port", TCP_PORT_IRC, dissect_irc);
+	old_dissector_add("tcp.port", TCP_PORT_IRC, dissect_irc);
 }
 
