@@ -1,8 +1,9 @@
-/* packet-gre.c
- * Routines for the Internet Security Association and Key Management Protocol (ISAKMP)
+/* packet-isakmp.c
+ * Routines for the Internet Security Association and Key Management Protocol
+ * (ISAKMP) (RFC 2408)
  * Brad Robel-Forrest <brad.robel-forrest@watchguard.com>
  *
- * $Id: packet-isakmp.c,v 1.22 2000/05/31 05:07:11 guy Exp $
+ * $Id: packet-isakmp.c,v 1.26 2000/08/13 14:08:18 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -39,17 +40,13 @@
 #endif
 
 #include <glib.h>
-#include "packet.h"
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
+#include "packet.h"
+
 static int proto_isakmp = -1;
 
 static gint ett_isakmp = -1;
@@ -180,22 +177,22 @@ struct isakmp_hdr {
 #define E_FLAG		0x01
 #define C_FLAG		0x02
 #define A_FLAG		0x04
-  guint32	message_id;
-  guint32	length;
+  guint8	message_id[4];
+  guint8	length[4];
 };
 
 struct sa_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
-  guint32	doi;
-  guint32	situation;
+  guint8	length[2];
+  guint8	doi[4];
+  guint8	situation[4];
 };
 
 struct proposal_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
   guint8	proposal_num;
   guint8	protocol_id;
   guint8	spi_size;
@@ -205,10 +202,10 @@ struct proposal_hdr {
 struct trans_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
   guint8	transform_num;
   guint8	transform_id;
-  guint16	reserved2;
+  guint8	reserved2[2];
 };
 
 #define TRANS_LEN(p)	(pntohs(&((struct trans_hdr *)(p))->length))
@@ -216,83 +213,83 @@ struct trans_hdr {
 struct ke_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
 };
 
 struct id_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
   guint8	id_type;
   guint8	protocol_id;
-  guint16	port;
+  guint8	port[2];
 };
 
 struct cert_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
   guint8	cert_enc;
 };
 
 struct certreq_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
   guint8	cert_type;
 };
 
 struct hash_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
 };
 
 struct sig_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
 };
 
 struct nonce_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
 };
 
 struct notif_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
-  guint32	doi;
+  guint8	length[2];
+  guint8	doi[4];
   guint8	protocol_id;
   guint8	spi_size;
-  guint16	msgtype;
+  guint8	msgtype[2];
 };
 
 struct delete_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
-  guint32	doi;
+  guint8	length[2];
+  guint8	doi[4];
   guint8	protocol_id;
   guint8	spi_size;
-  guint16	num_spis;
+  guint8	num_spis[2];
 };
 
 struct vid_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint16	length;
+  guint8	length[2];
 };
 
 struct cfg_hdr {
   guint8	next_payload;
   guint8	reserved;
-  guint8	length;
+  guint8	length[2];
   guint8	type;
   guint8	reserved2;
-  guint16	identifier;
+  guint8	identifier[2];
 };
 
 static void dissect_none(const u_char *, int, frame_data *, proto_tree *);
@@ -353,6 +350,8 @@ dissect_isakmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   
   struct isakmp_hdr *	hdr = (struct isakmp_hdr *)(pd + offset);
   guint32		len;
+
+  OLD_CHECK_DISPLAY_AS_DATA(proto_isakmp, pd, offset, fd, tree);
   
   if (check_col(fd, COL_PROTOCOL))
     col_add_str(fd, COL_PROTOCOL, "ISAKMP");
@@ -432,7 +431,7 @@ dissect_isakmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
           (*strfuncs[hdr->next_payload].func)(pd, offset, fd, isakmp_tree);
       }
       else
-        dissect_data(pd, offset, fd, isakmp_tree);
+        old_dissect_data(pd, offset, fd, isakmp_tree);
     }
   }
 }
@@ -482,7 +481,7 @@ dissect_sa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -538,7 +537,7 @@ dissect_proposal(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
       (*strfuncs[hdr->next_payload].func)(pd, next_hdr_offset, fd, tree);
   }
   else
-    dissect_data(pd, next_hdr_offset, fd, tree);
+    old_dissect_data(pd, next_hdr_offset, fd, tree);
 }
 
 static void
@@ -628,7 +627,7 @@ dissect_transform(const u_char *pd, int offset, frame_data *fd,
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -660,7 +659,7 @@ dissect_key_exch(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -717,7 +716,7 @@ dissect_id(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -753,7 +752,7 @@ dissect_cert(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -789,7 +788,7 @@ dissect_certreq(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -821,7 +820,7 @@ dissect_hash(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -853,7 +852,7 @@ dissect_sig(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -885,7 +884,7 @@ dissect_nonce(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -944,7 +943,7 @@ dissect_notif(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -999,7 +998,7 @@ dissect_delete(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -1031,7 +1030,7 @@ dissect_vid(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -1059,7 +1058,7 @@ dissect_config(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   offset += (sizeof(hdr->type) + sizeof(hdr->reserved2));
   
   proto_tree_add_text(ntree, NullTVB, offset, sizeof(hdr->identifier),
-                      "Identifier: %u",hdr->identifier);
+                      "Identifier: %u", pntohs(&hdr->identifier));
   offset += sizeof(hdr->identifier);
   length -= sizeof(*hdr);
   
@@ -1090,7 +1089,7 @@ dissect_config(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       (*strfuncs[hdr->next_payload].func)(pd, offset, fd, tree);
   }
   else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
  
 }
 
@@ -1431,5 +1430,5 @@ proto_register_isakmp(void)
 void
 proto_reg_handoff_isakmp(void)
 {
-	dissector_add("udp.port", UDP_PORT_ISAKMP, dissect_isakmp);
+	old_dissector_add("udp.port", UDP_PORT_ISAKMP, dissect_isakmp);
 }
