@@ -1,7 +1,7 @@
 /* packet-isl.c
  * Routines for Cisco ISL Ethernet header disassembly
  *
- * $Id: packet-isl.c,v 1.12 2000/05/31 05:07:15 guy Exp $
+ * $Id: packet-isl.c,v 1.15 2000/08/13 14:08:23 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -145,9 +145,11 @@ dissect_isl(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   guint8 type;
   guint16 length;
   tvbuff_t *next_tvb;
+
+  OLD_CHECK_DISPLAY_AS_DATA(proto_isl, pd, offset, fd, tree);
   
   if (!BYTES_ARE_IN_FRAME(offset, ISL_HEADER_SIZE)) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -209,7 +211,7 @@ dissect_isl(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   switch (type) {
 
   case TYPE_ETHER:
-    next_tvb = tvb_new_subset(pi.compat_top_tvb, offset+26, -1, -1);
+    next_tvb = tvb_create_from_top(offset+26);
     dissect_eth(next_tvb, &pi, tree);
     break;
 
@@ -226,13 +228,13 @@ dissect_isl(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			pd[offset+30]);
     proto_tree_add_uint(fh_tree, hf_isl_esize, NullTVB, offset+16, 1,
 			pd[offset+30]);
-    next_tvb = tvb_new_subset(pi.compat_top_tvb, offset+31, -1, -1);
+    next_tvb = tvb_create_from_top(offset+31);
     dissect_tr(next_tvb, &pi, tree);
     break;
 
   default:
-    next_tvb = tvb_new_subset(pi.compat_top_tvb, offset+26, -1, -1);
-    dissect_data_tvb(next_tvb, &pi, tree);
+    next_tvb = tvb_create_from_top(offset+26);
+    dissect_data(next_tvb, &pi, tree);
     break;
   }
 }
