@@ -7,7 +7,7 @@
  * Laurent Cazalet <laurent.cazalet@mailclub.net>
  * Thomas Parvais <thomas.parvais@advalvas.be>
  *
- * $Id: packet-l2tp.c,v 1.11 2000/06/07 11:37:08 gram Exp $
+ * $Id: packet-l2tp.c,v 1.15 2000/08/13 14:08:24 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -294,11 +294,13 @@ dissect_l2tp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   tvbuff_t	*tvb;
   tvbuff_t	*next_tvb;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_l2tp, pd, offset, fd, tree);
+
   pi.current_proto = "L2TP";
   if (check_col(fd, COL_PROTOCOL))	/* build output for closed L2tp frame displayed  */
         col_add_str(fd, COL_PROTOCOL, "L2TP"); 
 
-  tvb = tvb_new_subset(pi.compat_top_tvb, offset, -1, -1);
+  tvb = tvb_create_from_top(offset);
 
   control = tvb_get_ntohs(tvb, 0);
 
@@ -821,7 +823,8 @@ proto_register_l2tp(void)
 		&ett_l2tp_avp,
 	};
 
-	proto_l2tp = proto_register_protocol ("L2TP Protocol", "l2tp");
+	proto_l2tp = proto_register_protocol(
+		"Layer 2 Tunneling Protocol", "l2tp");
 	proto_register_field_array(proto_l2tp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 }
@@ -829,5 +832,5 @@ proto_register_l2tp(void)
 void
 proto_reg_handoff_l2tp(void)
 {
-	dissector_add("udp.port", UDP_PORT_L2TP, dissect_l2tp);
+	old_dissector_add("udp.port", UDP_PORT_L2TP, dissect_l2tp);
 }
