@@ -2,7 +2,7 @@
  * Routines for LAPD frame disassembly
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-lapd.c,v 1.11 2000/05/31 05:07:16 guy Exp $
+ * $Id: packet-lapd.c,v 1.13 2000/08/13 14:08:24 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -94,6 +94,8 @@ dissect_lapd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gboolean	is_response;
 	tvbuff_t	*next_tvb;
 
+	CHECK_DISPLAY_AS_DATA(proto_lapd, tvb, pinfo, tree);
+
 	pinfo->current_proto = "LAPD";
 
 	if (check_col(pinfo->fd, COL_PROTOCOL))
@@ -151,11 +153,11 @@ dissect_lapd(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			break;
 
 		default:
-			dissect_data_tvb(next_tvb, pinfo, tree);
+			dissect_data(next_tvb, pinfo, tree);
 			break;
 		}
 	} else
-		dissect_data_tvb(next_tvb, pinfo, tree);
+		dissect_data(next_tvb, pinfo, tree);
 }
 
 void
