@@ -2,7 +2,7 @@
  * Routines for IEEE 802.2 LLC layer
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-llc.c,v 1.65 2000/05/31 05:07:17 guy Exp $
+ * $Id: packet-llc.c,v 1.69 2000/08/25 06:31:25 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -278,6 +278,8 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	const guint8	*pd;
 	int		offset;
 
+	CHECK_DISPLAY_AS_DATA(proto_llc, tvb, pinfo, tree);
+    
 	pinfo->current_proto = "LLC";
 
 	if (check_col(pinfo->fd, COL_PROTOCOL)) {
@@ -323,17 +325,12 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	if (tree)
 		proto_item_set_len(ti, llc_header_len);
 
-	/*
-	 * XXX - do we want to append the SAP information to the stuff
-	 * "dissect_xdlc_control()" put in the COL_INFO column, rather
-	 * than overwriting it?
-	 */
 	if (is_snap) {
 		oui =	tvb_get_ntoh24(tvb, 3);
 		etype = tvb_get_ntohs(tvb, 6);
 
 		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO, "SNAP, OUI 0x%06X (%s), PID 0x%04X",
+			col_append_fstr(pinfo->fd, COL_INFO, "; SNAP, OUI 0x%06X (%s), PID 0x%04X",
 			    oui, val_to_str(oui, oui_vals, "Unknown"),
 			    etype);
 		}
@@ -360,7 +357,7 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				ethertype(etype, tvb, 8,
 				    pinfo, tree, llc_tree, hf_llc_type);
 			} else
-				dissect_data_tvb(next_tvb, pinfo, tree);
+				dissect_data(next_tvb, pinfo, tree);
 			break;
 
 		case OUI_CISCO:
@@ -395,11 +392,11 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					break;
 
 				default:
-					dissect_data_tvb(next_tvb, pinfo, tree);
+					dissect_data(next_tvb, pinfo, tree);
 					break;
 				}
 			} else
-				dissect_data_tvb(next_tvb, pinfo, tree);
+				dissect_data(next_tvb, pinfo, tree);
 			break;
 
 		case OUI_CABLE_BPDU:    /* DOCSIS cable modem spanning tree BPDU */
@@ -415,14 +412,14 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				proto_tree_add_uint(llc_tree,
 				    hf_llc_pid, tvb, 6, 2, etype);
 			}
-			dissect_data_tvb(next_tvb, pinfo, tree);
+			dissect_data(next_tvb, pinfo, tree);
 			break;
 		}
 	}
 	else {
 		if (check_col(pinfo->fd, COL_INFO)) {
-			col_add_fstr(pinfo->fd, COL_INFO, 
-			    "DSAP %s %s, SSAP %s %s",
+			col_append_fstr(pinfo->fd, COL_INFO, 
+			    "; DSAP %s %s, SSAP %s %s",
 			    val_to_str(dsap & SAP_MASK, sap_vals, "%02x"),
 			    dsap & DSAP_GI_BIT ?
 			      "Group" : "Individual",
@@ -434,17 +431,14 @@ dissect_llc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 		next_tvb = tvb_new_subset(tvb, llc_header_len, -1, -1);
 		if (XDLC_IS_INFORMATION(control)) {
-			tvb_compat(tvb, &pd, &offset);
 			/* non-SNAP */
-			offset += llc_header_len;
-
 			/* do lookup with the subdissector table */
 			if (!dissector_try_port(subdissector_table, dsap,
-			    pd, offset, pinfo->fd, tree)) {
-				dissect_data_tvb(next_tvb, pinfo, tree);
+			    next_tvb, pinfo, tree)) {
+				dissect_data(next_tvb, pinfo, tree);
 			}
 		} else {
-			dissect_data_tvb(next_tvb, pinfo, tree);
+			dissect_data(next_tvb, pinfo, tree);
 		}
 	}
 }
