@@ -2,7 +2,7 @@
  * Routines for LPR and LPRng packet disassembly
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-lpd.c,v 1.20 2000/05/31 05:07:17 guy Exp $
+ * $Id: packet-lpd.c,v 1.22 2000/08/13 14:08:27 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -78,6 +78,8 @@ dissect_lpd(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		"Bad job format, do not retry"
 	};
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_lpd, pd, offset, fd, tree);
+
 	/* rfc1179 states that all responses are 1 byte long */
 	if (END_OF_FRAME == 1) {
 		lpr_packet_type = response;
@@ -124,7 +126,7 @@ dissect_lpd(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 					strlen(printer), "Printer/options: %s", printer);
 			}
 			else {
-				dissect_data(pd, offset, fd, tree);
+				old_dissect_data(pd, offset, fd, tree);
 			}
 
 			if (printer) 
@@ -138,11 +140,11 @@ dissect_lpd(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 					lpd_server_code[response]);
 			}
 			else {
-				dissect_data(pd, offset, fd, tree);
+				old_dissect_data(pd, offset, fd, tree);
 			}
 		}
 		else {
-				dissect_data(pd, offset, fd, tree);
+				old_dissect_data(pd, offset, fd, tree);
 		}
 	}
 }
@@ -200,5 +202,5 @@ proto_register_lpd(void)
 void
 proto_reg_handoff_lpd(void)
 {
-  dissector_add("tcp.port", TCP_PORT_PRINTER, &dissect_lpd);
+  old_dissector_add("tcp.port", TCP_PORT_PRINTER, &dissect_lpd);
 }
