@@ -1,7 +1,7 @@
 /* packet-mapi.c
  * Routines for MSX mapi packet dissection
  *
- * $Id: packet-mapi.c,v 1.7 2000/05/31 05:07:18 guy Exp $
+ * $Id: packet-mapi.c,v 1.9 2000/08/13 14:08:27 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -55,6 +55,8 @@ dissect_mapi(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
 	proto_tree      *mapi_tree, *ti;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_mapi, pd, offset, fd, tree);
+
 	if (check_col(fd, COL_PROTOCOL))
 		col_add_str(fd, COL_PROTOCOL, "MAPI");
 
@@ -112,5 +114,5 @@ proto_register_mapi(void)
 void
 proto_reg_handoff_mapi(void)
 {
-	dissector_add("tcp.port", TCP_PORT_MAPI, dissect_mapi);
+	old_dissector_add("tcp.port", TCP_PORT_MAPI, dissect_mapi);
 }
