@@ -2,7 +2,7 @@
  * Routines for Mobile IP dissection
  * Copyright 2000, Stefan Raab <Stefan.Raab@nextel.com>
  *
- * $Id: packet-mip.c,v 1.4 2000/05/31 05:07:18 guy Exp $
+ * $Id: packet-mip.c,v 1.8 2000/08/13 14:08:28 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -38,17 +38,13 @@
 # include <netinet/in.h>
 #endif
 
+#include <string.h>
+#include <glib.h>
+
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
-#include <string.h>
-#include <glib.h>
 #include "packet.h"
 
 /* Initialize the protocol and registered fields */
@@ -149,7 +145,9 @@ dissect_mip(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	/* Make our own tvb until the function call includes one */
 	tvbuff_t	*tvb;
 	packet_info	*pinfo = &pi;
-	tvb = tvb_new_subset(pi.compat_top_tvb, offset, -1, -1);
+	tvb = tvb_create_from_top(offset);
+
+	CHECK_DISPLAY_AS_DATA(proto_mip, tvb, pinfo, tree);
 
 /* Make entries in Protocol column and Info column on summary display */
 
@@ -299,5 +297,5 @@ void proto_register_mip(void)
 void
 proto_reg_handoff_mip(void)
 {
-	dissector_add("udp.port", UDP_PORT_MIP, dissect_mip);
+	old_dissector_add("udp.port", UDP_PORT_MIP, dissect_mip);
 }
