@@ -1,8 +1,8 @@
-/* packet-rlogin.c
+/* packet-msproxy.c
  * Routines for Microsoft Proxy packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-msproxy.c,v 1.4 2000/05/31 16:49:42 gram Exp $
+ * $Id: packet-msproxy.c,v 1.11 2000/08/21 18:36:34 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -59,6 +59,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <glib.h>
+
+#ifdef NEED_SNPRINTF_H
+# include "snprintf.h"
+#endif
+
 #include "packet.h"
 #include "resolv.h"
 #include "globals.h"
@@ -68,17 +73,7 @@
 #include "packet-tcp.h"
 #include "packet-udp.h"
 
-
-#ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
-# include "snprintf.h"
-#endif
-
-#define CHECK_PACKET_LENGTH(X) if ((offset+X) > fd->cap_len){  \
+#define CHECK_PACKET_LENGTH(X) if (!BYTES_ARE_IN_FRAME(offset, X)){  \
  	proto_tree_add_text(tree, NullTVB, offset, 0, "****FRAME TOO SHORT***"); return;}
 
 extern void udp_hash_add(guint16 proto,
@@ -282,7 +277,7 @@ static void add_msproxy_conversation( hash_entry_t *hash_info){
 /* conversation data structure with the info needed to call the TCP or 	*/
 /* UDP port decoder.							*/
 
-/* NOTE: Currently this assume that the conversataion will be created 	*/
+/* NOTE: Currently this assume that the conversation will be created 	*/
 /* 	during a packet from the server.  If that changes, the pi.src	*/
 /*	and pi.dst will not be correct and this routine will have to 	*/
 /*	change.								*/
@@ -309,7 +304,7 @@ static void add_msproxy_conversation( hash_entry_t *hash_info){
 	new_conv_info->server_int_port = hash_info->server_int_port;
 	new_conv_info->proto = hash_info->proto;
 	
-	conversation->dissector = msproxy_sub_dissector;
+	old_conversation_set_dissector(conversation, msproxy_sub_dissector);
 }
 
 
@@ -323,7 +318,7 @@ static int display_application_name(const u_char *pd, int offset,
 
 	char temp[255];
 	
-	if ((offset+ 1) > fd->cap_len){ 
+	if (!IS_DATA_IN_FRAME(offset)){ 
  		proto_tree_add_text(tree, NullTVB, offset, 0, "****FRAME TOO SHORT***");
  		return 0;
  	}
@@ -1168,6 +1163,7 @@ static void dissect_msproxy(const u_char *pd, int offset, frame_data *fd, proto_
 	hash_entry_t *hash_info;
 	conversation_t *conversation;
 	
+	OLD_CHECK_DISPLAY_AS_DATA(proto_msproxy, pd, offset, fd, tree);
 
 	conversation = find_conversation( &pi.src, &pi.dst, pi.ptype,
 		pi.srcport, pi.destport);
@@ -1354,5 +1350,5 @@ proto_reg_handoff_msproxy(void) {
 
 	/* dissector install routine */ 
  
- 	dissector_add("udp.port", UDP_PORT_MSPROXY, dissect_msproxy);
+ 	old_dissector_add("udp.port", UDP_PORT_MSPROXY, dissect_msproxy);
 }
