@@ -1,6 +1,6 @@
 /* packet-nbipx.h
  *
- * $Id: packet-nbipx.h,v 1.2 2000/05/30 03:35:53 guy Exp $
+ * $Id: packet-nbipx.h,v 1.3 2000/08/11 13:34:08 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -22,5 +22,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_NBIPX_H__
+#define __PACKET_NBIPX_H__
 
 void dissect_nbipx(const u_char *, int, frame_data *, proto_tree *);
+
+#endif
