@@ -4,7 +4,7 @@
  * Gilbert Ramirez <gram@xiexie.org>
  * Much stuff added by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-nbns.c,v 1.43 2000/05/31 05:07:21 guy Exp $
+ * $Id: packet-nbns.c,v 1.45 2000/08/13 14:08:30 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -1133,6 +1133,8 @@ dissect_nbns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	guint16			id, flags, quest, ans, auth, add;
 	int			cur_off;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_nbns, pd, offset, fd, tree);
+
 	nbns_data_offset = offset;
 
 	if (check_col(fd, COL_PROTOCOL))
@@ -1140,7 +1142,7 @@ dissect_nbns(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 
 	if (pi.captured_len < NBNS_HDRLEN) {
 		col_add_str(fd, COL_INFO, "Short NBNS packet");
-		dissect_data(pd, offset, fd, tree);
+		old_dissect_data(pd, offset, fd, tree);
 		return;
 	}
 
@@ -1293,6 +1295,8 @@ dissect_nbdgm(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	int name_type;
 	int len;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_nbdgm, pd, offset, fd, tree);
+
 	header.msg_type = pd[offset];
 	
 	flags = pd[offset+1];
@@ -1586,6 +1590,8 @@ dissect_nbss(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	int		max_data;
 	int		is_cifs;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_nbss, pd, offset, fd, tree);
+
 	msg_type = pd[offset];
 
 	if (pi.match_port == TCP_PORT_CIFS) {
@@ -1759,8 +1765,8 @@ proto_register_nbt(void)
 void
 proto_reg_handoff_nbt(void)
 {
-  dissector_add("udp.port", UDP_PORT_NBNS, dissect_nbns);
-  dissector_add("udp.port", UDP_PORT_NBDGM, dissect_nbdgm);
-  dissector_add("tcp.port", TCP_PORT_NBSS, dissect_nbss);
-  dissector_add("tcp.port", TCP_PORT_CIFS, dissect_nbss);
+  old_dissector_add("udp.port", UDP_PORT_NBNS, dissect_nbns);
+  old_dissector_add("udp.port", UDP_PORT_NBDGM, dissect_nbdgm);
+  old_dissector_add("tcp.port", TCP_PORT_NBSS, dissect_nbss);
+  old_dissector_add("tcp.port", TCP_PORT_CIFS, dissect_nbss);
 }
