@@ -5,7 +5,7 @@
  * 
  * derived from the packet-nbns.c
  *
- * $Id: packet-netbios.c,v 1.20 2000/05/31 05:07:23 guy Exp $
+ * $Id: packet-netbios.c,v 1.22 2000/08/13 14:08:32 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -509,7 +509,7 @@ static void  dissect_netb_unknown(const u_char *data_ptr, int offset,
 
 {/* Handle any unknow commands, do nothing */
 
-/*	dissect_data( data_ptr, offset + NB_COMMAND + 1, fd, tree); */
+/*	old_dissect_data( data_ptr, offset + NB_COMMAND + 1, fd, tree); */
 }
 
 
@@ -974,6 +974,8 @@ static void dissect_netbios(const u_char *pd, int offset, frame_data *fd,
 /* Find NetBIOS marker EFFF, this is done because I have seen an extra LLC */
 /* byte on our network. This only checks for one extra LLC byte. */
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_netbios, pd, offset, fd, tree);
+
 	if (( pd[offset + 2] != 0xff) || ( pd[offset + 3] != 0xef)){
 		++offset;
 		if (( pd[offset + 2] != 0xff)
@@ -1086,5 +1088,5 @@ void proto_register_netbios(void)
 void
 proto_reg_handoff_netbios(void)
 {
-	dissector_add("llc.dsap", SAP_NETBIOS, dissect_netbios);
+	old_dissector_add("llc.dsap", SAP_NETBIOS, dissect_netbios);
 }
