@@ -2,7 +2,7 @@
  * Routines for nntp packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-nntp.c,v 1.10 2000/05/31 05:07:26 guy Exp $
+ * $Id: packet-nntp.c,v 1.13 2000/09/11 16:16:02 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -40,6 +40,7 @@
 #include <string.h>
 #include <glib.h>
 #include "packet.h"
+#include "strutil.h"
 
 static int proto_nntp = -1;
 static int hf_nntp_response = -1;
@@ -59,6 +60,8 @@ dissect_nntp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	int		linelen;
 	int		max_data = pi.captured_len - offset;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_nntp, pd, offset, fd, tree);
+
 	data = &pd[offset];
 	dataend = data + END_OF_FRAME;
 	if (dataend > data + max_data)
@@ -147,5 +150,5 @@ proto_register_nntp(void)
 void
 proto_reg_handoff_nntp(void)
 {
-  dissector_add("tcp.port", TCP_PORT_NNTP, dissect_nntp);
+  old_dissector_add("tcp.port", TCP_PORT_NNTP, dissect_nntp);
 }
