@@ -2,7 +2,7 @@
  * Routines for NTP packet dissection
  * Copyright 1999, Nathan Neulinger <nneul@umr.edu>
  *
- * $Id: packet-ntp.c,v 1.13 2000/05/31 05:07:26 guy Exp $
+ * $Id: packet-ntp.c,v 1.16 2000/08/13 14:08:25 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -45,11 +45,6 @@
 #include <glib.h>
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
@@ -241,6 +236,8 @@ dissect_ntp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	gchar buff[NTP_TS_SIZE];
 	int i;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_ntp, pd, offset, fd, tree);
+
 	/* get at least a full packet structure */
 	if ( !BYTES_ARE_IN_FRAME(offset, 48) ) /* 48 without keyid or mac */
 		return;
@@ -441,6 +438,6 @@ proto_register_ntp(void)
 void
 proto_reg_handoff_ntp(void)
 {
-	dissector_add("udp.port", UDP_PORT_NTP, dissect_ntp);
-	dissector_add("tcp.port", TCP_PORT_NTP, dissect_ntp);
+	old_dissector_add("udp.port", UDP_PORT_NTP, dissect_ntp);
+	old_dissector_add("tcp.port", TCP_PORT_NTP, dissect_ntp);
 }
