@@ -1,6 +1,6 @@
 /* packet-null.h
  *
- * $Id: packet-null.h,v 1.2 2000/05/19 05:18:26 guy Exp $
+ * $Id: packet-null.h,v 1.3 2000/08/11 13:34:05 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -22,5 +22,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_NULL_H__
+#define __PACKET_NULL_H__
+
 void capture_null(const u_char *, packet_counts *);
 void dissect_null(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+
+#endif
