@@ -2,7 +2,7 @@
  * Routines for OSPF packet disassembly
  * (c) Copyright Hannes R. Boehm <hannes@boehm.org>
  *
- * $Id: packet-ospf.c,v 1.24 2000/05/31 05:07:28 guy Exp $
+ * $Id: packet-ospf.c,v 1.28 2000/09/13 07:47:09 guy Exp $
  *
  * At this time, this module is able to analyze OSPF
  * packets as specified in RFC2328. MOSPF (RFC1584) and other
@@ -98,6 +98,8 @@ dissect_ospf(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
                                       {OSPF_LS_ACK,  "LS Acknowledge" },
                                       {0,             NULL            } };
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_ospf, pd, offset, fd, tree);
+
     memcpy(&ospfh, &pd[offset], sizeof(e_ospfhdr));
 
     packet_type = match_strval(ospfh.packet_type, pt_vals);
@@ -197,7 +199,7 @@ dissect_ospf(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	    break;
 	default:
             pi.captured_len = saved_len;
-            dissect_data(pd, offset, fd, tree); 
+            old_dissect_data(pd, offset, fd, tree); 
     }
     pi.captured_len = saved_len;
 }
@@ -308,8 +310,8 @@ dissect_ospf_db_desc(const u_char *pd, int offset, frame_data *fd, proto_tree *t
 
 	flags_offset=0;
 	if(( ospf_dbd.flags & OSPF_DBD_FLAG_MS ) == OSPF_DBD_FLAG_MS){
-	    strcpy( (char *)(flags + flags_offset), "_I_");
-	    flags_offset+=1;
+	    strcpy( (char *)(flags + flags_offset), "_MS_");
+	    flags_offset+=4;
 	}
 	if(( ospf_dbd.flags & OSPF_DBD_FLAG_M ) == OSPF_DBD_FLAG_M){
 	    strcpy((char *) (flags + flags_offset), "_M_");
@@ -660,6 +662,7 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
 
     /* data structures for the summary and ASBR LSAs */
     e_ospf_summary_lsa 	summary_lsa;
+    guint32             metric;
 
     /* data structures for the AS-External LSA */
     e_ospf_asexternal_lsa       asext_lsa;
@@ -691,6 +694,9 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
         case OSPF_LSTYPE_ASEXT:
 	    lsa_type="AS-external-LSA";
             break;
+        case OSPF_LSTYPE_ASEXT7:
+	    lsa_type="AS-external-LSA Type 7/NSSA";
+            break;
         case OSPF_LSTYPE_OP_LINKLOCAL:
 	    lsa_type="Opaque LSA, Link-local scope";
             break;
@@ -732,7 +738,7 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
 	                                             ip_to_str((guint8 *) &(lsa_hdr.adv_router)));
         proto_tree_add_text(ospf_lsa_tree, NullTVB, offset + 12, 4, "LS Sequence Number: 0x%04lx ", 
 	                                             (unsigned long)ntohl(lsa_hdr.ls_seq));
-        proto_tree_add_text(ospf_lsa_tree, NullTVB, offset + 16, 2, "LS Checksum: %d ", ntohs(lsa_hdr.ls_checksum));
+        proto_tree_add_text(ospf_lsa_tree, NullTVB, offset + 16, 2, "LS Checksum: %04x ", ntohs(lsa_hdr.ls_checksum));
 
         proto_tree_add_text(ospf_lsa_tree, NullTVB, offset + 18, 2, "Length: %d ", ntohs(lsa_hdr.length));
 
@@ -828,9 +834,15 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
                 memcpy(&summary_lsa, &pd[offset], sizeof(e_ospf_summary_lsa));
                 proto_tree_add_text(ospf_lsa_tree, NullTVB, offset, 4, "Netmask: %s", 
                                                  ip_to_str((guint8 *) &(summary_lsa.network_mask)));
+		/* this routine returns only the TOS 0 metric (even if there are more TOS metrics) */
+		memcpy(&metric, &pd[offset+4], 4);
+		metric = ntohl(metric) & 0x00ffffff ;
+		proto_tree_add_text(ospf_lsa_tree, NullTVB, offset + 5, 3, "Metric: %d", metric);
+				                 
                 /* returns only the TOS 0 metric (even if there are more TOS metrics) */
                 break;
             case(OSPF_LSTYPE_ASEXT):
+            case(OSPF_LSTYPE_ASEXT7):
                 memcpy(&summary_lsa, &pd[offset], sizeof(e_ospf_summary_lsa));
                 proto_tree_add_text(ospf_lsa_tree, NullTVB, offset, 4, "Netmask: %s", 
                                                   ip_to_str((guint8 *) &(summary_lsa.network_mask)));
@@ -902,5 +914,5 @@ proto_register_ospf(void)
 void
 proto_reg_handoff_ospf(void)
 {
-	dissector_add("ip.proto", IP_PROTO_OSPF, dissect_ospf);
+	old_dissector_add("ip.proto", IP_PROTO_OSPF, dissect_ospf);
 }
