@@ -2,7 +2,7 @@
  * Routines for PIM disassembly
  * (c) Copyright Jun-ichiro itojun Hagino <itojun@itojun.org>
  *
- * $Id: packet-pim.c,v 1.14 2000/05/31 05:07:28 guy Exp $
+ * $Id: packet-pim.c,v 1.17 2000/08/13 14:08:37 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -23,8 +23,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
- 
-#include "config.h"
+
+#ifdef HAVE_CONFIG_H 
+# include "config.h"
+#endif
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -34,16 +36,12 @@
 #include <netinet/in.h>
 #endif
 
+#include <glib.h>
+
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
-#include <glib.h>
 #include "packet.h"
 #include "packet-ip.h"
 #include "packet-ipv6.h"
@@ -195,6 +193,8 @@ dissect_pim(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     proto_tree *pimopt_tree = NULL;
 	proto_item *tiopt; 
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_pim, pd, offset, fd, tree);
+
     /* avoid alignment problem */
     memcpy(&pim, &pd[offset], sizeof(pim));
 
@@ -605,6 +605,6 @@ proto_register_pim(void)
 void
 proto_reg_handoff_pim(void)
 {
-    dissector_add("ip.proto", IP_PROTO_PIM, dissect_pim);
+    old_dissector_add("ip.proto", IP_PROTO_PIM, dissect_pim);
 }
 
