@@ -1,7 +1,7 @@
 /* packet-ppp.c
  * Routines for ppp packet disassembly
  *
- * $Id: packet-ppp.c,v 1.36 2000/05/31 05:07:29 guy Exp $
+ * $Id: packet-ppp.c,v 1.39 2000/08/13 14:08:38 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -996,7 +996,7 @@ dissect_ppp_stuff( tvbuff_t *tvb, packet_info *pinfo,
   tvb_compat(next_tvb, &next_pd, &next_offset);
 
   /* do lookup with the subdissector table */
-  if (dissector_try_port(subdissector_table, ppp_prot, next_pd, next_offset, pinfo->fd, tree))
+  if (old_dissector_try_port(subdissector_table, ppp_prot, next_pd, next_offset, pinfo->fd, tree))
       return TRUE;
 
   /* XXX - make "dissect_lcp()" and "dissect_ipcp()", have them just
@@ -1019,7 +1019,7 @@ dissect_ppp_stuff( tvbuff_t *tvb, packet_info *pinfo,
       if (check_col(pinfo->fd, COL_INFO))
         col_add_fstr(pinfo->fd, COL_INFO, "PPP %s (0x%04x)",
 		val_to_str(ppp_prot, ppp_vals, "Unknown"), ppp_prot);
-      dissect_data_tvb(next_tvb, pinfo, tree);
+      dissect_data(next_tvb, pinfo, tree);
       return FALSE;
   }
 }
@@ -1046,6 +1046,8 @@ dissect_mp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   int         first, last;
   tvbuff_t	*next_tvb;
 
+  CHECK_DISPLAY_AS_DATA(proto_mp, tvb, pinfo, tree);
+
   flags = tvb_get_guint8(tvb, 0);
   first = flags && MP_FRAG_FIRST;
   last  = flags && MP_FRAG_LAST;
@@ -1110,7 +1112,7 @@ dissect_payload_ppp( const u_char *pd, int offset, frame_data *fd, proto_tree *t
     fh_tree = proto_item_add_subtree(ti, ett_ppp);
   }
 
-  next_tvb = tvb_new_subset(pi.compat_top_tvb, offset, -1, -1);
+  next_tvb = tvb_create_from_top(offset);
   dissect_ppp_stuff(next_tvb, &pi, tree, fh_tree);
 }
 
@@ -1123,6 +1125,8 @@ dissect_ppp( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree ) {
   tvbuff_t   *next_tvb;
   guint8     byte0;
 
+  CHECK_DISPLAY_AS_DATA(proto_ppp, tvb, pinfo, tree);
+
   pinfo->current_proto = "PPP";
   byte0 = tvb_get_guint8(tvb, 0);
 
