@@ -1,7 +1,7 @@
-/* packet-arp.c
- * Routines for ARP packet disassembly
+/* packet-pppoe.c
+ * Routines for PPP Over Ethernet (PPPoE) packet disassembly (RFC2516)
  *
- * $Id: packet-pppoe.c,v 1.8 2000/05/11 08:15:35 gram Exp $
+ * $Id: packet-pppoe.c,v 1.11 2000/09/11 16:24:09 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -35,6 +35,7 @@
 #include "etypes.h"
 #include "packet.h"
 #include "packet-ppp.h"
+#include "strutil.h"
 
 static gint ett_pppoed = -1;
 static gint ett_pppoed_tags = -1;
@@ -251,6 +252,6 @@ proto_register_pppoed(void)
 void
 proto_reg_handoff_pppoe(void)
 {
-	dissector_add("ethertype", ETHERTYPE_PPPOED, dissect_pppoed);
-	dissector_add("ethertype", ETHERTYPE_PPPOES, dissect_pppoes);
+	old_dissector_add("ethertype", ETHERTYPE_PPPOED, dissect_pppoed);
+	old_dissector_add("ethertype", ETHERTYPE_PPPOES, dissect_pppoes);
 }
