@@ -1,7 +1,7 @@
 /* packet-radius.c
  * Routines for RADIUS packet disassembly
  *
- * $Id: packet-radius.c,v 1.13 2000/05/31 05:07:33 guy Exp $
+ * $Id: packet-radius.c,v 1.17 2000/08/13 14:08:35 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Johan Feyaerts
@@ -39,6 +39,7 @@
 #include <glib.h>
 #include "packet.h"
 #include "resolv.h"
+#include "packet-diameter.h"
 
 static int proto_radius = -1;
 static int hf_radius_length = -1;
@@ -498,14 +499,7 @@ guint32 totlen=0;
 }
 
 gchar *rd_match_strval(guint32 val, const value_string *vs) {
-	gchar		*result;
-	static gchar	undf[30];
-	result=match_strval(val,vs);
-	if (result == NULL ) {
-		sprintf(undf,"Undefined(%d)",val);
-		result=undf;
-	}
-	return result;
+	return val_to_str(val, vs, "Undefined (%u)");
 }
 
 gchar *rd_value_to_str(e_avphdr *avph,const u_char *pd, int offset)
@@ -680,6 +674,14 @@ proto_tree
 
   gchar *codestrval;
 
+  if (pd[offset] == 254) {
+	  /* We have a diameter packet */
+	  dissect_diameter(pd, offset, fd, tree);
+	  return;
+  }
+
+  OLD_CHECK_DISPLAY_AS_DATA(proto_radius, pd, offset, fd, tree);
+
   memcpy(&rh,&pd[offset],sizeof(e_radiushdr));
 
 
@@ -772,8 +774,8 @@ proto_register_radius(void)
 void
 proto_reg_handoff_radius(void)
 {
-	dissector_add("udp.port", UDP_PORT_RADIUS, dissect_radius);
-	dissector_add("udp.port", UDP_PORT_RADIUS_NEW, dissect_radius);
-	dissector_add("udp.port", UDP_PORT_RADACCT, dissect_radius);
-	dissector_add("udp.port", UDP_PORT_RADACCT_NEW, dissect_radius);
+	old_dissector_add("udp.port", UDP_PORT_RADIUS, dissect_radius);
+	old_dissector_add("udp.port", UDP_PORT_RADIUS_NEW, dissect_radius);
+	old_dissector_add("udp.port", UDP_PORT_RADACCT, dissect_radius);
+	old_dissector_add("udp.port", UDP_PORT_RADACCT_NEW, dissect_radius);
 }
