@@ -1,6 +1,6 @@
 /* packet-raw.h
  *
- * $Id: packet-raw.h,v 1.2 2000/05/19 21:47:37 gram Exp $
+ * $Id: packet-raw.h,v 1.3 2000/08/11 13:34:02 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -22,5 +22,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_RAW_H__
+#define __PACKET_RAW_H__
+
 void capture_raw(const u_char *, packet_counts *);
 void dissect_raw(tvbuff_t *, packet_info *, proto_tree *);
+
+#endif
