@@ -3,7 +3,7 @@
  * (c) Copyright Jun-ichiro itojun Hagino <itojun@itojun.org>
  * derived from packet-rip.c
  *
- * $Id: packet-ripng.c,v 1.12 2000/05/31 05:07:34 guy Exp $
+ * $Id: packet-ripng.c,v 1.14 2000/08/13 14:08:43 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -67,6 +67,8 @@ dissect_ripng(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     };
     const char *cmd;
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_ripng, pd, offset, fd, tree);
+
     /* avoid alignment problem */
     memcpy(&rip6, &pd[offset], sizeof(rip6));
   
@@ -156,5 +158,5 @@ proto_register_ripng(void)
 void
 proto_reg_handoff_ripng(void)
 {
-    dissector_add("udp.port", UDP_PORT_RIPNG, dissect_ripng);
+    old_dissector_add("udp.port", UDP_PORT_RIPNG, dissect_ripng);
 }
