@@ -2,7 +2,7 @@
  * Routines for rpc dissection
  * Copyright 1999, Uwe Girlich <Uwe.Girlich@philosys.de>
  * 
- * $Id: packet-rpc.c,v 1.31 2000/05/31 05:07:35 guy Exp $
+ * $Id: packet-rpc.c,v 1.41 2000/08/24 23:09:37 guy Exp $
  * 
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -41,19 +41,35 @@
 #include "conversation.h"
 #include "packet-rpc.h"
 
+/*
+ * See:
+ *
+ *	RFC 1831, "RPC: Remote Procedure Call Protocol Specification
+ *	Version 2";
+ *
+ *	RFC 1832, "XDR: External Data Representation Standard";
+ *
+ *	RFC 2203, "RPCSEC_GSS Protocol Specification".
+ *
+ * See also
+ *
+ *	RFC 2695, "Authentication Mechanisms for ONC RPC"
+ *
+ *	although we don't currently dissec AUTH_DES or AUTH_KERB.
+ */
 
 #define RPC_RM_FRAGLEN  0x7fffffffL
 
 static struct true_false_string yesno = { "Yes", "No" };
 
 
-static const value_string rpc_msg_type[3] = {
+static const value_string rpc_msg_type[] = {
 	{ RPC_CALL, "Call" },
 	{ RPC_REPLY, "Reply" },
 	{ 0, NULL }
 };
 
-static const value_string rpc_reply_state[3] = {
+static const value_string rpc_reply_state[] = {
 	{ MSG_ACCEPTED, "accepted" },
 	{ MSG_DENIED, "denied" },
 	{ 0, NULL }
@@ -64,10 +80,26 @@ const value_string rpc_auth_flavor[] = {
 	{ AUTH_UNIX, "AUTH_UNIX" },
 	{ AUTH_SHORT, "AUTH_SHORT" },
 	{ AUTH_DES, "AUTH_DES" },
+	{ RPCSEC_GSS, "RPCSEC_GSS" },
 	{ 0, NULL }
 };
 
-static const value_string rpc_accept_state[6] = {
+static const value_string rpc_authgss_proc[] = {
+	{ RPCSEC_GSS_DATA, "RPCSEC_GSS_DATA" },
+	{ RPCSEC_GSS_INIT, "RPCSEC_GSS_INIT" },
+	{ RPCSEC_GSS_CONTINUE_INIT, "RPCSEC_GSS_CONTINUE_INIT" },
+	{ RPCSEC_GSS_DESTROY, "RPCSEC_GSS_DESTROY" },
+	{ 0, NULL }
+};
+
+static const value_string rpc_authgss_svc[] = {
+	{ RPCSEC_GSS_SVC_NONE, "rpcsec_gss_svc_none" },
+	{ RPCSEC_GSS_SVC_INTEGRITY, "rpcsec_gss_svc_integrity" },
+	{ RPCSEC_GSS_SVC_PRIVACY, "rpcsec_gss_svc_privacy" },
+	{ 0, NULL }
+};
+
+static const value_string rpc_accept_state[] = {
 	{ SUCCESS, "RPC executed successfully" },
 	{ PROG_UNAVAIL, "remote hasn't exported program" },
 	{ PROG_MISMATCH, "remote can't support version #" },
@@ -76,18 +108,20 @@ static const value_string rpc_accept_state[6] = {
 	{ 0, NULL }
 };
 
-static const value_string rpc_reject_state[3] = {
+static const value_string rpc_reject_state[] = {
 	{ RPC_MISMATCH, "RPC_MISMATCH" },
 	{ AUTH_ERROR, "AUTH_ERROR" },
 	{ 0, NULL }
 };
 
-static const value_string rpc_auth_state[6] = {
+static const value_string rpc_auth_state[] = {
 	{ AUTH_BADCRED, "bad credential (seal broken)" },
 	{ AUTH_REJECTEDCRED, "client must begin new session" },
 	{ AUTH_BADVERF, "bad verifier (seal broken)" },
 	{ AUTH_REJECTEDVERF, "verifier expired or replayed" },
 	{ AUTH_TOOWEAK, "rejected for security reasons" },
+	{ RPCSEC_GSSCREDPROB, "GSS credential problem" },
+	{ RPCSEC_GSSCTXPROB, "GSS context problem" },
 	{ 0, NULL }
 };
 
@@ -112,6 +146,18 @@ static int hf_rpc_auth_machinename = -1;
 static int hf_rpc_auth_stamp = -1;
 static int hf_rpc_auth_uid = -1;
 static int hf_rpc_auth_gid = -1;
+static int hf_rpc_authgss_v = -1;
+static int hf_rpc_authgss_proc = -1;
+static int hf_rpc_authgss_seq = -1;
+static int hf_rpc_authgss_svc = -1;
+static int hf_rpc_authgss_ctx = -1;
+static int hf_rpc_authgss_major = -1;
+static int hf_rpc_authgss_minor = -1;
+static int hf_rpc_authgss_window = -1;
+static int hf_rpc_authgss_token = -1;
+static int hf_rpc_authgss_data_length = -1;
+static int hf_rpc_authgss_data = -1;
+static int hf_rpc_authgss_checksum = -1;
 static int hf_rpc_state_accept = -1;
 static int hf_rpc_state_reply = -1;
 static int hf_rpc_state_reject = -1;
@@ -126,6 +172,7 @@ static gint ett_rpc_string = -1;
 static gint ett_rpc_cred = -1;
 static gint ett_rpc_verf = -1;
 static gint ett_rpc_gids = -1;
+static gint ett_rpc_gss_data = -1;
 
 /* Hash table with info on RPC program numbers */
 static GHashTable *rpc_progs;
@@ -270,11 +317,27 @@ char *rpc_prog_name(guint32 prog)
 /*--------------------------------------*/
 
 /* static array, first quick implementation, I'll switch over to GList soon */ 
+typedef struct _rpc_call_info {
+	guint32	xid;
+	conversation_t *conversation;
+	guint32	req_num;	/* frame number of first request seen */
+	guint32	rep_num;	/* frame number of first reply seen */
+	guint32	prog;
+	guint32	vers;
+	guint32	proc;
+	guint32 flavor;
+	guint32 gss_proc;
+	guint32 gss_svc;
+	rpc_proc_info_value*	proc_info;
+} rpc_call_info;
+
+#define RPC_CALL_TABLE_LENGTH 1000
+
 rpc_call_info rpc_call_table[RPC_CALL_TABLE_LENGTH];
 guint32 rpc_call_index = 0;
 guint32 rpc_call_firstfree = 0;
 
-void
+static void
 rpc_call_insert(rpc_call_info *call)
 {
 	/* some space left? */
@@ -298,7 +361,7 @@ rpc_call_insert(rpc_call_info *call)
 }
 
 
-rpc_call_info*
+static rpc_call_info*
 rpc_call_lookup(rpc_call_info *call)
 {
 	int i;
@@ -345,9 +408,19 @@ int hfindex)
 }
 
 
+int
+dissect_rpc_bool_tvb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
+int hfindex, int offset)
+{
+	if (tree)
+		proto_tree_add_item(tree, hfindex, tvb, offset, 4, FALSE);
+	return offset + 4;
+}
+
+
 int
 dissect_rpc_uint32(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
-char* name, char* type)
+char* name)
 {
 	guint32 value;
 
@@ -364,9 +437,19 @@ char* name, char* type)
 }
 
 
+int
+dissect_rpc_uint32_tvb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
+int hfindex, int offset)
+{
+	if (tree)
+		proto_tree_add_item(tree, hfindex, tvb, offset, 4, FALSE);
+	return offset + 4;
+}
+
+
 int
 dissect_rpc_uint64(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
-char* name, char* type)
+char* name)
 {
 	guint32 value_low;
 	guint32 value_high;
@@ -389,6 +472,28 @@ char* name, char* type)
 }
 
 
+int
+dissect_rpc_uint64_tvb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
+int hfindex, int offset)
+{
+	guint32 value_low;
+	guint32 value_high;
+
+	value_high = tvb_get_ntohl(tvb, offset + 0);
+	value_low  = tvb_get_ntohl(tvb, offset + 4);
+
+	if (tree) {
+		if (value_high)
+			proto_tree_add_text(tree, tvb, offset, 8,
+				"%s: 0x%x%08x", proto_registrar_get_name(hfindex), value_high, value_low);
+		else
+			proto_tree_add_uint(tree, hfindex, tvb, offset, 8, value_low);
+	}
+
+	return offset + 8;
+}
+
+
 static int
 dissect_rpc_opaque_data(const u_char *pd, int offset, frame_data *fd,
     proto_tree *tree, int hfindex, gboolean string_data,
@@ -565,6 +670,23 @@ dissect_rpc_string(const u_char *pd, int offset, frame_data *fd,
 }
 
 
+int
+dissect_rpc_string_tvb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int hfindex, int offset, char **string_buffer_ret)
+{
+	const guint8 *pd;
+	int compat_offset;
+	int compat_offset_new;
+
+	tvb_compat(tvb, &pd, &compat_offset);
+	compat_offset += offset;
+	
+	compat_offset_new = dissect_rpc_string(pd, compat_offset, pinfo->fd,
+				tree, hfindex, string_buffer_ret);
+	offset += (compat_offset_new - compat_offset);
+	return offset;
+}
+
+
 int
 dissect_rpc_data(const u_char *pd, int offset, frame_data *fd,
     proto_tree *tree, int hfindex)
@@ -576,6 +698,23 @@ dissect_rpc_data(const u_char *pd, int offset, frame_data *fd,
 }
 
 
+int
+dissect_rpc_data_tvb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, int hfindex, int offset)
+{
+	const guint8 *pd;
+	int compat_offset;
+	int compat_offset_new;
+
+	tvb_compat(tvb, &pd, &compat_offset);
+	compat_offset += offset;
+	
+	compat_offset_new = dissect_rpc_data(pd, compat_offset, pinfo->fd,
+				tree, hfindex);
+	offset += (compat_offset_new - compat_offset);
+	return offset;
+}
+
+
 int
 dissect_rpc_list(const u_char *pd, int offset, frame_data *fd,
 	proto_tree *tree, dissect_function_t *rpc_list_dissector)
@@ -599,85 +738,136 @@ dissect_rpc_list(const u_char *pd, int offset, frame_data *fd,
 	return offset;
 }
 
-
-void
-dissect_rpc_auth( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
+static int
+dissect_rpc_authunix_cred(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
-        guint flavor;
-        guint length;
-        guint length_full;
+	guint stamp;
+	guint uid;
+	guint gid;
+	guint gids_count;
+	guint gids_i;
+	guint gids_entry;
+	proto_item *gitem;
+	proto_tree *gtree = NULL;
 
-	/* both checks are made outside */
-	/* if (!BYTES_ARE_IN_FRAME(offset,8)) return; */
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	stamp = EXTRACT_UINT(pd,offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_auth_stamp, NullTVB,
+			offset+0, 4, stamp);
+	offset += 4;
+
+	offset = dissect_rpc_string(pd,offset,fd,
+		tree,hf_rpc_auth_machinename,NULL);
+
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	uid = EXTRACT_UINT(pd,offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_auth_uid, NullTVB,
+			offset+0, 4, uid);
+	offset += 4;
+
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	gid = EXTRACT_UINT(pd,offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_auth_gid, NullTVB,
+			offset+0, 4, gid);
+	offset += 4;
+
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	gids_count = EXTRACT_UINT(pd,offset+0);
+	if (tree) {
+		gitem = proto_tree_add_text(tree, NullTVB, offset, 4+gids_count*4,
+		"Auxiliary GIDs");
+		gtree = proto_item_add_subtree(gitem, ett_rpc_gids);
+	}
+	offset += 4;
+	
+	if (!BYTES_ARE_IN_FRAME(offset,4*gids_count)) return offset;
+	for (gids_i = 0 ; gids_i < gids_count ; gids_i++) {
+		gids_entry = EXTRACT_UINT(pd,offset+0);
+		if (gtree)
+		proto_tree_add_uint(gtree, hf_rpc_auth_gid, NullTVB,
+			offset, 4, gids_entry);
+		offset+=4;
+	}
+	/* how can I NOW change the gitem to print a list with
+		the first 16 gids? */
+
+	return offset;
+}
+
+static int
+dissect_rpc_authgss_cred(const u_char *pd, int offset,
+			 frame_data *fd, proto_tree *tree)
+{
+	guint agc_v;
+	guint agc_proc;
+	guint agc_seq;
+	guint agc_svc;
+
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	agc_v = EXTRACT_UINT(pd, offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_authgss_v,
+				    NullTVB, offset+0, 4, agc_v);
+	offset += 4;
+	
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	agc_proc = EXTRACT_UINT(pd, offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_authgss_proc,
+				    NullTVB, offset+0, 4, agc_proc);
+	offset += 4;
+	
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	agc_seq = EXTRACT_UINT(pd, offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_authgss_seq,
+				    NullTVB, offset+0, 4, agc_seq);
+	offset += 4;
+	
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	agc_svc = EXTRACT_UINT(pd, offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_authgss_svc,
+				    NullTVB, offset+0, 4, agc_svc);
+	offset += 4;
+	
+	offset = dissect_rpc_data(pd,offset,fd,tree,
+				  hf_rpc_authgss_ctx);
+	
+	return offset;
+}
+
+static int
+dissect_rpc_cred( const u_char *pd, int offset, frame_data *fd, proto_tree *tree )
+{
+	guint flavor;
+	guint length;
+
+	proto_item *citem;
+	proto_tree *ctree;
+
+	if (!BYTES_ARE_IN_FRAME(offset,8)) return offset;
 	flavor = EXTRACT_UINT(pd,offset+0);
 	length = EXTRACT_UINT(pd,offset+4);
-	length_full = rpc_roundup(length);
-	/* if (!BYTES_ARE_IN_FRAME(offset+8,full_length)) return; */
+	length = rpc_roundup(length);
+	if (!BYTES_ARE_IN_FRAME(offset+8,length)) return offset;
 
 	if (tree) {
-		proto_tree_add_uint(tree, hf_rpc_auth_flavor, NullTVB, offset+0, 4,
-			flavor);
-		proto_tree_add_uint(tree, hf_rpc_auth_length, NullTVB, offset+4, 4,
-			length);
-	}
+		citem = proto_tree_add_text(tree, NullTVB, offset,
+					    8+length, "Credentials");
+		ctree = proto_item_add_subtree(citem, ett_rpc_cred);
+		proto_tree_add_uint(ctree, hf_rpc_auth_flavor, NullTVB,
+				    offset+0, 4, flavor);
+		proto_tree_add_uint(ctree, hf_rpc_auth_length, NullTVB,
+				    offset+4, 4, length);
 
-	offset += 8;
-
-	switch (flavor) {
-		case AUTH_UNIX: {
-			guint stamp;
-			guint uid;
-			guint gid;
-			guint gids_count;
-			guint gids_i;
-			guint gids_entry;
-			proto_item *gitem;
-			proto_tree *gtree = NULL;
-
-			if (!BYTES_ARE_IN_FRAME(offset,4)) return;
-			stamp = EXTRACT_UINT(pd,offset+0);
-			if (tree)
-				proto_tree_add_uint(tree, hf_rpc_auth_stamp, NullTVB,
-					offset+0, 4, stamp);
-			offset += 4;
-
-			offset = dissect_rpc_string(pd,offset,fd,
-				tree,hf_rpc_auth_machinename,NULL);
-
-			if (!BYTES_ARE_IN_FRAME(offset,4)) return;
-			uid = EXTRACT_UINT(pd,offset+0);
-			if (tree)
-				proto_tree_add_uint(tree, hf_rpc_auth_uid, NullTVB,
-					offset+0, 4, uid);
-			offset += 4;
-
-			if (!BYTES_ARE_IN_FRAME(offset,4)) return;
-			gid = EXTRACT_UINT(pd,offset+0);
-			if (tree)
-				proto_tree_add_uint(tree, hf_rpc_auth_gid, NullTVB,
-					offset+0, 4, gid);
-			offset += 4;
-
-			if (!BYTES_ARE_IN_FRAME(offset,4)) return;
-			gids_count = EXTRACT_UINT(pd,offset+0);
-			if (tree) {
-				gitem = proto_tree_add_text(tree, NullTVB, offset, 4+gids_count*4,
-				"Auxiliary GIDs");
-				gtree = proto_item_add_subtree(gitem, ett_rpc_gids);
-			}
-			offset += 4;
-			if (!BYTES_ARE_IN_FRAME(offset,4*gids_count)) return;
-			for (gids_i = 0 ; gids_i < gids_count ; gids_i++) {
-				gids_entry = EXTRACT_UINT(pd,offset+0);
-				if (gtree)
-				proto_tree_add_uint(gtree, hf_rpc_auth_gid, NullTVB,
-					offset, 4, gids_entry);
-				offset+=4;
-			}
-			/* how can I NOW change the gitem to print a list with
-				the first 16 gids? */
-		}
-		break;
+		switch (flavor) {
+		case AUTH_UNIX:
+			dissect_rpc_authunix_cred(pd, offset+8, fd, ctree);
+			break;
 		/*
 		case AUTH_SHORT:
 
@@ -690,64 +880,147 @@ dissect_rpc_auth( const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 
 		break;
 		*/
+		case RPCSEC_GSS:
+			dissect_rpc_authgss_cred(pd, offset+8, fd, ctree);
+			break;
 		default:
-			if (length_full) {
-				if (tree)
-				proto_tree_add_text(tree, NullTVB,offset,
-				length_full, "opaque data");
-			}
+			if (length)
+				proto_tree_add_text(ctree, NullTVB, offset+8,
+						    length,"opaque data");
+			break;
 	}
-}
-
-int
-dissect_rpc_cred( const u_char *pd, int offset, frame_data *fd, proto_tree *tree )
-{
-	guint length;
-	guint length_full;
-	proto_item *citem;
-	proto_tree *ctree;
-
-	if (!BYTES_ARE_IN_FRAME(offset,8)) return offset;
-	length = EXTRACT_UINT(pd,offset+4);
-	length_full = rpc_roundup(length);
-	if (!BYTES_ARE_IN_FRAME(offset+8,length_full)) return offset;
-
-	if (tree) {
-		citem = proto_tree_add_text(tree, NullTVB, offset, 8+length_full,
-			"Credentials");
-		ctree = proto_item_add_subtree(citem, ett_rpc_cred);
-		dissect_rpc_auth(pd, offset, fd, ctree);
 	}
-	offset += 8 + length_full;
+	offset += 8 + length;
 
 	return offset;
 }
 
-
-int
+static int
 dissect_rpc_verf( const u_char *pd, int offset, frame_data *fd, proto_tree *tree )
 {
-	unsigned int length;
-	unsigned int length_full;
+	guint flavor;
+	guint length;
+	
 	proto_item *vitem;
 	proto_tree *vtree;
 
 	if (!BYTES_ARE_IN_FRAME(offset,8)) return offset;
+	flavor = EXTRACT_UINT(pd,offset+0);
 	length = EXTRACT_UINT(pd,offset+4);
-	length_full = rpc_roundup(length);
-	if (!BYTES_ARE_IN_FRAME(offset+8,length_full)) return offset;
+	length = rpc_roundup(length);
+	if (!BYTES_ARE_IN_FRAME(offset+8,length)) return offset;
 
 	if (tree) {
-		vitem = proto_tree_add_text(tree, NullTVB, offset, 8+length_full,
-			"Verifier");
+		vitem = proto_tree_add_text(tree, NullTVB, offset,
+					    8+length, "Verifier");
 		vtree = proto_item_add_subtree(vitem, ett_rpc_verf);
-		dissect_rpc_auth(pd, offset, fd, vtree);
+		proto_tree_add_uint(vtree, hf_rpc_auth_flavor, NullTVB,
+				    offset+0, 4, flavor);
+
+		switch (flavor) {
+		case AUTH_UNIX:
+			proto_tree_add_uint(vtree, hf_rpc_auth_length, NullTVB,
+					    offset+4, 4, length);
+			dissect_rpc_authunix_cred(pd, offset+8, fd, vtree);
+			break;
+		case RPCSEC_GSS:
+			dissect_rpc_data(pd, offset+4, fd, vtree,
+					 hf_rpc_authgss_checksum);
+			break;
+		default:
+			proto_tree_add_uint(vtree, hf_rpc_auth_length, NullTVB,
+					    offset+4, 4, length);
+			if (length)
+				proto_tree_add_text(vtree, NullTVB, offset+8,
+						    length, "opaque data");
+			break;
+		}
 	}
-	offset += 8 + length_full;
+	offset += 8 + length;
 
 	return offset;
 }
 
+static int
+dissect_rpc_authgss_initarg(const u_char *pd, int offset,
+			    frame_data *fd, proto_tree *tree)
+{
+	offset = dissect_rpc_data(pd, offset, fd, tree, hf_rpc_authgss_token);
+	return offset;
+}
+
+static int
+dissect_rpc_authgss_initres(const u_char *pd, int offset,
+			    frame_data *fd, proto_tree *tree)
+{
+	int major, minor, window;
+	
+	offset = dissect_rpc_data(pd, offset, fd, tree, hf_rpc_authgss_ctx);
+	
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	major = EXTRACT_UINT(pd,offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_authgss_major, NullTVB,
+				    offset+0, 4, major);
+	offset += 4;
+
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	minor = EXTRACT_UINT(pd,offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_authgss_minor, NullTVB,
+				    offset+0, 4, minor);
+	offset += 4;
+
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	window = EXTRACT_UINT(pd,offset+0);
+	if (tree)
+		proto_tree_add_uint(tree, hf_rpc_authgss_window, NullTVB,
+				    offset+0, 4, window);
+	offset += 4;
+
+	offset = dissect_rpc_data(pd, offset, fd, tree, hf_rpc_authgss_token);
+
+	return offset;
+}
+
+static int
+dissect_rpc_authgss_integ_data(const u_char *pd, int offset,
+			       frame_data *fd, proto_tree *tree,
+			       dissect_function_t *dissect_function)
+{
+	guint32 length, seq;
+	
+	proto_item *gitem;
+	proto_tree *gtree;
+
+	if (!BYTES_ARE_IN_FRAME(offset, 8)) return offset;
+	length = EXTRACT_UINT(pd, offset+0);
+	length = rpc_roundup(length);
+	seq = EXTRACT_UINT(pd,offset+4);
+
+	if (tree) {
+		gitem = proto_tree_add_text(tree, NullTVB, offset,
+					    4+length, "GSS Data");
+		gtree = proto_item_add_subtree(gitem, ett_rpc_gss_data);
+		proto_tree_add_uint(gtree, hf_rpc_authgss_data_length,
+				    NullTVB, offset+0, 4, length);
+		proto_tree_add_uint(gtree, hf_rpc_authgss_seq,
+				    NullTVB, offset+4, 4, seq);
+		if (dissect_function != NULL)
+			offset = dissect_function(pd, offset, fd, gtree);
+	}
+	offset += 8 + length;
+	offset = dissect_rpc_data(pd, offset, fd, tree, hf_rpc_authgss_checksum);
+	return offset;
+}
+
+static int
+dissect_rpc_authgss_priv_data(const u_char *pd, int offset,
+			 frame_data *fd, proto_tree *tree)
+{
+	offset = dissect_rpc_data(pd, offset, fd, tree, hf_rpc_authgss_data);
+	return offset;
+}
 
 gboolean
 dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
@@ -763,6 +1036,9 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	unsigned int prog = 0;
 	unsigned int vers = 0;
 	unsigned int proc = 0;
+	unsigned int flavor = 0;
+	unsigned int gss_proc = 0;
+	unsigned int gss_svc = 0;
 	int	proto = 0;
 	int	ett = 0;
 
@@ -798,6 +1074,9 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 
 	dissect_function_t *dissect_function = NULL;
 
+	if (!proto_is_protocol_enabled(proto_rpc))
+	  return FALSE;
+
 	/* TCP uses record marking */
 	use_rm = (pi.ptype == PT_TCP);
 
@@ -948,12 +1227,19 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			return TRUE;
 		proc = EXTRACT_UINT(pd,offset+12);
 
+		/* Check for RPCSEC_GSS */
+		if (proc == 0 && BYTES_ARE_IN_FRAME(offset+16,28)) {
+			flavor = EXTRACT_UINT(pd, offset+16);
+			if (flavor == RPCSEC_GSS) {
+				gss_proc = EXTRACT_UINT(pd, offset+28);
+				gss_svc = EXTRACT_UINT(pd, offset+34);
+			}
+		}
 		key.prog = prog;
 		key.vers = vers;
 		key.proc = proc;
 
-		value = g_hash_table_lookup(rpc_procs,&key);
-		if (value != NULL) {
+		if ((value = g_hash_table_lookup(rpc_procs,&key)) != NULL) {
 			dissect_function = value->dissect_call;
 			procname = value->name;
 		}
@@ -964,6 +1250,7 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			sprintf(procname_static, "proc-%u", proc);
 			procname = procname_static;
 		}
+		
 		if (rpc_tree) {
 			proto_tree_add_uint_format(rpc_tree,
 				hf_rpc_procedure, NullTVB, offset+12, 4, proc,
@@ -998,24 +1285,38 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		rpc_call_msg.conversation = conversation;
 
 		/* look up the request */
-		if (rpc_call_lookup(&rpc_call_msg)) {
-			/* duplicate request */
-			if (check_col(fd, COL_INFO)) {
-				col_append_fstr(fd, COL_INFO, " dup XID 0x%x", xid);
-				if (rpc_tree) {
-					proto_tree_add_uint_hidden(rpc_tree,
-						hf_rpc_dup, NullTVB, 0,0, xid);
-					proto_tree_add_uint_hidden(rpc_tree,
-						hf_rpc_call_dup, NullTVB, 0,0, xid);
+		if ((rpc_call = rpc_call_lookup(&rpc_call_msg)) != NULL) {
+			/* We've seen a request with this XID, with the same
+			   source and destination, before - but was it
+			   *this* request? */
+			if (fd->num != rpc_call->req_num) {
+				/* No, so it's a duplicate request.
+				   Mark it as such. */
+				if (check_col(fd, COL_INFO)) {
+					col_append_fstr(fd, COL_INFO, " dup XID 0x%x", xid);
+					if (rpc_tree) {
+						proto_tree_add_uint_hidden(rpc_tree,
+							hf_rpc_dup, NullTVB, 0,0, xid);
+						proto_tree_add_uint_hidden(rpc_tree,
+							hf_rpc_call_dup, NullTVB, 0,0, xid);
+					}
 				}
 			}
 		}
 		else {
-			/* prepare the value data */
-			rpc_call_msg.replies = 0;
+			/* Prepare the value data.
+			   "req_num" and "rep_num" are frame numbers;
+			   frame numbers are 1-origin, so we use 0
+			   to mean "we don't yet know in which frame
+			   the reply for this call appears". */
+			rpc_call_msg.req_num = fd->num;
+			rpc_call_msg.rep_num = 0;
 			rpc_call_msg.prog = prog;
 			rpc_call_msg.vers = vers;
 			rpc_call_msg.proc = proc;
+			rpc_call_msg.flavor = flavor;
+			rpc_call_msg.gss_proc = gss_proc;
+			rpc_call_msg.gss_svc = gss_svc;
 			rpc_call_msg.proc_info = value;
 			/* store it */
 			rpc_call_insert(&rpc_call_msg);
@@ -1027,7 +1328,6 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		offset = dissect_rpc_verf(pd, offset, fd, rpc_tree);
 
 		/* go to the next dissector */
-		/* goto dissect_rpc_prog; */
 
 	} /* end of RPC call */
 	else if (msg_type == RPC_REPLY)
@@ -1037,6 +1337,15 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		prog = rpc_call->prog;
 		vers = rpc_call->vers;
 		proc = rpc_call->proc;
+		flavor = rpc_call->flavor;
+		gss_proc = rpc_call->gss_proc;
+		gss_svc = rpc_call->gss_svc;
+
+		/* Indicate the frame to which this is a reply. */
+		proto_tree_add_text(rpc_tree, NullTVB, 0, 0,
+		    "This is a reply to a request starting in frame %u",
+		    rpc_call->req_num);
+
 		if (rpc_call->proc_info != NULL) {
 			dissect_function = rpc_call->proc_info->dissect_reply;
 			if (rpc_call->proc_info->name != NULL) {
@@ -1052,7 +1361,6 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			sprintf(procname_static, "proc-%u", proc);
 			procname = procname_static;
 		}
-		rpc_call->replies++;
 
 		rpc_prog_key.prog = prog;
 		if ((rpc_prog = g_hash_table_lookup(rpc_progs,&rpc_prog_key)) == NULL) {
@@ -1092,14 +1400,25 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 				"Procedure: %s (%u)", procname, proc);
 		}
 
-		if (rpc_call->replies>1) {
-			if (check_col(fd, COL_INFO)) {
-				col_append_fstr(fd, COL_INFO, " dup XID 0x%x", xid);
-				if (rpc_tree) {
-					proto_tree_add_uint_hidden(rpc_tree,
-						hf_rpc_dup, NullTVB, 0,0, xid);
-					proto_tree_add_uint_hidden(rpc_tree,
-						hf_rpc_reply_dup, NullTVB, 0,0, xid);
+		if (rpc_call->rep_num == 0) {
+			/* We have not yet seen a reply to that call, so
+			   this must be the first reply; remember its
+			   frame number. */
+			rpc_call->rep_num = fd->num;
+		} else {
+			/* We have seen a reply to this call - but was it
+			   *this* reply? */
+			if (rpc_call->rep_num != fd->num) {
+				/* No, so it's a duplicate reply.
+				   Mark it as such. */
+				if (check_col(fd, COL_INFO)) {
+					col_append_fstr(fd, COL_INFO, " dup XID 0x%x", xid);
+					if (rpc_tree) {
+						proto_tree_add_uint_hidden(rpc_tree,
+							hf_rpc_dup, NullTVB, 0,0, xid);
+						proto_tree_add_uint_hidden(rpc_tree,
+							hf_rpc_reply_dup, NullTVB, 0,0, xid);
+					}
 				}
 			}
 		}
@@ -1125,8 +1444,7 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			offset += 4;
 			switch (accept_state) {
 				case SUCCESS:
-					/* now goto the lower protocol */
-					goto dissect_rpc_prog;
+					/* go to the next dissector */
 				break;
 				case PROG_MISMATCH:
 					if (!BYTES_ARE_IN_FRAME(offset,8))
@@ -1186,9 +1504,6 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		} 
 	} /* end of RPC reply */
 
-dissect_rpc_prog:
-	/* I know, goto is evil but it works as it is. */
-
 	/* now we know, that RPC was shorter */
 	if (rpc_item) {
 		proto_item_set_len(rpc_item, offset - offset_old);
@@ -1210,14 +1525,46 @@ dissect_rpc_prog:
 		}
 	}
 
-	/* call a specific dissection */
-	if (dissect_function != NULL) {
+	/* RPCSEC_GSS processing. */
+	if (flavor == RPCSEC_GSS) {
+		switch (gss_proc) {
+		case RPCSEC_GSS_INIT:
+		case RPCSEC_GSS_CONTINUE_INIT:
+			if (msg_type == RPC_CALL) {
+				offset = dissect_rpc_authgss_initarg(pd, offset, fd, ptree);
+			}
+			else {
+				offset = dissect_rpc_authgss_initres(pd, offset, fd, ptree);
+			}
+			break;
+		case RPCSEC_GSS_DATA:
+			if (gss_svc == RPCSEC_GSS_SVC_NONE) {
+				if (dissect_function != NULL && 
+					proto_is_protocol_enabled(proto))
+					offset = dissect_function(pd, offset, fd, ptree);
+			}
+			else if (gss_svc == RPCSEC_GSS_SVC_INTEGRITY) {
+				offset = dissect_rpc_authgss_integ_data(pd, offset, fd, ptree, 
+				(proto_is_protocol_enabled(proto) ? 
+				dissect_function : NULL));
+			}
+			else if (gss_svc == RPCSEC_GSS_SVC_PRIVACY) {
+				offset = dissect_rpc_authgss_priv_data(pd, offset, fd, ptree);
+			}
+			break;
+		default:
+			dissect_function = NULL;
+			break;
+		}
+	}
+	else if (dissect_function != NULL &&
+		proto_is_protocol_enabled(proto)) {
 		offset = dissect_function(pd, offset, fd, ptree);
 	}
 
 	/* dissect any remaining bytes (incomplete dissection) as pure data in
 	   the ptree */
-	dissect_data(pd, offset, fd, ptree);
+	old_dissect_data(pd, offset, fd, ptree);
 
 	return TRUE;
 }
@@ -1301,6 +1648,42 @@ proto_register_rpc(void)
 		{ &hf_rpc_auth_gid, {
 			"GID", "rpc.auth.gid", FT_UINT32, BASE_DEC,
 			NULL, 0, "GID" }},
+		{ &hf_rpc_authgss_v, {
+			"GSS Version", "rpc.authgss.version", FT_UINT32,
+			BASE_DEC, NULL, 0, "GSS Version" }},
+		{ &hf_rpc_authgss_proc, {
+			"GSS Procedure", "rpc.authgss.procedure", FT_UINT32,
+			BASE_DEC, VALS(rpc_authgss_proc), 0, "GSS Procedure" }},
+		{ &hf_rpc_authgss_seq, {
+			"GSS Sequence Number", "rpc.authgss.seqnum", FT_UINT32,
+			BASE_DEC, NULL, 0, "GSS Sequence Number" }},
+		{ &hf_rpc_authgss_svc, {
+			"GSS Service", "rpc.authgss.service", FT_UINT32,
+			BASE_DEC, VALS(rpc_authgss_svc), 0, "GSS Service" }},
+		{ &hf_rpc_authgss_ctx, {
+			"GSS Context", "rpc.authgss.context", FT_BYTES,
+			BASE_HEX, NULL, 0, "GSS Context" }},
+		{ &hf_rpc_authgss_major, {
+			"GSS Major Status", "rpc.authgss.major", FT_UINT32,
+			BASE_DEC, NULL, 0, "GSS Major Status" }},
+		{ &hf_rpc_authgss_minor, {
+			"GSS Minor Status", "rpc.authgss.minor", FT_UINT32,
+			BASE_DEC, NULL, 0, "GSS Minor Status" }},
+		{ &hf_rpc_authgss_window, {
+			"GSS Sequence Window", "rpc.authgss.window", FT_UINT32,
+			BASE_DEC, NULL, 0, "GSS Sequence Window" }},
+		{ &hf_rpc_authgss_token, {
+			"GSS Token", "rpc.authgss.token", FT_BYTES,
+			BASE_HEX, NULL, 0, "GSS Token" }},
+		{ &hf_rpc_authgss_data_length, {
+			"Length", "rpc.authgss.data.length", FT_UINT32,
+			BASE_DEC, NULL, 0, "Length" }},
+		{ &hf_rpc_authgss_data, {
+			"GSS Data", "rpc.authgss.data", FT_BYTES,
+			BASE_HEX, NULL, 0, "GSS Data" }},
+		{ &hf_rpc_authgss_checksum, {
+			"GSS Checksum", "rpc.authgss.checksum", FT_BYTES,
+			BASE_HEX, NULL, 0, "GSS Checksum" }},
 		{ &hf_rpc_auth_machinename, {
 			"Machine Name", "rpc.auth.machinename", FT_STRING, 
 			BASE_DEC, NULL, 0, "Machine Name" }},
@@ -1344,3 +1727,13 @@ proto_register_rpc(void)
 	rpc_progs = g_hash_table_new(rpc_prog_hash, rpc_prog_equal);
 	rpc_procs = g_hash_table_new(rpc_proc_hash, rpc_proc_equal);
 }
+
+
+void
+proto_reg_handoff_rpc(void)
+{
+	old_heur_dissector_add("tcp", dissect_rpc);
+	old_heur_dissector_add("udp", dissect_rpc);
+}
+
+
