@@ -1,5 +1,28 @@
-/* packet-rpc.h (c) 1999 Uwe Girlich */
-/* $Id: packet-rpc.h,v 1.14 2000/03/09 12:05:32 girlich Exp $ */
+/* packet-rpc.h
+ *
+ * $Id: packet-rpc.h,v 1.19 2000/08/24 22:58:56 guy Exp $
+ *
+ * (c) 1999 Uwe Girlich
+ *
+ * Ethereal - Network traffic analyzer
+ * By Gerald Combs <gerald@zing.org>
+ * Copyright 1998 Gerald Combs
+ *
+ * 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ */
 
 #ifndef __PACKET_RPC_H__
 #define __PACKET_RPC_H__
@@ -17,15 +40,11 @@
 #define AUTH_UNIX 1
 #define AUTH_SHORT 2
 #define AUTH_DES 3
+#define RPCSEC_GSS 6
 
 #define MSG_ACCEPTED 0
 #define MSG_DENIED 1
 
-#define AUTH_NULL 0
-#define AUTH_UNIX 1
-#define AUTH_SHORT 2
-#define AUTH_DES 3
-
 #define SUCCESS 0
 #define PROG_UNAVAIL 1
 #define PROG_MISMATCH 2
@@ -40,6 +59,17 @@
 #define AUTH_BADVERF 3
 #define AUTH_REJECTEDVERF 4
 #define AUTH_TOOWEAK 5
+#define RPCSEC_GSSCREDPROB 13
+#define RPCSEC_GSSCTXPROB 14
+
+#define RPCSEC_GSS_DATA 0
+#define RPCSEC_GSS_INIT 1
+#define RPCSEC_GSS_CONTINUE_INIT 2
+#define RPCSEC_GSS_DESTROY 3
+
+#define RPCSEC_GSS_SVC_NONE 1
+#define RPCSEC_GSS_SVC_INTEGRITY 2
+#define RPCSEC_GSS_SVC_PRIVACY 3
 
 typedef int (dissect_function_t)(const u_char* pd, int offset, frame_data* fd, proto_tree* tree);
 
@@ -72,23 +102,8 @@ typedef struct _rpc_prog_info_value {
 	char* progname;
 } rpc_prog_info_value;
 
-typedef struct _rpc_call_info {
-	guint32	xid;
-	conversation_t *conversation;
-	guint32	replies;
-	guint32	prog;
-	guint32	vers;
-	guint32	proc;
-	rpc_proc_info_value*	proc_info;
-} rpc_call_info;
-
-#define RPC_CALL_TABLE_LENGTH 1000
-
 extern const value_string rpc_auth_flavor[];
 
-extern void rpc_call_insert(rpc_call_info *call);
-extern rpc_call_info* rpc_call_lookup(rpc_call_info *call);
-
 extern void rpc_init_proc_table(guint prog, guint vers, const vsff *proc_table);
 extern void rpc_init_prog(int proto, guint32 prog, int ett);
 extern char *rpc_prog_name(guint32 prog);
@@ -100,16 +115,26 @@ extern unsigned int rpc_roundup(unsigned int a);
 gboolean dissect_rpc(const u_char *, int, frame_data *, proto_tree *);
 extern int dissect_rpc_bool(const u_char *pd, int offset, frame_data *fd,
 	proto_tree *tree, int hfindex);
+extern int dissect_rpc_bool_tvb(tvbuff_t *tvb, packet_info *pinfo,
+	proto_tree *tree, int hfindex, int offset);
 extern int dissect_rpc_string(const u_char *pd, int offset, frame_data *fd,
 	proto_tree *tree, int hfindex, char **string_buffer_ret);
+extern int dissect_rpc_string_tvb(tvbuff_t *tvb, packet_info *pinfo,
+	proto_tree *tree, int hfindex, int offset, char **string_buffer_ret);
 extern int dissect_rpc_data(const u_char *pd, int offset, frame_data *fd,
 	proto_tree *tree, int hfindex);
+extern int dissect_rpc_data_tvb(tvbuff_t *tvb, packet_info *pinfo,
+	proto_tree *tree, int hfindex, int offset);
 extern int dissect_rpc_list(const u_char *pd, int offset, frame_data *fd,
 	proto_tree *tree, dissect_function_t *rpc_list_dissector);
 extern int dissect_rpc_uint32(const u_char *pd, int offset, frame_data *fd,
-	proto_tree *tree, char* name, char* type);
+	proto_tree *tree, char* name);
+extern int dissect_rpc_uint32_tvb(tvbuff_t *tvb, packet_info *pinfo,
+	proto_tree *tree, int hfindex, int offset);
 extern int dissect_rpc_uint64(const u_char *pd, int offset, frame_data *fd,
-	proto_tree *tree, char* name, char* type);
+	proto_tree *tree, char* name);
+extern int dissect_rpc_uint64_tvb(tvbuff_t *tvb, packet_info *pinfo,
+	proto_tree *tree, int hfindex, int offset);
 
 
 #endif /* packet-rpc.h */
