@@ -3,7 +3,7 @@
  *
  * (c) Copyright Ashok Narayanan <ashokn@cisco.com>
  *
- * $Id: packet-rsvp.c,v 1.24 2000/06/02 13:24:12 gram Exp $
+ * $Id: packet-rsvp.c,v 1.27 2000/08/13 14:07:58 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -61,16 +61,12 @@
 # include <netinet/in.h>
 #endif
 
+#include <glib.h>
+
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
-#include <glib.h>
 #include "packet.h"
 #include "packet-ip.h"
 #include "packet-ipv6.h"
@@ -851,6 +847,8 @@ dissect_rsvp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
     struct e_in6_addr *ip6a;
     guint32 ip_addr;
 
+    OLD_CHECK_DISPLAY_AS_DATA(proto_rsvp, pd, offset, fd, tree);
+
     hdr = (rsvp_header *)&pd[offset];
     packet_type = match_strval(hdr->message_type, message_type_vals);
     if (check_col(fd, COL_PROTOCOL))
@@ -2037,5 +2035,5 @@ proto_register_rsvp(void)
 void
 proto_reg_handoff_rsvp(void)
 {
-	dissector_add("ip.proto", IP_PROTO_RSVP, dissect_rsvp);
+	old_dissector_add("ip.proto", IP_PROTO_RSVP, dissect_rsvp);
 }
