@@ -4,7 +4,7 @@
  * Jason Lango <jal@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-rtsp.c,v 1.14 2000/05/31 05:07:37 guy Exp $
+ * $Id: packet-rtsp.c,v 1.18 2000/09/11 16:16:03 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -43,6 +43,7 @@
 #include "packet-rtp.h"
 #include "packet-rtcp.h"
 #include "conversation.h"
+#include "strutil.h"
 
 static int proto_rtsp = -1;
 static gint ett_rtsp = -1;
@@ -125,14 +126,14 @@ rtsp_create_conversation(const u_char *trans_begin, const u_char *trans_end)
 
 	conv = conversation_new(&pi.src, &pi.dst, PT_UDP, s_data_port,
 		c_data_port, 0);
-	conv->dissector = dissect_rtp;
+	old_conversation_set_dissector(conv, dissect_rtp);
 
 	if (!c_mon_port || !s_mon_port)
 		return;
 
 	conv = conversation_new(&pi.src, &pi.dst, PT_UDP, s_mon_port,
 		c_mon_port, 0);
-	conv->dissector = dissect_rtcp;
+	old_conversation_set_dissector(conv, dissect_rtcp);
 }
 
 static void dissect_rtsp(const u_char *pd, int offset, frame_data *fd,
@@ -147,6 +148,8 @@ static void dissect_rtsp(const u_char *pd, int offset, frame_data *fd,
 	int		is_sdp = 0;
 	int		end_offset;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_rtsp, pd, offset, fd, tree);
+
 	data = &pd[offset];
 	dataend = data + END_OF_FRAME;
 	end_offset = offset + END_OF_FRAME;
@@ -386,5 +389,5 @@ proto_register_rtsp(void)
 void
 proto_reg_handoff_rtsp(void)
 {
-	dissector_add("tcp.port", TCP_PORT_RTSP, dissect_rtsp);
+	old_dissector_add("tcp.port", TCP_PORT_RTSP, dissect_rtsp);
 }
