@@ -4,7 +4,7 @@
  * Based on routines from tcpdump patches by
  *   Ken Hornstein <kenh@cmf.nrl.navy.mil>
  *
- * $Id: packet-rx.c,v 1.12 2000/05/31 05:07:38 guy Exp $
+ * $Id: packet-rx.c,v 1.14 2000/08/13 14:08:44 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -103,6 +103,8 @@ dissect_rx(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	struct rx_header *rxh;
 	int reply;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_rx, pd, offset, fd, tree);
+
 	rxh = (struct rx_header *) &pd[offset];
 
 	/* get at least a full packet structure */
@@ -250,6 +252,6 @@ proto_reg_handoff_rx(void)
 	/* Ports in the range UDP_PORT_RX_LOW to UDP_PORT_RX_HIGH
 	   are all used for various AFS services. */
 	for (port = UDP_PORT_RX_LOW; port <= UDP_PORT_RX_HIGH; port++)
-		dissector_add("udp.port", port, dissect_rx);
-	dissector_add("udp.port", UDP_PORT_RX_AFS_BACKUPS, dissect_rx);
+		old_dissector_add("udp.port", port, dissect_rx);
+	old_dissector_add("udp.port", UDP_PORT_RX_AFS_BACKUPS, dissect_rx);
 }
