@@ -1,10 +1,10 @@
-/* packet-tftp.c
+/* packet-sap.c
  * Routines for sap packet dissection
  * <draft-ietf-mmusic-sap-v2-03.txt>
  *
  * Heikki Vatiainen <hessu@cs.tut.fi>
  *
- * $Id: packet-sap.c,v 1.8 2000/05/31 05:07:39 guy Exp $
+ * $Id: packet-sap.c,v 1.11 2000/08/25 13:05:32 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -137,6 +137,8 @@ dissect_sap(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
         proto_item *si, *sif;
         proto_tree *sap_tree, *sap_flags_tree;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_sap, pd, offset, fd, tree);
+
         is_ipv6 = pd[offset]&MCAST_SAP_BIT_A;
         is_del = pd[offset]&MCAST_SAP_BIT_T;
         is_enc = pd[offset]&MCAST_SAP_BIT_E;
@@ -323,5 +325,5 @@ void proto_register_sap(void)
 void
 proto_reg_handoff_sap(void)
 {
-  dissector_add("udp.port", UDP_PORT_SAP, dissect_sap);
+  old_dissector_add("udp.port", UDP_PORT_SAP, dissect_sap);
 }
