@@ -4,7 +4,7 @@
  * Jason Lango <jal@netapp.com>
  * Liberally copied from packet-http.c, by Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-sdp.c,v 1.8 2000/05/31 05:07:40 guy Exp $
+ * $Id: packet-sdp.c,v 1.10 2000/09/11 16:16:03 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -39,6 +39,7 @@
 
 #include <glib.h>
 #include "packet.h"
+#include "strutil.h"
 
 static int proto_sdp = -1;
 
@@ -58,6 +59,8 @@ void dissect_sdp(const u_char *pd, int offset, frame_data *fd,
 	int		valuelen;
 	const char	*typename;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_sdp, pd, offset, fd, tree);
+
 	data = &pd[offset];
 	dataend = data + END_OF_FRAME;
 
