@@ -2,7 +2,7 @@
  * Routines for SNA
  * Gilbert Ramirez <gram@xiexie.org>
  *
- * $Id: packet-sna.c,v 1.16 2000/05/31 05:07:44 guy Exp $
+ * $Id: packet-sna.c,v 1.18 2000/08/13 14:08:58 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -310,6 +310,8 @@ dissect_sna(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	guint8		th_fid;
 	int		sna_header_len = 0, th_header_len = 0;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_sna, pd, offset, fd, tree);
+
 	/* SNA data should be printed in EBCDIC, not ASCII */
 	fd->flags.encoding = CHAR_EBCDIC;
 
@@ -364,7 +366,7 @@ dissect_sna(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 			th_header_len = dissect_fidf(pd, offset, fd, th_tree);
 			break;
 		default:
-			dissect_data(pd, offset+1, fd, tree);
+			old_dissect_data(pd, offset+1, fd, tree);
 	}
 
 	sna_header_len += th_header_len;
@@ -397,7 +399,7 @@ dissect_sna(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	}
 
 	if (IS_DATA_IN_FRAME(offset+1)) {
-		dissect_data(pd, offset, fd, tree);
+		old_dissect_data(pd, offset, fd, tree);
 	}
 }
 
@@ -1206,5 +1208,5 @@ proto_register_sna(void)
 void
 proto_reg_handoff_sna(void)
 {
-	dissector_add("llc.dsap", SAP_SNA_PATHCTRL, dissect_sna);
+	old_dissector_add("llc.dsap", SAP_SNA_PATHCTRL, dissect_sna);
 }
