@@ -2,7 +2,7 @@
  * Routines for SNMP (simple network management protocol)
  * D.Jorand (c) 1998
  *
- * $Id: packet-snmp.c,v 1.36 2000/05/31 05:07:46 guy Exp $
+ * $Id: packet-snmp.c,v 1.48 2000/09/08 07:14:01 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -45,7 +45,11 @@
 # include <netinet/in.h>
 #endif
 
-#define MAX_STRING_LEN 1024	/* TBC */
+#define MAX_STRING_LEN 2048	/* TBC */
+
+#ifdef linux
+#include <dlfcn.h>
+#endif
 
 #include <glib.h>
 
@@ -67,12 +71,12 @@
 #  include <ucd-snmp/mib.h>
 
    /*
-    * Sigh.  UCD SNMP 4.1.1 makes "snmp_set_full_objid()" a macro
-    * that calls "ds_set_boolean()" with the first two arguments
-    * being DS_LIBRARY_ID and DS_LIB_PRINT_FULL_OID; this means that,
+    * Sigh.  UCD SNMP 4.1.1 makes "snmp_set_suffix_only()" a macro
+    * that calls "ds_set_int()" with the first two arguments
+    * being DS_LIBRARY_ID and DS_LIB_PRINT_SUFFIX_ONLY; this means that,
     * when building with 4.1.1, we need to arrange that
     * <ucd-snmp/default_store.h> is included, to define those two values
-    * and to declare "ds_set_boolean()".
+    * and to declare "ds_int()".
     *
     * However:
     *
@@ -83,10 +87,10 @@
     *	   as that includes <ucd-snmp/snmp.h>, and that defines a whole
     *	   bunch of values that we also define ourselves.
     *
-    * So we only include it if "snmp_set_full_objid" is defined as
+    * So we only include it if "snmp_set_suffix_only" is defined as
     * a macro.
     */
-#  ifdef snmp_set_full_objid
+#  ifdef snmp_set_suffix_only
 #   include <ucd-snmp/default_store.h>
 #  endif
 
@@ -188,6 +192,13 @@
 
 #include "packet-snmp.h"
 
+/* Null string of type "guchar[]". */
+static const guchar nullstring[] = "";
+
+/* Take a pointer that may be null and return a pointer that's not null
+   by turning null pointers into pointers to the above null string. */
+#define	SAFE_STRING(s)	(((s) != NULL) ? (s) : nullstring)
+
 static int proto_snmp = -1;
 static int proto_smux = -1;
 
@@ -537,7 +548,7 @@ dissect_snmp_parse_error(const u_char *pd, int offset, frame_data *fd,
 		    "ERROR: Couldn't parse %s: %s", field_name, errstr);
 	}
 
-	dissect_data(pd, offset, fd, tree);
+	old_dissect_data(pd, offset, fd, tree);
 }
 
 static void
@@ -547,28 +558,73 @@ dissect_snmp_error(const u_char *pd, int offset, frame_data *fd,
 	if (check_col(fd, COL_INFO))
 		col_add_str(fd, COL_INFO, message);
 
-	dissect_data(pd, offset, fd, tree);
+	old_dissect_data(pd, offset, fd, tree);
 }
 
-static void
-format_oid(gchar *buf, subid_t *oid, guint oid_length)
+static gchar *
+format_oid(subid_t *oid, guint oid_length)
 {
-	int i;
-	int len;
+	char *result;
+	int result_len;
+	int len, i;
+	char *buf;
 
+	result_len = oid_length * 22;
+	result = g_malloc(result_len + 1);
+	buf = result;
 	len = sprintf(buf, "%lu", (unsigned long)oid[0]);
 	buf += len;
 	for (i = 1; i < oid_length;i++) {
 		len = sprintf(buf, ".%lu", (unsigned long)oid[i]);
 		buf += len;
 	}
+	return result;
 }
 
 #ifdef HAVE_SPRINT_VALUE
-static void
-format_value(gchar *buf, struct variable_list *variable, subid_t *variable_oid,
+static gchar *
+format_var(struct variable_list *variable, subid_t *variable_oid,
     guint variable_oid_length, gushort vb_type, guint vb_length)
 {
+	gchar *buf;
+
+	switch (vb_type) {
+
+	case SNMP_INTEGER:
+	case SNMP_COUNTER:
+	case SNMP_GAUGE:
+	case SNMP_TIMETICKS:
+		/* We don't know how long this will be, but let's guess it
+		   fits within 128 characters; that should be enough for an
+		   integral value plus some sort of type indication. */
+		buf = g_malloc(128);
+		break;
+
+	case SNMP_OCTETSTR:
+	case SNMP_IPADDR:
+	case SNMP_OPAQUE:
+	case SNMP_NSAP:
+	case SNMP_BITSTR:
+	case SNMP_COUNTER64:
+		/* We don't know how long this will be, but let's guess it
+		   fits within 128 characters plus 4 characters per octet. */
+		buf = g_malloc(128 + 4*vb_length);
+		break;
+
+	case SNMP_OBJECTID:
+		/* We don't know how long this will be, but let's guess it
+		   fits within 128 characters plus 32 characters per subid
+		   (10 digits plus period, or a subid name). */
+		buf = g_malloc(1024 + 32*vb_length);
+		break;
+
+	default:
+		/* Should not happen. */
+		g_assert_not_reached();
+		buf = NULL;
+		break;
+	}
+
 	variable->next_variable = NULL;
 	variable->name = variable_oid;
 	variable->name_length = variable_oid_length;
@@ -608,6 +664,7 @@ format_value(gchar *buf, struct variable_list *variable, subid_t *variable_oid,
 
 	case SNMP_OBJECTID:
 		variable->type = VALTYPE_OBJECTID;
+		vb_length *= sizeof (subid_t);	/* XXX - necessary? */
 		break;
 
 	case SNMP_BITSTR:
@@ -619,7 +676,9 @@ format_value(gchar *buf, struct variable_list *variable, subid_t *variable_oid,
 		break;
 	}
 	variable->val_len = vb_length;
+
 	sprint_value(buf, variable_oid, variable_oid_length, variable);
+	return buf;
 }
 #endif
 
@@ -644,7 +703,7 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 	subid_t *vb_oid;
 	guint vb_oid_length;
 
-	gchar vb_display_string[MAX_STRING_LEN]; /* TBC */
+	gchar *vb_display_string;
 
 #ifdef HAVE_SPRINT_VALUE
 	struct variable_list variable;
@@ -693,11 +752,12 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 #elif defined(HAVE_SNMP_SNMP_H)
 			variable.val.integer = &vb_integer_value;
 #endif
-			format_value(vb_display_string, &variable,
+			vb_display_string = format_var(&variable,
 			    variable_oid, variable_oid_length, vb_type,
 			    vb_length);
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 			    "Value: %s", vb_display_string);
+			g_free(vb_display_string);
 #else
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 			    "Value: %s: %d (%#x)", vb_type_name,
@@ -722,11 +782,12 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 #elif defined(HAVE_SNMP_SNMP_H)
 			variable.val.integer = &vb_uinteger_value;
 #endif
-			format_value(vb_display_string, &variable,
+			vb_display_string = format_var(&variable,
 			    variable_oid, variable_oid_length, vb_type,
 			    vb_length);
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 			    "Value: %s", vb_display_string);
+			g_free(vb_display_string);
 #else
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 			    "Value: %s: %u (%#x)", vb_type_name,
@@ -749,11 +810,12 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 		if (snmp_tree) {
 #ifdef HAVE_SPRINT_VALUE
 			variable.val.string = vb_octet_string;
-			format_value(vb_display_string, &variable,
+			vb_display_string = format_var(&variable,
 			    variable_oid, variable_oid_length, vb_type,
 			    vb_length);
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 			    "Value: %s", vb_display_string);
+			g_free(vb_display_string);
 #else
 			/*
 			 * If some characters are not printable, display
@@ -770,6 +832,7 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 				 * character, before we got to the end
 				 * of the string.
 				 */
+				vb_display_string = g_malloc(4*vb_length);
 				buf = &vb_display_string[0];
 				len = sprintf(buf, "%03u", vb_octet_string[0]);
 				buf += len;
@@ -781,6 +844,7 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 				proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 				    "Value: %s: %s", vb_type_name,
 				    vb_display_string);
+				g_free(vb_display_string);
 			} else {
 				proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 				    "Value: %s: %.*s", vb_type_name,
@@ -811,16 +875,17 @@ snmp_variable_decode(proto_tree *snmp_tree, subid_t *variable_oid,
 		if (snmp_tree) {
 #ifdef HAVE_SPRINT_VALUE
 			variable.val.objid = vb_oid;
-			format_value(vb_display_string, &variable,
+			vb_display_string = format_var(&variable,
 			    variable_oid, variable_oid_length, vb_type,
-			    vb_length*sizeof (subid_t));
+			    vb_length);
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 			    "Value: %s", vb_display_string);
 #else
-			format_oid(vb_display_string, vb_oid, vb_oid_length);
+			vb_display_string = format_oid(vb_oid, vb_oid_length);
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
 			    "Value: %s: %s", vb_type_name, vb_display_string);
 #endif
+			g_free(vb_display_string);
 		}
 		g_free(vb_oid);
 		break;
@@ -886,7 +951,7 @@ dissect_common_pdu(const u_char *pd, int offset, frame_data *fd,
 	guint timestamp;
 	guint timestamp_length;
 
-	gchar oid_string[MAX_STRING_LEN]; /* TBC */
+	gchar *oid_string;
 
 	guint variable_bindings_length;
 
@@ -989,9 +1054,10 @@ dissect_common_pdu(const u_char *pd, int offset, frame_data *fd,
 			return;
 		}
 		if (tree) {
-			format_oid(oid_string, enterprise, enterprise_length);
+			oid_string = format_oid(enterprise, enterprise_length);
 			proto_tree_add_text(tree, NullTVB, offset, length,
 			    "Enterprise: %s", oid_string);
+			g_free(oid_string);
 		}
 		g_free(enterprise);
 		offset += length;
@@ -1027,8 +1093,7 @@ dissect_common_pdu(const u_char *pd, int offset, frame_data *fd,
 		}
 		length = asn1.pointer - start;
 		if (tree) {
-			proto_tree_add_text(tree, NullTVB, offset,
-			     agent_address_length,
+			proto_tree_add_text(tree, NullTVB, offset, length,
 			    "Agent address: %s", ip_to_str(agent_address));
 		}
 		g_free(agent_address);
@@ -1129,7 +1194,7 @@ dissect_common_pdu(const u_char *pd, int offset, frame_data *fd,
 		sequence_length += length;
 
 		if (tree) {
-			format_oid(oid_string, variable_oid,
+			oid_string = format_oid(variable_oid,
 			    variable_oid_length);
 			
 #if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
@@ -1144,6 +1209,7 @@ dissect_common_pdu(const u_char *pd, int offset, frame_data *fd,
 			    "Object identifier %d: %s", vb_index,
 			    oid_string);
 #endif
+			g_free(oid_string);
 		}
 		offset += sequence_length;
 		variable_bindings_length -= sequence_length;
@@ -1261,7 +1327,8 @@ dissect_snmp_pdu(const u_char *pd, int offset, frame_data *fd,
 		}
 		if (tree) {
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
-			    "Community: %.*s", community_length, community);
+			    "Community: %.*s", community_length,
+			    SAFE_STRING(community));
 		}
 		g_free(community);
 		offset += length;
@@ -1349,23 +1416,6 @@ dissect_snmp_pdu(const u_char *pd, int offset, frame_data *fd,
 		}
 		offset += length;
 		switch(msgsec) {
-		case SNMP_SEC_V1:
-		case SNMP_SEC_V2C:
-			ret = asn1_octet_string_decode (&asn1, 
-			    &secparm, &secparm_length, &length);
-			if (ret != ASN1_ERR_NOERROR) {
-				dissect_snmp_parse_error(pd, offset, fd, tree, 
-				    "Message Security Parameters", ret);
-				return;
-			}
-			if (snmp_tree) {
-				proto_tree_add_text(snmp_tree, NullTVB, offset,
-				    length, "Message Security Parameters: %.*s",
-				    secparm_length, secparm);
-			}
-			g_free(secparm);
-			offset += length;
-			break;
 		case SNMP_SEC_USM:
 			start = asn1.pointer;
 			ret = asn1_header_decode (&asn1, &cls, &con, &tag,
@@ -1446,7 +1496,8 @@ dissect_snmp_pdu(const u_char *pd, int offset, frame_data *fd,
 			if (secur_tree) {
 				proto_tree_add_text(secur_tree, NullTVB, offset,
 				    length, "User Name: %.*s", 
-				    username_length, username);
+				    username_length,
+				    SAFE_STRING(username));
 			}
 			g_free(username);
 			offset += length;
@@ -1543,7 +1594,8 @@ dissect_snmp_pdu(const u_char *pd, int offset, frame_data *fd,
 		}
 		if (snmp_tree) {
 			proto_tree_add_text(snmp_tree, NullTVB, offset, length,
-			    "Context Name: %s", cname);
+			    "Context Name: %.*s", cname_length,
+			    SAFE_STRING(cname));
 		}
 		g_free(cname);
 		offset += length;
@@ -1598,7 +1650,7 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 	subid_t *regid;
 	guint regid_length;
 
-	gchar oid_string[MAX_STRING_LEN]; /* TBC */
+	gchar *oid_string;
 
 	proto_tree *smux_tree = NULL;
 	proto_item *item = NULL;
@@ -1628,6 +1680,7 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 		    "PDU type", ret);
 		return;
 	}
+
 	/* Dissect SMUX here */
 	if (cls == ASN1_APL && con == ASN1_CON && pdu_type == SMUX_MSG_OPEN) {
 		pdu_type_string = val_to_str(pdu_type, smux_types,
@@ -1659,9 +1712,10 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 			return;
 		}
 		if (tree) {
-			format_oid(oid_string, regid, regid_length);
+			oid_string = format_oid(regid, regid_length);
 			proto_tree_add_text(smux_tree, NullTVB, offset, length,
 			    "Registration: %s", oid_string);
+			g_free(oid_string);
 		}
 		g_free(regid);
 		offset += length;
@@ -1676,7 +1730,7 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 		if (tree) {
 			proto_tree_add_text(smux_tree, NullTVB, offset, length,
 			    "Application: %.*s", application_length,
-			     application);
+			     SAFE_STRING(application));
 		}
 		g_free(application);
 		offset += length;
@@ -1690,7 +1744,8 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 		}
 		if (tree) {
 			proto_tree_add_text(smux_tree, NullTVB, offset, length,
-			    "Password: %.*s", password_length, password);
+			    "Password: %.*s", password_length,
+			    SAFE_STRING(password));
 		}
 		g_free(password);
 		offset += length;
@@ -1740,9 +1795,10 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 			return;
 		}
 		if (tree) {
-			format_oid(oid_string, regid, regid_length);
+			oid_string = format_oid(regid, regid_length);
 			proto_tree_add_text(smux_tree, NullTVB, offset, length,
 			    "Registration: %s", oid_string);
+			g_free(oid_string);
 		}
 		g_free(regid);
 		offset += length;
@@ -1837,18 +1893,26 @@ dissect_smux_pdu(const u_char *pd, int offset, frame_data *fd,
 static void
 dissect_snmp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) 
 {
+	OLD_CHECK_DISPLAY_AS_DATA(proto_snmp, pd, offset, fd, tree);
 	dissect_snmp_pdu(pd, offset, fd, tree, "SNMP", proto_snmp, ett_snmp);
 }
 
 static void
 dissect_smux(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) 
 {
+	OLD_CHECK_DISPLAY_AS_DATA(proto_smux, pd, offset, fd, tree);
 	dissect_smux_pdu(pd, offset, fd, tree, proto_smux, ett_smux);
 }
 
 void
 proto_register_snmp(void)
 {
+#ifdef linux
+	void *libsnmp_handle;
+	int (*snmp_set_suffix_only_p)(int);
+	int (*ds_set_int_p)(int, int, int);
+#endif
+
         static hf_register_info hf[] = {
 		{ &hf_snmpv3_flags,
 		{ "SNMPv3 Flags", "snmpv3.flags", FT_UINT8, BASE_HEX, NULL,
@@ -1862,10 +1926,6 @@ proto_register_snmp(void)
 		{ &hf_snmpv3_flags_report,
 		{ "Reportable", "snmpv3.flags.report", FT_BOOLEAN, 8,
 		    TFS(&flags_set_truth), TH_REPORT, "" }},
-/*
-                { &hf_variable,
-                { "Name",           "snmp.abbreviation", TYPE, VALS_POINTER }},
-*/
         };
 	static gint *ett[] = {
 		&ett_snmp,
@@ -1879,9 +1939,114 @@ proto_register_snmp(void)
 	/* UCD or CMU SNMP */
 	init_mib();
 #ifdef HAVE_UCD_SNMP_SNMP_H
-	snmp_set_full_objid(TRUE);
-#endif
-#endif
+#ifdef linux
+	/* As per the comment near the beginning of the file, UCD SNMP 4.1.1
+	   changed "snmp_set_suffix_only()" from a function to a macro,
+	   removing "snmp_set_suffix_only()" from the library; this means
+	   that binaries that call "snmp_set_suffix_only()" and
+	   that are linked against shared libraries from earlier versions
+	   of the UCD SNMP library won't run with shared libraries from
+	   4.1.1.
+
+	   This is a problem on Red Hat Linux, as pre-6.2 releases
+	   came with pre-4.1.1 UCD SNMP, while 6.2 comes the 4.1.1.
+	   Versions of Ethereal built on pre-6.2 releases don't run
+	   on 6.2, and the current Ethereal RPMs are built on pre-6.2
+	   releases, causing problems when users running 6.2 download
+	   them and try to use them.
+
+	   Building the releases on 6.2 isn't necessarily the answer,
+	   as "snmp_set_suffix_only()" expands to a call to "ds_set_int()"
+	   with a second argument not supported by at least some pre-4.1.1
+	   versions of the library - it appears that the 4.0.1 library,
+	   at least, checks for invalid arguments and returns an error
+	   rather than stomping random memory, but that means that you
+	   won't get get OIDs displayed as module-name::sub-OID.
+
+	   So we use a trick similar to one I've seen mentioned as
+	   used in Windows applications to let you build binaries
+	   that run on many different versions of Windows 9x and
+	   Windows NT, that use features present on later versions
+	   if run on those later versions, but that avoid calling,
+	   when run on older versions, routines not present on those
+	   older versions.
+
+	   I.e., we load "libsnmp.so.0" with "dlopen()", and call
+	   "dlsym()" to try to find "snmp_set_suffix_only()"; if we
+	   don't find it, we make the appropriate call to
+	   "ds_set_int()" instead.  (We load "libsnmp.so.0" rather
+	   than "libsnmp.so" because, at least on RH 6.2, "libsnmp.so"
+	   exists only if you've loaded the libsnmp development package,
+	   which makes "libsnmp.so" a symlink to "libsnmp.so.0"; we
+	   don't want to force users to install it or to make said
+	   symlink by hand.)
+
+	   We do this only on Linux, for now, as we've only seen the
+	   problem on Red Hat; it may show up on other OSes that bundle
+	   UCD SNMP, or on OSes where it's not bundled but for which
+	   binary packages are built that link against a shared version
+	   of the UCD SNMP library.  If we run into one of those, we
+	   can do this under those OSes as well, *if* "dlopen()" makes
+	   the run-time linker use the same search rules as it uses when
+	   loading libraries with which the application is linked.
+
+	   (Perhaps we could use the GLib wrappers for run-time linking,
+	   *if* they're thin enough; however, as this code is currently
+	   used only on Linux, we don't worry about that for now.) */
+
+	libsnmp_handle = dlopen("libsnmp.so.0", RTLD_LAZY|RTLD_GLOBAL);
+	if (libsnmp_handle == NULL) {
+		/* We didn't find "libsnmp.so.0".
+
+		   This could mean that there is no SNMP shared library
+		   on this system, in which case we were linked statically,
+		   in which case whatever call the following line of code
+		   makes will presumably work, as we have the routine it
+		   calls wired into our binary.  (If we were linked
+		   dynamically with "-lsnmp", we would have failed to
+		   start.)
+
+		   It could also mean that there is an SNMP shared library
+		   on this system, but it's called something other than
+		   "libsnmp.so.0"; so far, we've seen the problem we're
+		   working around only on systems where the SNMP shared
+		   library is called "libsnmp.so.0", so we assume for now
+		   that systems with shared SNMP libraries named something
+		   other than "libsnmp.so.0" have an SNMP library that's
+		   not 4.1.1. */
+		snmp_set_suffix_only(2);
+	} else {
+		/* OK, we have it loaded.  Do we have
+		   "snmp_set_suffix_only()"? */
+		snmp_set_suffix_only_p = dlsym(libsnmp_handle,
+		    "snmp_set_suffix_only");
+		if (snmp_set_suffix_only_p != NULL) {
+			/* Yes - call it. */
+			(*snmp_set_suffix_only_p)(2);
+		} else {
+			/* No; do we have "ds_set_int()"? */
+			ds_set_int_p = dlsym(libsnmp_handle, "ds_set_int");
+			if (ds_set_int_p != NULL) {
+				/* Yes - cal it with DS_LIBRARY_ID,
+				   DS_LIB_PRINT_SUFFIX_ONLY, and 2 as
+				   arguments.
+
+				   We do *not* use DS_LIBRARY_ID or
+				   DS_LIB_PRINT_SUFFIX_ONLY by name, so that
+				   we don't require that Ethereal be built
+				   with versions of UCD SNMP that include
+				   that value; instead, we use their values
+				   in UCD SNMP 4.1.1, which are 0 and 4,
+				   respectively. */
+				(*ds_set_int_p)(0, 4, 2);
+			}
+		}
+	}
+#else /* linux */
+	snmp_set_suffix_only(2);
+#endif /* linux */
+#endif /* HAVE_UCD_SNMP_SNMP_H */
+#endif /* defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H) */
         proto_snmp = proto_register_protocol("Simple Network Management Protocol", "snmp");
         proto_smux = proto_register_protocol("SNMP Multiplex Protocol", "smux");
         proto_register_field_array(proto_snmp, hf, array_length(hf));
@@ -1891,10 +2056,10 @@ proto_register_snmp(void)
 void
 proto_reg_handoff_snmp(void)
 {
-	dissector_add("udp.port", UDP_PORT_SNMP, dissect_snmp);
-	dissector_add("udp.port", UDP_PORT_SNMP_TRAP, dissect_snmp);
-	dissector_add("tcp.port", TCP_PORT_SMUX, dissect_smux);
-	dissector_add("ethertype", ETHERTYPE_SNMP, dissect_snmp);
-	dissector_add("ipx.socket", IPX_SOCKET_SNMP_AGENT, dissect_snmp);
-	dissector_add("ipx.socket", IPX_SOCKET_SNMP_SINK, dissect_snmp);
+	old_dissector_add("udp.port", UDP_PORT_SNMP, dissect_snmp);
+	old_dissector_add("udp.port", UDP_PORT_SNMP_TRAP, dissect_snmp);
+	old_dissector_add("tcp.port", TCP_PORT_SMUX, dissect_smux);
+	old_dissector_add("ethertype", ETHERTYPE_SNMP, dissect_snmp);
+	old_dissector_add("ipx.socket", IPX_SOCKET_SNMP_AGENT, dissect_snmp);
+	old_dissector_add("ipx.socket", IPX_SOCKET_SNMP_SINK, dissect_snmp);
 }
