@@ -2,7 +2,7 @@
  * Routines for socks versions 4 &5  packet dissection
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com>
  *
- * $Id: packet-socks.c,v 1.5 2000/05/31 05:07:48 guy Exp $
+ * $Id: packet-socks.c,v 1.12 2000/09/11 16:16:07 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -63,6 +63,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <glib.h>
+
+#ifdef NEED_SNPRINTF_H
+# include "snprintf.h"
+#endif
+
 #include "packet.h"
 #include "resolv.h"
 #include "globals.h"
@@ -71,19 +76,10 @@
 
 #include "packet-tcp.h"
 #include "packet-udp.h"
-
-#ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
-# include "snprintf.h"
-#endif
+#include "strutil.h"
 
 
-
-#define CHECK_PACKET_LENGTH(X) if ((offset+X) > fd->cap_len){  \
+#define CHECK_PACKET_LENGTH(X) if (!BYTES_ARE_IN_FRAME(offset, X)){  \
         proto_tree_add_text(tree, NullTVB, offset, 0, "*** FRAME TOO SHORT ***"); \
         return; }
 
@@ -233,7 +229,7 @@ static int display_string( const u_char *pd, int offset, frame_data *fd,
 	char temp[ 256];
 	int length = GBYTE( pd, offset);
 
-	if ((offset + 8) > fd->cap_len){  
+	if (!BYTES_ARE_IN_FRAME(offset, 8)){  
        		proto_tree_add_text(tree, NullTVB, offset, 0, "*** FRAME TOO SHORT ***");
 	        return 0;
 	}
@@ -305,7 +301,7 @@ static int display_address( const u_char *pd, int offset,
 	++offset;
 
 	if ( a_type == 1){		/* IPv4 address */
-	   	if ( (offset + 4) > fd->cap_len) 
+	   	if (!BYTES_ARE_IN_FRAME(offset, 4)) 
        			proto_tree_add_text(tree, NullTVB, offset, 0, "*** FRAME TOO SHORT ***");
 
 		proto_tree_add_ipv4( tree, hf_socks_ip_dst, NullTVB, offset,
@@ -318,7 +314,7 @@ static int display_address( const u_char *pd, int offset,
 			"Remote name");
 	}
 	else if ( a_type == 4){	/* IPv6 address */
-		if ((offset + 16) > fd->cap_len) 
+		if (!BYTES_ARE_IN_FRAME(offset, 16)) 
        			proto_tree_add_text(tree, NullTVB, offset, 0, "*** FRAME TOO SHORT ***");
 
 		proto_tree_add_ipv6( tree, hf_socks_ip6_dst, NullTVB, offset,
@@ -438,7 +434,7 @@ void new_udp_conversation( socks_hash_entry_t *hash_info){
 			
 	g_assert( conversation);
 	
-	conversation->dissector = socks_udp_dissector;
+	old_conversation_set_dissector(conversation, socks_udp_dissector);
 }
 
 
@@ -739,7 +735,7 @@ static void state_machine_v5( socks_hash_entry_t *hash_info, const u_char *pd,
 		hash_info->state = Connecting;	/* change state		*/
 		hash_info->connect_row = get_packet_ptr;	
 
-		if (( offset+ 1) > fd->cap_len){ 
+		if (!BYTES_ARE_IN_FRAME(offset, 1)){ 
 			hash_info->state = Done;	/* change state		*/
 	        	return; 
 	        }
@@ -775,7 +771,7 @@ static void state_machine_v5( socks_hash_entry_t *hash_info, const u_char *pd,
 
 		guint temp;
 
-		if (( offset+ 1) > fd->cap_len){ 
+		if (!BYTES_ARE_IN_FRAME(offset, 1)){ 
 			hash_info->state = Done;	/* change state		*/
 	        	return; 
 	        }
@@ -793,7 +789,7 @@ static void state_machine_v5( socks_hash_entry_t *hash_info, const u_char *pd,
 
 		offset = get_address_v5( pd, offset, hash_info);
 
-		if (( offset+ 1) > fd->cap_len){ 
+		if (!BYTES_ARE_IN_FRAME(offset, 1)){ 
 			hash_info->state = Done;
 	        	return; 
 	        }
@@ -827,7 +823,7 @@ static void state_machine_v5( socks_hash_entry_t *hash_info, const u_char *pd,
 			offset = get_address_v5( pd, offset, hash_info);
 
 	/* save server udp port and create upd conversation */
-			if (( offset+ 2) > fd->cap_len){ 
+			if (!BYTES_ARE_IN_FRAME(offset, 2)){ 
 				hash_info->state = Done;
 		        	return; 
 		        }
@@ -958,6 +954,7 @@ dissect_socks(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	socks_hash_entry_t *hash_info;
 	conversation_t *conversation;
 	
+	OLD_CHECK_DISPLAY_AS_DATA(proto_socks, pd, offset, fd, tree);
 
 	conversation = find_conversation( &pi.src, &pi.dst, pi.ptype,
 		pi.srcport, pi.destport);
@@ -1150,5 +1147,5 @@ proto_reg_handoff_socks(void) {
 
 	/* dissector install routine */ 
  
- 	dissector_add("tcp.port", TCP_PORT_SOCKS, dissect_socks);
+ 	old_dissector_add("tcp.port", TCP_PORT_SOCKS, dissect_socks);
 }
