@@ -2,7 +2,7 @@
  * Routines for SSCOP (Q.2110, Q.SAAL) frame disassembly
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: packet-sscop.c,v 1.8 2000/05/29 08:57:40 guy Exp $
+ * $Id: packet-sscop.c,v 1.10 2000/08/13 14:09:03 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -144,6 +144,8 @@ dissect_sscop(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   int pad_len;
   tvbuff_t *next_tvb;
 
+  CHECK_DISPLAY_AS_DATA(proto_sscop, tvb, pinfo, tree);
+
   pinfo->current_proto = "SSCOP";
 
   reported_length = tvb_reported_length(tvb);	/* frame length */
@@ -308,7 +310,7 @@ dissect_sscop(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
       if (pdu_type == SSCOP_SD)
         dissect_q2931(next_tvb, pinfo, tree);
       else
-        dissect_data_tvb(next_tvb, pinfo, tree);
+        dissect_data(next_tvb, pinfo, tree);
     }
     break;
   }
