@@ -1,7 +1,7 @@
 /* packet-tacacs.c
  * Routines for cisco tacacs/tacplus/AAA packet dissection
  *
- * $Id: packet-tacacs.c,v 1.5 2000/05/31 05:07:49 guy Exp $
+ * $Id: packet-tacacs.c,v 1.7 2000/08/13 14:09:05 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -57,6 +57,8 @@ dissect_tacacs(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
 	proto_tree      *tacacs_tree, *ti;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_tacacs, pd, offset, fd, tree);
+
 	if (check_col(fd, COL_PROTOCOL))
 		col_add_str(fd, COL_PROTOCOL, "TACACS");
 
@@ -157,6 +159,6 @@ proto_register_tacacs(void)
 void
 proto_reg_handoff_tacacs(void)
 {
-	dissector_add("udp.port", UDP_PORT_TACACS, dissect_tacacs);
-	dissector_add("tcp.port", TCP_PORT_TACACS, dissect_tacplus);
+	old_dissector_add("udp.port", UDP_PORT_TACACS, dissect_tacacs);
+	old_dissector_add("tcp.port", TCP_PORT_TACACS, dissect_tacplus);
 }
