@@ -1,7 +1,7 @@
 /* packet-tcp.c
  * Routines for TCP packet disassembly
  *
- * $Id: packet-tcp.c,v 1.75 2000/05/31 05:07:49 guy Exp $
+ * $Id: packet-tcp.c,v 1.82 2000/09/11 16:16:10 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -37,24 +37,23 @@
 
 #include <stdio.h>
 #include <glib.h>
-#include "globals.h"
-#include "resolv.h"
-#include "follow.h"
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
+#include "globals.h"
+#include "resolv.h"
+#include "follow.h"
+#include "prefs.h"
 #include "plugins.h"
 #include "packet-tcp.h"
-
 #include "packet-ip.h"
-#include "packet-rpc.h"
+#include "conversation.h"
+#include "strutil.h"
+
+/* Place TCP summary in proto tree */
+gboolean g_tcp_summary_in_tree = TRUE;
 
 extern FILE* data_out_file;
 
@@ -358,25 +357,15 @@ static const true_false_string flags_set_truth = {
 /* can call to it, ie. socks	*/
 
 void
-decode_tcp_ports( const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
-	int src_port, int dst_port) {
-  dissector_t sub_dissector;
-
+decode_tcp_ports(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
+	int src_port, int dst_port)
+{
 /* determine if this packet is part of a conversation and call dissector */
 /* for the conversation if available */
 
-  sub_dissector = find_conversation_dissector( &pi.src, &pi.dst, PT_TCP,
-		src_port, dst_port);
-  if (sub_dissector){
-	(sub_dissector)(pd, offset, fd, tree);
+  if (old_try_conversation_dissector(&pi.src, &pi.dst, PT_TCP,
+		src_port, dst_port, pd, offset, fd, tree))
 	return;
-  }
-
-  /* ONC RPC.  We can't base this on anything in the TCP header; we have
-     to look at the payload.  If "dissect_rpc()" returns TRUE, it was
-     an RPC packet, otherwise it's some other type of packet. */
-  if (dissect_rpc(pd, offset, fd, tree))
-    return;
 
   /* try to apply the plugins */
 #ifdef HAVE_PLUGINS
@@ -397,16 +386,16 @@ decode_tcp_ports( const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 #endif
 
   /* do lookup with the subdissector table */
-  if (dissector_try_port(subdissector_table, src_port, pd, offset, fd, tree) ||
-      dissector_try_port(subdissector_table, dst_port, pd, offset, fd, tree))
+  if (old_dissector_try_port(subdissector_table, src_port, pd, offset, fd, tree) ||
+      old_dissector_try_port(subdissector_table, dst_port, pd, offset, fd, tree))
     return;
 
   /* do lookup with the heuristic subdissector table */
-  if (dissector_try_heuristic(heur_subdissector_list, pd, offset, fd, tree))
+  if (old_dissector_try_heuristic(heur_subdissector_list, pd, offset, fd, tree))
     return;
 
   /* Oh, well, we don't know this; dissect it as data. */
-  dissect_data(pd, offset, fd, tree);
+  old_dissect_data(pd, offset, fd, tree);
 }
 
 
@@ -423,6 +412,8 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   guint      optlen;
   guint      packet_max = pi.len;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_tcp, pd, offset, fd, tree);
+
   /* To do: Check for {cap len,pkt len} < struct len */
   /* Avoids alignment problems on many architectures. */
   memcpy(&th, &pd[offset], sizeof(e_tcphdr));
@@ -474,7 +465,12 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   }
   
   if (tree) {
-    ti = proto_tree_add_protocol_format(tree, proto_tcp, NullTVB, offset, hlen, "Transmission Control Protocol, Src Port: %s (%u), Dst Port: %s (%u), Seq: %u, Ack: %u", get_tcp_port(th.th_sport), th.th_sport, get_tcp_port(th.th_dport), th.th_dport, th.th_seq, th.th_ack);
+    if (g_tcp_summary_in_tree) {
+	    ti = proto_tree_add_protocol_format(tree, proto_tcp, NullTVB, offset, hlen, "Transmission Control Protocol, Src Port: %s (%u), Dst Port: %s (%u), Seq: %u, Ack: %u", get_tcp_port(th.th_sport), th.th_sport, get_tcp_port(th.th_dport), th.th_dport, th.th_seq, th.th_ack);
+    }
+    else {
+	    ti = proto_tree_add_item(tree, proto_tcp, NullTVB, offset, hlen, FALSE);
+    }
     tcp_tree = proto_item_add_subtree(ti, ett_tcp);
     proto_tree_add_uint_format(tcp_tree, hf_tcp_srcport, NullTVB, offset, 2, th.th_sport,
 	"Source port: %s (%u)", get_tcp_port(th.th_sport), th.th_sport);
@@ -526,8 +522,29 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   
   /* Check the packet length to see if there's more data
      (it could be an ACK-only packet) */
-  if (packet_max > offset) 
-    decode_tcp_ports( pd, offset, fd, tree, th.th_sport, th.th_dport);
+  if (packet_max > offset) {
+    if (th.th_flags & TH_RST) {
+      /*
+       * RFC1122 says:
+       *
+       *	4.2.2.12  RST Segment: RFC-793 Section 3.4
+       *
+       *	  A TCP SHOULD allow a received RST segment to include data.
+       *
+       *	  DISCUSSION
+       * 	       It has been suggested that a RST segment could contain
+       * 	       ASCII text that encoded and explained the cause of the
+       *	       RST.  No standard has yet been established for such
+       *	       data.
+       *
+       * so for segments with RST we just display the data as text.
+       */
+      proto_tree_add_text(tcp_tree, NullTVB, offset, END_OF_FRAME,
+			    "Reset cause: %s",
+			    format_text(&pd[offset], END_OF_FRAME));
+    } else
+      decode_tcp_ports( pd, offset, fd, tree, th.th_sport, th.th_dport);
+  }
  
   if( data_out_file ) {
     reassemble_tcp( th.th_seq,		/* sequence number */
@@ -617,18 +634,26 @@ proto_register_tcp(void)
 		&ett_tcp_options,
 		&ett_tcp_option_sack,
 	};
+	module_t *tcp_module;
 
 	proto_tcp = proto_register_protocol ("Transmission Control Protocol", "tcp");
 	proto_register_field_array(proto_tcp, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 
-/* subdissector code */
+	/* subdissector code */
 	subdissector_table = register_dissector_table("tcp.port");
 	register_heur_dissector_list("tcp", &heur_subdissector_list);
+
+	/* Register a configuration preferences */
+	tcp_module = prefs_register_module("tcp", "TCP", NULL);
+	prefs_register_bool_preference(tcp_module, "tcp_summary_in_tree",
+	    "Show TCP summary in protocol tree",
+"Whether the TCP summary line should be shown in the protocol tree",
+	    &g_tcp_summary_in_tree);
 }
 
 void
 proto_reg_handoff_tcp(void)
 {
-	dissector_add("ip.proto", IP_PROTO_TCP, dissect_tcp);
+	old_dissector_add("ip.proto", IP_PROTO_TCP, dissect_tcp);
 }
