@@ -2,7 +2,7 @@
  * Routines for telnet packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-telnet.c,v 1.13 2000/05/31 05:07:50 guy Exp $
+ * $Id: packet-telnet.c,v 1.16 2000/09/11 16:16:11 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -42,6 +42,7 @@
 #include <string.h>
 #include <glib.h>
 #include "packet.h"
+#include "strutil.h"
 
 static int proto_telnet = -1;
 
@@ -297,6 +298,8 @@ dissect_telnet(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
         proto_tree      *telnet_tree, *ti;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_telnet, pd, offset, fd, tree);
+
 	if (check_col(fd, COL_PROTOCOL))
 		col_add_str(fd, COL_PROTOCOL, "TELNET");
 
@@ -373,5 +376,5 @@ proto_register_telnet(void)
 void
 proto_reg_handoff_telnet(void)
 {
-	dissector_add("tcp.port", TCP_PORT_TELNET, dissect_telnet);
+	old_dissector_add("tcp.port", TCP_PORT_TELNET, dissect_telnet);
 }
