@@ -1,6 +1,6 @@
 /* packet-tftp.h
  *
- * $Id: packet-tftp.h,v 1.1 2000/02/15 21:03:18 gram Exp $
+ * $Id: packet-tftp.h,v 1.2 2000/08/11 13:33:58 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -22,5 +22,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#ifndef __PACKET_TFTP_H__
+#define __PACKET_TFTP_H__
 
 void dissect_tftp(const u_char *, int, frame_data *, proto_tree *);
+
+#endif
