@@ -1,7 +1,7 @@
 /* packet-udp.c
  * Routines for UDP packet disassembly
  *
- * $Id: packet-udp.c,v 1.72 2000/05/31 05:07:52 guy Exp $
+ * $Id: packet-udp.c,v 1.75 2000/08/13 14:09:07 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -48,7 +48,7 @@
 #include "packet-udp.h"
 
 #include "packet-ip.h"
-#include "packet-rpc.h"
+#include "conversation.h"
 #include "packet-tftp.h"
 #include "packet-vines.h"
 
@@ -76,31 +76,22 @@ typedef struct _e_udphdr {
 #define UDP_PORT_VINES	573
 
 static dissector_table_t udp_dissector_table;
+static heur_dissector_list_t heur_subdissector_list;
 
 /* Determine if there is a sub-dissector and call it.  This has been */
 /* separated into a stand alone routine to other protocol dissectors */
 /* can call to it, ie. socks	*/
 
 void
-decode_udp_ports( const u_char *pd, int offset, frame_data *fd,
-	proto_tree *tree, int uh_sport, int uh_dport){
-  dissector_t sub_dissector;
-
+decode_udp_ports(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
+	int uh_sport, int uh_dport)
+{
 /* determine if this packet is part of a conversation and call dissector */
 /* for the conversation if available */
 
-  sub_dissector = find_conversation_dissector( &pi.src, &pi.dst, PT_UDP,
-		uh_sport, uh_dport);
-  if (sub_dissector){
-	(sub_dissector)(pd, offset, fd, tree);
+  if (old_try_conversation_dissector(&pi.src, &pi.dst, PT_UDP,
+		uh_sport, uh_dport, pd, offset, fd, tree))
 	return;
-  }
-
-  /* ONC RPC.  We can't base this on anything in the UDP header; we have
-     to look at the payload.  If "dissect_rpc()" returns TRUE, it was
-     an RPC packet, otherwise it's some other type of packet. */
-  if (dissect_rpc(pd, offset, fd, tree))
-    return;
 
   /* try to apply the plugins */
 #ifdef HAVE_PLUGINS
@@ -121,8 +112,12 @@ decode_udp_ports( const u_char *pd, int offset, frame_data *fd,
 #endif
 
   /* do lookup with the subdissector table */
-  if (dissector_try_port(udp_dissector_table, uh_sport, pd, offset, fd, tree) ||
-      dissector_try_port(udp_dissector_table, uh_dport, pd, offset, fd, tree))
+  if (old_dissector_try_port(udp_dissector_table, uh_sport, pd, offset, fd, tree) ||
+      old_dissector_try_port(udp_dissector_table, uh_dport, pd, offset, fd, tree))
+    return;
+
+  /* do lookup with the heuristic subdissector table */
+  if (old_dissector_try_heuristic(heur_subdissector_list, pd, offset, fd, tree))
     return;
 
   /* XXX - we should do these with the subdissector table as well. */
@@ -132,10 +127,10 @@ decode_udp_ports( const u_char *pd, int offset, frame_data *fd,
     dissect_vines_frp(pd, offset, fd, tree);
   } else if (PORT_IS(UDP_PORT_TFTP)) {
     /* This is the first point of call, but it adds a dynamic call */
-    dissector_add("udp.port", MAX(uh_sport, uh_dport), dissect_tftp);  /* Add to table */
+    old_dissector_add("udp.port", MAX(uh_sport, uh_dport), dissect_tftp);  /* Add to table */
     dissect_tftp(pd, offset, fd, tree);
   } else
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
 }
 
 
@@ -146,8 +141,10 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   proto_tree *udp_tree;
   proto_item *ti;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_udp, pd, offset, fd, tree);
+
   if (!BYTES_ARE_IN_FRAME(offset, sizeof(e_udphdr))) {
-    dissect_data(pd, offset, fd, tree);
+    old_dissect_data(pd, offset, fd, tree);
     return;
   }
 
@@ -227,10 +224,11 @@ proto_register_udp(void)
 
 /* subdissector code */
 	udp_dissector_table = register_dissector_table("udp.port");
+	register_heur_dissector_list("udp", &heur_subdissector_list);
 }
 
 void
 proto_reg_handoff_udp(void)
 {
-	dissector_add("ip.proto", IP_PROTO_UDP, dissect_udp);
+	old_dissector_add("ip.proto", IP_PROTO_UDP, dissect_udp);
 }
