@@ -1,7 +1,7 @@
 /* packet-vlan.c
  * Routines for VLAN 802.1Q ethernet header disassembly
  *
- * $Id: packet-vlan.c,v 1.16 2000/05/31 05:07:53 guy Exp $
+ * $Id: packet-vlan.c,v 1.19 2000/08/13 14:09:06 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -74,6 +74,8 @@ dissect_vlan(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   guint16 tci,encap_proto;
   tvbuff_t *next_tvb;
 
+  OLD_CHECK_DISPLAY_AS_DATA(proto_vlan, pd, offset, fd, tree);
+
   if (!BYTES_ARE_IN_FRAME(offset, 2*sizeof(guint16))) {
     return;
   }
@@ -98,11 +100,11 @@ dissect_vlan(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     proto_tree_add_uint(vlan_tree, hf_vlan_id, NullTVB, offset, 2, tci);
   }
 
-  next_tvb = tvb_new_subset(pi.compat_top_tvb, offset+4, -1, -1); /* XXX - should TRY() like dissect_eth() */
+  next_tvb = tvb_create_from_top(offset+4); /* XXX - should TRY() like dissect_eth() */
   if ( encap_proto <= IEEE_802_3_MAX_LEN) {
     if ( pd[offset+4] == 0xff && pd[offset+5] == 0xff ) {
-      dissect_ipx(pd,offset+4,fd,tree);
-	} else {
+      dissect_ipx(next_tvb, &pi, tree);
+    } else {
       dissect_llc(next_tvb, &pi, tree);
     }
   } else {
@@ -139,5 +141,5 @@ proto_register_vlan(void)
 void
 proto_reg_handoff_vlan(void)
 {
-	dissector_add("ethertype", ETHERTYPE_VLAN, dissect_vlan);
+	old_dissector_add("ethertype", ETHERTYPE_VLAN, dissect_vlan);
 }
