@@ -2,7 +2,7 @@
  * Routines for Web Cache Coordination Protocol dissection
  * Jerry Talkington <jerryt@netapp.com>
  *
- * $Id: packet-wccp.c,v 1.8 2000/05/31 05:07:54 guy Exp $
+ * $Id: packet-wccp.c,v 1.10 2000/08/13 14:09:09 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -102,6 +102,8 @@ dissect_wccp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	guint32 cache_count;
 	int i;
 
+	OLD_CHECK_DISPLAY_AS_DATA(proto_wccp, pd, offset, fd, tree);
+
 	if(check_col(fd, COL_PROTOCOL)) {
 		col_add_str(fd, COL_PROTOCOL, "WCCP");
 	}
@@ -195,7 +197,7 @@ dissect_wccp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			proto_tree_add_uint(wccp_tree, hf_wccp_version, NullTVB,
 			    offset, 4, wccp_version);
 			offset += 4;
-			dissect_data(pd, offset, fd, wccp_tree);
+			old_dissect_data(pd, offset, fd, wccp_tree);
 			break;
 		}
 	}
@@ -341,5 +343,5 @@ proto_register_wccp(void)
 void
 proto_reg_handoff_wccp(void)
 {
-	dissector_add("udp.port", UDP_PORT_WCCP, dissect_wccp);
+	old_dissector_add("udp.port", UDP_PORT_WCCP, dissect_wccp);
 }
