@@ -2,7 +2,7 @@
  * Routines for x25 packet disassembly
  * Olivier Abad <oabad@cybercable.fr>
  *
- * $Id: packet-x25.c,v 1.32 2000/05/31 05:07:54 guy Exp $
+ * $Id: packet-x25.c,v 1.35 2000/08/13 14:09:13 deniel Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -1412,6 +1412,12 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     modulo = ((bytes0_1 & 0x2000) ? 128 : 8);
     vc     = (int)(bytes0_1 & 0x0FFF);
 
+    if (modulo == 8) {
+      CHECK_DISPLAY_AS_DATA(proto_x25, tvb, pinfo, tree);
+    } else {
+      CHECK_DISPLAY_AS_DATA(proto_ex25, tvb, pinfo, tree);
+    }
+
     if (bytes0_1 & 0x8000) toa = TRUE;
     else toa = FALSE;
 
@@ -1542,7 +1548,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		    case PRT_ID_ISO_8073:
 			/* ISO 8073 COTP */
 			x25_hash_add_proto_start(vc, pinfo->fd->abs_secs,
-					 pinfo->fd->abs_usecs, dissect_cotp);
+					 pinfo->fd->abs_usecs, dissect_ositp);
 			break;
 
 		    default:
@@ -1927,7 +1933,7 @@ dissect_x25(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    dissect_ip(next_pd, next_offset, pinfo->fd, tree);
 	}
 	else {
-	    dissect_data(next_pd, next_offset, pinfo->fd, tree);
+	    dissect_data(next_tvb, pinfo, tree);
 	}
     }
 }
