@@ -1,7 +1,7 @@
 /* packet.c
  * Routines for packet disassembly
  *
- * $Id: packet.c,v 1.94 2000/05/31 05:07:57 guy Exp $
+ * $Id: packet.c,v 1.108 2000/09/13 20:17:21 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -43,7 +43,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+
+#ifdef HAVE_STDARG_H
 #include <stdarg.h>
+#endif
+
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
@@ -86,17 +90,11 @@
 #include "packet-v120.h"
 #include "packet-vines.h"
 
-#ifndef __RESOLV_H__
 #include "resolv.h"
-#endif
-
-#ifndef __TVBUFF_H__
 #include "tvbuff.h"
-#endif
-
 #include "plugins.h"
 
-extern capture_file  cf;
+extern capture_file  cfile;
 
 static int proto_frame = -1;
 static int hf_frame_arrival_time = -1;
@@ -109,6 +107,15 @@ static int proto_malformed = -1;
 
 static gint ett_frame = -1;
 
+static void display_signed_time(gchar *, int, gint32, gint32);
+
+/* Protocol-specific data attched to a frame_data structure - protocol
+   index and opaque pointer. */
+typedef struct _frame_proto_data {
+  int proto;
+  void *proto_data;
+} frame_proto_data;
+
 GMemChunk *frame_proto_data_area = NULL;
 
 /* 
@@ -363,11 +370,13 @@ abs_time_to_str(struct timeval *abs_time)
         return cur;
 }
 
+#define	REL_TIME_LEN	(1+10+1+6+1)
+
 gchar *
 rel_time_to_str(struct timeval *rel_time)
 {
         static gchar *cur;
-        static char str[3][10+1+6+1];
+        static char str[3][REL_TIME_LEN];
 
         if (cur == &str[0][0]) {
                 cur = &str[1][0];
@@ -377,196 +386,27 @@ rel_time_to_str(struct timeval *rel_time)
                 cur = &str[0][0];
         }
 
-	sprintf(cur, "%ld.%06ld", (long)rel_time->tv_sec,
-		(long)rel_time->tv_usec);
-
+	display_signed_time(cur, REL_TIME_LEN, rel_time->tv_sec,
+	    rel_time->tv_usec);
         return cur;
 }
 
-/*
- * Given a pointer into a data buffer, and to the end of the buffer,
- * find the end of the (putative) line at that position in the data
- * buffer.
- * Return a pointer to the EOL character(s) in "*eol".
- */
-const u_char *
-find_line_end(const u_char *data, const u_char *dataend, const u_char **eol)
+static void
+display_signed_time(gchar *buf, int buflen, gint32 sec, gint32 usec)
 {
-  const u_char *lineend;
+	char *sign;
 
-  lineend = memchr(data, '\n', dataend - data);
-  if (lineend == NULL) {
-    /*
-     * No LF - line is probably continued in next TCP segment.
-     */
-    lineend = dataend;
-    *eol = dataend;
-  } else {
-    /*
-     * Is the LF at the beginning of the line?
-     */
-    if (lineend > data) {
-      /*
-       * No - is it preceded by a carriage return?
-       * (Perhaps it's supposed to be, but that's not guaranteed....)
-       */
-      if (*(lineend - 1) == '\r') {
-        /*
-	 * Yes.  The EOL starts with the CR.
-	 */
-        *eol = lineend - 1;
-      } else {
-        /*
-         * No.  The EOL starts with the LF.
-         */
-        *eol = lineend;
-
-        /*
-         * I seem to remember that we once saw lines ending with LF-CR
-         * in an HTTP request or response, so check if it's *followed*
-         * by a carriage return.
-         */
-        if (lineend < (dataend - 1) && *(lineend + 1) == '\r') {
-          /*
-           * It's <non-LF><LF><CR>; say it ends with the CR.
-           */
-          lineend++;
-        }
-      }
-    }
-
-    /*
-     * Point to the character after the last character.
-     */
-    lineend++;
-  }
-  return lineend;
-}
-
-#define	MAX_COLUMNS_LINE_DETAIL	62
-
-/*
- * Get the length of the next token in a line, and the beginning of the
- * next token after that (if any).
- * Return 0 if there is no next token.
- */
-int
-get_token_len(const u_char *linep, const u_char *lineend,
-	      const u_char **next_token)
-{
-  const u_char *tokenp;
-  int token_len;
-
-  tokenp = linep;
-  
-  /*
-   * Search for a blank, a CR or an LF, or the end of the buffer.
-   */
-  while (linep < lineend && *linep != ' ' && *linep != '\r' && *linep != '\n')
-      linep++;
-  token_len = linep - tokenp;
-
-  /*
-   * Skip trailing blanks.
-   */
-  while (linep < lineend && *linep == ' ')
-    linep++;
-
-  *next_token = linep;
-
-  return token_len;
-}
-
-/*
- * Given a string, generate a string from it that shows non-printable
- * characters as C-style escapes, and return a pointer to it.
- */
-gchar *
-format_text(const u_char *string, int len)
-{
-  static gchar fmtbuf[MAX_COLUMNS_LINE_DETAIL + 3 + 4 + 1];
-  gchar *fmtbufp;
-  int column;
-  const u_char *stringend = string + len;
-  u_char c;
-  int i;
-
-  column = 0;
-  fmtbufp = &fmtbuf[0];
-  while (string < stringend) {
-    if (column >= MAX_COLUMNS_LINE_DETAIL) {
-      /*
-       * Put "..." and quit.
-       */
-      strcpy(fmtbufp, " ...");
-      fmtbufp += 4;
-      break;
-    }
-    c = *string++;
-    if (isprint(c)) {
-      *fmtbufp++ = c;
-      column++;
-    } else {
-      *fmtbufp++ =  '\\';
-      column++;
-      switch (c) {
-
-      case '\\':
-	*fmtbufp++ = '\\';
-	column++;
-	break;
-
-      case '\a':
-	*fmtbufp++ = 'a';
-	column++;
-	break;
-
-      case '\b':
-	*fmtbufp++ = 'b';
-	column++;
-	break;
-
-      case '\f':
-	*fmtbufp++ = 'f';
-	column++;
-	break;
-
-      case '\n':
-	*fmtbufp++ = 'n';
-	column++;
-	break;
-
-      case '\r':
-	*fmtbufp++ = 'r';
-	column++;
-	break;
-
-      case '\t':
-	*fmtbufp++ = 't';
-	column++;
-	break;
-
-      case '\v':
-	*fmtbufp++ = 'v';
-	column++;
-	break;
-
-      default:
-	i = (c>>6)&03;
-	*fmtbufp++ = i + '0';
-	column++;
-	i = (c>>3)&07;
-	*fmtbufp++ = i + '0';
-	column++;
-	i = (c>>0)&07;
-	*fmtbufp++ = i + '0';
-	column++;
-	break;
-      }
-    }
-  }
-  *fmtbufp = '\0';
-  return fmtbuf;
+	/* If the microseconds part of the time stamp is negative,
+	   print its absolute value and, if the seconds part isn't
+	   (the seconds part should be zero in that case), stick
+	   a "-" in front of the entire time stamp. */
+	sign = "";
+	if (usec < 0) {
+		usec = -usec;
+		if (sec >= 0)
+			sign = "-";
+	}
+	snprintf(buf, buflen, "%s%d.%06d", sign, sec, usec);
 }
 
 
@@ -694,13 +534,34 @@ decode_numeric_bitfield(guint32 val, guint32 mask, int width,
   return buf;
 }
 
+gboolean
+col_get_writable(frame_data *fd)
+{
+  if (fd) {
+
+    return (fd->cinfo ? fd->cinfo->writable : FALSE);
+
+  }
+
+  return FALSE;
+
+}
+
+void
+col_set_writable(frame_data *fd, gboolean writable)
+{
+	if (fd->cinfo) {
+		fd->cinfo->writable = writable;
+	}
+}
+
 /* Checks to see if a particular packet information element is needed for
    the packet list */
 gint
 check_col(frame_data *fd, gint el) {
   int i;
 
-  if (fd->cinfo) {
+  if (fd->cinfo && fd->cinfo->writable) {
     for (i = 0; i < fd->cinfo->num_cols; i++) {
       if (fd->cinfo->fmt_matx[i][el])
         return TRUE;
@@ -715,14 +576,15 @@ col_add_fstr(frame_data *fd, gint el, gchar *format, ...) {
   va_list ap;
   int     i;
   size_t  max_len;
+
+  if (el == COL_INFO)
+	max_len = COL_MAX_INFO_LEN;
+  else
+	max_len = COL_MAX_LEN;
   
   va_start(ap, format);
   for (i = 0; i < fd->cinfo->num_cols; i++) {
     if (fd->cinfo->fmt_matx[i][el]) {
-      if (el == COL_INFO)
-	max_len = COL_MAX_INFO_LEN;
-      else
-	max_len = COL_MAX_LEN;
       vsnprintf(fd->cinfo->col_data[i], max_len, format, ap);
     }
   }
@@ -733,12 +595,13 @@ col_add_str(frame_data *fd, gint el, const gchar* str) {
   int    i;
   size_t max_len;
 
+  if (el == COL_INFO)
+	max_len = COL_MAX_INFO_LEN;
+  else
+	max_len = COL_MAX_LEN;
+  
   for (i = 0; i < fd->cinfo->num_cols; i++) {
     if (fd->cinfo->fmt_matx[i][el]) {
-      if (el == COL_INFO)
-	max_len = COL_MAX_INFO_LEN;
-      else
-	max_len = COL_MAX_LEN;
       strncpy(fd->cinfo->col_data[i], str, max_len);
       fd->cinfo->col_data[i][max_len - 1] = 0;
     }
@@ -752,14 +615,15 @@ col_append_fstr(frame_data *fd, gint el, gchar *format, ...) {
   int     i;
   size_t  len, max_len;
   
+  if (el == COL_INFO)
+	max_len = COL_MAX_INFO_LEN;
+  else
+	max_len = COL_MAX_LEN;
+  
   va_start(ap, format);
   for (i = 0; i < fd->cinfo->num_cols; i++) {
     if (fd->cinfo->fmt_matx[i][el]) {
       len = strlen(fd->cinfo->col_data[i]);
-      if (el == COL_INFO)
-	max_len = COL_MAX_INFO_LEN;
-      else
-	max_len = COL_MAX_LEN;
       vsnprintf(&fd->cinfo->col_data[i][len], max_len - len, format, ap);
     }
   }
@@ -770,13 +634,14 @@ col_append_str(frame_data *fd, gint el, gchar* str) {
   int    i;
   size_t len, max_len;
 
+  if (el == COL_INFO)
+	max_len = COL_MAX_INFO_LEN;
+  else
+	max_len = COL_MAX_LEN;
+  
   for (i = 0; i < fd->cinfo->num_cols; i++) {
     if (fd->cinfo->fmt_matx[i][el]) {
       len = strlen(fd->cinfo->col_data[i]);
-      if (el == COL_INFO)
-	max_len = COL_MAX_LEN;
-      else
-	max_len = COL_MAX_INFO_LEN;
       strncat(fd->cinfo->col_data[i], str, max_len - len);
       fd->cinfo->col_data[i][max_len - 1] = 0;
     }
@@ -803,15 +668,15 @@ col_set_abs_time(frame_data *fd, int col)
 static void
 col_set_rel_time(frame_data *fd, int col)
 {
-  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%d.%06d", fd->rel_secs,
-    fd->rel_usecs);
+  display_signed_time(fd->cinfo->col_data[col], COL_MAX_LEN,
+	fd->rel_secs, fd->rel_usecs);
 }
 
 static void
 col_set_delta_time(frame_data *fd, int col)
 {
-  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%d.%06d", fd->del_secs,
-    fd->del_usecs);
+  display_signed_time(fd->cinfo->col_data[col], COL_MAX_LEN,
+	fd->del_secs, fd->del_usecs);
 }
 
 /* Add "command-line-specified" time.
@@ -920,6 +785,13 @@ col_set_port(frame_data *fd, int col, port_type ptype, guint32 port,
 {
   switch (ptype) {
 
+  case PT_SCTP:
+    if (is_res)
+      strncpy(fd->cinfo->col_data[col], get_sctp_port(port), COL_MAX_LEN);
+    else
+      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%u", port);
+    break;
+    
   case PT_TCP:
     if (is_res)
       strncpy(fd->cinfo->col_data[col], get_tcp_port(port), COL_MAX_LEN);
@@ -1126,7 +998,7 @@ dissect_packet(union wtap_pseudo_header *pseudo_header, const u_char *pd,
 	proto_tree *fh_tree;
 	proto_item *ti;
 	struct timeval tv;
-	tvbuff_t *tvb;
+	static tvbuff_t *tvb;
 
 	/* Put in frame header information. */
 	if (tree) {
@@ -1168,12 +1040,16 @@ dissect_packet(union wtap_pseudo_header *pseudo_header, const u_char *pd,
 	pi.len = fd->pkt_len;
 	pi.captured_len = fd->cap_len;
 
-	tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len);
 	pi.fd = fd;
-	pi.compat_top_tvb = tvb;
 	pi.pseudo_header = pseudo_header;
+	pi.current_proto = "Frame";
+
+	col_set_writable(fd, TRUE);
 
 	TRY {
+		tvb = tvb_new_real_data(pd, fd->cap_len, fd->pkt_len);
+		pi.compat_top_tvb = tvb;
+
 		switch (fd->lnk_t) {
 			case WTAP_ENCAP_ETHERNET :
 				dissect_eth(tvb, &pi, tree);
@@ -1267,7 +1143,7 @@ p_add_proto_data(frame_data *fd, int proto, void *proto_data)
 void *
 p_get_proto_data(frame_data *fd, int proto)
 {
-  frame_proto_data temp;
+  frame_proto_data temp, *p1;
   GSList *item;
 
   temp.proto = proto;
@@ -1275,7 +1151,10 @@ p_get_proto_data(frame_data *fd, int proto)
 
   item = g_slist_find_custom(fd->pfd, (gpointer *)&temp, p_compare);
 
-  if (item) return (void *)item->data;
+  if (item) {
+    p1 = (frame_proto_data *)item->data;
+    return p1->proto_data;
+  }
 
   return NULL;
 
@@ -1284,7 +1163,19 @@ p_get_proto_data(frame_data *fd, int proto)
 void
 p_rem_proto_data(frame_data *fd, int proto)
 {
+  frame_proto_data temp;
+  GSList *item;
 
+  temp.proto = proto;
+  temp.proto_data = NULL;
+
+  item = g_slist_find_custom(fd->pfd, (gpointer *)&temp, p_compare);
+
+  if (item) {
+
+    fd->pfd = g_slist_remove(fd->pfd, item);
+
+  }
 
 }
 
@@ -1331,6 +1222,23 @@ proto_register_frame(void)
 
 static GHashTable *dissector_tables = NULL;
 
+/*
+ * XXX - for now, we support having both "old" dissectors, with packet
+ * data pointer, packet offset, frame_data pointer, and protocol tree
+ * pointer arguments, and "new" dissectors, with tvbuff pointer,
+ * packet_info pointer, and protocol tree pointer arguments.
+ *
+ * Nuke this and go back to storing a pointer to the dissector when
+ * the last old-style dissector is gone.
+ */
+typedef struct {
+	gboolean is_old_dissector;
+	union {
+		old_dissector_t	old;
+		dissector_t	new;
+	} dissector;
+} dtbl_entry_t;
+
 /* Finds a dissector table by field name. */
 static dissector_table_t
 find_dissector_table(const char *name)
@@ -1339,27 +1247,42 @@ find_dissector_table(const char *name)
 	return g_hash_table_lookup( dissector_tables, name );
 }
 
-/* lookup a dissector based upon pattern. */
-dissector_t
-dissector_lookup( dissector_table_t table, guint32 pattern)
-{
-	return g_hash_table_lookup( table, GUINT_TO_POINTER( pattern));
-}
-
-
 /* add an entry, lookup the dissector table for the specified field name,  */
 /* if a valid table found, add the subdissector */
 void
-dissector_add(const char *name, guint32 pattern, dissector_t dissector)
+old_dissector_add(const char *name, guint32 pattern, old_dissector_t dissector)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
+	dtbl_entry_t *dtbl_entry;
 
 /* sanity check */
 	g_assert( sub_dissectors);
 
+	dtbl_entry = g_malloc(sizeof (dtbl_entry_t));
+	dtbl_entry->is_old_dissector = TRUE;
+	dtbl_entry->dissector.old = dissector;
+
 /* do the table insertion */
     	g_hash_table_insert( sub_dissectors, GUINT_TO_POINTER( pattern),
-    	 (gpointer)dissector);
+    	 (gpointer)dtbl_entry);
+}
+
+void
+dissector_add(const char *name, guint32 pattern, dissector_t dissector)
+{
+	dissector_table_t sub_dissectors = find_dissector_table( name);
+	dtbl_entry_t *dtbl_entry;
+
+/* sanity check */
+	g_assert( sub_dissectors);
+
+	dtbl_entry = g_malloc(sizeof (dtbl_entry_t));
+	dtbl_entry->is_old_dissector = FALSE;
+	dtbl_entry->dissector.new = dissector;
+
+/* do the table insertion */
+    	g_hash_table_insert( sub_dissectors, GUINT_TO_POINTER( pattern),
+    	 (gpointer)dtbl_entry);
 }
 
 /* delete the entry for this dissector at this pattern */
@@ -1370,30 +1293,120 @@ dissector_add(const char *name, guint32 pattern, dissector_t dissector)
 /*	If temporary dissectors are deleted, then the original dissector must */
 /*	be available. */
 void
-dissector_delete(const char *name, guint32 pattern, dissector_t dissector)
+old_dissector_delete(const char *name, guint32 pattern, old_dissector_t dissector)
 {
 	dissector_table_t sub_dissectors = find_dissector_table( name);
+	dtbl_entry_t *dtbl_entry;
 
 /* sanity check */
 	g_assert( sub_dissectors);
 
-/* remove the hash table entry */
-	g_hash_table_remove( sub_dissectors, GUINT_TO_POINTER( pattern));
+	/*
+	 * Find the entry.
+	 */
+	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	    GUINT_TO_POINTER(pattern));
+
+	if (dtbl_entry != NULL) {
+		/*
+		 * Found - remove it.
+		 */
+		g_hash_table_remove(sub_dissectors, GUINT_TO_POINTER(pattern));
+
+		/*
+		 * Now free up the entry.
+		 */
+		g_free(dtbl_entry);
+	}
+}
+
+void
+dissector_delete(const char *name, guint32 pattern, dissector_t dissector)
+{
+	dissector_table_t sub_dissectors = find_dissector_table( name);
+	dtbl_entry_t *dtbl_entry;
+
+/* sanity check */
+	g_assert( sub_dissectors);
+
+	/*
+	 * Find the entry.
+	 */
+	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	    GUINT_TO_POINTER(pattern));
+
+	if (dtbl_entry != NULL) {
+		/*
+		 * Found - remove it.
+		 */
+		g_hash_table_remove(sub_dissectors, GUINT_TO_POINTER(pattern));
+
+		/*
+		 * Now free up the entry.
+		 */
+		g_free(dtbl_entry);
+	}
 }
 
 /* Look for a given port in a given dissector table and, if found, call
    the dissector with the arguments supplied, and return TRUE, otherwise
-   return FALSE. */
+   return FALSE.
+
+   If the arguments supplied don't match the arguments to the dissector,
+   do the appropriate translation. */
 gboolean
-dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
+old_dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
     const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
-	dissector_t subdissector;
+	dtbl_entry_t *dtbl_entry;
+	tvbuff_t *tvb;
 
-	subdissector = dissector_lookup(sub_dissectors, port);
-	if (subdissector != NULL) {
+	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	    GUINT_TO_POINTER(port));
+	if (dtbl_entry != NULL) {
 		pi.match_port = port;
-		(subdissector)(pd, offset, fd, tree);
+		if (dtbl_entry->is_old_dissector)
+			(*dtbl_entry->dissector.old)(pd, offset, fd, tree);
+		else {
+			/*
+			 * Old dissector calling new dissector; use
+			 * "tvb_create_from_top()" to remap.
+			 *
+			 * XXX - what about the "pd" argument?  Do
+			 * any dissectors not just pass that along and
+			 * let the "offset" argument handle stepping
+			 * through the packet?
+			 */
+			tvb = tvb_create_from_top(offset);
+			(*dtbl_entry->dissector.new)(tvb, &pi, tree);
+		}
+		return TRUE;
+	} else
+		return FALSE;
+}
+
+gboolean
+dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
+    tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+{
+	dtbl_entry_t *dtbl_entry;
+	const guint8 *pd;
+	int offset;
+
+	dtbl_entry = g_hash_table_lookup(sub_dissectors,
+	    GUINT_TO_POINTER(port));
+	if (dtbl_entry != NULL) {
+		pi.match_port = port;
+		if (dtbl_entry->is_old_dissector) {
+			/*
+			 * New dissector calling old dissector; use
+			 * "tvb_compat()" to remap.
+			 */
+			tvb_compat(tvb, &pd, &offset);
+			(*dtbl_entry->dissector.old)(pd, offset, pinfo->fd,
+			    tree);
+		} else
+			(*dtbl_entry->dissector.new)(tvb, pinfo, tree);
 		return TRUE;
 	} else
 		return FALSE;
@@ -1422,6 +1435,23 @@ register_dissector_table(const char *name)
 
 static GHashTable *heur_dissector_lists = NULL;
 
+/*
+ * XXX - for now, we support having both "old" dissectors, with packet
+ * data pointer, packet offset, frame_data pointer, and protocol tree
+ * pointer arguments, and "new" dissectors, with tvbuff pointer,
+ * packet_info pointer, and protocol tree pointer arguments.
+ *
+ * Nuke this and go back to storing a pointer to the dissector when
+ * the last old-style dissector is gone.
+ */
+typedef struct {
+	gboolean is_old_dissector;
+	union {
+		old_heur_dissector_t	old;
+		heur_dissector_t	new;
+	} dissector;
+} heur_dtbl_entry_t;
+
 /* Finds a heuristic dissector table by field name. */
 static heur_dissector_list_t *
 find_heur_dissector_list(const char *name)
@@ -1431,28 +1461,96 @@ find_heur_dissector_list(const char *name)
 }
 
 void
-heur_dissector_add(const char *name, heur_dissector_t dissector)
+old_heur_dissector_add(const char *name, old_heur_dissector_t dissector)
 {
 	heur_dissector_list_t *sub_dissectors = find_heur_dissector_list(name);
+	heur_dtbl_entry_t *dtbl_entry;
 
 	/* sanity check */
 	g_assert(sub_dissectors != NULL);
 
+	dtbl_entry = g_malloc(sizeof (heur_dtbl_entry_t));
+	dtbl_entry->is_old_dissector = TRUE;
+	dtbl_entry->dissector.old = dissector;
+
 	/* do the table insertion */
-	*sub_dissectors = g_slist_append(*sub_dissectors, (gpointer)dissector);
+	*sub_dissectors = g_slist_append(*sub_dissectors, (gpointer)dtbl_entry);
+}
+
+void
+heur_dissector_add(const char *name, heur_dissector_t dissector)
+{
+	heur_dissector_list_t *sub_dissectors = find_heur_dissector_list(name);
+	heur_dtbl_entry_t *dtbl_entry;
+
+	/* sanity check */
+	g_assert(sub_dissectors != NULL);
+
+	dtbl_entry = g_malloc(sizeof (heur_dtbl_entry_t));
+	dtbl_entry->is_old_dissector = FALSE;
+	dtbl_entry->dissector.new = dissector;
+
+	/* do the table insertion */
+	*sub_dissectors = g_slist_append(*sub_dissectors, (gpointer)dtbl_entry);
+}
+
+gboolean
+old_dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
+    const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
+{
+	GSList *entry;
+	heur_dtbl_entry_t *dtbl_entry;
+	tvbuff_t *tvb = NULL;
+
+	for (entry = sub_dissectors; entry != NULL; entry = g_slist_next(entry)) {
+		dtbl_entry = (heur_dtbl_entry_t *)entry->data;
+		if (dtbl_entry->is_old_dissector) {
+			if ((*dtbl_entry->dissector.old)(pd, offset, fd, tree))
+				return TRUE;
+		} else {
+			/*
+			 * Old dissector calling new dissector; use
+			 * "tvb_create_from_top()" to remap.
+			 *
+			 * XXX - what about the "pd" argument?  Do
+			 * any dissectors not just pass that along and
+			 * let the "offset" argument handle stepping
+			 * through the packet?
+			 */
+			if (tvb == NULL)
+				tvb = tvb_create_from_top(offset);
+			if ((*dtbl_entry->dissector.new)(tvb, &pi, tree))
+				return TRUE;
+		}
+	}
+	return FALSE;
 }
 
 gboolean
 dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
-    const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
+    tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	heur_dissector_t subdissector;
 	GSList *entry;
+	heur_dtbl_entry_t *dtbl_entry;
+	const guint8 *pd = NULL;
+	int offset;
 
 	for (entry = sub_dissectors; entry != NULL; entry = g_slist_next(entry)) {
-		subdissector = (heur_dissector_t)entry->data;
-		if ((subdissector)(pd, offset, fd, tree))
-			return TRUE;
+		dtbl_entry = (heur_dtbl_entry_t *)entry->data;
+		if (dtbl_entry->is_old_dissector) {
+			/*
+			 * New dissector calling old dissector; use
+			 * "tvb_compat()" to remap.
+			 */
+			if (pd == NULL)
+				tvb_compat(tvb, &pd, &offset);
+			if ((*dtbl_entry->dissector.old)(pd, offset, pinfo->fd,
+			    tree))
+				return TRUE;
+		} else {
+			if ((*dtbl_entry->dissector.new)(tvb, pinfo, tree))
+				return TRUE;
+		}
 	}
 	return FALSE;
 }
