@@ -1,7 +1,7 @@
 /* packet.h
  * Definitions for packet disassembly structures and routines
  *
- * $Id: packet.h,v 1.189 2000/05/26 22:08:16 guy Exp $
+ * $Id: packet.h,v 1.201 2000/09/11 16:16:12 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -27,45 +27,13 @@
 #ifndef __PACKET_H__
 #define __PACKET_H__
 
-#ifndef __WTAP_H__
 #include "wiretap/wtap.h"
-#endif
-
-#ifndef __PROTO_H__
 #include "proto.h"
-#endif
-
-#ifndef __TVBUFF_H__
 #include "tvbuff.h"
-#endif
+#include "pint.h"
 
-
-/* Pointer versions of ntohs and ntohl.  Given a pointer to a member of a
- * byte array, returns the value of the two or four bytes at the pointer.
- * The pletoh[sl] versions return the little-endian representation.
- */
-
-#define pntohs(p)  ((guint16)                       \
-                    ((guint16)*((guint8 *)p+0)<<8|  \
-                     (guint16)*((guint8 *)p+1)<<0))
-
-#define pntohl(p)  ((guint32)*((guint8 *)p+0)<<24|  \
-                    (guint32)*((guint8 *)p+1)<<16|  \
-                    (guint32)*((guint8 *)p+2)<<8|   \
-                    (guint32)*((guint8 *)p+3)<<0)
-
-#define pletohs(p) ((guint16)                       \
-                    ((guint16)*((guint8 *)p+1)<<8|  \
-                     (guint16)*((guint8 *)p+0)<<0))
-
-#define pletohl(p) ((guint32)*((guint8 *)p+3)<<24|  \
-                    (guint32)*((guint8 *)p+2)<<16|  \
-                    (guint32)*((guint8 *)p+1)<<8|   \
-                    (guint32)*((guint8 *)p+0)<<0)
-
-
-#define hi_nibble(b) ((b & 0xf0) >> 4)
-#define lo_nibble(b) (b & 0x0f)
+#define hi_nibble(b) (((b) & 0xf0) >> 4)
+#define lo_nibble(b) ((b) & 0x0f)
 
 /* Useful when you have an array whose size you can tell at compile-time */
 #define array_length(x)	(sizeof x / sizeof x[0])
@@ -73,7 +41,7 @@
 /* Useful when highlighting regions inside a dissect_*() function. With this
  * macro, you can highlight from an arbitrary offset to the end of the
  * packet (which may come before the end of the frame).
- * See dissect_data() for an example.
+ * See old_dissect_data() for an example.
  */
 #define END_OF_FRAME	(pi.captured_len - offset)
 
@@ -94,6 +62,7 @@ typedef struct _column_info {
   gint      *col_width; /* Column widths to use during a "-S" capture */
   gchar    **col_title; /* Column titles */
   gchar    **col_data;  /* Column data */
+  gboolean   writable;  /* Are we stil writing to the columns? */
 } column_info;
 
 #define COL_MAX_LEN 256
@@ -119,11 +88,6 @@ typedef enum {
 	CHAR_EBCDIC	 = 1	/* EBCDIC */
 } char_enc;
 
-typedef struct _frame_proto_data {
-  int proto;
-  void *proto_data;
-} frame_proto_data;
-
 /* XXX - some of this stuff is used only while a packet is being dissected;
    should we keep around a separate data structure for that, to save
    memory? */
@@ -134,12 +98,12 @@ typedef struct _frame_data {
   guint32      num;       /* Frame number */
   guint32      pkt_len;   /* Packet length */
   guint32      cap_len;   /* Amount actually captured */
-  guint32      rel_secs;  /* Relative seconds */
-  guint32      rel_usecs; /* Relative microseconds */
+  gint32       rel_secs;  /* Relative seconds (yes, it can be negative) */
+  gint32       rel_usecs; /* Relative microseconds (yes, it can be negative) */
   guint32      abs_secs;  /* Absolute seconds */
   guint32      abs_usecs; /* Absolute microseconds */
-  guint32      del_secs;  /* Delta seconds */
-  guint32      del_usecs; /* Delta microseconds */
+  gint32       del_secs;  /* Delta seconds (yes, it can be negative) */
+  gint32       del_usecs; /* Delta microseconds (yes, it can be negative) */
   long         file_off;  /* File offset */
   column_info *cinfo;     /* Column formatting information */
   int          lnk_t;     /* Per-packet encapsulation/data-link type */
@@ -147,6 +111,7 @@ typedef struct _frame_data {
 	unsigned int passed_dfilter	: 1; /* 1 = display, 0 = no display */
   	unsigned int encoding		: 2; /* Character encoding (ASCII, EBCDIC...) */
 	unsigned int visited		: 1; /* Has this packet been visited yet? 1=Yes,0=No*/
+	unsigned int marked             : 1; /* 1 = marked by user, 0 = normal */
   } flags;
 } frame_data;
 
@@ -177,6 +142,7 @@ typedef struct _address {
 /* Types of port numbers Ethereal knows about. */
 typedef enum {
   PT_NONE,		/* no port number */
+  PT_SCTP,              /* SCTP */
   PT_TCP,		/* TCP */
   PT_UDP,		/* UDP */
   PT_NCP		/* NCP connection */
@@ -223,27 +189,29 @@ typedef struct true_false_string {
 typedef GHashTable* dissector_table_t;
 
 /* types for sub-dissector lookup */
-typedef void (*dissector_t)(const u_char *, int, frame_data *, proto_tree *);
+typedef void (*old_dissector_t)(const u_char *, int, frame_data *, proto_tree *);
+typedef void (*dissector_t)(tvbuff_t *, packet_info *, proto_tree *);
 
 /* a protocol uses the function to register a sub-dissector table */
 dissector_table_t register_dissector_table(const char *name);
 
-/* dissector lookup routine.  called by protocol dissector to find a sub-dissector */
-dissector_t dissector_lookup( dissector_table_t table, guint32 pattern);
-
 /* Add a sub-dissector to a dissector table.  Called by the protocol routine */
 /* that wants to register a sub-dissector.  */
+void old_dissector_add(const char *abbrev, guint32 pattern, old_dissector_t dissector);
 void dissector_add(const char *abbrev, guint32 pattern, dissector_t dissector);
 
 /* Add a sub-dissector to a dissector table.  Called by the protocol routine */
 /* that wants to de-register a sub-dissector.  */
+void old_dissector_delete(const char *name, guint32 pattern, old_dissector_t dissector);
 void dissector_delete(const char *name, guint32 pattern, dissector_t dissector);
 
 /* Look for a given port in a given dissector table and, if found, call
    the dissector with the arguments supplied, and return TRUE, otherwise
    return FALSE. */
-gboolean dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
+gboolean old_dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
     const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
+gboolean dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
+    tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 
 /* List of "heuristic" dissectors (which get handed a packet, look at it,
    and either recognize it as being for their protocol, dissect it, and
@@ -252,7 +220,9 @@ gboolean dissector_try_port(dissector_table_t sub_dissectors, guint32 port,
 typedef GSList *heur_dissector_list_t;
 
 /* Type of a heuristic dissector */
-typedef gboolean (*heur_dissector_t)(const u_char *, int, frame_data *,
+typedef gboolean (*old_heur_dissector_t)(const u_char *, int, frame_data *,
+	proto_tree *);
+typedef gboolean (*heur_dissector_t)(tvbuff_t *, packet_info *,
 	proto_tree *);
 
 /* A protocol uses this function to register a heuristic dissector list */
@@ -260,22 +230,16 @@ void register_heur_dissector_list(const char *name, heur_dissector_list_t *list)
 
 /* Add a sub-dissector to a heuristic dissector list.  Called by the
    protocol routine that wants to register a sub-dissector.  */
+void old_heur_dissector_add(const char *name, old_heur_dissector_t dissector);
 void heur_dissector_add(const char *name, heur_dissector_t dissector);
 
 /* Try all the dissectors in a given heuristic dissector list until
    we find one that recognizes the protocol, in which case we return
    TRUE, or we run out of dissectors, in which case we return FALSE. */
-gboolean dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
+gboolean old_dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
     const u_char *pd, int offset, frame_data *fd, proto_tree *tree);
-
-/* Many of the structs and definitions below and in packet-*.c files
- * were taken from include files in the Linux distribution. */
-
-typedef struct tcp_extra_data {
-  int match_port;
-  int sport;
-  int dport;
-} tcp_extra_data;
+gboolean dissector_try_heuristic(heur_dissector_list_t sub_dissectors,
+    tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 
 /* Utility routines used by packet*.c */
 gchar*     ether_to_str(const guint8 *);
@@ -288,11 +252,6 @@ gchar*	   abs_time_to_str(struct timeval*);
 gchar*	   rel_time_to_str(struct timeval*);
 gchar*     time_secs_to_str(guint32);
 gchar*     bytes_to_str(const guint8 *, int);
-const u_char *find_line_end(const u_char *data, const u_char *dataend,
-    const u_char **eol);
-int        get_token_len(const u_char *linep, const u_char *lineend,
-    const u_char **next_token);
-gchar*     format_text(const u_char *line, int len);
 gchar*     val_to_str(guint32, const value_string *, const char *);
 gchar*     match_strval(guint32, const value_string*);
 char * decode_bitfield_value(char *buf, guint32 val, guint32 mask, int width);
@@ -302,6 +261,8 @@ const char *decode_enumerated_bitfield(guint32 val, guint32 mask, int width,
   const value_string *tab, const char *fmt);
 const char *decode_numeric_bitfield(guint32 val, guint32 mask, int width,
   const char *fmt);
+
+void	   col_set_writable(frame_data *fd, gboolean writable);
 gint       check_col(frame_data *, gint);
 #if __GNUC__ == 2
 void       col_add_fstr(frame_data *, gint, gchar *, ...)
@@ -345,8 +306,8 @@ void init_dissect_rpc(void);
  */
 void dissect_packet(union wtap_pseudo_header *, const u_char *, frame_data *,
     proto_tree *);
-void dissect_data(const u_char *, int, frame_data *, proto_tree *);
-void dissect_data_tvb(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
+void old_dissect_data(const u_char *, int, frame_data *, proto_tree *);
+void dissect_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 
 
 /* These functions are in ethertype.c */
