@@ -1,7 +1,7 @@
 /* plugins.c
  * plugin routines
  *
- * $Id: plugins.c,v 1.18 2000/06/07 02:36:59 gram Exp $
+ * $Id: plugins.c,v 1.24 2000/09/14 10:03:02 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -43,8 +43,14 @@
 
 #include <string.h>
 #include <stdlib.h>
+
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -67,15 +73,18 @@ plugin *plugin_list;
 guint32 enabled_plugins_number;
 
 #ifdef WIN32
-static gchar std_plug_dir[] = "c:/program files/ethereal/plugins/0.8.9";
-static gchar local_plug_dir[] = "c:/ethereal/plugins/0.8.9";
+static gchar std_plug_dir[] = "c:/program files/ethereal/plugins/0.8.12";
+static gchar local_plug_dir[] = "c:/ethereal/plugins/0.8.12";
 #else
-static gchar std_plug_dir[] = "/usr/lib/ethereal/plugins/0.8.9";
-static gchar local_plug_dir[] = "/usr/local/lib/ethereal/plugins/0.8.9";
+static gchar std_plug_dir[] = "/usr/lib/ethereal/plugins/0.8.12";
+static gchar local_plug_dir[] = "/usr/local/lib/ethereal/plugins/0.8.12";
 #endif
 static gchar *user_plug_dir = NULL;
 static gchar *plugin_status_file = NULL;
 
+#define PLUGINS_STATUS		"plugins.status"
+#define PLUGINS_DIR_NAME	"plugins"
+
 /*
  * add a new plugin to the list
  * returns :
@@ -254,6 +263,7 @@ plugin_replace_filter(const gchar *name, const gchar *version,
  * gryphon.so 0.8.0 1
  * tcp.port == 7000
  */
+
 int
 save_plugin_status()
 {
@@ -262,18 +272,21 @@ save_plugin_status()
     plugin *pt_plug;
 
     if (!plugin_status_file) {
-	plugin_status_file = (gchar *)g_malloc(strlen(get_home_dir()) + 26);
-	sprintf(plugin_status_file, "%s/%s/plugins.status", get_home_dir(), PF_DIR);
+	plugin_status_file = (gchar *)g_malloc(strlen(get_home_dir()) + 
+					       strlen(PF_DIR) +
+					       strlen(PLUGINS_STATUS) + 3);
+	sprintf(plugin_status_file, "%s/%s/%s", 
+		get_home_dir(), PF_DIR, PLUGINS_STATUS);
     }
     statusfile=fopen(plugin_status_file, "w");
     if (!statusfile) {
 	pf_path = g_malloc(strlen(get_home_dir()) + strlen(PF_DIR) + 2);
 	sprintf(pf_path, "%s/%s", get_home_dir(), PF_DIR);
-	#ifdef WIN32
+#ifdef WIN32
 	mkdir(pf_path);
-	#else
+#else
 	mkdir(pf_path, 0755);
-	#endif
+#endif
 	g_free(pf_path);
 	statusfile=fopen(plugin_status_file, "w");
 	if (!statusfile) return -1;
@@ -346,9 +359,10 @@ check_plugin_status(gchar *name, gchar *version, GModule *handle,
 static void
 plugins_scan_dir(const char *dirname)
 {
+#define FILENAME_LEN	1024
     DIR           *dir;             /* scanned directory */
     struct dirent *file;            /* current file */
-    gchar          filename[512];   /* current file name */
+    gchar          filename[FILENAME_LEN];   /* current file name */
     GModule       *handle;          /* handle returned by dlopen */
     gchar         *name;
     gchar         *version;
@@ -368,8 +382,11 @@ plugins_scan_dir(const char *dirname)
 
     if (!plugin_status_file)
     {
-	plugin_status_file = (gchar *)g_malloc(strlen(get_home_dir()) + 26);
-	sprintf(plugin_status_file, "%s/%s/plugins.status", get_home_dir(), PF_DIR);
+	plugin_status_file = (gchar *)g_malloc(strlen(get_home_dir()) + 
+					       strlen(PF_DIR) +
+					       strlen(PLUGINS_STATUS) + 3);
+	sprintf(plugin_status_file, "%s/%s/%s", 
+		get_home_dir(), PF_DIR, PLUGINS_STATUS);
     }
     statusfile = fopen(plugin_status_file, "r");
 
@@ -385,31 +402,36 @@ plugins_scan_dir(const char *dirname)
             dot = strrchr(file->d_name, '.');
             if (dot == NULL || strcmp(dot, LT_LIB_EXT) != 0) continue;
 
-	    sprintf(filename, "%s/%s", dirname, file->d_name);
+	    snprintf(filename, FILENAME_LEN, "%s/%s", dirname, file->d_name);
 	    if ((handle = g_module_open(filename, 0)) == NULL) continue;
 	    name = (gchar *)file->d_name;
 	    if (g_module_symbol(handle, "version", (gpointer*)&version) == FALSE)
 	    {
+	        g_warning("The plugin %s has no version symbol", name);
 		g_module_close(handle);
 		continue;
 	    }
 	    if (g_module_symbol(handle, "protocol", (gpointer*)&protocol) == FALSE)
 	    {
+	        g_warning("The plugin %s has no protocol symbol", name);
 		g_module_close(handle);
 		continue;
 	    }
 	    if (g_module_symbol(handle, "filter_string", (gpointer*)&filter_string) == FALSE)
 	    {
+	        g_warning("The plugin %s has no filter_string symbol", name);
 		g_module_close(handle);
 		continue;
 	    }
 	    if (dfilter_compile(filter_string, &filter) != 0) {
+	        g_warning("The plugin %s has a non compilable filter", name);
 		g_module_close(handle);
 		continue;
 	    }
 	    if (g_module_symbol(handle, "dissector", (gpointer*)&dissector) == FALSE) {
 		if (filter != NULL)
 		    dfilter_destroy(filter);
+	        g_warning("The plugin %s has no dissector symbol", name);
 		g_module_close(handle);
 		continue;
 	    }
@@ -551,8 +573,11 @@ init_plugins()
 	}
 	if (!user_plug_dir)
 	{
-	    user_plug_dir = (gchar *)g_malloc(strlen(get_home_dir()) + 19);
-	    sprintf(user_plug_dir, "%s/%s/plugins", get_home_dir(), PF_DIR);
+	    user_plug_dir = (gchar *)g_malloc(strlen(get_home_dir()) +
+					      strlen(PF_DIR) +
+					      strlen(PLUGINS_DIR_NAME) + 3);
+	    sprintf(user_plug_dir, "%s/%s/%s", get_home_dir(), 
+		    PF_DIR, PLUGINS_DIR_NAME);
 	}
 	plugins_scan_dir(user_plug_dir);
     }
