@@ -1,7 +1,7 @@
 /* proto.c
  * Routines for protocol tree
  *
- * $Id: proto.c,v 1.68 2000/05/31 17:10:09 gram Exp $
+ * $Id: proto.c,v 1.80 2000/08/30 02:50:02 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,41 +31,19 @@
 # include <sys/types.h>
 #endif
 
-#ifndef _STDIO_H
 #include <stdio.h>
-#endif
-
-#include <stdarg.h>
-
-#ifndef _STRING_H
 #include <string.h>
-#endif
+#include <glib.h>
 
 #ifdef NEED_SNPRINTF_H
 # include "snprintf.h"
 #endif
 
-#ifndef __G_LIB_H__
-#include <glib.h>
-#endif
-
-#ifndef __PROTO_H__
-#include "proto.h"
-#endif
-
-#ifndef __PACKET_H__
 #include "packet.h"
-#endif
-
-#ifndef __RESOLV_H__
 #include "resolv.h"
-#endif
-
-#ifndef __REGISTER_H__
 #include "register.h"
-#endif
-
 #include "packet-ipv6.h"
+#include "proto.h"
 
 #define cVALS(x) (const value_string*)(x)
 
@@ -87,11 +65,13 @@ static char* hfinfo_int_vals_format(header_field_info *hfinfo);
 static char* hfinfo_int_format(header_field_info *hfinfo);
 
 static gboolean check_for_protocol_or_field_id(GNode *node, gpointer data);
-static gboolean check_for_field_within_protocol(GNode *node, gpointer data);
 
 static proto_item*
 proto_tree_add_node(proto_tree *tree, field_info *fi);
 
+static field_info *
+alloc_field_info(int hfindex, tvbuff_t *tvb, gint start, gint length);
+
 static proto_item *
 proto_tree_add_pi(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint length,
 		field_info **pfi);
@@ -242,6 +222,13 @@ proto_tree_free(proto_tree *tree)
 	g_node_destroy((GNode*)tree);
 }
 
+/* We accept a void* instead of a field_info* to satisfy CLEANUP_POP */
+static void
+free_field_info(void *fi)
+{
+	g_mem_chunk_free(gmc_field_info, (field_info*)fi);
+}
+
 static gboolean
 proto_tree_free_node(GNode *node, gpointer data)
 {
@@ -252,9 +239,13 @@ proto_tree_free_node(GNode *node, gpointer data)
 			g_mem_chunk_free(gmc_item_labels, fi->representation);
 		if (fi->hfinfo->type == FT_STRING)
 			g_free(fi->value.string);
+		else if (fi->hfinfo->type == FT_STRINGZ)
+			g_free(fi->value.string);
+		else if (fi->hfinfo->type == FT_UINT_STRING)
+			g_free(fi->value.string);
 		else if (fi->hfinfo->type == FT_BYTES) 
 			g_free(fi->value.bytes);
-		g_mem_chunk_free(gmc_field_info, fi);
+		free_field_info(fi);
 	}
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
 }	
@@ -300,6 +291,45 @@ proto_tree_add_text(proto_tree *tree, tvbuff_t *tvb, gint start, gint length,
 	return pi;
 }
 
+/* Add a text-only node to the proto_tree (va_list version) */
+proto_item *
+proto_tree_add_text_valist(proto_tree *tree, tvbuff_t *tvb, gint start, 
+	gint length, const char *format, va_list ap)
+{
+	proto_item	*pi;
+
+	pi = proto_tree_add_notext(tree, tvb, start, length);
+	if (pi == NULL)
+		return(NULL);
+
+	proto_tree_set_representation(pi, format, ap);
+
+	return pi;
+}
+
+/* Add a text-only node for debugging purposes. The caller doesn't need
+ * to worry about tvbuff, start, or length. Debug message gets sent to
+ * STDOUT, too */
+proto_item *
+proto_tree_add_debug_text(proto_tree *tree, const char *format, ...)
+{
+	proto_item	*pi;
+	va_list		ap;
+
+	pi = proto_tree_add_notext(tree, NULL, 0, 0);
+	if (pi == NULL)
+		return(NULL);
+
+	va_start(ap, format);
+	proto_tree_set_representation(pi, format, ap);
+	vprintf(format, ap);
+	va_end(ap);
+	printf("\n");
+
+	return pi;
+}
+
+
 static guint32
 get_uint_value(tvbuff_t *tvb, gint offset, gint length, gboolean little_endian)
 {
@@ -378,14 +408,21 @@ proto_item *
 proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
     gint start, gint length, gboolean little_endian)
 {
-	proto_item	*pi;
 	field_info	*new_fi;
-	guint32		value;
+	proto_item	*pi;
+	guint32		value, n;
+	char		*string;
+	int		found_length;
 
-	pi = proto_tree_add_pi(tree, hfindex, tvb, start, length, &new_fi);
-	if (pi == NULL)
+	new_fi = alloc_field_info(hfindex, tvb, start, length);
+
+	if (new_fi == NULL)
 		return(NULL);
 
+	/* Register a cleanup function in case on of our tvbuff accesses
+	 * throws an exception. We need to clean up new_fi. */
+	CLEANUP_PUSH(free_field_info, new_fi);
+
 	switch(new_fi->hfinfo->type) {
 		case FT_NONE:
 			/* no value to set for FT_NONE */
@@ -425,9 +462,9 @@ proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 			break;
 
 		case FT_IPXNET:
-			g_assert(length == 4);	/* 2? */
-			tvb_memcpy(tvb, (guint8 *)&value, start, 4);
-			proto_tree_set_ipxnet(new_fi, value);
+			g_assert(length == 4);
+			proto_tree_set_ipxnet(new_fi,
+			    get_uint_value(tvb, start, 4, FALSE));
 			break;
 
 		case FT_IPv6:
@@ -445,11 +482,46 @@ proto_tree_add_item(proto_tree *tree, int hfindex, tvbuff_t *tvb,
 			proto_tree_set_string_tvb(new_fi, tvb, start, length);
 			break;
 
-		default:
-			g_error("new_fi->hfinfo->type %d not handled\n", new_fi->hfinfo->type);
-			g_assert_not_reached();
+		case FT_STRINGZ:
+			/* This g_strdup'ed memory is freed in proto_tree_free_node() */
+			string = g_malloc(length);
+
+			CLEANUP_PUSH(g_free, string);
+
+			found_length = tvb_get_nstringz(tvb, start, length, string);
+			if (found_length < 1) {
+				found_length = tvb_get_nstringz0(tvb, start, length, string);
+			}
+
+			CLEANUP_POP;
+
+			proto_tree_set_string(new_fi, string);
+			new_fi->length = found_length + 1;
+
 			break;
+
+		case FT_UINT_STRING:
+			/* This g_strdup'ed memory is freed in proto_tree_free_node() */
+			n = get_uint_value(tvb, start, length, little_endian);
+			proto_tree_set_string_tvb(new_fi, tvb, start + 1, n);
+
+			/* Instead of calling proto_item_set_len(), since we don't yet
+			 * have a proto_item, we set the field_info's length ourselves. */
+			new_fi->length = n + 1;
+			break;
+		default:
+                        g_error("new_fi->hfinfo->type %d (%s) not handled\n",
+					new_fi->hfinfo->type,
+					proto_registrar_ftype_name(new_fi->hfinfo->type));
+                        g_assert_not_reached();
+                        break;
+
 	}
+	CLEANUP_POP;
+
+	/* Don't add to proto_item to proto_tree until now so that any exceptions
+	 * raised by a tvbuff access method doesn't leave junk in the proto_tree. */
+	pi = proto_tree_add_node(tree, new_fi);
 
 	return pi;
 }
@@ -911,8 +983,10 @@ proto_tree_set_string(field_info *fi, const char* value)
 static void
 proto_tree_set_string_tvb(field_info *fi, tvbuff_t *tvb, gint start, gint length)
 {
-	/* This g_strdup'ed memory is freed in proto_tree_free_node() */
-	fi->value.string = tvb_memdup(tvb, start, length);
+	/* This memory is freed in proto_tree_free_node() */
+	fi->value.string = g_malloc(length + 1);
+	tvb_memcpy(tvb, fi->value.string, start, length);
+	fi->value.string[length] = '\0';
 }
 
 /* Add a FT_ETHER to a proto_tree */
@@ -1304,6 +1378,21 @@ proto_tree_add_pi(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint
 	if (!tree)
 		return(NULL);
 
+	fi = alloc_field_info(hfindex, tvb, start, length);
+	pi = proto_tree_add_node(tree, fi);
+
+	if (pfi) {
+		*pfi = fi;
+	}
+
+	return pi;
+}
+
+static field_info *
+alloc_field_info(int hfindex, tvbuff_t *tvb, gint start, gint length)
+{
+	field_info	*fi;
+
 	fi = g_mem_chunk_alloc(gmc_field_info);
 
 	g_assert(hfindex >= 0 && hfindex < gpa_hfinfo->len);
@@ -1318,13 +1407,7 @@ proto_tree_add_pi(proto_tree *tree, int hfindex, tvbuff_t *tvb, gint start, gint
 	fi->visible = proto_tree_is_visible;
 	fi->representation = NULL;
 
-	pi = proto_tree_add_node(tree, fi);
-
-	if (pfi) {
-		*pfi = fi;
-	}
-
-	return pi;
+	return fi;
 }
 
 /* Set representation of a proto_tree entry, if the protocol tree is to
@@ -1361,6 +1444,13 @@ proto_item_set_len(proto_item *pi, gint length)
 	fi->length = length;
 }
 
+int
+proto_item_get_len(proto_item *pi)
+{
+	field_info *fi = (field_info*) (((GNode*)pi)->data);
+	return fi->length;
+}
+
 proto_tree*
 proto_tree_create_root(void)
 {
@@ -1391,10 +1481,45 @@ proto_register_protocol(char *name, char *abbrev)
 	hfinfo->bitshift = 0;
 	hfinfo->blurb = "";
 	hfinfo->parent = -1; /* this field differentiates protos and fields */
+	hfinfo->display = TRUE; /* XXX protocol is enabled by default */
 
 	return proto_register_field_init(hfinfo, hfinfo->parent);
 }
 
+
+/*
+ * XXX - In the future, we might need a hash table or list of procotol
+ * characteristics that will be fill in each time proto_register_protocol is 
+ * called.
+ * A protocol entry could contain the display flag among others (such as the
+ * address of the dissector function for intance). The access to an entry
+ * by protocol abbrev (which shall be unique) would be faster than the actual
+ * way.
+ */
+
+gboolean 
+proto_is_protocol_enabled(int n)
+{
+	struct header_field_info *hfinfo;
+
+	hfinfo = proto_registrar_get_nth(n);
+	if (hfinfo)
+		return (hfinfo->display);
+	else
+		return FALSE;
+
+}
+
+void 
+proto_set_decoding(int n, gboolean enabled)
+{
+	struct header_field_info *hfinfo;
+
+	hfinfo = proto_registrar_get_nth(n);
+	if (hfinfo)
+		hfinfo->display = enabled;
+}
+
 /* for use with static arrays only, since we don't allocate our own copies
 of the header_field_info struct contained withing the hf_register_info struct */
 void
@@ -1534,31 +1659,31 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 
 		case FT_DOUBLE:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %g", fi->hfinfo->name,
+				"%s: %g", hfinfo->name,
 				fi->value.floating);
 			break;
 
 		case FT_ABSOLUTE_TIME:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s", fi->hfinfo->name,
+				"%s: %s", hfinfo->name,
 				abs_time_to_str(&fi->value.time));
 			break;
 
 		case FT_RELATIVE_TIME:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s seconds", fi->hfinfo->name,
+				"%s: %s seconds", hfinfo->name,
 				rel_time_to_str(&fi->value.time));
 			break;
 
 		case FT_IPXNET:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: 0x%08X (%s)", fi->hfinfo->name,
+				"%s: 0x%08X (%s)", hfinfo->name,
 				fi->value.numeric, get_ipxnet_name(fi->value.numeric));
 			break;
 
 		case FT_ETHER:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s (%s)", fi->hfinfo->name,
+				"%s: %s (%s)", hfinfo->name,
 				ether_to_str(fi->value.ether),
 				get_ether_name(fi->value.ether));
 			break;
@@ -1566,26 +1691,31 @@ proto_item_fill_label(field_info *fi, gchar *label_str)
 		case FT_IPv4:
 			n_addr = ipv4_get_net_order_addr(&fi->value.ipv4);
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s (%s)", fi->hfinfo->name,
+				"%s: %s (%s)", hfinfo->name,
 				get_hostname(n_addr),
 				ip_to_str((guint8*)&n_addr));
 			break;
 
 		case FT_IPv6:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s (%s)", fi->hfinfo->name,
+				"%s: %s (%s)", hfinfo->name,
 				get_hostname6((struct e_in6_addr *)fi->value.ipv6),
 				ip6_to_str((struct e_in6_addr*)fi->value.ipv6));
 			break;
 	
 		case FT_STRING:
+		case FT_STRINGZ:
+		case FT_UINT_STRING:
 			snprintf(label_str, ITEM_LABEL_LENGTH,
-				"%s: %s", fi->hfinfo->name, fi->value.string);
+				"%s: %s", hfinfo->name, fi->value.string);
 			break;
 
 		default:
-			g_error("hfinfo->type %d not handled\n", fi->hfinfo->type);
-			break;
+                        g_error("hfinfo->type %d (%s) not handled\n",
+					hfinfo->type,
+					proto_registrar_ftype_name(hfinfo->type));
+                        g_assert_not_reached();
+                        break;
 	}
 }
 
@@ -2027,6 +2157,8 @@ proto_registrar_get_length(int n)
 		case FT_BYTES:
 		case FT_BOOLEAN:
 		case FT_STRING:
+		case FT_STRINGZ:
+		case FT_UINT_STRING:
 		case FT_DOUBLE:
 		case FT_ABSOLUTE_TIME:
 		case FT_RELATIVE_TIME:
@@ -2060,40 +2192,53 @@ proto_registrar_get_length(int n)
 	return -1;
 }
 
-/* Looks for a protocol or a field in a proto_tree. Returns TRUE if
- * it exists anywhere, or FALSE if it exists nowhere. */
-gboolean
-proto_check_for_protocol_or_field(proto_tree* tree, int id)
+
+/* =================================================================== */
+/* used when calling proto search functions */
+typedef struct {
+	int			target;
+	int			parent;
+	const guint8		*packet_data;
+	guint			packet_len;
+	gboolean		halt_on_first_hit;
+	GNodeTraverseFunc	traverse_func; /* for traverse_subtree_for_field() */
+	union {
+		GPtrArray		*ptr_array;
+		GNode			*node;
+	} 			result;
+} proto_tree_search_info;
+
+/* Looks for a protocol at the top layer of the tree. The protocol can occur
+ * more than once, for those encapsulated protocols. For each protocol subtree
+ * that is found, the callback function is called.
+ */
+static void
+proto_find_protocol_multi(proto_tree* tree, int target, GNodeTraverseFunc callback,
+			proto_tree_search_info *sinfo)
 {
-	proto_tree_search_info	sinfo;
+	g_assert(callback != NULL);
+	g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2, callback, (gpointer*)sinfo);
+}
 
-	sinfo.target = id;
-	sinfo.result.node = NULL;
-	sinfo.parent = -1;
-	sinfo.traverse_func = NULL;
+/* Calls a traversal function for all subtrees where:
+ * 1. Subtree is immediate child of root node. That is, subtree is a "protocol"
+ * 2. Subtree has finfo such that finfo->hfinfo->id == sinfo->parent
+ */
+static gboolean
+traverse_subtree_for_field(GNode *node, gpointer data)
+{
+	field_info		*fi = (field_info*) (node->data);
+	proto_tree_search_info	*sinfo = (proto_tree_search_info*) data;
 
-	/* do a quicker check if target is a protocol */
-	if (proto_registrar_is_protocol(id) == TRUE) {
-		proto_find_protocol_multi(tree, id, &check_for_protocol_or_field_id, &sinfo);
+	if (fi) { /* !fi == the top most container node which holds nothing */
+		if (fi->hfinfo->id == sinfo->parent) {
+			g_node_traverse(node, G_IN_ORDER, G_TRAVERSE_ALL, -1,
+					sinfo->traverse_func, sinfo);
+			if (sinfo->result.node)
+				return sinfo->halt_on_first_hit; /* halt? continue? */
+		}
 	}
-	else {
-		/* find the field's parent protocol */
-		sinfo.parent = proto_registrar_get_parent(id);
-
-		/* Go through each protocol subtree, checking if the protocols
-		 * is the parent protocol of the field that we're looking for.
-		 * We may have protocols that occur more than once (e.g., IP in IP),
-		 * so we do indeed have to check all protocol subtrees, looking
-		 * for the parent protocol. That's why proto_find_protocol()
-		 * is not used --- it assumes a protocol occurs only once. */
-		g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2,
-						check_for_field_within_protocol, &sinfo);
-	}
-
-	if (sinfo.result.node)
-		return TRUE;
-	else
-		return FALSE;
+	return FALSE; /* keep traversing */
 }
 
 static gboolean
@@ -2111,46 +2256,99 @@ check_for_protocol_or_field_id(GNode *node, gpointer data)
 	return FALSE; /* keep traversing */
 }
 
+/* Looks for a protocol or a field in a proto_tree. Returns TRUE if
+ * it exists anywhere, or FALSE if it exists nowhere. */
+gboolean
+proto_check_for_protocol_or_field(proto_tree* tree, int id)
+{
+	proto_tree_search_info	sinfo;
+
+	sinfo.target		= id;
+	sinfo.result.node	= NULL;
+	sinfo.parent		= -1;
+	sinfo.traverse_func	= check_for_protocol_or_field_id;
+	sinfo.halt_on_first_hit	= TRUE;
+
+	/* do a quicker check if target is a protocol */
+	if (proto_registrar_is_protocol(id) == TRUE) {
+		proto_find_protocol_multi(tree, id, check_for_protocol_or_field_id, &sinfo);
+	}
+	else {
+		/* find the field's parent protocol */
+		sinfo.parent = proto_registrar_get_parent(id);
+
+		/* Go through each protocol subtree, checking if the protocols
+		 * is the parent protocol of the field that we're looking for.
+		 * We may have protocols that occur more than once (e.g., IP in IP),
+		 * so we do indeed have to check all protocol subtrees, looking
+		 * for the parent protocol. That's why proto_find_protocol()
+		 * is not used --- it assumes a protocol occurs only once. */
+		g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2,
+						traverse_subtree_for_field, &sinfo);
+	}
+
+	if (sinfo.result.node)
+		return TRUE;
+	else
+		return FALSE;
+}
+
+
+
 static gboolean
-check_for_field_within_protocol(GNode *node, gpointer data)
+get_finfo_ptr_array(GNode *node, gpointer data)
 {
 	field_info		*fi = (field_info*) (node->data);
 	proto_tree_search_info	*sinfo = (proto_tree_search_info*) data;
 
 	if (fi) { /* !fi == the top most container node which holds nothing */
-		if (fi->hfinfo->id == sinfo->parent) {
-			g_node_traverse(node, G_IN_ORDER, G_TRAVERSE_ALL, -1,
-					check_for_protocol_or_field_id, sinfo);
-			if (sinfo->result.node)
-				return TRUE; /* halt traversal */
+		if (fi->hfinfo->id == sinfo->target) {
+			if (!sinfo->result.ptr_array) {
+				sinfo->result.ptr_array = g_ptr_array_new();
+			}
+			g_ptr_array_add(sinfo->result.ptr_array,
+					(gpointer)fi);
+			return FALSE; /* keep traversing */
 		}
 	}
 	return FALSE; /* keep traversing */
 }
 
-/* Looks for a protocol at the top layer of the tree. The protocol can occur
- * more than once, for those encapsulated protocols. For each protocol subtree
- * that is found, the callback function is called.
- */
-void
-proto_find_protocol_multi(proto_tree* tree, int target, GNodeTraverseFunc callback,
-			proto_tree_search_info *sinfo)
+/* Return GPtrArray* of field_info pointers for all hfindex that appear in tree
+ * (we assume that a field will only appear under its registered parent's subtree) */
+GPtrArray*
+proto_get_finfo_ptr_array(proto_tree *tree, int id)
 {
-	g_assert(callback != NULL);
-	g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2, callback, (gpointer*)sinfo);
-}
+	proto_tree_search_info	sinfo;
 
-/* Simple wrappter to traverse all nodes, calling the sinfo traverse function with sinfo as an arg */
-gboolean
-proto_get_field_values(proto_tree* subtree, proto_tree_search_info *sinfo)
-{
-	/* Don't try to check value of top-level NULL GNode */
-	if (!((GNode*)subtree)->data) {
-		return FALSE; /* don't halt */
+	sinfo.target		= id;
+	sinfo.result.ptr_array	= NULL;
+	sinfo.parent		= -1;
+	sinfo.traverse_func	= get_finfo_ptr_array;
+	sinfo.halt_on_first_hit	= FALSE;
+
+	/* do a quicker check if target is a protocol */
+	if (proto_registrar_is_protocol(id) == TRUE) {
+		proto_find_protocol_multi(tree, id, get_finfo_ptr_array, &sinfo);
 	}
-	g_node_traverse((GNode*)subtree, G_IN_ORDER, G_TRAVERSE_ALL, -1, sinfo->traverse_func, (gpointer*)sinfo);
-	return FALSE; /* don't halt */
+	else {
+		/* find the field's parent protocol */
+		sinfo.parent = proto_registrar_get_parent(id);
+
+		/* Go through each protocol subtree, checking if the protocols
+		 * is the parent protocol of the field that we're looking for.
+		 * We may have protocols that occur more than once (e.g., IP in IP),
+		 * so we do indeed have to check all protocol subtrees, looking
+		 * for the parent protocol. That's why proto_find_protocol()
+		 * is not used --- it assumes a protocol occurs only once. */
+		sinfo.traverse_func = get_finfo_ptr_array;
+		g_node_traverse((GNode*)tree, G_IN_ORDER, G_TRAVERSE_ALL, 2,
+						traverse_subtree_for_field, &sinfo);
+	}
+
+	return sinfo.result.ptr_array;
 }
+	
 
 /* Dumps the contents of the registration database to stdout. An indepedent program can take
  * this output and format it into nice tables or HTML or whatever.
@@ -2192,73 +2390,91 @@ proto_registrar_dump(void)
 			parent_hfinfo = proto_registrar_get_nth(hfinfo->parent);
 			g_assert(parent_hfinfo);
 
-			switch(hfinfo->type) {
-			case FT_NONE:
-				enum_name = "FT_NONE";
-				break;
-			case FT_BOOLEAN:
-				enum_name = "FT_BOOLEAN";
-				break;
-			case FT_UINT8:
-				enum_name = "FT_UINT8";
-				break;
-			case FT_UINT16:
-				enum_name = "FT_UINT16";
-				break;
-			case FT_UINT24:
-				enum_name = "FT_UINT24";
-				break;
-			case FT_UINT32:
-				enum_name = "FT_UINT32";
-				break;
-			case FT_INT8:
-				enum_name = "FT_INT8";
-				break;
-			case FT_INT16:
-				enum_name = "FT_INT16";
-				break;
-			case FT_INT24:
-				enum_name = "FT_INT24";
-				break;
-			case FT_INT32:
-				enum_name = "FT_INT32";
-				break;
-			case FT_DOUBLE:
-				enum_name = "FT_DOUBLE";
-				break;
-			case FT_ABSOLUTE_TIME:
-				enum_name = "FT_ABSOLUTE_TIME";
-				break;
-			case FT_RELATIVE_TIME:
-				enum_name = "FT_RELATIVE_TIME";
-				break;
-			case FT_STRING:
-				enum_name = "FT_STRING";
-				break;
-			case FT_ETHER:
-				enum_name = "FT_ETHER";
-				break;
-			case FT_BYTES:
-				enum_name = "FT_BYTES";
-				break;
-			case FT_IPv4:
-				enum_name = "FT_IPv4";
-				break;
-			case FT_IPv6:
-				enum_name = "FT_IPv6";
-				break;
-			case FT_IPXNET:
-				enum_name = "FT_IPXNET";
-				break;
-			case FT_TEXT_ONLY:
-				enum_name = "FT_TEXT_ONLY";
-				break;
-			default:
-				enum_name = "UNKNOWN";
-				break;
-			}
+			enum_name = proto_registrar_ftype_name(hfinfo->type);
 			printf("F\t%s\t%s\t%s\t%s\n", hfinfo->name, hfinfo->abbrev,
 				enum_name,parent_hfinfo->abbrev);
 		}
 	}
 }
+
+
+/* Returns a string representing the field type */
+const char*
+proto_registrar_ftype_name(enum ftenum ftype)
+{
+	const char	*enum_name = NULL;
+
+	switch(ftype) {
+		case FT_NONE:
+			enum_name = "FT_NONE";
+			break;
+		case FT_BOOLEAN:
+			enum_name = "FT_BOOLEAN";
+			break;
+		case FT_UINT8:
+			enum_name = "FT_UINT8";
+			break;
+		case FT_UINT16:
+			enum_name = "FT_UINT16";
+			break;
+		case FT_UINT24:
+			enum_name = "FT_UINT24";
+			break;
+		case FT_UINT32:
+			enum_name = "FT_UINT32";
+			break;
+		case FT_INT8:
+			enum_name = "FT_INT8";
+			break;
+		case FT_INT16:
+			enum_name = "FT_INT16";
+			break;
+		case FT_INT24:
+			enum_name = "FT_INT24";
+			break;
+		case FT_INT32:
+			enum_name = "FT_INT32";
+			break;
+		case FT_DOUBLE:
+			enum_name = "FT_DOUBLE";
+			break;
+		case FT_ABSOLUTE_TIME:
+			enum_name = "FT_ABSOLUTE_TIME";
+			break;
+		case FT_RELATIVE_TIME:
+			enum_name = "FT_RELATIVE_TIME";
+			break;
+		case FT_UINT_STRING:
+			enum_name = "FT_UINT_STRING";
+			break;
+		case FT_STRING:
+			enum_name = "FT_STRING";
+			break;
+		case FT_STRINGZ:
+			enum_name = "FT_STRINGZ";
+			break;
+		case FT_ETHER:
+			enum_name = "FT_ETHER";
+			break;
+		case FT_BYTES:
+			enum_name = "FT_BYTES";
+			break;
+		case FT_IPv4:
+			enum_name = "FT_IPv4";
+			break;
+		case FT_IPv6:
+			enum_name = "FT_IPv6";
+			break;
+		case FT_IPXNET:
+			enum_name = "FT_IPXNET";
+			break;
+		case FT_TEXT_ONLY:
+			enum_name = "FT_TEXT_ONLY";
+			break;
+		case NUM_FIELD_TYPES:
+			g_assert_not_reached();
+			break;
+	}
+	g_assert(enum_name);
+	return enum_name;
+}
