@@ -1,7 +1,7 @@
 /* proto.h
  * Definitions for protocol display
  *
- * $Id: proto.h,v 1.34 2000/05/31 18:32:07 gram Exp $
+ * $Id: proto.h,v 1.42 2000/08/30 02:50:03 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -28,22 +28,21 @@
 #define __PROTO_H__
 
 #ifdef HAVE_SYS_TIME_H
-# ifndef _SYS_TIME_H
-#  include <sys/time.h>
-# endif
+# include <sys/time.h>
+#endif
+
+#ifdef HAVE_STDARG_H
+# include <stdarg.h>
+#else
+# include <varargs.h>
 #endif
 
 #ifdef HAVE_WINSOCK_H
 # include <winsock.h>
 #endif
 
-#ifndef __IPV4_H__
 #include "ipv4.h"
-#endif
-
-#ifndef __TVBUFF_H__
 #include "tvbuff.h"
-#endif
 
 /* needs glib.h */
 typedef GNode proto_tree;
@@ -59,6 +58,21 @@ struct value_string;
 /* ... and similarly, */
 #define TFS(x)	(struct true_false_string*)(x)
 
+/* check protocol activation */
+#define OLD_CHECK_DISPLAY_AS_DATA(index, pd, offset, fd, tree) {\
+	if (!proto_is_protocol_enabled(index)) {		\
+		old_dissect_data(pd, offset, fd, tree);		\
+		return;						\
+	}							\
+  }
+
+#define CHECK_DISPLAY_AS_DATA(index, tvb, pinfo, tree) {	\
+	if (!proto_is_protocol_enabled(index)) {		\
+		dissect_data(tvb, pinfo, tree);			\
+		return;						\
+	}							\
+  }
+
 /* field types */
 enum ftenum {
 	FT_NONE,	/* used for protocol labels (thus no field type) */
@@ -75,6 +89,8 @@ enum ftenum {
 	FT_ABSOLUTE_TIME,
 	FT_RELATIVE_TIME,
 	FT_STRING,
+	FT_STRINGZ,	/* for use with proto_tree_add_item() */
+	FT_UINT_STRING,	/* for use with proto_tree_add_item() */
 	FT_ETHER,
 	FT_BYTES,
 	FT_IPv4,
@@ -139,20 +155,6 @@ typedef struct field_info {
 } field_info;
 
 
-/* used when calling proto search functions */
-typedef struct proto_tree_search_info {
-	int			target;
-	int			parent;
-	const guint8		*packet_data;
-	guint			packet_len;
-	GNodeTraverseFunc	traverse_func;
-	union {
-		GArray			*array;
-		GNode			*node;
-	} 			result;
-} proto_tree_search_info;
-
-
 /* For use while converting dissectors to use tvbuff's */
 #define NullTVB NULL
 
@@ -173,6 +175,10 @@ void proto_item_set_text(proto_item *ti, const char *format, ...);
 /* Set length of proto_item after having already been created. */
 void proto_item_set_len(proto_item *ti, gint length);
 
+/* Get length of proto_item. Useful after using proto_tree_add_item()
+ * to add a variable-length field (e.g., FT_NSTRING_UINT8) */
+int proto_item_get_len(proto_item *ti);
+
 /* Creates new proto_tree root */
 proto_tree* proto_tree_create_root(void);
 
@@ -449,12 +455,26 @@ proto_tree_add_text(proto_tree *tree, tvbuff_t *tvb, gint start, gint length, co
 	...);
 #endif
 
+proto_item *
+proto_tree_add_text_valist(proto_tree *tree, tvbuff_t *tvb, gint start,
+	gint length, const char *format, va_list ap);
 
 /* Add a node with no text */
 proto_item *
 proto_tree_add_notext(proto_tree *tree, tvbuff_t *tvb, gint start, gint length);
 
 
+/* Useful for quick debugging. Also sends string to STDOUT, so don't
+ * leave call to this function in production code. */
+#if __GNUC__ == 2
+proto_item *
+proto_tree_add_debug_text(proto_tree *tree, const char *format, ...)
+	__attribute__((format (printf, 2, 3)));
+#else
+proto_item *
+proto_tree_add_debug_text(proto_tree *tree, const char *format, ...);
+#endif
+
 void
 proto_item_fill_label(field_info *fi, gchar *label_str);
 
@@ -480,24 +500,27 @@ int proto_registrar_get_parent(int n);
 /* Is item #n a protocol? */
 gboolean proto_registrar_is_protocol(int n);
 
+/* Is item #n decoding enabled ? */
+gboolean proto_is_protocol_enabled(int n);
+
+/* Enable / Disable protocol */
+void proto_set_decoding(int n, gboolean enabled);
+
 /* Get length of registered field according to field type.
  * 0 means undeterminable at registration time.
  * -1 means unknown field */
 gint proto_registrar_get_length(int n);
 
 /* Checks for existence any protocol or field within a tree.
+ * "Protocols" are assumed to be a child of the [empty] root node.
  * TRUE = found, FALSE = not found */
 gboolean proto_check_for_protocol_or_field(proto_tree* tree, int id);
 
-/* Search for a protocol subtree, which can occur more than once, and for each successful
- * find, call the calback function, passing sinfo as the second argument */
-void proto_find_protocol_multi(proto_tree* tree, int target, GNodeTraverseFunc callback,
-			proto_tree_search_info *sinfo);
-
-/* Just a wrapper to call sinfo->traverse_func() for all nodes in the subtree, with the GNode
- * and sinfo as the two arguments to sinfo->traverse_func(). Useful when you have to process
- * all nodes in a subtree. */
-gboolean proto_get_field_values(proto_tree* subtree, proto_tree_search_info *sinfo);
+/* Return GPtrArray* of field_info pointers for all hfindex that appear in
+ * tree. Assume that a field will only appear under its registered parent's
+ * subtree, and that the parent's subtree is a child of the
+ * [empty] root node. */
+GPtrArray* proto_get_finfo_ptr_array(proto_tree *tree, int hfindex);
 
 /* Dumps a glossary of the protocol and field registrations to STDOUT */
 void proto_registrar_dump(void);
@@ -519,4 +542,12 @@ extern gboolean	     *tree_is_expanded;
 /* Number of elements in that array. */
 extern int           num_tree_types;
 
+/* glib doesn't have g_ptr_array_len of all things!*/
+#ifndef g_ptr_array_len
+#define g_ptr_array_len(a)      ((a)->len)
+#endif
+
+/* Returns a string representing the field type */
+const char* proto_registrar_ftype_name(enum ftenum ftype);
+
 #endif /* proto.h */
