@@ -1,7 +1,7 @@
 /* resolv.c
  * Routines for network object lookup
  *
- * $Id: resolv.c,v 1.23 2000/01/29 16:41:14 gram Exp $
+ * $Id: resolv.c,v 1.28 2000/08/19 08:26:03 guy Exp $
  *
  * Laurent Deniel <deniel@worldnet.fr>
  *
@@ -69,6 +69,10 @@
 # include <setjmp.h>
 #endif
 
+#ifdef NEED_INET_ATON_H
+# include "inet_aton.h"
+#endif
+
 #ifdef NEED_INET_V6DEFS_H
 # include "inet_v6defs.h"
 #endif
@@ -92,6 +96,7 @@
 typedef struct hashname {
   u_int			addr;
   u_char   		name[MAXNAMELEN];
+  gboolean              is_dummy_entry;	/* name is IP address in dot format */
   struct hashname 	*next;
 } hashname_t;
 
@@ -110,7 +115,7 @@ typedef struct hashmanuf {
 typedef struct hashether {
   u_char 		addr[6];
   char 			name[MAXNAMELEN];
-  gboolean		is_name_from_file;
+  gboolean		is_dummy_entry;		/* not a complete entry */
   struct hashether     	*next;
 } hashether_t;
 
@@ -133,6 +138,7 @@ typedef struct _ipxnet
 static hashname_t 	*host_table[HASHHOSTSIZE];
 static hashname_t 	*udp_port_table[HASHPORTSIZE];
 static hashname_t 	*tcp_port_table[HASHPORTSIZE];
+static hashname_t       *sctp_port_table[HASHPORTSIZE];
 static hashether_t 	*eth_table[HASHETHSIZE];
 static hashmanuf_t 	*manuf_table[HASHMANUFSIZE];
 static hashipxnet_t 	*ipxnet_table[HASHIPXNETSIZE];
@@ -157,7 +163,7 @@ gchar *g_manuf_path   = EPATH_MANUF;	/* may only be changed before the   */
  *  Local function definitions 
  */
 
-static u_char *serv_name_lookup(u_int port, u_int proto)
+static u_char *serv_name_lookup(u_int port, port_type proto)
 {
 
   hashname_t *tp;
@@ -167,14 +173,18 @@ static u_char *serv_name_lookup(u_int port, u_int proto)
   int i;
 
   switch(proto) {
-  case IPPROTO_UDP:
+  case PT_UDP:
     table = udp_port_table;
     serv_proto = "udp";
     break;
-  case IPPROTO_TCP:
+  case PT_TCP:
     table = tcp_port_table;
     serv_proto = "tcp";
     break;
+  case PT_SCTP:
+    table = sctp_port_table;
+    serv_proto = "sctp";
+    break;
   default:
     /* not yet implemented */
     return NULL;
@@ -206,7 +216,8 @@ static u_char *serv_name_lookup(u_int port, u_int proto)
   tp->addr = port;
   tp->next = NULL;
 
-  if ((servp = getservbyport(htons(port), serv_proto)) == NULL) {
+  if (!g_resolving_actif || 
+      (servp = getservbyport(htons(port), serv_proto)) == NULL) {
     /* unknown port */
     sprintf(tp->name, "%d", port);
   } else {
@@ -230,13 +241,15 @@ static void abort_network_query(int sig)
 }
 #endif /* AVOID_DNS_TIMEOUT */
 
-static u_char *host_name_lookup(u_int addr)
+static u_char *host_name_lookup(u_int addr, gboolean *found)
 {
 
   hashname_t * volatile tp;
   hashname_t **table = host_table;
   struct hostent *hostp;
 
+  *found = TRUE;
+
   tp = table[ addr & (HASHHOSTSIZE - 1)];
 
   if( tp == NULL ) {
@@ -245,6 +258,8 @@ static u_char *host_name_lookup(u_int addr)
   } else {  
     while(1) {
       if( tp->addr == addr ) {
+	if (tp->is_dummy_entry)
+	  *found = FALSE;
 	return tp->name;
       }
       if (tp->next == NULL) {
@@ -260,64 +275,73 @@ static u_char *host_name_lookup(u_int addr)
   tp->addr = addr;
   tp->next = NULL;
 
+  if (g_resolving_actif) {
 #ifdef AVOID_DNS_TIMEOUT
     
-  /* Quick hack to avoid DNS/YP timeout */
+    /* Quick hack to avoid DNS/YP timeout */
   
-  if (!setjmp(hostname_env)) {
-    signal(SIGALRM, abort_network_query);
-    alarm(DNS_TIMEOUT);
+    if (!setjmp(hostname_env)) {
+      signal(SIGALRM, abort_network_query);
+      alarm(DNS_TIMEOUT);
 #endif
-    hostp = gethostbyaddr((char *)&addr, 4, AF_INET);
+      hostp = gethostbyaddr((char *)&addr, 4, AF_INET);
 #ifdef AVOID_DNS_TIMEOUT
-    alarm(0);
+      alarm(0);
 #endif
-    if (hostp != NULL) {
-      strncpy(tp->name, hostp->h_name, MAXNAMELEN);
-      tp->name[MAXNAMELEN-1] = '\0';
-      return tp->name;
+      if (hostp != NULL) {
+	strncpy(tp->name, hostp->h_name, MAXNAMELEN);
+	tp->name[MAXNAMELEN-1] = '\0';
+	tp->is_dummy_entry = FALSE;
+	return tp->name;
+      }
+#ifdef AVOID_DNS_TIMEOUT
     }
-#ifdef AVOID_DNS_TIMEOUT
-  }
 #endif
+  }
 
   /* unknown host or DNS timeout */
 
   sprintf(tp->name, "%s", ip_to_str((guint8 *)&addr));  
+  tp->is_dummy_entry = TRUE;
+  *found = FALSE;
 
   return (tp->name);
 
 } /* host_name_lookup */
 
-static u_char *host_name_lookup6(struct e_in6_addr *addr)
+static u_char *host_name_lookup6(struct e_in6_addr *addr, gboolean *found)
 {
   static u_char name[MAXNAMELEN];
 #ifdef INET6
   struct hostent *hostp;
 
+  if (g_resolving_actif) {
 #ifdef AVOID_DNS_TIMEOUT
     
-  /* Quick hack to avoid DNS/YP timeout */
-  
-  if (!setjmp(hostname_env)) {
-    signal(SIGALRM, abort_network_query);
-    alarm(DNS_TIMEOUT);
+    /* Quick hack to avoid DNS/YP timeout */
+    
+    if (!setjmp(hostname_env)) {
+      signal(SIGALRM, abort_network_query);
+      alarm(DNS_TIMEOUT);
 #endif /* AVOID_DNS_TIMEOUT */
-    hostp = gethostbyaddr((char *)addr, sizeof(*addr), AF_INET6);
+      hostp = gethostbyaddr((char *)addr, sizeof(*addr), AF_INET6);
 #ifdef AVOID_DNS_TIMEOUT
-    alarm(0);
+      alarm(0);
 #endif
-    if (hostp != NULL) {
-      strncpy(name, hostp->h_name, MAXNAMELEN);
-      name[MAXNAMELEN-1] = '\0';
-      return name;
+      if (hostp != NULL) {
+	strncpy(name, hostp->h_name, MAXNAMELEN);
+	name[MAXNAMELEN-1] = '\0';
+	*found = TRUE;
+	return name;
+      }
+#ifdef AVOID_DNS_TIMEOUT
     }
-#ifdef AVOID_DNS_TIMEOUT
-  }
 #endif
+  }
 
   /* unknown host or DNS timeout */
 #endif /* INET6 */
+  *found = FALSE;
   sprintf(name, "%s", ip6_to_str(addr));  
   return (name);
 }
@@ -484,7 +508,7 @@ static ether_t *get_ethent(int six_bytes)
 
 } /* get_ethent */
 
-static ether_t *get_ethbyname(u_char *name)
+static ether_t *get_ethbyname(const u_char *name)
 {
   ether_t *eth;
   
@@ -585,10 +609,6 @@ static void initialize_ethers(void)
 {
   ether_t *eth;
 
-#ifdef DEBUG_RESOLV
-  signal(SIGSEGV, SIG_IGN);
-#endif
-
   /* Set g_pethers_path here, but don't actually do anything
    * with it. It's used in get_ethbyname() and get_ethbyaddr()
    */
@@ -611,7 +631,7 @@ static void initialize_ethers(void)
 
 } /* initialize_ethers */
 
-static hashether_t *add_eth_name(u_char *addr, u_char *name)
+static hashether_t *add_eth_name(const u_char *addr, const u_char *name)
 {
   hashether_t *tp;
   hashether_t **table = eth_table;
@@ -627,6 +647,15 @@ static hashether_t *add_eth_name(u_char *addr, u_char *name)
       (hashether_t *)g_malloc(sizeof(hashether_t));
   } else {  
     while(1) {
+      if (memcmp(tp->addr, addr, sizeof(tp->addr)) == 0) {
+	/* address already known */
+	if (!tp->is_dummy_entry) {
+	  return tp;
+	} else {
+	  /* replace this dummy (manuf) entry with a real name */
+	  break;
+	}
+      }
       if (tp->next == NULL) {
 	tp->next = (hashether_t *)g_malloc(sizeof(hashether_t));
 	tp = tp->next;
@@ -640,6 +669,7 @@ static hashether_t *add_eth_name(u_char *addr, u_char *name)
   strncpy(tp->name, name, MAXNAMELEN);
   tp->name[MAXNAMELEN-1] = '\0';
   tp->next = NULL;
+  tp->is_dummy_entry = FALSE;
 
   return tp;
 
@@ -689,19 +719,19 @@ static u_char *eth_name_lookup(const u_char *addr)
       sprintf(tp->name, "%s_%02x:%02x:%02x", 
 	      manufp->name, addr[3], addr[4], addr[5]);
 
-    tp->is_name_from_file = FALSE;
+    tp->is_dummy_entry = TRUE;
 
   } else {
     strncpy(tp->name, eth->name, MAXNAMELEN);
     tp->name[MAXNAMELEN-1] = '\0';
-    tp->is_name_from_file = TRUE;
+    tp->is_dummy_entry = FALSE;
   }
 
   return (tp->name);
 
 } /* eth_name_lookup */
 
-static u_char *eth_addr_lookup(u_char *name)
+static u_char *eth_addr_lookup(const u_char *name)
 {
   ether_t *eth;
   hashether_t *tp;
@@ -873,10 +903,6 @@ static ipxnet_t *get_ipxnetbyaddr(guint32 addr)
 static void initialize_ipxnets(void)
 {
 
-#ifdef DEBUG_RESOLV
-  signal(SIGSEGV, SIG_IGN);
-#endif
-
   /* Set g_pipxnets_path here, but don't actually do anything
    * with it. It's used in get_ipxnetbyname() and get_ipxnetbyaddr()
    */
@@ -1004,21 +1030,26 @@ static u_int ipxnet_addr_lookup(u_char *name, gboolean *success)
 
 extern u_char *get_hostname(u_int addr) 
 {
+  gboolean found;
+
   if (!g_resolving_actif)
     return ip_to_str((guint8 *)&addr);
 
-  return host_name_lookup(addr);
+  return host_name_lookup(addr, &found);
 }
 
 extern gchar *get_hostname6(struct e_in6_addr *addr) 
 {
+  gboolean found;
+
 #ifdef INET6
   if (!g_resolving_actif)
     return ip6_to_str(addr);
   if (IN6_IS_ADDR_LINKLOCAL(addr) || IN6_IS_ADDR_MULTICAST(addr))
     return ip6_to_str(addr);
 #endif
-  return host_name_lookup6(addr);
+
+  return host_name_lookup6(addr, &found);
 }
 
 extern void add_host_name(u_int addr, u_char *name)
@@ -1034,6 +1065,15 @@ extern void add_host_name(u_int addr, u_char *name)
       (hashname_t *)g_malloc(sizeof(hashname_t));
   } else {  
     while(1) {
+      if (tp->addr == addr) {
+	/* address already known */
+	if (!tp->is_dummy_entry) {
+	  return;
+	} else {
+	  /* replace this dummy entry with the new one */
+	  break;
+	}
+      }
       if (tp->next == NULL) {
 	tp->next = (hashname_t *)g_malloc(sizeof(hashname_t));
 	tp = tp->next;
@@ -1047,6 +1087,7 @@ extern void add_host_name(u_int addr, u_char *name)
   tp->name[MAXNAMELEN-1] = '\0';
   tp->addr = addr;
   tp->next = NULL;
+  tp->is_dummy_entry = FALSE;
 
 } /* add_host_name */
 
@@ -1063,11 +1104,11 @@ extern u_char *get_udp_port(u_int port)
     } else {  
       cur = &str[0][0];
     }
-    sprintf(cur, "%d", port);
+    sprintf(cur, "%u", port);
     return cur;
   }
 
-  return serv_name_lookup(port, IPPROTO_UDP);
+  return serv_name_lookup(port, PT_UDP);
 
 } /* get_udp_port */
 
@@ -1084,14 +1125,35 @@ extern u_char *get_tcp_port(u_int port)
     } else {  
       cur = &str[0][0];
     }
-    sprintf(cur, "%d", port);
+    sprintf(cur, "%u", port);
     return cur;
   }
 
-  return serv_name_lookup(port, IPPROTO_TCP);
+  return serv_name_lookup(port, PT_TCP);
 
 } /* get_tcp_port */
 
+extern u_char *get_sctp_port(u_int port) 
+{
+  static gchar  str[3][MAXNAMELEN];
+  static gchar *cur;
+
+  if (!g_resolving_actif) {
+    if (cur == &str[0][0]) {
+      cur = &str[1][0];
+    } else if (cur == &str[1][0]) {  
+      cur = &str[2][0];
+    } else {  
+      cur = &str[0][0];
+    }
+    sprintf(cur, "%u", port);
+    return cur;
+  }
+
+  return serv_name_lookup(port, PT_SCTP);
+
+} /* get_sctp_port */
+
 extern u_char *get_ether_name(const u_char *addr)
 {
   if (!g_resolving_actif)
@@ -1145,7 +1207,7 @@ u_char *get_ether_name_if_known(const u_char *addr)
   else { 
     while(1) {
       if (memcmp(tp->addr, addr, sizeof(tp->addr)) == 0) {
-	      if (tp->is_name_from_file) {
+	      if (!tp->is_dummy_entry) {
 		/* A name was found, and its origin is an ethers file */
 		return tp->name;
 	      }
@@ -1181,6 +1243,23 @@ extern u_char *get_ether_addr(u_char *name)
 
 } /* get_ether_addr */
 
+extern void add_ether_byip(u_int ip, const u_char *eth)
+{
+
+  u_char *host;
+  gboolean found;
+
+  /* first check that IP address can be resolved */
+
+  if ((host = host_name_lookup(ip, &found)) == NULL)
+    return;
+  
+  /* ok, we can add this entry in the ethers hashtable */
+
+  if (found)
+    add_eth_name(eth, host);
+
+} /* add_ether_byip */
 
 extern u_char *get_ipxnet_name(const guint32 addr)
 {
