@@ -1,6 +1,6 @@
 /* tethereal.c
  *
- * $Id: tethereal.c,v 1.29 2000/05/19 23:06:12 gram Exp $
+ * $Id: tethereal.c,v 1.45 2000/09/10 06:44:37 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,23 +31,29 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <locale.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
 #include <errno.h>
+
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
+
 #include <signal.h>
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
@@ -77,7 +83,6 @@
 #include "column.h"
 #include "print.h"
 #include "resolv.h"
-#include "follow.h"
 #include "util.h"
 #include "ui_util.h"
 #include "conversation.h"
@@ -117,7 +122,7 @@ static void wtap_dispatch_cb_print(u_char *, const struct wtap_pkthdr *, int,
 static gchar *col_info(frame_data *, gint);
 
 packet_info  pi;
-capture_file cf;
+capture_file cfile;
 FILE        *data_out_file = NULL;
 guint        main_ctx, file_ctx;
 ts_type timestamp_type = RELATIVE;
@@ -130,14 +135,14 @@ print_usage(void)
   fprintf(stderr, "This is GNU t%s %s, compiled with %s\n", PACKAGE,
 	  VERSION, comp_info_str);
 #ifdef HAVE_LIBPCAP
-  fprintf(stderr, "t%s [ -vVh ] [ -c count ] [ -D ] [ -f <filter expression> ]\n", PACKAGE);
-  fprintf(stderr, "\t[ -F <capture file type> ] [ -i iface ] [ -n ] [ -r infile ]\n");
-  fprintf(stderr, "\t[ -R <filter expression> ] [ -s snaplen ] [ -t <time stamp format> ]\n");
-  fprintf(stderr, "\t[ -w savefile ] [ -x ]\n");
+  fprintf(stderr, "t%s [ -vVh ] [ -c count ] [ -f <capture filter> ]\n", PACKAGE);
+  fprintf(stderr, "\t[ -F <capture file type> ] [ -i interface ] [ -n ]\n");
+  fprintf(stderr, "\t[ -o <preference setting> ] ... [ -r infile ] [ -R <read filter> ]\n");
+  fprintf(stderr, "\t[ -s snaplen ] [ -t <time stamp format> ] [ -w savefile ] [ -x ]\n");
 #else
-  fprintf(stderr, "t%s [ -vVh ] [ -D ] [ -F <capture file type> ] [ -n ] [ -r infile ]\n", PACKAGE);
-  fprintf(stderr, "\t[ -R <filter expression> ] [ -t <time stamp format> ] [ -w savefile ]\n");
-  fprintf(stderr, "\t[ -x ]\n");
+  fprintf(stderr, "t%s [ -vVh ] [ -F <capture file type> ] [ -n ]\n", PACKAGE);
+  fprintf(stderr, "\t[ -o <preference setting> ] ... [ -r infile ] [ -R <read filter> ]\n");
+  fprintf(stderr, "\t[ -t <time stamp format> ] [ -w savefile ] [ -x ]\n");
 #endif
   fprintf(stderr, "Valid file type arguments to the \"-F\" flag:\n");
   for (i = 0; i < WTAP_NUM_FILE_TYPES; i++) {
@@ -161,7 +166,8 @@ main(int argc, char *argv[])
   extern char          pcap_version[];
 #endif
 #endif
-  char                *pf_path;
+  char                *gpf_path, *pf_path;
+  int                  gpf_open_errno, pf_open_errno;
   int                  err;
 #ifdef HAVE_LIBPCAP
   gboolean             capture_filter_specified = FALSE;
@@ -176,47 +182,63 @@ main(int argc, char *argv[])
   dfilter             *rfcode = NULL;
   e_prefs             *prefs;
 
+  /* Register all dissectors; we must do this before checking for the
+     "-G" flag, as the "-G" flag dumps a list of fields registered
+     by the dissectors, and we must do it before we read the preferences,
+     in case any dissectors register preferences. */
+  dissect_init();
+
+  /* Now register the preferences for any non-dissector modules.
+     We must do that before we read the preferences as well. */
+  prefs_register_modules();
+
   /* If invoked with the "-G" flag, we dump out a glossary of
      display filter symbols.
 
      We do this here to mirror what happens in the GTK+ version, although
      it's not necessary here. */
   if (argc >= 2 && strcmp(argv[1], "-G") == 0) {
-    dissect_init();
     proto_registrar_dump();
     exit(0);
   }
 
-  prefs = read_prefs(&pf_path);
+  /* Set the C-language locale to the native environment. */
+  setlocale(LC_ALL, "");
+
+  prefs = read_prefs(&gpf_open_errno, &gpf_path, &pf_open_errno, &pf_path);
+  if (gpf_path != NULL) {
+    fprintf(stderr, "Can't open global preferences file \"%s\": %s.\n", pf_path,
+        strerror(gpf_open_errno));
+  }
   if (pf_path != NULL) {
-    fprintf(stderr, "Can't open preferences file \"%s\": %s.\n", pf_path,
-        strerror(errno));
+    fprintf(stderr, "Can't open your preferences file \"%s\": %s.\n", pf_path,
+        strerror(pf_open_errno));
   }
     
   /* Initialize the capture file struct */
-  cf.plist		= NULL;
-  cf.plist_end		= NULL;
-  cf.wth		= NULL;
-  cf.filename		= NULL;
-  cf.user_saved		= FALSE;
-  cf.is_tempfile	= FALSE;
-  cf.rfcode		= NULL;
-  cf.dfilter		= NULL;
-  cf.dfcode		= NULL;
+  cfile.plist		= NULL;
+  cfile.plist_end		= NULL;
+  cfile.wth		= NULL;
+  cfile.filename		= NULL;
+  cfile.user_saved		= FALSE;
+  cfile.is_tempfile	= FALSE;
+  cfile.rfcode		= NULL;
+  cfile.dfilter		= NULL;
+  cfile.dfcode		= NULL;
 #ifdef HAVE_LIBPCAP
-  cf.cfilter		= g_strdup("");
+  cfile.cfilter		= g_strdup("");
 #endif
-  cf.iface		= NULL;
-  cf.save_file		= NULL;
-  cf.save_file_fd	= -1;
-  cf.snap		= WTAP_MAX_PACKET_SIZE;
-  cf.count		= 0;
-  cf.cinfo.num_cols	= prefs->num_cols;
-  cf.cinfo.col_fmt      = (gint *) g_malloc(sizeof(gint) * cf.cinfo.num_cols);
-  cf.cinfo.fmt_matx	= (gboolean **) g_malloc(sizeof(gboolean *) * cf.cinfo.num_cols);
-  cf.cinfo.col_width	= (gint *) g_malloc(sizeof(gint) * cf.cinfo.num_cols);
-  cf.cinfo.col_title    = (gchar **) g_malloc(sizeof(gchar *) * cf.cinfo.num_cols);
-  cf.cinfo.col_data	= (gchar **) g_malloc(sizeof(gchar *) * cf.cinfo.num_cols);
+  cfile.iface		= NULL;
+  cfile.save_file		= NULL;
+  cfile.save_file_fd	= -1;
+  cfile.snap		= WTAP_MAX_PACKET_SIZE;
+  cfile.count		= 0;
+  cfile.cinfo.num_cols	= prefs->num_cols;
+  cfile.cinfo.col_fmt      = (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
+  cfile.cinfo.fmt_matx	= (gboolean **) g_malloc(sizeof(gboolean *) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_width	= (gint *) g_malloc(sizeof(gint) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_title    = (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
+  cfile.cinfo.col_data	= (gchar **) g_malloc(sizeof(gchar *) * cfile.cinfo.num_cols);
 
   /* Assemble the compile-time options */
   snprintf(comp_info_str, 256,
@@ -262,7 +284,7 @@ main(int argc, char *argv[])
    );
     
   /* Now get our args */
-  while ((opt = getopt(argc, argv, "c:Df:F:hi:nr:R:s:t:vw:Vx")) != EOF) {
+  while ((opt = getopt(argc, argv, "c:Df:F:hi:no:r:R:s:t:vw:Vx")) != EOF) {
     switch (opt) {
       case 'c':        /* Capture xxx packets */
 #ifdef HAVE_LIBPCAP
@@ -272,13 +294,10 @@ main(int argc, char *argv[])
         arg_error = TRUE;
 #endif
         break;
-      case 'D':        /* Turn off DSCP printing */
-	g_ip_dscp_actif = FALSE;
-	break;
       case 'f':
 #ifdef HAVE_LIBPCAP
         capture_filter_specified = TRUE;
-	cf.cfilter = g_strdup(optarg);
+	cfile.cfilter = g_strdup(optarg);
 #else
         capture_option_specified = TRUE;
         arg_error = TRUE;
@@ -298,7 +317,7 @@ main(int argc, char *argv[])
         break;
       case 'i':        /* Use interface xxx */
 #ifdef HAVE_LIBPCAP
-        cf.iface = g_strdup(optarg);
+        cfile.iface = g_strdup(optarg);
 #else
         capture_option_specified = TRUE;
         arg_error = TRUE;
@@ -307,6 +326,21 @@ main(int argc, char *argv[])
       case 'n':        /* No name resolution */
 	g_resolving_actif = 0;
 	break;
+      case 'o':        /* Override preference from command line */
+        switch (prefs_set_pref(optarg)) {
+
+	case PREFS_SET_SYNTAX_ERR:
+          fprintf(stderr, "tethereal: Invalid -o flag \"%s\"\n", optarg);
+          exit(1);
+          break;
+
+        case PREFS_SET_NO_SUCH_PREF:
+          fprintf(stderr, "tethereal: -o flag \"%s\" specifies unknown preference\n",
+			optarg);
+          exit(1);
+          break;
+        }
+        break;
       case 'r':        /* Read capture file xxx */
         cf_name = g_strdup(optarg);
         break;
@@ -315,7 +349,7 @@ main(int argc, char *argv[])
         break;
       case 's':        /* Set the snapshot (capture) length */
 #ifdef HAVE_LIBPCAP
-        cf.snap = atoi(optarg);
+        cfile.snap = atoi(optarg);
 #else
         capture_option_specified = TRUE;
         arg_error = TRUE;
@@ -341,7 +375,7 @@ main(int argc, char *argv[])
         exit(0);
         break;
       case 'w':        /* Write to capture file xxx */
-        cf.save_file = g_strdup(optarg);
+        cfile.save_file = g_strdup(optarg);
 	break;
       case 'V':        /* Verbose */
         verbose = TRUE;
@@ -371,13 +405,18 @@ main(int argc, char *argv[])
 "tethereal: Capture filters were specified both with \"-f\" and with additional command-line arguments\n");
         exit(2);
       }
-      cf.cfilter = get_args_as_string(argc, argv, optind);
+      cfile.cfilter = get_args_as_string(argc, argv, optind);
 #else
       capture_option_specified = TRUE;
 #endif
     }
   }
 
+  /* Notify all registered modules that have had any of their preferences
+     changed either from one of the preferences file or from the command
+     line that its preferences have changed. */
+  prefs_apply_all();
+
 #ifndef HAVE_LIBPCAP
   if (capture_option_specified)
     fprintf(stderr, "This version of Tethereal was not built with support for capturing packets.\n");
@@ -386,25 +425,23 @@ main(int argc, char *argv[])
     print_usage();
 
   /* Build the column format array */  
-  for (i = 0; i < cf.cinfo.num_cols; i++) {
-    cf.cinfo.col_fmt[i] = get_column_format(i);
-    cf.cinfo.col_title[i] = g_strdup(get_column_title(i));
-    cf.cinfo.fmt_matx[i] = (gboolean *) g_malloc0(sizeof(gboolean) *
+  for (i = 0; i < cfile.cinfo.num_cols; i++) {
+    cfile.cinfo.col_fmt[i] = get_column_format(i);
+    cfile.cinfo.col_title[i] = g_strdup(get_column_title(i));
+    cfile.cinfo.fmt_matx[i] = (gboolean *) g_malloc0(sizeof(gboolean) *
       NUM_COL_FMTS);
-    get_column_format_matches(cf.cinfo.fmt_matx[i], cf.cinfo.col_fmt[i]);
-    if (cf.cinfo.col_fmt[i] == COL_INFO)
-      cf.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_INFO_LEN);
+    get_column_format_matches(cfile.cinfo.fmt_matx[i], cfile.cinfo.col_fmt[i]);
+    if (cfile.cinfo.col_fmt[i] == COL_INFO)
+      cfile.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_INFO_LEN);
     else
-      cf.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+      cfile.cinfo.col_data[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
   }
 
-  if (cf.snap < 1)
-    cf.snap = WTAP_MAX_PACKET_SIZE;
-  else if (cf.snap < MIN_PACKET_SIZE)
-    cf.snap = MIN_PACKET_SIZE;
+  if (cfile.snap < 1)
+    cfile.snap = WTAP_MAX_PACKET_SIZE;
+  else if (cfile.snap < MIN_PACKET_SIZE)
+    cfile.snap = MIN_PACKET_SIZE;
   
-  dissect_init();   /* Init anything that needs initializing */
-
   if (rfilter != NULL) {
     if (dfilter_compile(rfilter, &rfcode) != 0) {
       fprintf(stderr, "tethereal: %s\n", dfilter_error_msg);
@@ -412,14 +449,14 @@ main(int argc, char *argv[])
       exit(2);
     }
   }
-  cf.rfcode = rfcode;
+  cfile.rfcode = rfcode;
   if (cf_name) {
-    err = open_cap_file(cf_name, FALSE, &cf);
+    err = open_cap_file(cf_name, FALSE, &cfile);
     if (err != 0) {
       dissect_cleanup();
       exit(2);
     }
-    err = load_cap_file(&cf, out_file_type);
+    err = load_cap_file(&cfile, out_file_type);
     if (err != 0) {
       dissect_cleanup();
       exit(2);
@@ -430,7 +467,7 @@ main(int argc, char *argv[])
        do we have support for live captures? */
 #ifdef HAVE_LIBPCAP
     /* Yes; did the user specify an interface to use? */
-    if (cf.iface == NULL) {
+    if (cfile.iface == NULL) {
         /* No - pick the first one from the list of interfaces. */
         if_list = get_interface_list(&err, err_str);
         if (if_list == NULL) {
@@ -447,7 +484,7 @@ main(int argc, char *argv[])
             }
             exit(2);
         }
-        cf.iface = g_strdup(if_list->data);	/* first interface */
+        cfile.iface = g_strdup(if_list->data);	/* first interface */
         free_interface_list(if_list);
     }
     capture(packet_count, out_file_type);
@@ -474,6 +511,10 @@ capture(int packet_count, int out_file_type)
   void        (*oldhandler)(int);
   int         err, inpkts;
   char        errmsg[1024+1];
+#ifndef _WIN32
+  static const char ppamsg[] = "can't find PPA for ";
+  char       *libpcap_warn;
+#endif
 
   /* Initialize the table of conversations. */
   conversation_init();
@@ -485,34 +526,67 @@ capture(int packet_count, int out_file_type)
   ld.pdh            = NULL;
 
   /* Open the network interface to capture from it. */
-  ld.pch = pcap_open_live(cf.iface, cf.snap, 1, 1000, err_str);
+  ld.pch = pcap_open_live(cfile.iface, cfile.snap, 1, 1000, err_str);
 
   if (ld.pch == NULL) {
-    /* Well, we couldn't start the capture.
-       If this is a child process that does the capturing in sync
-       mode or fork mode, it shouldn't do any UI stuff until we pop up the
-       capture-progress window, and, since we couldn't start the
-       capture, we haven't popped it up. */
+    /* Well, we couldn't start the capture. */
+#ifdef _WIN32
+    /* On Win32 OSes, the capture devices are probably available to all
+       users; don't warn about permissions problems.
+
+       Do, however, warn that Token Ring and PPP devices aren't supported. */
+    snprintf(errmsg, sizeof errmsg,
+	"The capture session could not be initiated (%s).\n"
+	"Please check that you have the proper interface specified.\n"
+	"\n"
+	"Note that the driver Tethereal uses for packet capture on Windows\n"
+	"doesn't support capturing on Token Ring interfaces, and doesn't\n"
+	"support capturing on PPP/WAN interfaces in Windows NT/2000.\n",
+	err_str);
+#else
+      /* If we got a "can't find PPA for XXX" message, warn the user (who
+         is running Ethereal on HP-UX) that they don't have a version
+	 of libpcap patched to properly handle HP-UX (the patched version
+	 says "can't find /dev/dlpi PPA for XXX" rather than "can't find
+	 PPA for XXX"). */
+      if (strncmp(err_str, ppamsg, sizeof ppamsg - 1) == 0)
+	libpcap_warn =
+	  "\n\n"
+	  "You are running Tethereal with a version of the libpcap library\n"
+	  "that doesn't handle HP-UX network devices well; this means that\n"
+	  "Tethereal may not be able to capture packets.\n"
+	  "\n"
+	  "To fix this, you will need to download the source to Tethereal\n"
+	  "from ethereal.zing.org if you have not already done so, read\n"
+	  "the instructions in the \"README.hpux\" file in the source\n"
+	  "distribution, download the source to libpcap if you have not\n"
+	  "already done so, patch libpcap as per the instructions, rebuild\n"
+	  "and install libpcap, and then build Tethereal (if you have already\n"
+	  "built Tethereal from source, do a \"make distclean\" and re-run\n"
+	  "configure before building).";
+      else
+	libpcap_warn = "";
     snprintf(errmsg, sizeof errmsg,
       "The capture session could not be initiated (%s).\n"
       "Please check to make sure you have sufficient permissions, and that\n"
-      "you have the proper interface specified.", err_str);
+      "you have the proper interface specified.%s", err_str, libpcap_warn);
+#endif
     goto error;
   }
 
-  if (cf.cfilter) {
+  if (cfile.cfilter) {
     /* A capture filter was specified; set it up. */
-    if (pcap_lookupnet (cf.iface, &netnum, &netmask, err_str) < 0) {
+    if (pcap_lookupnet (cfile.iface, &netnum, &netmask, err_str) < 0) {
       snprintf(errmsg, sizeof errmsg,
         "Can't use filter:  Couldn't obtain netmask info (%s).", err_str);
       goto error;
     }
-    if (pcap_compile(ld.pch, &cf.fcode, cf.cfilter, 1, netmask) < 0) {
+    if (pcap_compile(ld.pch, &cfile.fcode, cfile.cfilter, 1, netmask) < 0) {
       snprintf(errmsg, sizeof errmsg, "Unable to parse filter string (%s).",
 	pcap_geterr(ld.pch));
       goto error;
     }
-    if (pcap_setfilter(ld.pch, &cf.fcode) < 0) {
+    if (pcap_setfilter(ld.pch, &cfile.fcode) < 0) {
       snprintf(errmsg, sizeof errmsg, "Can't install filter (%s).",
 	pcap_geterr(ld.pch));
       goto error;
@@ -520,51 +594,19 @@ capture(int packet_count, int out_file_type)
   }
 
   ld.linktype = wtap_pcap_encap_to_wtap_encap(pcap_datalink(ld.pch));
-  if (cf.save_file != NULL) {
+  if (cfile.save_file != NULL) {
     /* Set up to write to the capture file. */
     if (ld.linktype == WTAP_ENCAP_UNKNOWN) {
       strcpy(errmsg, "The network you're capturing from is of a type"
                " that Tethereal doesn't support.");
       goto error;
     }
-    ld.pdh = wtap_dump_open(cf.save_file, out_file_type,
+    ld.pdh = wtap_dump_open(cfile.save_file, out_file_type,
 		ld.linktype, pcap_snapshot(ld.pch), &err);
 
     if (ld.pdh == NULL) {
-      /* We couldn't set up to write to the capture file. */
-      switch (err) {
-
-      case WTAP_ERR_UNSUPPORTED_FILE_TYPE:
-        strcpy(errmsg, "Tethereal does not support writing capture files in that format.");
-        break;
-
-      case WTAP_ERR_UNSUPPORTED_ENCAP:
-      case WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED:
-        strcpy(errmsg, "Tethereal cannot save this capture in that format.");
-        break;
-
-      case WTAP_ERR_CANT_OPEN:
-        strcpy(errmsg, "The file to which the capture would be written"
-                 " couldn't be created for some unknown reason.");
-        break;
-
-      case WTAP_ERR_SHORT_WRITE:
-        strcpy(errmsg, "A full header couldn't be written to the file"
-                 " to which the capture would be written.");
-        break;
-
-      default:
-        if (err < 0) {
-          sprintf(errmsg, "The file to which the capture would be"
-                       " written (\"%s\") could not be opened: Error %d.",
-   			cf.save_file, err);
-        } else {
-          sprintf(errmsg, "The file to which the capture would be"
-                       " written (\"%s\") could not be opened: %s.",
- 			cf.save_file, strerror(err));
-        }
-        break;
-      }
+      snprintf(errmsg, sizeof errmsg, file_open_error_message(errno, TRUE),
+		cfile.save_file);
       goto error;
     }
   }
@@ -581,16 +623,21 @@ capture(int packet_count, int out_file_type)
 #endif
 
   /* Let the user know what interface was chosen. */
-  printf("Capturing on %s\n", cf.iface);
+  printf("Capturing on %s\n", cfile.iface);
 
   inpkts = pcap_loop(ld.pch, packet_count, capture_pcap_cb, (u_char *) &ld);
   pcap_close(ld.pch);
 
+  /* Send a newline if we were printing packet counts to stdout */
+  if (cfile.save_file != NULL) {
+    printf("\n");
+  }
+
   return TRUE;
 
 error:
-  g_free(cf.save_file);
-  cf.save_file = NULL;
+  g_free(cfile.save_file);
+  cfile.save_file = NULL;
   fprintf(stderr, "tethereal: %s\n", errmsg);
   if (ld.pch != NULL)
     pcap_close(ld.pch);
@@ -612,12 +659,11 @@ capture_pcap_cb(u_char *user, const struct pcap_pkthdr *phdr,
   whdr.len = phdr->len;
   whdr.pkt_encap = ld->linktype;
 
-  args.cf = &cf;
+  args.cf = &cfile;
   args.pdh = ld->pdh;
   if (ld->pdh) {
     wtap_dispatch_cb_write((u_char *)&args, &whdr, 0, NULL, pd);
-    cf.count++;
-    printf("\r%u ", cf.count);
+    printf("\r%u ", cfile.count);
     fflush(stdout);
   } else {
     wtap_dispatch_cb_print((u_char *)&args, &whdr, 0, NULL, pd);
@@ -753,17 +799,19 @@ fill_in_fdata(frame_data *fdata, capture_file *cf,
 
   fdata->next = NULL;
   fdata->prev = NULL;
-  fdata->pkt_len  = phdr->len;
-  fdata->cap_len  = phdr->caplen;
+  fdata->pfd = NULL;
+  fdata->num = cf->count;
+  fdata->pkt_len = phdr->len;
+  fdata->cap_len = phdr->caplen;
   fdata->file_off = offset;
+  fdata->cinfo = NULL;
   fdata->lnk_t = phdr->pkt_encap;
   fdata->abs_secs  = phdr->ts.tv_sec;
   fdata->abs_usecs = phdr->ts.tv_usec;
+  fdata->flags.passed_dfilter = 0;
   fdata->flags.encoding = CHAR_ASCII;
   fdata->flags.visited = 0;
-  fdata->cinfo = NULL;
-
-  fdata->num = cf->count;
+  fdata->flags.marked = 0;
 
   /* If we don't have the time stamp of the first packet in the
      capture, it's because this is the first packet.  Save the time
@@ -773,15 +821,6 @@ fill_in_fdata(frame_data *fdata, capture_file *cf,
     firstusec = fdata->abs_usecs;
   }
 
-  /* Get the time elapsed between the first packet and this packet. */
-  cf->esec = fdata->abs_secs - firstsec;
-  if (firstusec <= fdata->abs_usecs) {
-    cf->eusec = fdata->abs_usecs - firstusec;
-  } else {
-    cf->eusec = (fdata->abs_usecs + 1000000) - firstusec;
-    cf->esec--;
-  }
-  
   /* If we don't have the time stamp of the previous displayed packet,
      it's because this is the first displayed packet.  Save the time
      stamp of this packet as the time stamp of the previous displayed
@@ -792,18 +831,23 @@ fill_in_fdata(frame_data *fdata, capture_file *cf,
   }
 
   /* Get the time elapsed between the first packet and this packet. */
-  fdata->rel_secs = cf->esec;
-  fdata->rel_usecs = cf->eusec;
+  compute_timestamp_diff(&fdata->rel_secs, &fdata->rel_usecs,
+		fdata->abs_secs, fdata->abs_usecs, firstsec, firstusec);
+
+  /* If it's greater than the current elapsed time, set the elapsed time
+     to it (we check for "greater than" so as not to be confused by
+     time moving backwards). */
+  if (cf->esec < fdata->rel_secs
+	|| (cf->esec == fdata->rel_secs && cf->eusec < fdata->rel_usecs)) {
+    cf->esec = fdata->rel_secs;
+    cf->eusec = fdata->rel_usecs;
+  }
   
   /* Get the time elapsed between the previous displayed packet and
      this packet. */
+  compute_timestamp_diff(&fdata->del_secs, &fdata->del_usecs,
+		fdata->abs_secs, fdata->abs_usecs, prevsec, prevusec);
   fdata->del_secs = fdata->abs_secs - prevsec;
-  if (prevusec <= fdata->abs_usecs) {
-    fdata->del_usecs = fdata->abs_usecs - prevusec;
-  } else {
-    fdata->del_usecs = (fdata->abs_usecs + 1000000) - prevusec;
-    fdata->del_secs--;
-  }
   prevsec = fdata->abs_secs;
   prevusec = fdata->abs_usecs;
 
@@ -920,7 +964,7 @@ wtap_dispatch_cb_print(u_char *user, const struct wtap_pkthdr *phdr, int offset,
 }
 
 char *
-file_open_error_message(int err, int for_writing)
+file_open_error_message(int err, gboolean for_writing)
 {
   char *errmsg;
   static char errmsg_errno[1024+1];
@@ -928,7 +972,7 @@ file_open_error_message(int err, int for_writing)
   switch (err) {
 
   case WTAP_ERR_NOT_REGULAR_FILE:
-    errmsg = "The file \"%s\" is invalid.";
+    errmsg = "The file \"%s\" is a \"special file\" or socket or other non-regular file.";
     break;
 
   case WTAP_ERR_FILE_UNKNOWN_FORMAT:
@@ -984,9 +1028,14 @@ file_open_error_message(int err, int for_writing)
       errmsg = "You do not have permission to read the file \"%s\".";
     break;
 
+  case EISDIR:
+    errmsg = "\"%s\" is a directory (folder), not a file.";
+    break;
+
   default:
-    sprintf(errmsg_errno, "The file \"%%s\" could not be opened: %s.",
-				wtap_strerror(err));
+    snprintf(errmsg_errno, sizeof(errmsg_errno),
+	     "The file \"%%s\" could not be opened: %s.",
+	     wtap_strerror(err));
     errmsg = errmsg_errno;
     break;
   }
@@ -1045,7 +1094,6 @@ open_cap_file(char *fname, gboolean is_tempfile, capture_file *cf)
   cf->esec      = 0;
   cf->eusec     = 0;
   cf->snap      = wtap_snapshot_length(cf->wth);
-  cf->update_progbar = FALSE;
   cf->progbar_quantum = 0;
   cf->progbar_nextstep = 0;
   firstsec = 0, firstusec = 0;
