@@ -9,7 +9,7 @@
  * 		the data of a backing tvbuff, or can be a composite of
  * 		other tvbuffs.
  *
- * $Id: tvbuff.c,v 1.4 2000/05/29 08:57:42 guy Exp $
+ * $Id: tvbuff.c,v 1.17 2000/09/13 20:30:06 gram Exp $
  *
  * Copyright (c) 2000 by Gilbert Ramirez <gram@xiexie.org>
  *
@@ -33,43 +33,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#ifndef __TVBUFF_H__
-#include "tvbuff.h"
+#ifdef HAVE_CONFIG_H
+# include "config.h"
 #endif
 
 #include <string.h>
 
-/* Pointer versions of ntohs and ntohl.  Given a pointer to a member of a
- * byte array, returns the value of the two or four bytes at the pointer.
- * The pletoh[sl] versions return the little-endian representation.
- */
-
-#define pntohs(p)  ((guint16)                       \
-                    ((guint16)*((guint8 *)p+0)<<8|  \
-                     (guint16)*((guint8 *)p+1)<<0))
-
-#define pntohl(p)  ((guint32)*((guint8 *)p+0)<<24|  \
-                    (guint32)*((guint8 *)p+1)<<16|  \
-                    (guint32)*((guint8 *)p+2)<<8|   \
-                    (guint32)*((guint8 *)p+3)<<0)
-
-#define pntoh24(p) ((guint32)*((guint8 *)p+0)<<16|  \
-                    (guint32)*((guint8 *)p+1)<<8|  \
-                    (guint32)*((guint8 *)p+2)<<0)
-
-#define pletohs(p) ((guint16)                       \
-                    ((guint16)*((guint8 *)p+1)<<8|  \
-                     (guint16)*((guint8 *)p+0)<<0))
-
-#define pletohl(p) ((guint32)*((guint8 *)p+3)<<24|  \
-                    (guint32)*((guint8 *)p+2)<<16|  \
-                    (guint32)*((guint8 *)p+1)<<8|   \
-                    (guint32)*((guint8 *)p+0)<<0)
-
-#define pletoh24(p) ((guint32)*((guint8 *)p+2)<<16|  \
-                     (guint32)*((guint8 *)p+1)<<8|  \
-                     (guint32)*((guint8 *)p+0)<<0)
-
+#include "pint.h"
+#include "tvbuff.h"
+#include "strutil.h"
 
 typedef struct {
 	/* The backing tvbuff_t */
@@ -207,6 +179,15 @@ tvb_new(tvbuff_type type)
 	return tvb;
 }
 
+/* We accept a void* instead of a field_info* to satisfy CLEANUP_POP */
+static void
+tvb_free_void(void *tvb)
+{
+	tvb_free((tvbuff_t*)tvb);
+}
+
+
+
 void
 tvb_free(tvbuff_t* tvb)
 {
@@ -306,7 +287,10 @@ tvb_set_real_data(tvbuff_t* tvb, const guint8* data, guint length, gint reported
 {
 	g_assert(tvb->type == TVBUFF_REAL_DATA);
 	g_assert(!tvb->initialized);
-	g_assert(reported_length >= -1);
+
+	if (reported_length < -1) {
+		THROW(ReportedBoundsError);
+	}
 
 	tvb->real_data		= (gpointer) data;
 	tvb->length		= length;
@@ -320,8 +304,13 @@ tvb_new_real_data(const guint8* data, guint length, gint reported_length)
 	tvbuff_t	*tvb;
 
 	tvb = tvb_new(TVBUFF_REAL_DATA);
+
+	CLEANUP_PUSH(tvb_free_void, tvb);
+
 	tvb_set_real_data(tvb, data, length, reported_length);
 
+	CLEANUP_POP;
+
 	return tvb;
 }
 
@@ -454,6 +443,10 @@ tvb_set_subset(tvbuff_t *tvb, tvbuff_t *backing,
 	g_assert(tvb->type == TVBUFF_SUBSET);
 	g_assert(!tvb->initialized);
 
+	if (reported_length < -1) {
+		THROW(ReportedBoundsError);
+	}
+
 	check_offset_length(backing, backing_offset, backing_length,
 			&tvb->tvbuffs.subset.offset,
 			&tvb->tvbuffs.subset.length);
@@ -461,7 +454,7 @@ tvb_set_subset(tvbuff_t *tvb, tvbuff_t *backing,
 	tvb_increment_usage_count(backing, 1);
 	tvb->tvbuffs.subset.tvb		= backing;
 	tvb->length			= tvb->tvbuffs.subset.length;
-	g_assert(reported_length >= -1);
+
 	if (reported_length == -1) {
 		tvb->reported_length	= backing->reported_length - tvb->tvbuffs.subset.offset;
 	}
@@ -485,8 +478,13 @@ tvb_new_subset(tvbuff_t *backing, gint backing_offset, gint backing_length, gint
 	tvbuff_t	*tvb;
 
 	tvb = tvb_new(TVBUFF_SUBSET);
+
+	CLEANUP_PUSH(tvb_free_void, tvb);
+
 	tvb_set_subset(tvb, backing, backing_offset, backing_length, reported_length);
 
+	CLEANUP_POP;
+
 	return tvb;
 }
 
@@ -621,7 +619,7 @@ tvb_reported_length(tvbuff_t* tvb)
 
 
 
-guint8*
+static guint8*
 first_real_data_ptr(tvbuff_t *tvb)
 {
 	tvbuff_t	*member;
@@ -641,7 +639,7 @@ first_real_data_ptr(tvbuff_t *tvb)
 	return NULL;
 }
 
-int
+static int
 offset_from_real_beginning(tvbuff_t *tvb, int counter)
 {
 	tvbuff_t	*member;
@@ -746,6 +744,22 @@ ensure_contiguous(tvbuff_t *tvb, gint offset, gint length)
 	return NULL;
 }
 
+static const guint8*
+guint8_find(const guint8* haystack, size_t haystacklen, guint8 needle)
+{
+	const guint8	*b;
+	int		i;
+
+	for (b = haystack, i = 0; i < haystacklen; i++, b++) {
+		if (*b == needle) {
+			return b;
+		}
+	}
+
+	return NULL;
+}
+
+
 
 /************** ACCESSORS **************/
 
@@ -812,6 +826,7 @@ tvb_memcpy(tvbuff_t *tvb, guint8* target, gint offset, gint length)
 {
 	guint	abs_offset, abs_length;
 
+	g_assert(length >= -1);
 	check_offset_length(tvb, offset, length, &abs_offset, &abs_length);
 
 	if (tvb->real_data) {
@@ -874,6 +889,15 @@ tvb_get_ntohs(tvbuff_t *tvb, gint offset)
 	return pntohs(ptr);
 }
 
+guint32
+tvb_get_ntoh24(tvbuff_t *tvb, gint offset)
+{
+	guint8* ptr;
+
+	ptr = ensure_contiguous(tvb, offset, 3);
+	return pntoh24(ptr);
+}
+
 guint32
 tvb_get_ntohl(tvbuff_t *tvb, gint offset)
 {
@@ -883,14 +907,16 @@ tvb_get_ntohl(tvbuff_t *tvb, gint offset)
 	return pntohl(ptr);
 }
 
-guint32
-tvb_get_ntoh24(tvbuff_t *tvb, gint offset)
+#ifdef G_HAVE_GINT64
+guint64
+tvb_get_ntohll(tvbuff_t *tvb, gint offset)
 {
 	guint8* ptr;
 
-	ptr = ensure_contiguous(tvb, offset, 3);
-	return pntoh24(ptr);
+	ptr = ensure_contiguous(tvb, offset, sizeof(guint64));
+	return pntohll(ptr);
 }
+#endif
 
 guint16
 tvb_get_letohs(tvbuff_t *tvb, gint offset)
@@ -901,6 +927,15 @@ tvb_get_letohs(tvbuff_t *tvb, gint offset)
 	return pletohs(ptr);
 }
 
+guint32
+tvb_get_letoh24(tvbuff_t *tvb, gint offset)
+{
+	guint8* ptr;
+
+	ptr = ensure_contiguous(tvb, offset, 3);
+	return pletoh24(ptr);
+}
+
 guint32
 tvb_get_letohl(tvbuff_t *tvb, gint offset)
 {
@@ -910,11 +945,196 @@ tvb_get_letohl(tvbuff_t *tvb, gint offset)
 	return pletohl(ptr);
 }
 
-guint32
-tvb_get_letoh24(tvbuff_t *tvb, gint offset)
+#ifdef G_HAVE_GINT64
+guint64
+tvb_get_letohll(tvbuff_t *tvb, gint offset)
 {
 	guint8* ptr;
 
-	ptr = ensure_contiguous(tvb, offset, 3);
-	return pletoh24(ptr);
+	ptr = ensure_contiguous(tvb, offset, sizeof(guint64));
+	return pletohll(ptr);
+}
+#endif
+
+
+/* Find first occurence of needle in tvbuff, starting at offset. Searches
+ * at most maxlength number of bytes. Returns the offset of the found needle,
+ * or -1 if not found. Will not throw an exception, even if maxlength exceeds
+ * boundary of tvbuff; in that case, -1 will be returned if the boundary is
+ * reached before finding needle. */
+gint
+tvb_find_guint8(tvbuff_t *tvb, gint offset, guint maxlength, guint8 needle)
+{
+	guint		abs_offset, junk_length;
+	const guint8	*result;
+	guint		limit;
+
+	check_offset_length(tvb, offset, 0, &abs_offset, &junk_length);
+
+	/* Only search to end of tvbuff, w/o throwing exception. */
+	if (tvb_length_remaining(tvb, abs_offset) < maxlength) {
+		limit = maxlength - (tvb_length(tvb) - abs_offset);
+	}
+	else {
+		limit = maxlength;
+	}
+
+	/* If we have real data, perform our search now. */
+	if (tvb->real_data) {
+		result = guint8_find(tvb->real_data + abs_offset, limit, needle);
+		if (result == NULL) {
+			return -1;
+		}
+		else {
+			return result - tvb->real_data;
+		}
+	}
+
+	switch(tvb->type) {
+		case TVBUFF_REAL_DATA:
+			g_assert_not_reached();
+
+		case TVBUFF_SUBSET:
+			return tvb_find_guint8(tvb->tvbuffs.subset.tvb,
+					abs_offset - tvb->tvbuffs.subset.offset,
+					limit, needle);
+
+		case TVBUFF_COMPOSITE:
+			g_assert_not_reached();
+			/* XXX - return composite_find_guint8(tvb, offset, limit, needle); */
+	}
+
+	g_assert_not_reached();
+	return -1;
+}
+
+/* Find length of string by looking for end of string ('\0'), up to
+ * 'max_length' characters'. Returns -1 if 'max_length' reached
+ * before finding EOS. */
+gint
+tvb_strnlen(tvbuff_t *tvb, gint offset, guint maxlength)
+{
+	gint	result_offset;
+	guint	abs_offset, junk_length;
+
+	check_offset_length(tvb, offset, 0, &abs_offset, &junk_length);
+
+	result_offset = tvb_find_guint8(tvb, abs_offset, maxlength, 0);
+
+	if (result_offset == -1) {
+		return -1;
+	}
+	else {
+		return result_offset;
+	}
+}
+
+/*
+ * Implement strneql etc
+ */
+
+/* Call strncmp after checking if enough chars left, otherwise return -1 */
+gint
+tvb_strneql(tvbuff_t *tvb, gint offset, guint8 *str, gint size)
+{
+  guint8 *ptr;
+
+  ptr = ensure_contiguous(tvb, offset, size);
+
+  if (ptr) {
+
+    int cmp = strncmp(ptr, str, size);
+
+    return (cmp == 0 ? 0 : -1);  /* Make it -1 if comparison failed */
+
+  }
+  else {
+
+    return -1;   /* Not enough chars in the TVB */
+
+  }
+
+}
+
+/*
+ * Format the data in the tvb from offset for length ...
+ */
+
+guint8 *
+tvb_format_text(tvbuff_t *tvb, gint offset, gint size)
+{
+  guint8 *ptr;
+  gint len = size;
+
+  if ((ptr = ensure_contiguous(tvb, offset, size)) == NULL) {
+
+    len = tvb_length_remaining(tvb, offset);
+    ptr = ensure_contiguous(tvb, offset, len);
+
+  }
+
+  return format_text(ptr, len);
+ 
+}
+
+/* Looks for a stringz (NUL-terminated string) in tvbuff and copies
+ * no more than maxlength number of bytes, including terminating NUL, to buffer.
+ * Returns length of string (not including terminating NUL), or -1 if the string was
+ * truncated in the buffer due to not having reached the terminating NUL.
+ * In this way, it acts like snprintf().
+ */
+gint
+tvb_get_nstringz(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer)
+{
+	gint	stringlen, NUL_offset;
+	guint	abs_offset, junk_length;
+	gint	limit;
+
+	check_offset_length(tvb, offset, 0, &abs_offset, &junk_length);
+
+	if (maxlength == 0) {
+		buffer[0] = 0;
+		return 0;
+	}
+
+	/* Only copy to end of tvbuff, w/o throwing exception. */
+	if (tvb_length_remaining(tvb, abs_offset) < maxlength) {
+		limit = maxlength - (tvb_length(tvb) - abs_offset);
+	}
+	else {
+		limit = maxlength;
+	}
+
+	NUL_offset = tvb_strnlen(tvb, abs_offset, limit);
+
+	/* If NUL wasn't found, copy the data and return -1 */
+	if (NUL_offset == -1) {
+		tvb_memcpy(tvb, buffer, abs_offset, limit);
+		return -1;
+	}
+
+	/* Copy the string to buffer */
+	stringlen = NUL_offset - abs_offset;
+	tvb_memcpy(tvb, buffer, abs_offset, stringlen + 1);
+	return stringlen;
+}
+
+/* Like tvb_get_nstringz(), but never returns -1. The string is guaranteed to
+ * have a terminating NUL. If the string was truncated when copied into buffer,
+ * a NUL is placed at the end of buffer to terminate it.
+ */
+gint
+tvb_get_nstringz0(tvbuff_t *tvb, gint offset, guint maxlength, guint8* buffer)
+{
+	gint	len;
+
+	len = tvb_get_nstringz(tvb, offset, maxlength, buffer);
+
+	if (len == -1) {
+		buffer[maxlength] = 0;
+		return maxlength - 1;
+	}
+	else {
+		return len;
+	}
 }
