@@ -1,7 +1,7 @@
 /* util.c
  * Utility routines
  *
- * $Id: util.c,v 1.39 2000/03/21 06:51:59 guy Exp $
+ * $Id: util.c,v 1.44 2000/09/10 06:44:38 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -47,11 +47,6 @@
 #endif
 
 #ifdef NEED_SNPRINTF_H
-# ifdef HAVE_STDARG_H
-#  include <stdarg.h>
-# else
-#  include <varargs.h>
-# endif
 # include "snprintf.h"
 #endif
 
@@ -92,6 +87,51 @@ typedef int mode_t;	/* for win32 */
 
 #endif
 
+/*
+ * Given a pathname, return:
+ *
+ *	the errno, if an attempt to "stat()" the file fails;
+ *
+ *	EISDIR, if the attempt succeeded and the file turned out
+ *	to be a directory;
+ *
+ *	0, if the attempt succeeded and the file turned out not
+ *	to be a directory.
+ */
+
+/*
+ * Visual C++ on Win32 systems doesn't define these.  (Old UNIX systems don't
+ * define them either.)
+ *
+ * Visual C++ on Win32 systems doesn't define S_IFIFO, it defines _S_IFIFO.
+ */
+#ifndef S_ISREG
+#define S_ISREG(mode)   (((mode) & S_IFMT) == S_IFREG)
+#endif
+#ifndef S_IFIFO
+#define S_IFIFO	_S_IFIFO
+#endif
+#ifndef S_ISFIFO
+#define S_ISFIFO(mode)  (((mode) & S_IFMT) == S_IFIFO)
+#endif
+#ifndef S_ISDIR
+#define S_ISDIR(mode)   (((mode) & S_IFMT) == S_IFDIR)
+#endif
+
+int
+test_for_directory(const char *path)
+{
+	struct stat statb;
+
+	if (stat(path, &statb) < 0)
+		return errno;
+
+	if (S_ISDIR(statb.st_mode))
+		return EISDIR;
+	else
+		return 0;
+}
+
 /*
  * Given a pathname, return a pointer to the last pathname separator
  * character in the pathname, or NULL if the pathname contains no
@@ -468,6 +508,8 @@ get_interface_list(int *err, char *err_str)
 	int     sock = socket(AF_INET, SOCK_DGRAM, 0);
 	struct search_user_data user_data;
 	pcap_t *pch;
+	int len, lastlen;
+	char *buf;
 
 	if (sock < 0) {
 		sprintf(err_str, "Error opening socket: %s",
@@ -476,19 +518,36 @@ get_interface_list(int *err, char *err_str)
 	}
 
 	/*
-	 * Since we have to grab the interface list all at once, we'll
-	 * make plenty of room.
+	 * This code came from: W. Richard Stevens: "UNIX Network Programming",
+	 * Networking APIs: Sockets and XTI, Vol 1, page 434.
 	 */
-	ifc.ifc_len = 1024 * sizeof(struct ifreq);
-	ifc.ifc_buf = malloc(ifc.ifc_len);
-
-	if (ioctl(sock, SIOCGIFCONF, &ifc) < 0 ||
-	    ifc.ifc_len < sizeof(struct ifreq)) {
-		sprintf(err_str, "SIOCGIFCONF error getting list of interfaces: %s",
-		    strerror(errno));
-		goto fail;
-	 }
-
+	lastlen = 0;
+	len = 100 * sizeof(struct ifreq);
+	for ( ; ; ) {
+		buf = g_malloc(len);
+		ifc.ifc_len = len;
+		ifc.ifc_buf = buf;
+		memset (buf, 0, len);
+		if (ioctl(sock, SIOCGIFCONF, &ifc) < 0) {
+			if (errno != EINVAL || lastlen != 0) {
+				sprintf(err_str,
+					"SIOCGIFCONF ioctl error getting list of interfaces: %s",
+					strerror(errno));
+				goto fail;
+			}
+		} else {
+			if (ifc.ifc_len < sizeof(struct ifreq)) {
+				sprintf(err_str,
+					"SIOCGIFCONF ioctl gave too small return buffer");
+				goto fail;
+			}
+			if (ifc.ifc_len == lastlen)
+				break;			/* success, len has not changed */
+			lastlen = ifc.ifc_len;
+		}
+		len += 10 * sizeof(struct ifreq);	/* increment */
+		g_free(buf);
+	}
 	ifr = (struct ifreq *) ifc.ifc_req;
 	last = (struct ifreq *) ((char *) ifr + ifc.ifc_len);
 	while (ifr < last) {
@@ -573,7 +632,7 @@ get_interface_list(int *err, char *err_str)
 #endif
 	}
 
-	free(ifc.ifc_buf);
+	g_free(ifc.ifc_buf);
 	close(sock);
 
 	if (il == NULL) {
@@ -589,7 +648,7 @@ fail:
 		g_list_foreach(il, free_if_cb, NULL);
 		g_list_free(il);
 	}
-	free(ifc.ifc_buf);
+	g_free(ifc.ifc_buf);
 	close(sock);
 	*err = CANT_GET_INTERFACE_LIST;
 	return NULL;
@@ -745,3 +804,47 @@ get_home_dir(void)
 
 	return home;
 }
+
+/* Compute the difference between two seconds/microseconds time stamps. */
+void
+compute_timestamp_diff(gint *diffsec, gint *diffusec,
+	guint32 sec1, guint32 usec1, guint32 sec2, guint32 usec2)
+{
+  if (sec1 == sec2) {
+    /* The seconds part of the first time is the same as the seconds
+       part of the second time, so if the microseconds part of the first
+       time is less than the microseconds part of the second time, the
+       first time is before the second time.  The microseconds part of
+       the delta should just be the difference between the microseconds
+       part of the first time and the microseconds part of the second
+       time; don't adjust the seconds part of the delta, as it's OK if
+       the microseconds part is negative. */
+
+    *diffsec = sec1 - sec2;
+    *diffusec = usec1 - usec2;
+  } else if (sec1 <= sec2) {
+    /* The seconds part of the first time is less than the seconds part
+       of the second time, so the first time is before the second time.
+
+       Both the "seconds" and "microseconds" value of the delta
+       should have the same sign, so if the difference between the
+       microseconds values would be *positive*, subtract 1,000,000
+       from it, and add one to the seconds value. */
+    *diffsec = sec1 - sec2;
+    if (usec2 >= usec1) {
+      *diffusec = usec1 - usec2;
+    } else {
+      *diffusec = (usec1 - 1000000) - usec2;
+      (*diffsec)++;
+    }
+  } else {
+    /* Oh, good, we're not caught in a chronosynclastic infindibulum. */
+    *diffsec = sec1 - sec2;
+    if (usec2 <= usec1) {
+      *diffusec = usec1 - usec2;
+    } else {
+      *diffusec = (usec1 + 1000000) - usec2;
+      (*diffsec)--;
+    }
+  }
+}
