@@ -1,6 +1,6 @@
 /* file.c
  *
- * $Id: file.c,v 1.53 2000/05/25 09:00:20 guy Exp $
+ * $Id: file.c,v 1.60 2000/08/11 13:32:36 deniel Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
@@ -25,10 +25,17 @@
 #endif
 
 #include <stdio.h>
-#include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+
 #include <errno.h>
 
 #ifdef HAVE_IO_H
@@ -50,6 +57,7 @@
 #include "netxray.h"
 #include "toshiba.h"
 #include "i4btrace.h"
+#include "csids.h"
 
 /* The open_file_* routines should return:
  *
@@ -94,8 +102,11 @@ static int (*open_routines[])(wtap *, int *) = {
 	ascend_open,
 	toshiba_open,
 	i4btrace_open,
+	csids_open,
 };
 
+#define	N_FILE_TYPES	(sizeof open_routines / sizeof open_routines[0])
+
 int wtap_def_seek_read(wtap *wth, int seek_off,
 	union wtap_pseudo_header *pseudo_header, guint8 *pd, int len)
 {
@@ -104,7 +115,24 @@ int wtap_def_seek_read(wtap *wth, int seek_off,
 	return file_read(pd, sizeof(guint8), len, wth->random_fh);
 }
 
-#define	N_FILE_TYPES	(sizeof open_routines / sizeof open_routines[0])
+/*
+ * Visual C++ on Win32 systems doesn't define these.  (Old UNIX systems don't
+ * define them either.)
+ *
+ * Visual C++ on Win32 systems doesn't define S_IFIFO, it defines _S_IFIFO.
+ */
+#ifndef S_ISREG
+#define S_ISREG(mode)   (((mode) & S_IFMT) == S_IFREG)
+#endif
+#ifndef S_IFIFO
+#define S_IFIFO	_S_IFIFO
+#endif
+#ifndef S_ISFIFO
+#define S_ISFIFO(mode)  (((mode) & S_IFMT) == S_IFIFO)
+#endif
+#ifndef S_ISDIR
+#define S_ISDIR(mode)   (((mode) & S_IFMT) == S_IFDIR)
+#endif
 
 /* Opens a file and prepares a wtap struct.
    If "do_random" is TRUE, it opens the file twice; the second open
@@ -124,12 +152,13 @@ wtap* wtap_open_offline(const char *filename, int *err, gboolean do_random)
 		*err = errno;
 		return NULL;
 	}
-#ifndef WIN32
 	if (! S_ISREG(statb.st_mode) && ! S_ISFIFO(statb.st_mode)) {
-		*err = WTAP_ERR_NOT_REGULAR_FILE;
+		if (S_ISDIR(statb.st_mode))
+			*err = EISDIR;
+		else
+			*err = WTAP_ERR_NOT_REGULAR_FILE;
 		return NULL;
 	}
-#endif
 
 	errno = ENOMEM;
 	wth = g_malloc(sizeof(wtap));
@@ -145,7 +174,8 @@ wtap* wtap_open_offline(const char *filename, int *err, gboolean do_random)
 
 	/* Open the file */
 	errno = WTAP_ERR_CANT_OPEN;
-	if (!(wth->fd = open(filename, O_RDONLY|O_BINARY))) {
+	wth->fd = open(filename, O_RDONLY|O_BINARY);
+	if (wth->fd < 0) {
 		*err = errno;
 		g_free(wth);
 		return NULL;
@@ -225,12 +255,16 @@ const static struct file_type_info {
 	{ "libpcap (tcpdump, Ethereal, etc.)", "libpcap",
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
-	/* WTAP_FILE_PCAP_MODIFIED */
-	{ "modified libpcap (tcpdump)", "modlibpcap",
+	/* WTAP_FILE_PCAP_SS990417 */
+	{ "Red Hat Linux 6.1 libpcap (tcpdump)", "rh6_1libpcap",
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
-	/* WTAP_FILE_PCAP_RH_6_1 */
-	{ "Red Hat Linux 6.1 libpcap (tcpdump)", "rh6_1libpcap",
+	/* WTAP_FILE_PCAP_SS990915 */
+	{ "SuSE Linux 6.3 libpcap (tcpdump)", "suse6_3libpcap",
+	  libpcap_dump_can_write_encap, libpcap_dump_open },
+
+	/* WTAP_FILE_PCAP_SS991029 */
+	{ "modified libpcap (tcpdump)", "modlibpcap",
 	  libpcap_dump_can_write_encap, libpcap_dump_open },
 
 	/* WTAP_FILE_LANALYZER */
@@ -273,8 +307,8 @@ const static struct file_type_info {
 	{ "Network Associates Sniffer (Windows-based) 1.1", "ngwsniffer_1_1",
 	  netxray_dump_can_write_encap, netxray_dump_open_1_1 },
 
-	/* WTAP_FILE_NETXRAY_2_001 */
-	{ "Network Associates Sniffer (Windows-based) 2.001", NULL,
+	/* WTAP_FILE_NETXRAY_2_00x */
+	{ "Network Associates Sniffer (Windows-based) 2.00x", NULL,
 	  NULL, NULL },
 
 	/* WTAP_FILE_RADCOM */
@@ -297,6 +331,10 @@ const static struct file_type_info {
 	{ "I4B ISDN trace", NULL,
 	  NULL, NULL },
 
+        /* WTAP_FILE_CSIDS */
+        { "CSIDS IPLog", NULL,
+          NULL, NULL },
+
 };
 
 /* Name that should be somewhat descriptive. */
