@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h,v 1.72 2000/05/25 09:00:24 guy Exp $
+ * $Id: wtap.h,v 1.78 2000/09/07 05:34:23 gram Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@xiexie.org>
@@ -103,34 +103,38 @@
 #define WTAP_FILE_UNKNOWN			0
 #define WTAP_FILE_WTAP				1
 #define WTAP_FILE_PCAP				2
-#define WTAP_FILE_PCAP_MODIFIED			3
-#define WTAP_FILE_PCAP_RH_6_1			4
-#define WTAP_FILE_LANALYZER			5
-#define WTAP_FILE_NGSNIFFER_UNCOMPRESSED	6
-#define WTAP_FILE_NGSNIFFER_COMPRESSED		7
-#define WTAP_FILE_SNOOP				8
-#define WTAP_FILE_IPTRACE_1_0			9
-#define WTAP_FILE_IPTRACE_2_0			10
-#define WTAP_FILE_NETMON_1_x			11
-#define WTAP_FILE_NETMON_2_x			12
-#define WTAP_FILE_NETXRAY_1_0			13
-#define WTAP_FILE_NETXRAY_1_1			14
-#define WTAP_FILE_NETXRAY_2_001			15
-#define WTAP_FILE_RADCOM			16
-#define WTAP_FILE_ASCEND			17
-#define WTAP_FILE_NETTL				18
-#define WTAP_FILE_TOSHIBA			19
-#define WTAP_FILE_I4BTRACE			20
+#define WTAP_FILE_PCAP_SS990417			3
+#define WTAP_FILE_PCAP_SS990915			4
+#define WTAP_FILE_PCAP_SS991029			5
+#define WTAP_FILE_LANALYZER			6
+#define WTAP_FILE_NGSNIFFER_UNCOMPRESSED	7
+#define WTAP_FILE_NGSNIFFER_COMPRESSED		8
+#define WTAP_FILE_SNOOP				9
+#define WTAP_FILE_IPTRACE_1_0			10
+#define WTAP_FILE_IPTRACE_2_0			11
+#define WTAP_FILE_NETMON_1_x			12
+#define WTAP_FILE_NETMON_2_x			13
+#define WTAP_FILE_NETXRAY_1_0			14
+#define WTAP_FILE_NETXRAY_1_1			15
+#define WTAP_FILE_NETXRAY_2_00x			16
+#define WTAP_FILE_RADCOM			17
+#define WTAP_FILE_ASCEND			18
+#define WTAP_FILE_NETTL				19
+#define WTAP_FILE_TOSHIBA			20
+#define WTAP_FILE_I4BTRACE			21
+#define WTAP_FILE_CSIDS				22
 
 /* last WTAP_FILE_ value + 1 */
-#define WTAP_NUM_FILE_TYPES			21
+#define WTAP_NUM_FILE_TYPES			23
 
 /*
  * Maximum packet size we'll support.
  */
 #define	WTAP_MAX_PACKET_SIZE			65535
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -269,8 +273,19 @@ typedef struct wtap_dumper wtap_dumper;
  * a negative number, indicating the type of error, on other failures.
  */
 struct wtap* wtap_open_offline(const char *filename, int *err, gboolean do_random);
-int wtap_loop(wtap *wth, int, wtap_handler, u_char*, int*);
-int wtap_read(wtap *wth, int *err);
+
+/* Returns TRUE if entire loop-reading was successful. If read failure
+ * happened, FALSE is returned and err is set. */
+gboolean wtap_loop(wtap *wth, int, wtap_handler, u_char*, int *err);
+
+/* Returns TRUE if read was successful. FALSE if failure. data_offset is
+ * set the the offset in the file where the data for the read packet is
+ * located. */
+gboolean wtap_read(wtap *wth, int *err, int *data_offset);
+
+struct wtap_pkthdr *wtap_phdr(wtap *wth);
+union wtap_pseudo_header *wtap_pseudoheader(wtap *wth);
+guint8 *wtap_buf_ptr(wtap *wth);
 
 FILE* wtap_file(wtap *wth);
 int wtap_fd(wtap *wth);
